#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   server_getAllFolders.py
#
#       サーバ側のfolder、solver名などを孫folderまで確認する
#
#   16/09/19    新規作成
#      10/31    importできるようにメインを修正
#

import os
import sys
import glob

def getSolverBC(selectDir):
    fileName = selectDir + "/system/controlDict"
    solver = ""
    BCb = ""
    BCc = ""
    if os.path.isfile(fileName) == True:
        f=open(fileName)
        flag = 0
        BCb = "a"; BCc = "n"
        for line in f.readlines():
            lineList = line.split()
            if len(lineList) >= 2:
                if lineList[0] == "application":
                    solver = getItemCont(lineList[1])
                    flag += 1
                elif lineList[0] == "writeFormat":
                    wformat = getItemCont(lineList[1])
                    if wformat == "binary":
                        BCb = "B"
                    else:
                        BCb = "a"
                    flag += 1
                elif lineList[0] == "writeCompression":
                    comp = getItemCont(lineList[1])
                    if comp == "compressed" or comp == "yes" or comp == "on":
                        BCc = "C"
                    else:
                        BCc = "n"
                    flag += 1
            if flag >= 3:
                break
        f.close()
    BC = BCb + BCc
    return [solver, BC]

def getItemCont(cont):
    if cont[-1] == ";":
        ans = cont[:-1]
    else:
        if len(cont) > 0:
            ans = cont
        else:
            ans = ""
    return ans

#
#  getNumberResult
#    計算結果folderの数を取得
def getNumberResult(selectDir):
    folders = getFoldersOnlyCurrDir(selectDir)
    ans = []
    for folder in folders:
        try:
            a = float(folder.split("/")[-1])
            ans.append(a)
        except:
            a = 1.0
    nRst = len(ans)
    if nRst < 2:
        for folder in folders:
            if folder.split("/")[-1] == "processor0":
                subFolders = getFoldersOnlyCurrDir(selectDir + "/processor0")
                ans = []
                for subFolder in subFolders:
                    try:
                        a = float(subFolder.split("/")[-1])
                        ans.append(a)
                    except:
                        a = 1.0
                nRst = len(ans)
                break
    #並列数を確認
    ip = 0
    for folder in folders:
        if folder.split("/")[-1][:len("processor")] == "processor":
            ip += 1
    #戻り値を設定
    ansR = []
    if nRst != 0:
        ansR.append(" " + str(nRst))
        ansR.append(" " + str(min(ans)))
        if min(ans) == max(ans):
            ansR.append("b")
        else:
            ansR.append(" " + str(max(ans)))
        if ip == 0:
            ansR.append(" b")
        else:
            ansR.append(str(ip))
    else:
        ansR = ["b", "b", "b", "b"]
    return ansR

#
#  checkFolder
#    folderのチェック
def checkFolder(folderDir):
    ans = ""
    if os.path.isdir(folderDir) == True:
        ans = "folder"
        folderName = folderDir.split("/")[-1]
        try:
            a=float(folderName)
            #数字フォルダ
            ans = "hide"
        except:
            if (folderName == "system" or
                folderName == "constant" or
                folderName == "solve.log.analyzed" or
                folderName == "TreeFoamServer"):
                #禁止フォルダ
                ans = "hide"
            elif folderName[0] == ".":
                ans = "hide"
            elif folderName.find(".log") >= 0:
                ans = "hide"
            elif folderName.find("log.") >= 0:
                ans = "hide"
            else:
                controlDict = folderDir + "/system/controlDict"
                if os.path.isfile(controlDict)==True:
                    #caseフォルダ
                    ans = "case"
                elif len(glob.glob(folderDir+"/*.hdf")) != 0:
                    ans = "salome" 
    else:
        #ファイル
        ans = "file"
    return ans

#
#　getFoldersOnlyCurrDir
#  　currDirのみのFolderを取得する
#
def getFoldersOnlyCurrDir(currDir):
    folders = []
    try:
        allItem = os.listdir(currDir)
        for item in allItem:
            newFolder = currDir + "/" + item
            if os.path.isdir(newFolder) == True:
                folders.append(newFolder)
    except:
        folders=[]
    return folders

#
#  writeResult
#    結果を書き出し
def writeResult(idxFolders):
    for idxFolder in idxFolders:
        line = " ".join(idxFolder)
        print line

#
#  getFoldersUntilGroundChildren
#    孫folderまでのfolderを取得する
def getFoldersUntilGroundChildren(selDir):
    cFolders = getFoldersOnlyCurrDir(selDir)
    folders = []
    for cFolder in cFolders:
        check = checkFolder(cFolder)
        if check != "hide":
            gFolders = getFoldersOnlyCurrDir(cFolder)
            folders += gFolders
    idxFolders = []
    for folder in folders:
        check = checkFolder(folder)
        if check != "hide":
            idx = [folder, check]
            if check == "case":
                [solver, BC] = getSolverBC(folder)
                nRst = getNumberResult(folder)
                idx += [solver, BC] + nRst
            idxFolders.append(idx)
    return idxFolders


#メイン
if __name__ == "__main__":
    selDir = sys.argv[1]
    selDir = os.path.expanduser(selDir)
    selDir = os.path.abspath(selDir)
    idxFolders = getFoldersUntilGroundChildren(selDir)
    writeResult(idxFolders)




