#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   server_getUntilSelSolveFolders.py
#
#       サーバ側のfolderとその内容を、selCase、solveCaseまで取得する
#
#   16/10/31    新規作成
#      11/02    folderを4世代まで取得する様に修正
#      12/19    rootDir/****部がselCaseDir or solveCaseDirの場合、正しくreload
#               できていなかったので、これを修正。
#

import os
import sys
import glob
import server_getAllFolders


#
#  deleteSameData
#    同じdataを削除
def deleteSameData(folders):
    folders.sort()
    newFolders = []
    newFolders.append(folders[0])
    i = 1
    while i < len(folders):
        if folders[i-1] != folders[i]:
            newFolders.append(folders[i])
        i += 1
    return newFolders

#
#  getFoldersUntilSelSolveCase
#    folderDirとその内容を取得
def getFoldersUntilSelSolveCase(rootDir, selCaseDir, solveCaseDir):

    #  getNeedFolders
    #    必要なfolderを取得する
    def getNeedFolders(folders):
        newFolders = []
        for folder in folders:
            res = server_getAllFolders.checkFolder(folder)
            if res == "folder" or res == "case" or res == "salome":
                newFolders.append(folder)
        return newFolders

    #  addSubFoldersInDirs
    #    folders内のsubFolderを追加して返す
    def addSubFoldersInDirs(folders):
        newFolders = []
        for folder in folders:
            subFolders = server_getAllFolders.getFoldersOnlyCurrDir(folder)
            subFolders = getNeedFolders(subFolders)
            for subFolder in subFolders:
                newFolders.append(subFolder.decode("utf-8"))
        newFolders += folders
        return newFolders

    #  getFoldersUntilCaseDir
    #    caseDirまでの全てのfolderを取得
    def getFoldersUntilCaseDir(caseDir, folderDir):
        newFolders = []
        i = 0
        currDir = ""
        for folder in folderDir:
            n = len(folder)
            if folder == caseDir[:n].encode("utf-8"):
                if i < n:
                    i = n
                    currDir = folder
        names = caseDir[len(currDir)+1:].split("/")
        for name in names:
            if name != "":
                currDir += "/" + name
            folders = server_getAllFolders.getFoldersOnlyCurrDir(currDir)
            folders = getNeedFolders(folders)
            for folder in folders:
                newFolders.append(folder.decode("utf-8"))
        #親folderを基準にしているので、もう1段下まで求めておく。
        newFolders = addSubFoldersInDirs(newFolders)
        return newFolders

    #rootDirのfoldersを取得
    folderDir = []
    folders = server_getAllFolders.getFoldersOnlyCurrDir(rootDir)
    folders = getNeedFolders(folders)
    for folder in folders:
        folderDir.append(folder)
    #folderDir.sort()
    #selCase系列のfolderを取得（selCaseの親folderから開始）
    if selCaseDir == rootDir:
        caseDir = rootDir
    else:
        caseDir = "/".join(selCaseDir.split("/")[:-1])
    newFolders = getFoldersUntilCaseDir(caseDir, folderDir)
    folderDir += newFolders
    #solveCase系列のfolderを取得
    if solveCaseDir == rootDir:
        caseDir = rootDir
    else:
        caseDir = "/".join(solveCaseDir.split("/")[:-1])
    newFolders = getFoldersUntilCaseDir(caseDir, folderDir)
    folderDir += newFolders
    #子folderを取得（自身のfolder）
    folderDir = addSubFoldersInDirs(folderDir)
    folderDir = deleteSameData(folderDir)
    #孫folderを取得
    folderDir = addSubFoldersInDirs(folderDir)
    folderDir = deleteSameData(folderDir)
    #曾孫folderを取得
    folderDir = addSubFoldersInDirs(folderDir)
    folderDir = deleteSameData(folderDir)
    #最終folderを取得
    folderDir = addSubFoldersInDirs(folderDir)
    folderDir = deleteSameData(folderDir)
    #folderの種類を設定
    folderConts = []
    for folder in folderDir:
        folType = server_getAllFolders.checkFolder(folder)
        if folType == "case" or folType == "salome" or folType == "folder":
            folderConts.append([folder, folType])
    #caseFolderに内容を設定
    for i in range(len(folderConts)):
        folderCont = folderConts[i]
        if folderCont[1] == "case":
            folder = folderCont[0]
            [solver, BC] = server_getAllFolders.getSolverBC(folder)
            nRst = server_getAllFolders.getNumberResult(folder)
            folderConts[i] += [solver, BC] + nRst
    return folderConts


#メイン
if __name__ == "__main__":
    selCaseDir = sys.argv[1]
    solveCaseDir = sys.argv[2]
    selCaseDir = os.path.expanduser(selCaseDir)
    selCaseDir = os.path.abspath(selCaseDir)
    solveCaseDir = os.path.expanduser(solveCaseDir)
    solveCaseDir = os.path.abspath(solveCaseDir)
    rootDir = "~"
    rootDir = os.path.expanduser(rootDir)
    rootDir = os.path.abspath(rootDir)
    idxFolders = getFoldersUntilSelSolveCase(rootDir, selCaseDir, solveCaseDir)
    server_getAllFolders.writeResult(idxFolders)




