#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       changeDictionaryAllRegions.py
#
#           changeDictionaryAllRegionsDictの内容に従って、各regionに
#           changeDictionaryRegionsDictを作り、各regionに対し、changeDictionaryを
#           実行してdictionaryを書き換える。
#
#   13/01/18    file名を受けて実行する様に修正
#      06/01    helpを追加
#      10/15    国際化のため、修正
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#

import os
import sys
import glob
import getFoamContents
import setFoamContents

import logFileCreater; logFileCreater.log()


caseDir = ""
contentsDict = ""

#
#  getEachRegions
#    AllregionsDictから書き換えるregionを取得
def getEachRegions(name):
    global caseDir, contentsDict
    fileName = caseDir + "/system/" + name
    f=open(fileName)
    contentsDict = f.read()
    f.close()
    p = getFoamContents.skipFoamFileMem(contentsDict, 0)
    regions = []
    while p<len(contentsDict):
        [[region, dummy], p] = getFoamContents.getNextKeywordAndContentsMem(contentsDict, p)
        if region != "":
            regions.append(region)
    return regions

#
#  setChangeDictionaryRegionsDict
#    各reginのchangeDictionaryDictを作成し、各regionにセット
def setChangeDictionaryRegionsDict(regions):
    global caseDir, contentsDict
    error = ""
    #errorチェック
    for name in regions:
        fileName = fileName = caseDir + "/system/" + name
        if len(glob.glob(fileName)) == 0:
            error = _(u"region「") + name + _(u"」が存在しません。")
            break
    if error != "":
        return error

    #foamFile作成
    foamFile = setFoamContents.makeFoamHeader()
    foamFile += setFoamContents.makeFoamFile("2.0", "ascii", "dictionary", "system", "changeDictionaryDict")
    footer = setFoamContents.makeFooter()

    #changeDictionaryDictを作成
    p=0
    for name in regions:
        [contents, p] = getFoamContents.getKeywordContentsMem(name, contentsDict, p)
        fileName = caseDir + "/system/" + name + "/changeDictionaryDict"
        f=open(fileName, "w")
        f.write(foamFile + contents + footer)
        f.close()
    return error

#
#  runEachChangeDictionary
#    各regionのchangeDictionaryを実行
def runEachChangeDictionary(regions):
    for name in regions:
        comm = "changeDictionary -region " + name
        print comm
        os.system(comm)

#
#  printHelp
#    helpを出力
def printHelp():
    cont  = _(u"\n*** changeDictionaryAllRegions.pyの使い方 ***\n\n")
    cont += _(u"<概要>\n")
    cont += _(u"multiRegionモデルの境界条件を設定するコマンド。\n")
    cont += _(u"境界条件は、dictFileの指示に従って設定される。\n\n")
    cont += _(u"<使い方>\n")
    cont += _(u"changeDictionaryAllRegionsDict.py [dictFile] [option]\n\n")
    cont += _(u"[dictFile]\n")
    cont += _(u"    dictFileの指示に従って境界条件を設定する。\n")
    cont += _(u"    省略時はchangeDictionaryAllRegionsDictが使われる\n\n")
    cont += _(u"[option]\n")
    cont += _(u"    -h          helpを表示\n")
    cont += _(u"    -help       　同上\n")
    cont += _(u"    --help        同上\n\n")
    cont += _(u"***********************************************\n")
    print cont.encode("utf-8")



#
#  メイン
#
import gettext
treeFoamPath = os.environ["TreeFoamPath"]
gettext.install("treefoam", treeFoamPath + "/data/locale")

if len(sys.argv) > 1:
    comm = sys.argv[1]
    if comm == "-h" or comm == "--help" or comm == "-help":
        printHelp()
        exit()
    else:
        fileName = comm
else:
    fileName = "changeDictionaryAllRegionsDict"
#  caseDirを取得
caseDir = os.getcwd()
#  変更するregionを取得
regions = getEachRegions(fileName)
#  changeDictionaryを作成し各regionにセット
error = setChangeDictionaryRegionsDict(regions)
#  changeDictionaryを実行
if error == "":
    runEachChangeDictionary(regions)
else:
    print error.encode("utf-8")

