#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   gridEditor
#
#       grid editorを起動する
#
#       使い方
#       gridEditor.py <caseDir> <timeFolder> <polyMeshFolder>
#       例：gridEditor.py ~/cavity 0 constant/polyMesh
#
#   11/08/11    新規作成
#   12/01/21    バグ修正
#   12/03/25    起動時に読み込むfieldを設定
#   13/04/19    fieldが無い場合でも起動するように修正
#      05/24    timeFolder取得方法を変更
#      10/15    国際化のため、修正
#

import os
import os.path
import sys
import glob
import string
import pyTreeFoam
import getFoamContents
import selectFieldsForEditDialog
import openGridEditorDialog



nTreeFoam = "-1"        #ここから起動したgridEditorのnTeeFoamは「-1」
TreeFoamPath = ""
caseDir = ""
editDir = ""            #fieldDir
polyMeshDir = ""


#
#  showGridEditor
#
def showGridEditor(editDir, polyMeshDir, fields):
    if len(fields) == 0:
        foldersFiles = pyTreeFoam.getFoldersFiles(editDir)
        fields = foldersFiles[1]

    #title, dirを設定
    caseFolder = editDir.split("/")[-2]
    timeFolder = editDir.split("/")[-1]
    title = ": " + caseFolder + "/" + timeFolder
    caseDir = string.join(editDir.split("/")[:-2], "/")
    #openGridEditorDialog.Show(nTreeFoam, title, "", "" , editDir, timeFolder)
    openGridEditorDialog.Show(nTreeFoam, title, "", editDir, timeFolder)


    """
    #Grid編集
    if len(fields) > 0:
        #title, dirを設定
        caseFolder = editDir.split("/")[-2]
        timeFolder = editDir.split("/")[-1]
        title = ": " + caseFolder + "/" + timeFolder
        caseDir = string.join(editDir.split("/")[:-2], "/")
        #openGridEditorDialog.Show(nTreeFoam, title, "", "" , editDir, timeFolder)
        openGridEditorDialog.Show(nTreeFoam, title, "", editDir, timeFolder)
    else:
        mess = u"error:Fieldが選択されていません。\n  Fieldを選択してください。"
        print mess
    """


#  getInstallFolder
#    TreeFoamのインストールpathを取得
def getInstallFolder():
    homeFolder = os.path.expanduser("~")
    fileName = homeFolder + "/.treeFoamInstallFolder"
    contents = getFileContents(fileName)[0]
    return contents

#  getFileContents
#    ファイル内容を読み込む
def getFileContents(fileName):
    f=open(fileName); contents=f.readlines(); f.close()
    return contents

#  setFileContents
#    ファイル内容を書き込む
def setFileContents(fileName, contents):
    f=open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()

#  checkDir
#    入力されたDirのチェック
def checkDir(a1, a2, a3):
    global caseDir, editDir, polyMeshDir
    error = ""
    caseDir = os.path.abspath(a1)
    editDir = os.path.abspath(a2)
    polyMeshDir = os.path.abspath(a3)
    if len(glob.glob(a1)) == 0:
        error = _(u"指定されたcaseDirは、存在していません。\n ") + caseDir
    elif len(glob.glob(a2)) == 0:
        error = _(u"指定されたeditDirは、存在していません。\n ") + editDir
    elif len(glob.glob(a3)) == 0:
        error = _(u"指定されたpolyMeshDirは、存在しません。\n ") + polyMeshDir
    if error != "":
        error = "error:" + error + "\n"
    return error

#  getTimeFolder
#    timeFolderを取得
def getTimeFolder(a1):
    caseDir = os.path.abspath(a1)
    timeFolder = getFoamContents.getCurrTimeFolder(caseDir)
    return timeFolder

#  checkFields
#    fieldsのチェック
def checkFields():
    global fields, editDir
    wFields = []
    for field in fields:
        fieldDir = editDir + "/" + field
        if len(glob.glob(fieldDir)) != 0:
            wFields.append(field)
        else:
            print _(u"「") + field + _(u"」が存在しません。")
    if len(wFields) == 0:
        print _(u"読み込むfieldが無かったので、全てのfieldを選択しました。")
    fields = wFields

#
#  getCommand
#    command取得
def getCommand():
    global dirCaseNum, fields
    comm = "run"
    i=1
    while i<len(sys.argv):
        val = sys.argv[i]
        if val == "-h" or val == "-help" or val == "--help":
            comm = "help"
            i = len(sys.argv)
        elif val == "-f" or val == "-fields" or val == "--fields":
            i += 1
            flag = 0
            fields = []
            while i<len(sys.argv) and flag==0:
                val = sys.argv[i]
                if val[0] != "-":
                    fields.append(val)
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        elif val == "-d" or val == "-directory" or val == "--directory":
            i += 1
            flag = 0
            dirCaseNum = []
            while i<len(sys.argv) and flag == 0:
                j=0
                val = sys.argv[i]
                if val[0] != "-":
                    dirCaseNum.append(val)
                    j += 1
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        else:
            flag = 0
            dirCaseNum = []
            while i<len(sys.argv) and flag == 0:
                j=0
                val = sys.argv[i]
                if val[0] != "-":
                    dirCaseNum.append(val)
                    j += 1
                    i += 1
                else:
                    flag = 1
                    i -= 1
            comm = "run"
        i += 1
    return comm


#
#  メイン
#  ------
#
#言語の設定
import gettext
treeFoamPath = os.environ["TreeFoamPath"]
gettext.install("treefoam", treeFoamPath + "/data/locale")

#変数クリア
error = ""
a1=""
a2=""
a3=""
dirCaseNum = []
fields = []

#TreeFoamのインストールpath取得
TreeFoamPath = getInstallFolder()

#commnad取得
comm = getCommand()

#各コマンドの実行
if comm == "help":
    error = _(u"使い方\n")
    error += _(u"  gridEditor [option]\n")
    error += _(u"  option\n")
    error += _(u"      -d      読み込むdirectoryを設定\n")
    error += _(u"              <caseDir> <fieldFolder> <polyMeshFolder>を設定\n")
    error += _(u"              省略時は、currentDirがcaseDirになる。\n")
    error += _(u"      -directory   同上\n")
    error += _(u"      --directory  同上\n")
    error += _(u"      -h      helpを表示\n")
    error += _(u"      -help     同上\n")
    error += _(u"      --help    同上\n")
    error += _(u"\n")
    error += _(u"  ex: gridEditor -d ~/cavity 0 constant/polyMesh\n")
    error += _(u"      gridEditor -d ~/cavity 0\n")
    error += _("      gridEditor -d ~/cavity\n")
    error += _("      gridEditor ~/cavity\n")

elif comm == "run":
    if len(dirCaseNum) == 0:
        a1 = "."
        a2 = getTimeFolder(a1)
        a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 1:
        a1 = dirCaseNum[0]
        a2 = getTimeFolder(a1)
        a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 2:
        a1 = dirCaseNum[0]
        a2 = dirCaseNum[1]
        if len(a2.split("/")) == 2:
            folder = a2.split("/")[1]
            a3 = "constant/" + folder + "/polyMesh"
        else:
            a3 = "constant/polyMesh"
    elif len(dirCaseNum) == 3:
        a1 = dirCaseNum[0]
        a2 = dirCaseNum[1]
        a3 = dirCaseNum[2]        
    else:
        error = _(u"error:アーギュメントの数が合いません。\n gridEditor <caseDir> <fieldFolder> <polyMeshFolder>\n Ex: gridEditor ~/cavity 0 constant/polyMesh")


if error != "":
    print "\n" + error
else:
    #dirのチェック
    #print a1, a2, a3
    error = checkDir(a1, a2, a3)
    if error != "":
        print "\n" + error
    else:
        #gridRditor起動
        showGridEditor(editDir, polyMeshDir, fields)

