#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  makePatch.py
#
#    internal patch作成（createBaffles）した後、faceNoが変わっているので
#    constant/polyMesh/setsフォルダ内のsetファイルのfaceNoを修正する。
#
#       programed s.Fujii
#           09/08/21    新規作成
#              09/06    subCase内のメッシュデータを修正
#              12/22    faceZonesのデータが１行で済む様な少ないデータの読み込み時に
#                       エラーが発生していたのを修正
#           10/02/14    currentDirectoryの設定方法を「os.getcwd()」に変更。
#           10/04/13    上記変更によりdirectoryが狂いエラーが発生していたがこれを修正。
#
#

import os
import sys

#
#  readFaceZoneFile
#    faceZonesファイル読み込み
def readFaceZoneFile(wdir):
  fileName = wdir + "/constant/polyMesh/faceZones"
  work = []
  f = open(fileName)
  line = f.readline()
  while line[0] != "(":
    line = f.readline()
  line = f.readline()
  while line[0] != "/":         # get faceZone data
    work.append(line)
    line = f.readline()
  f.close
  return work

#
#  setFaceZoneNo
#  -------------
#    faceZoneNoをsetsfileに書き込み
def setFaceZoneNo(wdir,fileData):
  i=0
  faceZoneData = get1FaceZone(i,fileData)
  while (i < len(fileData)-1) and (len(faceZoneData) > 1):
    writeFaceZoneNo(wdir,faceZoneData)
    i = nextFaceZonePoint(i,fileData)
    faceZoneData = get1FaceZone(i,fileData)

#  get1FaceZone
#    1ヶのFaceZonedataを取得
def get1FaceZone(i,fileData):
  data = []
  data.append(fileData[i])
  while (i < len(fileData)-1) and (fileData[i][:10] != "faceLabels"):
    i = i + 1
  if i < len(fileData)-1:
    if fileData[i+2][0] == "(":
      while (fileData[i][0] != ")") and (i < len(fileData)):
        data.append(fileData[i])
        i=i+1
      if fileData[i][0] == ")":
        data.append(fileData[i])
    else:                           #dataが「4(1 2 33 11);」形式の１行の場合
      line = fileData[i]
      j=0
      while line[j] != ">" and j < len(line):
        j=j+1
      j=j+1
      while line[j] != "(" and j < len(line):
        data.append(line[j]); j=j+1
      data.append("\n")
      data.append(line[j]+"\n"); j=j+1
      while line[j] != ")" and j < len(line):
        data.append(line[j]); j=j+1
      data.append("\n")
      data.append(")\n;\n")
  return data

#  writeFaceZoneNo
#    faceZoneDataを書き込む
def writeFaceZoneNo(wdir,faceZoneData):
  data = []; wdata = []
  name = faceZoneData[0][:-1]
  fileName = wdir + "/constant/polyMesh/sets/" + name
  f = open(fileName)
  for line in f.readlines():
    data.append(line)
  i=0
  while data[i][0] != "(":
    i=i+1
  ie=i-1
  ii=0
  while ii<ie:
    wdata.append(data[ii])
    ii=ii+1 
  j=0
  while faceZoneData[j][0] != "(":
    j=j+1
  j=j-1
  while faceZoneData[j][0] != ")":
    wdata.append(faceZoneData[j])
    i=i+1;j=j+1
  wdata.append(faceZoneData[j])
  f.close()
  f=open(fileName,"w")
  for line in wdata:
    f.write(line)
  f.close()

#  nextFaceZonePoint
#    ポインタを次に進める。
def nextFaceZonePoint(i,fileData):
  while (fileData[i][0] != "}") and (i < len(fileData)-1):
    i=i+1
  i=i+1
  while (len(fileData[i]) < 3) and (i < len(fileData)-1):
    i=i+1
  return i


#
#　メイン
#
wdir = os.getcwd()                  #current Directoryから始める。
fileData = readFaceZoneFile(wdir)
setFaceZoneNo(wdir,fileData)


