#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       paraSnappyHexMesh.py
#
#           snappyHexMeshを並列処理する。
#           mesh作成後、reconstructParMeshを実行し、結果を再構築する。
#
#   16/08/29    新規作成
#      08/30    cellLevel作成を追加。
#      09/08    cellLevel作成部分を削除（圧縮、binary、{0}形式もあり、要検討）
#      09/10    cellLevel作成部分を修正し、追加
#


import sys
import os
import glob
import pyTreeFoam
import getFoamContents
import setFoamContents

#  readFile
#    ファイルの読み込み
def readFile(fileName):
    f = open(fileName)
    cont = f.read()
    f.close()
    return cont

#  writeFile
#    ファイルの書き込み
def writeFile(fileName, cont):
    f=open(fileName, "w")
    f.write(cont)
    f.close()

#  runCommandReturnStat
#    
def runCommandReturnStat(comm):
    errFile = "log.error"
    bashrc = pyTreeFoam.getBashrcFoam()
    cont = ""
    cont += ". " + bashrc + "; "
    cont += comm + " 2> " + errFile
    os.system(cont)
    if os.path.getsize(errFile) == 0:
        stat = "OK"
    else:
        stat = "ERROR"
    return stat

#
#  createDecomposeParDict
#    decomposeParDict作成
def createDecomposeParDict(nCpu, nSep):
    stat = "OK"
    if len(glob.glob("system/decomposeParDict")) == 0:
        #decomposeDict無い場合、defaultをコピー
        ver = pyTreeFoam.getOFversion()
        treeFoamPath = os.getenv("TreeFoamPath")
        dictFolders = glob.glob(treeFoamPath + "/data/OFDict/*")
        dictFolders.sort()
        readFolder = []
        for dictFolder in dictFolders:
            folder = dictFolder.split("/")[-1]
            verDict = folder.split("-")[-1]
            if verDict > ver:
                break
            else:
                readFolder = dictFolder
        comm = "cp " + readFolder + "/system/decomposeParDict system"
        os.system(comm)
    #fileの読み込み
    cont = readFile("system/decomposeParDict")
    #nCpu, simpleをセット
    newLine = "numberOfSubdomains  " + nCpu + ";"
    [cont, p, keyword] = getFoamContents.replace1lineMem(cont, 0, newLine)
    newLine = "method          simple;"
    [cont, p, keyword] = getFoamContents.replace1lineMem(cont, 0, newLine)
    #分割数nSepをセット
    p = 0
    line = " "
    while line != "" or p >= len(cont):
        [lineCont, p, kind] = getFoamContents.get1lineMem(cont, p)
        [keyword, pp] = getFoamContents.getKeywordMem(lineCont, 0)
        if keyword == "simpleCoeffs":
            [pairCont, pp] = getFoamContents.getMiddlePairMem(lineCont, 0)
            repLine = "n           (" + " ".join(nSep) + ");"
            [pairCont, pp, keyword] = getFoamContents.replace1lineMem(pairCont, 0, repLine)
            newLineCont = "simpleCoeffs\n{" + pairCont + "}"
            [cont, p, keyword] = getFoamContents.replace1lineMem(cont, 0, newLineCont)
            break
    #fileの書き込み
    writeFile("system/decomposeParDict", cont)
    return stat

#
#  decomposePar
#    blockMeshを分割する
def decomposePar():
    if len(glob.glob("processor*")) != 0:
        comm = "rm -rf processor*"
        os.system(comm)
    comm = "decomposePar"
    stat = runCommandReturnStat(comm)
    if stat != "OK":
        print "***** Error in decomposePar. check in solve.log ********"
        print
    return stat

#
#  runSnappy
#    snappyHexMeshを並列処理する。
def runSnappy(nCpu):
    comm = "mpirun -np " + nCpu + " snappyHexMesh -parallel"
    stat = runCommandReturnStat(comm)
    if stat != "OK":
        print "***** Error in snappyHexMesh. check in solve.log ********"
        print
    return stat

#
#  reconsuructMesh
#    分割されたmeshを再構築する
def reconstructMesh():
    [pre] = getFoamContents.controlDict("system", ["writePrecision"])
    comm = "reconstructParMesh -latestTime -cellDist -mergeTol 1e-" + pre
    stat = runCommandReturnStat(comm)
    if stat != "OK":
        print "***** Error in reconstructParMesh. check in solve.log ********"
        print
    return stat

#  getNumbersInFile
#    ファイル内の（）内数値を文字で取得
def getNumbersInFile(fileName):
    #f=open(fileName); cont=f.read(); f.close()
    cont = getFoamContents.readFileFull(fileName)
    if getFoamContents.checkBinary(cont) == "ascii":
        #asciiデータの場合
        p = getFoamContents.skipFoamFileMem(cont, 0)
        [data, pp] = getFoamContents.getSmallPairMemF(cont, p)
        if len(data) == 0:
            #{}dataを取得
            [nData, pp] = getFoamContents.getKeywordMem(cont, p)
            [val, pp] = getFoamContents.getMiddlePairMem(cont, p)
            #data作成
            data = ""
            for i in range(int(nData)):
                data += val + "\n"
        nums = data.split()
    else:
        #binaryデータの場合
        [ascii, data] = getFoamContents.separateLabelListContents(cont)
        vals = getFoamContents.structBinData(data[0])
        nums = []
        for i in range(len(vals)):
            nums.append(str(vals[i]))
    return nums

#
#  createCellLevel
#    cellLevelを作成
#    圧縮、binary、***{0}形式もあり、要検討
def createCellLevel():
    timeFolders = pyTreeFoam.getTimeFolders(".")
    timeFolders.sort()
    timeFolder = timeFolders[-1]
    fileName = timeFolder + "/cellDecomposition"
    nums = getNumbersInFile(fileName)
    proNums = []
    for num in nums:
        proNums.append([num, ""])

    procs = glob.glob("processor*")
    for proc in procs:
        proNo = proc[len("processor"):]
        fileName = proc + "/" + timeFolder + "/polyMesh/cellLevel"
        levels = getNumbersInFile(fileName)
        j = 0
        for i in range(len(nums)):
            if proNums[i][0] == proNo:
                proNums[i][1] = levels[j]
                j += 1

    #header作成(ascii形式）
    cont = getFoamContents.readFile2000Bytes(fileName)
    p = getFoamContents.skipFoamFileMem(cont, 0)
    p += 1
    while p < len(cont):
        chara = cont[p]
        if chara == "\n":
            break
        p += 1
    cont = cont[:p] + "\n\n"
    [foamLine, ps, kind] = getFoamContents.get1lineMem(cont, 0)
    [pairCont, p] = getFoamContents.getMiddlePairMem(foamLine, 0)
    chgLine = "format      ascii;"
    [pairCont, p, keyword] = getFoamContents.replace1lineMem(pairCont, 0, chgLine)
    newFoamLine = "FoamFile\n{\n" + pairCont + "}"
    [cont, p, keyword] = getFoamContents.replace1lineMem(cont, 0, newFoamLine)
    #dataセット
    cont += str(len(proNums)) + "\n"
    cont += "(\n"
    for num in proNums:
        cont += num[1] + "\n"
    cont += ")\n"
    #footerセット
    cont += setFoamContents.makeFooter()
    #file保存
    fileName = timeFolder + "/polyMesh/cellLevel"
    getFoamContents.writeFileFull(fileName, cont)
    print "create 'cellLevel' from each processors."

#
#  setPolyMesh
#    再構築されたmeshをconstantにコピーする
def setPolyMesh():
    timeFolders = pyTreeFoam.getTimeFolders(".")
    timeFolders.sort()
    comm = "rm -rf ./constant/polyMesh/*"
    print comm
    os.system(comm)
    comm = "cp ./" + timeFolders[-1] + "/polyMesh/* " + "./constant/polyMesh"
    print comm
    os.system(comm)
    if len(timeFolders) > 1:
        comm = "rm -rf " + timeFolders[-1]
    print comm
    os.system(comm)
    comm = "rm -rf ./processor*"
    print comm
    os.system(comm)     
    return


if __name__ == "__main__":
    nCpu = sys.argv[1]
    x = sys.argv[2]
    y = sys.argv[3]
    z = sys.argv[4]
    nSep = [x,y,z]
    #decomposeParDict作成
    stat = createDecomposeParDict(nCpu, nSep)
    if stat != "OK":
        exit()
    #mesh分割
    stat = decomposePar()
    if stat != "OK":
        exit()
    #snappyを並列処理
    stat = runSnappy(nCpu)
    if stat != "OK":
        exit()
    #meshを再構築
    stat = reconstructMesh()
    if stat != "OK":
        exit()
    #cellLevelを作成
    createCellLevel()
    #meshをconstantにcopyし、余分なfile、folderを削除
    setPolyMesh()

