#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   runGridEditor.py
#
#       gridEditor起動用（TreeFoamから起動する時に使用）
#         fieldのerrorCheck実施後これを起動する事。
#         ここでは、dialogが表示できないので、errorCheckを実施していない。
#
#       ＜使い方＞
#       runGridEditor.py <nTreeFoam> <title> <boundaryDir> <fieldDir>
#        nTreeFoam:     TreeFoamの起動No
#        title:         geridEditorのタイトル（case名、timeFolder名を含む）
#        boundaryDir:   polyMesh/baoundaryのdirectory
#        fieldDir:      timeFolderのdirectory
#
#   12/0919     新規作成
#      09/21    importするmoduleを整理
#   13/04/28    存在しないfieldがdisplayFieldsにあった場合、そのfieldを削除する。
#      05/01    全field表示設定の場合は、displayFieldsをrenameする。
#      07/11    boundaryDirを再取得（latest, start, firstTime, constantの順で探す）
#      10/13    多言語化対応
#   15/03/08    バグ修正。（表示順が正しく表示されなかったので、修正）
#      07/05    gridEditorの起動方法を修正。
#      11/28    gridEditorの起動方法を修正。（fieldの表示順の変更時も設定を保存。）
#

import os
import glob
import pyTreeFoam
import getFoamContents
from sys import argv
from editBoundaryConditionsDialog import Show

#
# getFields
#   編集するfieldを取得
def getFields():
    fileName = pyTreeFoam.getSaveDisplayFieldsFileName(fieldDir)
    if len(glob.glob(fileName)) == 0:
        fields = pyTreeFoam.getFieldNames(fieldDir)
        fields.sort()
    else:
        fieldsD = pyTreeFoam.getDisplayFields(fileName)
        fieldsC = pyTreeFoam.getFieldNames(fieldDir)
        fields = []
        for field in fieldsD:
            for name in fieldsC:
                if field == name:
                    fields.append(field)
                    break
        #fields.sort()
        #fieldsC.sort()
        """
        if len(fields) == len(fieldsC):
            comm = "mv " + fileName + " " + fileName + ".bak"
            os.system(comm)
            fields = []
        """
    return fields


#言語の設定
import gettext
treeFoamPath = os.environ["TreeFoamPath"]
gettext.install("treefoam", treeFoamPath + "/data/locale")

#引数を取得
nTreeFoam   = argv[1]
title       = argv[2]
boundaryDir = argv[3]
fieldDir    = argv[4]

#fieldを取得
fields = getFields()

#boundaryDirを取得
folder = fieldDir.split("/")[-1]
try:
    a=float(folder)
    currTime = folder
    caseDir = "/".join(fieldDir.split("/")[:-1])
    region = "."
except:
    currTime = fieldDir.split("/")[-2]
    caseDir = "/".join(fieldDir.split("/")[:-2])
    region = folder
relativeDir = getFoamContents.getCurrMeshDir(caseDir, currTime, region, "boundary")
boundaryDir = caseDir + "/" + relativeDir

#gridEditorを起動
#Show(nTreeFoam, title, boundaryDir, fieldDir, fields)
var = nTreeFoam + " '" + title + "' " + " ".join([boundaryDir] + [fieldDir] + fields)
comm = "editBoundaryConditionsDialog.py " + var
os.system(comm)

