from unv2x import *

if __name__=='__main__':
    helpmsg=""" 
	========================================================
	 unv2gmsh: Convert UNV file from Salome to GMSH v1 mesh
	               J.Cugnoni, www.caelinux.com, 2007
	========================================================
	
        Usage: unv2gmsh unvfile prefix
	
        What it does: read unvfile, create an internal FEM structure
                      in memory and writes the prefix.msh file
		      
        Groups:
        Salome does not write a geometrical/physical id number which are needed
        in GMSH mesh format. To circumvent this problem, each group of elements
        defined in the UNV mesh will be associated to a Physical id number in
        the generated GMSH. The mapping table will be displayed in standard
        output and written to prefix.map. If one element belongs to several 
	groups, it will receive the physical id of the last group.
    
        Element type mapping from UNV to GMSH
        # UNV id,   GMSH id:  description
        ---------------------------------------- 
          UNV 41, GMSH 2: triangle, 3 nodes
          UNV 42, GMSH 9: triangle, 6 nodes
          UNV 44, GMSH 3: quadrangle, 4 nodes
          UNV 45, GMSH 16: quadrangle, 8 nodes
          UNV 111, GMSH 4: tetra, 4 nodes
          UNV 112, GMSH 6: prism, 6 nodes
          UNV 113, GMSH 18: prism, 15 nodes
          UNV 115, GMSH 5: hexa, 8 nodes
          UNV 116, GMSH 17: hexa, 20 nodes
          UNV 118, GMSH 11: tetra, 10 nodes
          UNV 11, GMSH 1: line, 2 nodes
          UNV 22, GMSH 8: line, 3 nodes
	  
	  TODO: check element connectivity numbering...
	  	  
    """
    
    # usage : elementMapping[UnvType]=GmshType
    elementMapping={41:2,42:9,44:3,45:16,111:4,112:6,113:18,115:5,116:17,118:11,11:1,22:8}
    # usage : connectivityMapping[UnvType]=VectorOfNodeIndicesInGMSHformat
    connectivityMapping={41:[0,1,2],42:[0,2,4,1,3,5],22:[0,2,1],11:[0,1],118:[0,2,4,9,1,3,5,6,8,7],115:[0,1,2,3,4,5,6,7],116:[0,2,4,6,12,14,16,18,1,7,8,3,9,5,10,11,13,19,15,17],113:[0,2,4,9,11,13,1,5,6,3,7,8,10,14,13],44:[0,1,2,3],45:[0,2,4,6,1,3,5,7],112:[0,1,2,3,4,5],111:[0,1,2,3]}

    
    if len(sys.argv)==3:
        unvfile=sys.argv[1]
        prefix=sys.argv[2]
        ls=os.linesep
        flag='    -1' + ls
        # read UNV file in FEM object structure
        UNV=UNVParser(unvfile)
        FEM=UNV.parse()
        # write GMSH file
        # 
        fil=open(prefix + '.msh','w')
        fmap=open(prefix + '.map','w')
        # nodes         
        fil.write('$NOD'+ls)
        fil.write('%d'%(len(FEM.nodes)) + ls)
        for node in FEM.nodes:
            fil.write(('%d %g %g %g'+ls) % (node.id,node.coords[0],node.coords[1],node.coords[2]))
        fil.write('$ENDNOD'+ls)
	
		
        # elements
        #  first create mapping btw elem. groups and physical ids
	#  physical id = 0 is the default value	
	elemIds=[elem.id for elem in FEM.elems]
	physIds=dict(zip(elemIds,[0]*len(elemIds)))
	physId=0
	txt=ls + ' Mapping of UNV groups <=> GMSH physical ID '+ls+'  Default  <=>  0'+ls
	print(txt)
	fmap.write(txt)
	FEM.nelemsets=len(FEM.elemsets)
        for group in FEM.elemsets:	
            physId=physId+1
	    txt=('  %s  <=>  %d '+ls)%(group.name.strip(),physId)
	    print(txt)
	    fmap.write(txt)
            for i in group.items:
	        physIds[i]=physId	
	# write ELM section in GMSH file
	fil.write('$ELM'+ls)
        fil.write('%d'%(len(FEM.elems)) + ls)
        for elem in FEM.elems:
            nnodes=elem.nnodes
            dat=[elem.id,elementMapping[elem.type],physIds[elem.id],physIds[elem.id],elem.nnodes]	    
	    dat.extend([elem.cntvt[i] for i in connectivityMapping[elem.type]])
            fil.write((('%d ' * len(dat)) + ls) % tuple(dat))
        fil.write('$ENDELM'+ls)
        fmap.close()
        fil.close()

    else:
        print(helpmsg)
