#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  unv2gmshToFoamNameConvert.py
#    face名、volume名をFoam側に設定する。
#
#       09/08/02    Face名又はCell名の合計が10ヶを越える場合、名称変換が正常にできなかった
#                   事を修正。setsフォルダ内のファイル内容についても名称変換。
#       09/08/31    エラーメッセージの内容を充実。
#       10/04/15    バグ修正。同じ文字列を含んだ名前を変換する時、うまく変換できなかった
#       10/08/09    未定義のvolume名をcellZone_defaultとして定義を追加
#       12/08/05    変換後の名称を「cellZone_」→「cells_」に変更
#       13/04/30    変換後の名称を「cells_xxx」→「xxx」に変更。
#          10/05    圧縮・binaryファイルに対応
#          10/10    多言語化
#          10/27    mesh.unvを任意の場所に変更
#          12/20    バグ修正（modelDirが絶対参照の場合、エラー発生）
#       14/09/23    setCellFaceZoneName:バグ修正（dirに「.」文字があるとｴﾗｰ発生）
#

import os
import sys
import glob
import getFoamContents
from editBoundaryConditionsDialog import readConfigData as readGridEditorConfig

gmshFileName = "salomeNameList.txt"
foamFileName = "foamNameList.txt"
gmshFile = ""
foamFile = ""
workdir = ""
modelDir = ""
salomeName = []
foamName = []; foamNameNo = []
faceZoneNo = []
cellZoneNo = []


#
#  setCurrentDirectory
#    Current directoryをworkdirにセットし、gmshFileとfoamFileのdirを取得
#    workdir
def setCurrentDirectory():
    global workdir, gmshFile, foamFile
    workdir = os.getcwd()
    if modelDir[0] == "/":
        gmshFile = modelDir + "/" + gmshFileName
        foamFile = modelDir + "/" + foamFileName
    else:
        gmshFile = workdir + "/" + modelDir + "/" + gmshFileName
        foamFile = workdir + "/" + modelDir + "/" + foamFileName




#
#  getSalomeName
#
def getSalomeName():
    global workdir,gmshFile,salomeName
    print _(u"Salome側（unv file）のグループ名を取得...")
    salomeName=[]
    name = []; sname = []
    #f = open(workdir + '/' + gmshFile)
    f = open(gmshFile)
    for a in f.readlines():
        name.append(a[2:])
    for a in name:
        i=0; b=""
        while a[i] != " ":
            b=b+a[i]
            i=i+1
        salomeName.append(b)
    f.close

#
#  getFormName
#
def getFoamName():
    global workdir,foamFile
    print _(u"Foam側で設定したpatch名を取得...")
    name = []
    #f = open(workdir + '/' + foamFile)
    f = open(foamFile)
    for a in f.readlines():
        b = a.split()
        b[0]=int(b[0])
        name.append(b)
    f.close
    name.sort()
    i=0
    while i < len(name):
        a=(name[i])[1]
        foamName.append(a)
        i=i+1

#
#  checkSalomeFoamName
#　　未定義のvolume名を検出し、cellZone_defaultとして定義
def checkSalomeFoamName():
    global workdir, gmshFile, foamFile
    #f = open(workdir + "/" + foamFile)
    f = open(foamFile)
    foamCont = []
    for line in f.readlines():
        foamCont.append(line)
    f.close()
    a = foamCont[1].split()
    if (a[0] == "0"):                 #「0」のcellZoneがあるか？
        a = foamCont[0].split()
        b = foamCont[1].split()
        if (a[1][:4] == "cell"):        #あれば「-1」に変更
            foamCont[0] = "-1 " + a[1] + "\n"
        if (b[1][:4] == "cell"):
            foamCont[1] = "-1 " + b[1] + "\n"
        foamCont.sort()
        print _(u"定義されていないcellZoneがあります。defaultで定義しました")

        #f = open(workdir + "/" + foamFile, "w")
        f = open(foamFile, "w")
        i=0
        while i<len(foamCont):
            f.write(foamCont[i])
            i=i+1
        f.close()

        salomeCont = []
        salomeCont.append("  default -1\n")    
        #f=open(workdir + "/" + gmshFile)
        f=open(gmshFile)
        for line in f.readlines():
            salomeCont.append(line)
        f.close()
        #f=open(workdir + "/" + gmshFile, "w")
        f=open(gmshFile, "w")
        i=0
        while i<len(salomeCont):
            f.write(salomeCont[i])
            i=i+1
        f.close()

        getSalomeName()
    

#
# changePatchName
#
def changePatchName():
    global workdir, salomeName, foamName, foamNameNo
    print _(u"boundaryファイルに設定されたpatch名をSalome側の名称に変換...")
    boundaryFile = workdir + '/constant/polyMesh/boundary'
    f=open(boundaryFile)
    contents = f.readlines()
    f.close

    i=0
    for line in contents:
        name = line.split()
        if len(name) == 1:
            s = 0
            for Fname in foamName:
                if name[0] == Fname:
                    Sname = salomeName[s]
                    cont = exchangeValue(Fname, Sname, line)
                    contents[i] = cont
                    break
                s=s+1
        i=i+1

        f = open(boundaryFile, "w")
        for line in contents:
            f.write(line)
        f.close()

def exchangeValue(name, cname,line):
    i = line.index(name)
    lineR = line[:i] + cname + "\n"
    return lineR


#
#  setCellFaceZoneName
#  
def setCellFaceZoneName():
    global workdir
    print _(u"faceZones, cellZones, sets内ファイル名をsalome側名称に変換...")

    #faceZoneファイルの修正
    #  ファイルの読込
    fileName = workdir + "/constant/polyMesh/faceZones"
    fileName = getFoamContents.checkFileName(fileName)
    command = "cp " + fileName + " " + fileName + ".bak"
    os.system(command)
    [ascii, data] = getFoamContents.readFileSeparate(fileName)
    contents = ascii.split("\n")
    #  faceZone名の取得
    zoneName = getFoamContents.faceZoneNamesF(workdir)
    for name in zoneName:
        if name[0:4] == "face":
            No = "patch" + name[9:]
            i = 0
            for fName in foamName:
                if No == fName:
                    faceZoneNo.append(i)
                i = i + 1
    #　faceZone名の変換
    for i in faceZoneNo:
        salomeN = "faceZone_"+salomeName[i]
        foamN = "faceZone_"+(foamName[i])[5:]
        n=0
        for line in contents:
            lineName = line.split()
            if len(lineName) != 0:
                contName = lineName[0]
                if contName == foamN:
                    #salomeNameに置き換え
                    contents[n] = salomeName[i]
                    break
            n=n+1
    #  ファイルの書き込み
    newAscii = "\n".join(contents)
    getFoamContents.writeFileSeparate(fileName, [newAscii, data])

    #cellZoneファイルの修正
    #  ファイルの読込
    fileName = workdir + "/constant/polyMesh/cellZones"
    fileName = getFoamContents.checkFileName(fileName)
    command = "cp " + fileName + " " + fileName + ".bak"
    os.system(command)
    [ascii, data] = getFoamContents.readFileSeparate(fileName)
    contents = ascii.split("\n")
    #  cellZone名の取得
    i= 0
    for nameF in foamName:
        if nameF[0:4] == "cell":
            cellZoneNo.append(i)
        i=i+1
    #  cellZone名の置き換え
    for i in cellZoneNo:
        Oname = foamName[i]
        Cname = salomeName[i]
        n=0
        for line in contents:
            lineName = line.split()
            if len(lineName) != 0:
                contName = lineName[0]
                if contName == Oname:
                    #salomeNameに置き換え
                    contents[n] = Cname
                    break
            n=n+1
    #　結果を書き込み
    ascii = "\n".join(contents)
    getFoamContents.writeFileSeparate(fileName, [ascii, data])

    #setsフォルダ内のファイル名修正
    fileDir = workdir + "/constant/polyMesh/sets/"
    #faceZone名をチェック
    for i in faceZoneNo:
        Oname = fileDir + "faceZone_" + (foamName[i])[5:]
        Oname = getFoamContents.checkFileName(Oname)
        #圧縮ファイルのチェック
        if len(Oname.split(".")) == 1:
            #非圧縮のfileName
            Cname = fileDir + salomeName[i]
        else:
            #圧縮のfileName
            Cname = fileDir + salomeName[i] + "." + Oname.split(".")[1]
        #setsファイル内容をsalomeNameに置き換え
        [cont, data] = getFoamContents.readFileSeparate(Oname)
        newCont = replaceObjectNameInFoamFile(cont, salomeName[i])
        getFoamContents.writeFileSeparate(Cname, [newCont, data])
        command = "rm " + Oname
        os.system(command)

    #cellZone名をチェック
    for i in cellZoneNo:
        Oname = fileDir + foamName[i]
        Oname = getFoamContents.checkFileName(Oname)
        #圧縮ファイルのチェック
        #names = Oname.split(".")
        names = (Oname.split("/")[-1]).split(".")
        if len(names) == 1:
            #非圧縮のfileName
            Cname = fileDir + salomeName[i]
        else:
            #圧縮のfileName
            Cname = fileDir + salomeName[i] + "." + names[1]
        #setsファイルの置き換え
        [cont, data] = getFoamContents.readFileSeparate(Oname)
        newCont = replaceObjectNameInFoamFile(cont, salomeName[i])
        getFoamContents.writeFileSeparate(Cname, [newCont, data])
        command = "rm " + Oname
        os.system(command)

#
#  replaceObjectNameInFoamFile
#    FoamFile内のobject名を変更する
def replaceObjectNameInFoamFile(cont, objectName):
    p = getFoamContents.getKeywordPointerMem("object", cont, 0)
    [dummy, p] = getFoamContents.getKeywordMem(cont, p)
    [oldName, p] = getFoamContents.getKeywordMem(cont, p)
    pe = p - (len(oldName) + 1)
    newCont = cont[:pe] + objectName + cont[p-1:]
    return newCont
 
#
#  writeConvertMap
#    グループ名変換結果の書き出し
#
def writeConvertMap():
    global wordir, modelDir
    #writeFileName = workdir + "/model/map.map"
    if modelDir[0] == "/":
        writeFileName = modelDir + "/map.map"
    else:
        writeFileName = workdir + "/" + modelDir + "/map.map"
    f = open(writeFileName, "w")
    cont = _(u"\nSalome 側のグループ名の変換結果\n\n")
    #salome名、gmsh名、foam名の対応
    i = 0
    for name in salomeName:
        cont = cont + name + "\t" + str(i) + "\t" + foamName[i] +"\n"
        i = i + 1
    #faceZone名の書出し
    cont = cont + "\nfaces\n"
    for n in faceZoneNo:
        line = "    " + foamName[n] + "  " + salomeName[n] + "\n"
        cont = cont + line
    #cellZone名の書出し
    cont = cont + "\ncells\n"
    for n in cellZoneNo:
        line = "    " + foamName[n] + "  " + salomeName[n] + "\n"
        cont = cont + line
    f.write(cont.encode("utf-8"))
    f.close
  


#
# メイン
#
import gettext
treeFoamPath = os.environ["TreeFoamPath"]
gettext.install("treefoam", treeFoamPath + "/data/locale")

modelDir = sys.argv[1]
setCurrentDirectory()
getSalomeName()
getFoamName()
checkSalomeFoamName()       #volumeの定義をチェック

if len(salomeName) == len(foamName):
    changePatchName()
    setCellFaceZoneName()
    writeConvertMap()
else:
    f = open("./model/error.txt","w")
    message = _(u"unv2gmshToFoam 変換エラー\n")
    message += _(u"Salome側のグループ数と変換後のFoam側のグループ数が合いません。\n")
    message += _(u"（標準出力の出力結果でsalomeとfoamのグループ名を確認してください。）")
    message += _(u"Salome側でVolumeをグループ化していない可能性があります。\n")
    message += _(u"Patch名のみグループ化している場合、「ideasUnvToFoam」を使って変換してください。\n")
    message += _(u"faceがオーバラップして定義されている、または同じ面を違う名前で定義した場合は、エラーが発生します。")
    f.write(message)
    f.close
    print _(u"\nSalome側のグループ数と変換後のFoam側のグループ数が合いません。")
    print _(u"Salomeグループ名:")
    print salomeName
    print _(u"Foamグループ名:")
    print foamName
    print

