#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   createAndChangeCaseDialog.py
#
#       caseの作成、solver、meshの入れ替え
#
#   13/03/31    新規作成
#      04/06    空patchのboundaryFieldクリア追加
#      04/08    openFolderを追加
#      04/10    現在caseのtimeFolder内のメッシュ入れ替えを追加
#      04/15    newCaseの作成場所の指定を追加
#      04/20    ボタンの動作を変更
#      04/27    notebook方式に変更、設定を記憶させる
#               cell数が同じメッシュを入れ替えた場合、internalFieldをクリアしない。
#               timeFolder内のpolyMeshは、初めの10ヶと最後の10ヶをチェックする。
#      05/08    meshの一部入れ替えの場合エラー発生。これを修正
#      05/09    mesh入れ替え時の方向icon追加
#      06/09    tutorialsCaseのチェックをゆるめる。
#      07/14    meshDirの検索方法をglobに変更
#      07/20    mesh入れ替え:コピー先のdirも選択できるように修正
#      07/21    mesh入れ替え:dirを選択した場合、反対側のボタンを非enableに設定
#      07/27    mesh入れ替え:表示を変更
#      08/15    newCase,solver入れ替えの表示方法修正
#      08/21    clearOnlyNullBoundaryFields:fieldNamesの取得方法変更
#               バグ修正：隠しファイルが有った場合、エラー発生
#      09/6     newCase作成時、case名を入力できる様修正
#      09/25    meshCopy時:binaryMeshのコピーに対応
#      10/04    multiRegionの場合、meshを一括コピーできる様に対応
#      10/05    OpenFOAM用のcaseでない場合、pane_2, pane_3をunenableに設定
#      10/15    国際化による修正
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/03/01    case作成時、currentCaseのtimeフォルダが存在しない場合エラーが発生
#               していたので、修正。addNullTimeFolder内を修正
#      09/02    clearNullBoundaryFieldsInternalFields:バグ修正
#      09/22    addNullTimeFolder:nullTimeはfirstTimeがmustに変更
#      10/23    copyMesh:cell数が異なる場合、internalFieldをクリアするか訊く様に修正
#      11/09    copyMesh:checkCurrMeshを追加
#      12/22    readMeshLocation,getMeshToCopyCase:「constant」をmeshが無くても追加
#   15/02/12    getMeshToCopyCase:バグ修正（ｺﾋﾟｰ先のdirを選択時ｴﾗｰ発生）
#      07/13    clipBoardの場所をTreeFoam/dataに変更。initialize部を修正。
#      07/17    dialog起動方法修正
#   17/02/26    initialize:solverの有無でfoamCaseかどうかを確認していたが、
#               constant, systemフォルダの有無で確認するように修正
#
#
#-----------code生成後の修正項目-------------------------------------------
#
#   ・アイコンのDirを書き直す（絶対path→相対path[./icons/***.png]）
#

# generated by wxGlade 0.6.5 on Sun Mar 31 16:01:43 2013

import wx
import os
import glob
import gio
import pyTreeFoam
import getFoamContents
import setFoamContents
import gaugeDDialog
from createNewCaseDDialog import Show as createNewCaseDDialogShow
from getOpenFolderDDialog import Show as getOpenFolderDDialogShow

import logFileCreater; logFileCreater.log()

# begin wxGlade: extracode
# end wxGlade

#clipBoard = os.environ["TreeFoamPath"] + "/data/clipBoard"
clipBoard = os.environ["TreeFoamUserPath"] + "/data/clipBoard"
#createNewCase_data = os.environ["TreeFoamPath"] + "/data/createNewCase_data"
createNewCase_data = os.environ["TreeFoamUserPath"] + "/data/createNewCase_data"
toCopyMeshDirs = []
selMeshDirs = []
currCaseDir = ""
selCaseDir = ""
selCaseCopyDir = ""
toCopyCaseDir = ""
foamCase = True         #OpenFOAM用のcaseか否か

selNote = 0             #選択しているcase
newCase_loc = "./"
newCase_tuto = "yes"
solver_tuto = "yes"




class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        savePath = os.getcwd()
        os.chdir(os.environ["TreeFoamPath"])
        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)
        self.label_1 = wx.StaticText(self, -1, _(u"caseを作成、変更します"))
        self.notebook_1 = wx.Notebook(self, -1, style=0)
        self.notebook_1_pane_1 = wx.Panel(self.notebook_1, -1)
        self.bitmap_1 = wx.StaticBitmap(self.notebook_1_pane_1, -1, wx.Bitmap("./icons/newCase.png", wx.BITMAP_TYPE_ANY))
        self.label_2 = wx.StaticText(self.notebook_1_pane_1, -1, _(u"tutorialsまたは、任意のdirからcaseをコピーして、\n新しくcaseを作成します。\n"))
        self.radio_btn_3 = wx.RadioButton(self.notebook_1_pane_1, -1, _("tutorials"), style=wx.RB_GROUP)
        self.radio_btn_4 = wx.RadioButton(self.notebook_1_pane_1, -1, _(u"その他case"))
        self.button_9 = wx.Button(self.notebook_1_pane_1, -1, _(u"case取得..."))
        self.text_ctrl_4 = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.sizer_35_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, _(u"source（コピー元）"))
        self.label_14 = wx.StaticText(self.notebook_1_pane_1, -1, _(u"デフォルトの作成場所：解析case内"))
        self.label_7 = wx.StaticText(self.notebook_1_pane_1, -1, _(u"作成場所:"))
        self.text_ctrl_1 = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.button_8 = wx.Button(self.notebook_1_pane_1, -1, _(u"参照..."))
        self.label_11 = wx.StaticText(self.notebook_1_pane_1, -1, _(u"  case名:"))
        self.text_ctrl_2 = wx.TextCtrl(self.notebook_1_pane_1, -1, "")
        self.sizer_40_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, _("newCase"))
        self.button_2 = wx.Button(self.notebook_1_pane_1, -1, _(u"コピー開始"))
        self.sizer_25_staticbox = wx.StaticBox(self.notebook_1_pane_1, -1, "")
        self.notebook_1_pane_2 = wx.Panel(self.notebook_1, -1)
        self.label_10 = wx.StaticText(self.notebook_1_pane_2, -1, _("solver"))
        self.label_3 = wx.StaticText(self.notebook_1_pane_2, -1, _(u"現在の解析caseのsolverを、選択したcaseのsolverと入れ替えます。\nsolverが必要なfield、properties、dict類もコピーし、設定します。\n入れ替え後、boundaryFieldを修正します。"))
        self.radio_btn_1 = wx.RadioButton(self.notebook_1_pane_2, -1, _("tutorials"), style=wx.RB_GROUP)
        self.radio_btn_2 = wx.RadioButton(self.notebook_1_pane_2, -1, _(u"その他case"))
        self.button_7 = wx.Button(self.notebook_1_pane_2, -1, _(u"case取得..."))
        self.text_ctrl_5 = wx.TextCtrl(self.notebook_1_pane_2, -1, "")
        self.label_15 = wx.StaticText(self.notebook_1_pane_2, -1, _(u"現在のsolver:"))
        self.label_12 = wx.StaticText(self.notebook_1_pane_2, -1, _(u"新しいsolver:"))
        self.label_13 = wx.StaticText(self.notebook_1_pane_2, -1, "")
        self.sizer_36_staticbox = wx.StaticBox(self.notebook_1_pane_2, -1, _(u"case選択"))
        self.button_3 = wx.Button(self.notebook_1_pane_2, -1, _(u"コピー開始"))
        self.sizer_31_staticbox = wx.StaticBox(self.notebook_1_pane_2, -1, "")
        self.notebook_1_pane_3 = wx.Panel(self.notebook_1, -1)
        self.bitmap_3 = wx.StaticBitmap(self.notebook_1_pane_3, -1, wx.Bitmap("./icons/editMesh.png", wx.BITMAP_TYPE_ANY))
        self.label_4 = wx.StaticText(self.notebook_1_pane_3, -1, _(u"解析caseのmeshを、選択したcaseのmeshと入れ替えます。\nmultiRegionの場合、regionのmeshを複数選択して、コピー先をconstant又は\ntimeFolderを選択する事で、regionのmeshを一括してコピーできます。\n入れ替え後、internalField、boundaryFieldを修正します。"))
        self.label_6 = wx.StaticText(self.notebook_1_pane_3, -1, _(u"case名:"))
        self.label_9 = wx.StaticText(self.notebook_1_pane_3, -1, _(u"polyMash場所"))
        self.button_5 = wx.Button(self.notebook_1_pane_3, -1, _(u"case変更(元)"))
        self.list_box_1 = wx.ListBox(self.notebook_1_pane_3, -1, choices=[], style=wx.LB_MULTIPLE)
        self.sizer_20_staticbox = wx.StaticBox(self.notebook_1_pane_3, -1, _(u"source（コピー元）"))
        self.bitmap_2 = wx.StaticBitmap(self.notebook_1_pane_3, -1, wx.Bitmap("./icons/runSolver.png", wx.BITMAP_TYPE_ANY))
        self.label_8 = wx.StaticText(self.notebook_1_pane_3, -1, _(u"case名:"))
        self.label_5 = wx.StaticText(self.notebook_1_pane_3, -1, _(u"コピー場所"))
        self.button_1 = wx.Button(self.notebook_1_pane_3, -1, _(u"case変更(先)"))
        self.list_box_2 = wx.ListBox(self.notebook_1_pane_3, -1, choices=[], style=wx.LB_MULTIPLE)
        self.sizer_22_staticbox = wx.StaticBox(self.notebook_1_pane_3, -1, _(u"toCopy（コピー先）"))
        self.button_6 = wx.Button(self.notebook_1_pane_3, -1, _(u"コピー開始"))
        self.sizer_16_staticbox = wx.StaticBox(self.notebook_1_pane_3, -1, "")
        self.button_4 = wx.Button(self, -1, _(u"閉じる"))

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.onSelCaseStartCopy, self.button_9)
        self.Bind(wx.EVT_BUTTON, self.onReferenceDir, self.button_8)
        self.Bind(wx.EVT_BUTTON, self.onNewCaseCopy, self.button_2)
        self.Bind(wx.EVT_BUTTON, self.onSelCaseRemoveSolver, self.button_7)
        self.Bind(wx.EVT_BUTTON, self.onSolverCopy, self.button_3)
        self.Bind(wx.EVT_BUTTON, self.onSelectMeshCopyCase, self.button_5)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenSourceFolder, self.list_box_1)
        self.Bind(wx.EVT_BUTTON, self.onSelectToCopyCase, self.button_1)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenCurrFolder, self.list_box_2)
        self.Bind(wx.EVT_BUTTON, self.onCopyMesh, self.button_6)
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.onChangePage, self.notebook_1)
        self.Bind(wx.EVT_BUTTON, self.onClose, self.button_4)
        # end wxGlade

        os.chdir(savePath)
        #toolTipの追加
        self.list_box_1.SetToolTipString(_(u"ダブルクリックするとfolderを開きます。"))
        self.list_box_2.SetToolTipString(_(u"ダブルクリックするとfolderを開きます。"))
        #unenable object
        self.text_ctrl_1.SetValue("./")
        #初期化(dialogセットデータの読み込み)
        self.initialize()
        if foamCase == True:
            #meshLocationの読み込み
            self.readMeshLocation()


    def __set_properties(self):
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle(_(u"新しいcaseの作成"))
        self.button_9.SetMinSize((85, 30))
        self.label_7.SetMinSize((70, 18))
        self.button_8.SetMinSize((60, 28))
        self.label_11.SetMinSize((70, 18))
        self.text_ctrl_2.SetMinSize((200, 28))
        self.label_10.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.button_7.SetMinSize((98, 30))
        self.button_5.SetMinSize((98, 25))
        self.list_box_1.SetMinSize((100, 100))
        self.button_1.SetMinSize((98, 25))
        # end wxGlade


    def __do_layout(self):
        # begin wxGlade: MyFrame.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_14 = wx.BoxSizer(wx.VERTICAL)
        sizer_15 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_24 = wx.BoxSizer(wx.VERTICAL)
        sizer_10 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_16_staticbox.Lower()
        sizer_16 = wx.StaticBoxSizer(self.sizer_16_staticbox, wx.VERTICAL)
        sizer_38 = wx.BoxSizer(wx.VERTICAL)
        sizer_39 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_17 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_22_staticbox.Lower()
        sizer_22 = wx.StaticBoxSizer(self.sizer_22_staticbox, wx.VERTICAL)
        sizer_18 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_37 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_20_staticbox.Lower()
        sizer_20 = wx.StaticBoxSizer(self.sizer_20_staticbox, wx.VERTICAL)
        sizer_19 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_11 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_13 = wx.BoxSizer(wx.VERTICAL)
        sizer_12 = wx.BoxSizer(wx.VERTICAL)
        sizer_23 = wx.BoxSizer(wx.VERTICAL)
        sizer_6 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_31_staticbox.Lower()
        sizer_31 = wx.StaticBoxSizer(self.sizer_31_staticbox, wx.VERTICAL)
        sizer_46 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_47 = wx.BoxSizer(wx.VERTICAL)
        sizer_48 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_36_staticbox.Lower()
        sizer_36 = wx.StaticBoxSizer(self.sizer_36_staticbox, wx.VERTICAL)
        sizer_42 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_50 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_32 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_34 = wx.BoxSizer(wx.VERTICAL)
        sizer_45 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_33 = wx.BoxSizer(wx.VERTICAL)
        sizer_7 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_9 = wx.BoxSizer(wx.VERTICAL)
        sizer_8 = wx.BoxSizer(wx.VERTICAL)
        sizer_21 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_25_staticbox.Lower()
        sizer_25 = wx.StaticBoxSizer(self.sizer_25_staticbox, wx.VERTICAL)
        sizer_49 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_43 = wx.BoxSizer(wx.VERTICAL)
        sizer_44 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_40_staticbox.Lower()
        sizer_40 = wx.StaticBoxSizer(self.sizer_40_staticbox, wx.VERTICAL)
        sizer_41 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_29 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_35_staticbox.Lower()
        sizer_35 = wx.StaticBoxSizer(self.sizer_35_staticbox, wx.VERTICAL)
        sizer_26 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_28 = wx.BoxSizer(wx.VERTICAL)
        sizer_30 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_27 = wx.BoxSizer(wx.VERTICAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_5 = wx.BoxSizer(wx.VERTICAL)
        sizer_4 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.label_1, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_4.Add(self.bitmap_1, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 20)
        sizer_3.Add(sizer_4, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_5.Add(self.label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_3.Add(sizer_5, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_2.Add(sizer_3, 0, wx.EXPAND, 0)
        sizer_27.Add(self.radio_btn_3, 0, wx.LEFT | wx.TOP, 5)
        sizer_27.Add(self.radio_btn_4, 0, wx.LEFT | wx.BOTTOM, 5)
        sizer_26.Add(sizer_27, 0, wx.EXPAND, 0)
        sizer_30.Add(self.button_9, 0, wx.LEFT, 10)
        sizer_30.Add(self.text_ctrl_4, 1, wx.EXPAND, 0)
        sizer_28.Add(sizer_30, 0, wx.EXPAND, 0)
        sizer_26.Add(sizer_28, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_35.Add(sizer_26, 0, wx.EXPAND, 0)
        sizer_25.Add(sizer_35, 0, wx.ALL | wx.EXPAND, 10)
        sizer_40.Add(self.label_14, 0, wx.LEFT, 75)
        sizer_29.Add(self.label_7, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 8)
        sizer_29.Add(self.text_ctrl_1, 1, 0, 0)
        sizer_29.Add(self.button_8, 0, wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_40.Add(sizer_29, 0, wx.TOP | wx.EXPAND, 0)
        sizer_41.Add(self.label_11, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 8)
        sizer_41.Add(self.text_ctrl_2, 0, 0, 0)
        sizer_40.Add(sizer_41, 0, wx.BOTTOM | wx.EXPAND, 5)
        sizer_25.Add(sizer_40, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 10)
        sizer_44.Add(self.button_2, 0, wx.ALL, 5)
        sizer_43.Add(sizer_44, 0, wx.ALIGN_RIGHT | wx.ALIGN_BOTTOM, 0)
        sizer_49.Add(sizer_43, 1, wx.ALIGN_BOTTOM, 0)
        sizer_25.Add(sizer_49, 1, wx.EXPAND, 0)
        sizer_2.Add(sizer_25, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_21.Add(sizer_2, 1, wx.ALL | wx.EXPAND, 10)
        self.notebook_1_pane_1.SetSizer(sizer_21)
        sizer_8.Add(self.label_10, 0, wx.ALL, 15)
        sizer_7.Add(sizer_8, 0, wx.EXPAND, 0)
        sizer_9.Add(self.label_3, 0, wx.SHAPED, 0)
        sizer_7.Add(sizer_9, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7, 0, wx.EXPAND, 0)
        sizer_33.Add(self.radio_btn_1, 0, wx.LEFT, 5)
        sizer_33.Add(self.radio_btn_2, 0, wx.LEFT | wx.BOTTOM, 5)
        sizer_32.Add(sizer_33, 0, wx.EXPAND, 0)
        sizer_45.Add(self.button_7, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_45.Add(self.text_ctrl_5, 1, wx.EXPAND, 0)
        sizer_34.Add(sizer_45, 1, wx.EXPAND, 0)
        sizer_32.Add(sizer_34, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_36.Add(sizer_32, 0, wx.EXPAND, 0)
        sizer_50.Add(self.label_15, 0, wx.LEFT, 120)
        sizer_36.Add(sizer_50, 1, wx.EXPAND, 0)
        sizer_42.Add(self.label_12, 0, wx.LEFT, 120)
        sizer_42.Add(self.label_13, 0, 0, 0)
        sizer_36.Add(sizer_42, 1, wx.EXPAND, 0)
        sizer_31.Add(sizer_36, 0, wx.ALL | wx.EXPAND, 10)
        sizer_48.Add(self.button_3, 0, wx.ALL | wx.ALIGN_RIGHT, 5)
        sizer_47.Add(sizer_48, 0, wx.ALIGN_RIGHT, 0)
        sizer_46.Add(sizer_47, 1, wx.ALIGN_BOTTOM, 0)
        sizer_31.Add(sizer_46, 1, wx.EXPAND, 0)
        sizer_6.Add(sizer_31, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_23.Add(sizer_6, 1, wx.ALL | wx.EXPAND, 10)
        self.notebook_1_pane_2.SetSizer(sizer_23)
        sizer_12.Add(self.bitmap_3, 0, wx.LEFT | wx.RIGHT, 20)
        sizer_11.Add(sizer_12, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_13.Add(self.label_4, 0, 0, 0)
        sizer_11.Add(sizer_13, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10.Add(sizer_11, 0, wx.EXPAND, 0)
        sizer_20.Add(self.label_6, 0, wx.TOP, 5)
        sizer_19.Add(self.label_9, 0, wx.ALIGN_BOTTOM, 0)
        sizer_19.Add(self.button_5, 0, wx.LEFT, 50)
        sizer_20.Add(sizer_19, 0, wx.LEFT | wx.EXPAND, 10)
        sizer_20.Add(self.list_box_1, 1, wx.ALL | wx.EXPAND, 5)
        sizer_17.Add(sizer_20, 1, wx.ALL | wx.EXPAND, 5)
        sizer_37.Add(self.bitmap_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_17.Add(sizer_37, 0, wx.EXPAND, 0)
        sizer_22.Add(self.label_8, 0, wx.TOP, 5)
        sizer_18.Add(self.label_5, 0, wx.ALIGN_BOTTOM, 0)
        sizer_18.Add(self.button_1, 0, wx.LEFT, 50)
        sizer_22.Add(sizer_18, 0, wx.LEFT | wx.EXPAND, 10)
        sizer_22.Add(self.list_box_2, 1, wx.ALL | wx.EXPAND, 5)
        sizer_17.Add(sizer_22, 1, wx.ALL | wx.EXPAND, 5)
        sizer_16.Add(sizer_17, 1, wx.EXPAND, 0)
        sizer_39.Add(self.button_6, 0, wx.ALL, 5)
        sizer_38.Add(sizer_39, 0, wx.ALIGN_RIGHT, 0)
        sizer_16.Add(sizer_38, 0, wx.EXPAND, 0)
        sizer_10.Add(sizer_16, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_24.Add(sizer_10, 1, wx.ALL | wx.EXPAND, 10)
        self.notebook_1_pane_3.SetSizer(sizer_24)
        self.notebook_1.AddPage(self.notebook_1_pane_1, _(u"newCaseの作成"))
        self.notebook_1.AddPage(self.notebook_1_pane_2, _(u"solverの入れ替え"))
        self.notebook_1.AddPage(self.notebook_1_pane_3, _(u"meshの入れ替え"))
        sizer_1.Add(self.notebook_1, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 15)
        sizer_15.Add(self.button_4, 0, wx.RIGHT | wx.BOTTOM, 15)
        sizer_14.Add(sizer_15, 0, wx.ALIGN_RIGHT, 0)
        sizer_1.Add(sizer_14, 0, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()
        # end wxGlade


    #case取得
    def onSelCaseStartCopy(self, event):  # wxGlade: MyFrame.<event_handler>
        self.getSourceCaseName(event)

    #newCase作成場所参照ボタン
    def onReferenceDir(self, event):  # wxGlade: MyFrame.<event_handler>
        self.referenceDir(event)

    #newCase作成開始
    def onNewCaseCopy(self, event):  # wxGlade: MyFrame.<event_handler>
        self.newCaseCopy(event)

    #solver入れ替えのcase取得
    def onSelCaseRemoveSolver(self, event):  # wxGlade: MyFrame.<event_handler>
        #self.selCaseRemoveSolver(event)
        self.getSourceSolverName(event)

    #solver入れ替え開始
    def onSolverCopy(self, event):  # wxGlade: MyFrame.<event_handler>
        self.solverCopy(event)

    #コピー元（source）のmeshを取得
    def onSelectMeshCopyCase(self, event):  # wxGlade: MyFrame.<event_handler>
        self.getMeshCopyCase(event)

    #コピー先（toCopy）のmeshを取得
    def onSelectToCopyCase(self, event):  # wxGlade: MyFrame.<event_handler>
        self.getMeshToCopyCase(event)

    #meshCopy開始
    def onCopyMesh(self, event):  # wxGlade: MyFrame.<event_handler>
        self.copyMesh(event)

    #閉じる
    def onClose(self, event):  # wxGlade: MyFrame.<event_handler>
        self.ending()
        self.Destroy()

    #openSourceFolder
    def onOpenSourceFolder(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openSourceFolder(event)

    #openCurrFolder
    def onOpenCurrFolder(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openCurrFolder(event)

    #page切り替え時
    def onChangePage(self, event):  # wxGlade: MyFrame.<event_handler>
        self.changePage(event)
        event.Skip()


    #
    #  changePage
    #    noteBookのpage切り替え時
    def changePage(self, event):
        #solver名を修正
        newSolverCaseDir = self.text_ctrl_5.GetValue()
        if newSolverCaseDir != "":
            newSolver = pyTreeFoam.getSolver(newSolverCaseDir)
            self.label_13.SetLabel(newSolver)


    #
    #  getSourceCaseName
    #    sourceCase名を取得
    def getSourceCaseName(self, event):
        global currCaseDir, selCaseCopyDir
        selCaseCopyDir = self.selectCase(currCaseDir)
        if selCaseCopyDir != "":
            self.text_ctrl_4.SetValue(selCaseCopyDir)
            newCaseName = selCaseCopyDir.split("/")[-1]
            self.text_ctrl_2.SetValue(newCaseName)

    #
    #  newCaseCopy
    #    newCase作成開始
    def newCaseCopy(self, event):
        global currCaseDir, selCaseCopyDir
        self.startCaseCopy(selCaseCopyDir, currCaseDir)


    #  selectCase
    #    copyするcaseを選択
    def selectCase(self, currCaseDir):
        if self.radio_btn_3.GetValue() == True:
            #tutorialsから入れ替え
            title = _(u"newCaseの作成")
            mess = _(u"tutorialsからcaseをコピーして、newCaseを作成します。\ncaseを選択してください。")
            [status, selCaseCopyDir] = createNewCaseDDialogShow(title, mess)
            if status != "OK":
                return ""
        elif self.radio_btn_4.GetValue() == True:
            #その他caseから入れ替え
            title = _(u"newCaseの作成")
            message = _(u"選択したcaseと同じcaseを新しく作成します。\ncaseを選択してください。")
            rootDir = os.environ["HOME"]
            currDir = currCaseDir
            selDir = currCaseDir
            [status, selCaseCopyDir] = getOpenFolderDDialogShow(title, message,
                                        rootDir, currDir, selDir)
            if status != "OK":
                return ""
        else:
            #HelyxOS用case作成
            inputTextDDialog.title = _(u"HelyxOS用case作成")
            inputTextDDialog.message = _(u"新規case名を入力してください。")
            inputTextDDialog.inputText = u"newCase"
            dialog = inputTextDDialog.MyDialog(None)
            result = dialog.ShowModal()
            if inputTextDDialog.status == "OK":
                newCaseName = inputTextDDialog.inputText
            else:
                return ""
            #case名のチェック
            pasteDir = self.text_ctrl_1.GetValue()
            if pasteDir[0] != "/":
                pasteDir = os.path.abspath(pasteDir)
            [folders, files] = pyTreeFoam.getFoldersFiles(pasteDir)
            baseName = newCaseName
            i=0
            while i<10:
                flag = 0
                for folder in folders:
                    if folder == newCaseName:
                        newCaseName = baseName + "_" + str(i)
                        flag = 1
                        break
                if flag == 0:
                    break
            if i<10:
                newCaseDir = pasteDir + "/" + newCaseName
                comm = "mkdir " + newCaseDir
                os.system(comm)
                version = runHelyxosDialog.getHelyxosVersion()
                helyxCaseDir = runHelyxosDialog.getNewCaseDir(version)
                runHelyxosDialog.checkAndCopyConstant(newCaseDir, helyxCaseDir)
                runHelyxosDialog.checkAndCopyLog(newCaseDir, helyxCaseDir)
                runHelyxosDialog.checkAndCopySystem(newCaseDir, helyxCaseDir)
                title = _(u"newCaseの作成")
                message = _(u"HelyxOS用のcaseを作成しました。")
                self.okDialog(title, message)
                return ""
            else:
                title = _(u"エラー")
                message = _(u"同じfolder名があるため、新しいcaseが作成できません。\ncase名を変更してください。")
                self.errDialog(title, message)
                return ""
        return selCaseCopyDir

    #  startCaseCopy
    #    caseCopy開始
    def startCaseCopy(self, selCaseCopyDir, currCaseDir):
        pasteDir = self.text_ctrl_1.GetValue()
        newCaseName = self.text_ctrl_2.GetValue()
        if pasteDir[0] != "/":
            pasteDir = os.path.abspath(pasteDir)
        error = copyOtherCase(pasteDir, newCaseName, selCaseCopyDir)
        if error != "":
            title = _(u"エラー")
            self.errDialog(title, error)
        else:
            title = _(u"newCaseの作成")
            mess = _(u"newCaseを作成しました。")
            self.okDialog(title, mess)

    #
    #  referenceDir
    #    newCase作成場所の参照ボタン
    def referenceDir(self, event):
        title = _(u"newCaseの作成")
        message = _(u"newCaseの作成場所を指定してください。")
        rootDir = os.environ["HOME"]
        #selDir = currCaseDir
        pasteDir = self.text_ctrl_1.GetValue()
        if pasteDir[0] != "/":
            selDir = os.path.abspath(pasteDir)
        else:
            selDir = pasteDir
        currDir = currCaseDir
        [status, pasteDir] = getOpenFolderDDialogShow(title, message, rootDir, currDir, selDir)
        if status != "OK":
            return

        self.text_ctrl_1.SetValue(pasteDir)

    #
    #  getSourceSolverName
    #    sourceCase名(solver)を取得
    def getSourceSolverName(self, event):
        global currCaseDir
        if self.radio_btn_1.GetValue() == True:
            #tutorialsから入れ替え
            title = _(u"solverの入れ替え")
            mess = _(u"選択したtutorialsCase内のsolverと入れ替えます。\ncaseを選択してください。")
            [status, tutoCaseDir] = createNewCaseDDialogShow(title, mess)
        else:
            #その他caseから入れ替え
            title = _(u"solverの入れ替え")
            message = _(u"現在のcaseのsolverを、選択したcaseのsolverに入れ替えます。\ncaseを選択してください。")
            rootDir = os.environ["HOME"]
            currDir = currCaseDir
            selDir = currCaseDir
            [status, tutoCaseDir] = getOpenFolderDDialogShow(title, message,
                                        rootDir, currDir, selDir)
        if status == "OK":
            self.text_ctrl_5.SetValue(tutoCaseDir)
            newSolver = pyTreeFoam.getSolver(tutoCaseDir)
            self.label_13.SetLabel(newSolver)

    #
    #  solverCopy
    #    solverコピー開始
    def solverCopy(self, event):
        global currCaseDir
        tutoCaseDir = self.text_ctrl_5.GetValue()
        if tutoCaseDir == "":
            return
        #solver入れ替え開始
        title = _(u"エラー")
        msg = _(u"solverコピー時、エラーが発生しました。")
        error = copySolver(tutoCaseDir, currCaseDir)
        if error != "":
            self.errDialog(title, msg)
            return
        error = pasteSolver(currCaseDir)
        if error != "":
            self.errDialog(title, msg)
            return
        error = clearNullBoundaryFields(currCaseDir)
        if error != "":
            self.errDialog(title, msg)
            return
        title = _(u"solverの入れ替え")
        msg = _(u"solverを入れ替えました。")
        print msg
        self.okDialog(title, msg)


    #
    #  readMeshLocation
    #    currCaseのpolyMeshDirを取得
    def readMeshLocation(self):
        global currCaseDir, toCopyMeshDirs, selCaseDir, selMeshDirs
        #受け入れ側のfolderを確認（regionがあるかどうか）
        toCopyDir = currCaseDir
        self.label_8.SetLabel(_(u"case名:") + toCopyDir.split("/")[-1])
        toCopyMeshDirs = getPolyMeshDirsInCase(toCopyDir)
        #  constantFolder有無チェック
        toCopyMeshDirs = checkConstantFolder(toCopyDir, toCopyMeshDirs)
        #  timeFolderを1ヶ追加
        toCopyMeshDirs = addNullTimeFolder(toCopyDir, toCopyMeshDirs)
        self.setDirsToListBox(self.list_box_2, toCopyDir, toCopyMeshDirs)
        #コピー元
        selCaseDir = toCopyDir
        selMeshDirs = []
        trueMeshDirs = getPolyMeshDirsInCase(toCopyDir)
        for meshDir in trueMeshDirs:
            selMeshDirs.append(meshDir+"/polyMesh")
        self.label_6.SetLabel(_(u"case名:") + selCaseDir.split("/")[-1])
        self.setDirsToListBox(self.list_box_1, selCaseDir, selMeshDirs)
        return
        
    #
    #  getMeshCopyCase
    #    コピー元のcaseDirを取得
    def getMeshCopyCase(self, event):
        global selMeshDirs, selCaseDir, currCaseDir
        title = _(u"meshの入れ替え")
        message = _(u"コピー先caseの「constant/polyMesh」を、選択したcaseのpolyMeshに入れ替えます。\ncaseを選択してください。")
        rootDir = os.environ["HOME"]
        #selDir = currCaseDir
        selDir = selCaseDir
        currDir = currCaseDir
        [status, selCaseDir] = getOpenFolderDDialogShow(title, message, rootDir, currDir, selDir)
        if status != "OK":
            return status
        self.label_6.SetLabel(_(u"case名:") + selCaseDir.split("/")[-1])

        #コピー元のpolyMeshを確認
        selMeshDirs = []
        meshDirs = getPolyMeshDirsInCase(selCaseDir)
        for meshDir in meshDirs:
            selMeshDirs.append(meshDir+"/polyMesh")
        self.setDirsToListBox(self.list_box_1, selCaseDir, selMeshDirs)

        #ボタンのenable設定
        if selCaseDir == currCaseDir:
            self.button_1.Enable(True)
        else:
            self.button_1.Enable(False)

    #
    #  getMeshToCopyCase
    #    コピー先のcaseDir取得
    def getMeshToCopyCase(self, event):
        global toCopyMeshDirs, toCopyCaseDir, currCaseDir
        title = _(u"meshの入れ替え")
        message = _(u"選択したcaseの「constant/polyMesh」を、コピー元caseのpolyMeshに入れ替えます。\ncaseを選択してください。")
        rootDir = os.environ["HOME"]
        selDir = toCopyCaseDir
        currDir = currCaseDir
        [status, toCopyCaseDir] = getOpenFolderDDialogShow(title, message, rootDir, currDir, selDir)
        if status != "OK":
            return status
        self.label_8.SetLabel(_(u"case名:") + toCopyCaseDir.split("/")[-1])

        #コピー先のpolyMeshを確認
        toCopyMeshDirs = []
        meshDirs = getPolyMeshDirsInCase(toCopyCaseDir)
        #meshDirs = checkConstantFolder(toCopyDir, toCopyMeshDirs)
        meshDirs = checkConstantFolder(toCopyCaseDir, meshDirs)
        meshDirs = addNullTimeFolder(toCopyCaseDir, meshDirs)
        for meshDir in meshDirs:
            toCopyMeshDirs.append(meshDir)
        self.setDirsToListBox(self.list_box_2, toCopyCaseDir, toCopyMeshDirs)

        #ボタンのenable設定
        if toCopyCaseDir == currCaseDir:
            self.button_5.Enable(True)
        else:
            self.button_5.Enable(False)

    #
    #  copyMesh
    #    meshをcopyする
    def copyMesh(self, event):
        global toCopyMeshDirs, selMeshDirs, selCaseDir, toCopyCaseDir
        if len(toCopyMeshDirs)==0 or len(selMeshDirs)==0:
            title = _(u"エラー")
            mess = _(u"コピー元、またはコピー先を選択してください。")
            self.errDialog(title, mess)
            return
        #選択しているmeshを取得
        ns = self.list_box_1.GetSelections()
        nc = self.list_box_2.GetSelections()
        if len(nc)!=1:
            title = _(u"エラー")
            mess = _(u"コピー先は、複数選択できません。")
            self.errDialog(title, mess)
            return

        #コピー元を複数選択している場合
        if len(ns) > 1:
            toCopyName = self.list_box_2.GetString(nc[0])
            if len(toCopyName.split("/")) != 1:
                mess = _(u"コピー元を複数選択する場合は、コピー先を「constant」または「timefilder」に設定してください。")
                self.errDialog(_(u"エラー"), mess)
                return
            #sourceMeshを取得
            sources = []
            for n in ns:
                source = selCaseDir + "/" + self.list_box_1.GetString(n)
                sources.append(source)
            #エラーチェック
            errFlag = 0
            for source in sources:
                meshLoc = source.split("/")[-2]
                if meshLoc == "constant":
                    errFlag = 1
                try:
                    a = float(meshLoc)
                    errFlag = 1
                except:
                    pass
                if errFlag == 1:
                    break
            if errFlag == 1:
                mess = _(u"コピー元を複数選択する場合は、regionのメッシュを選択してください。")
                self.errDialog(_(u"エラー"), mess)
                return
            #sourceMeshをクリップボードにコピー
            toCopy = toCopyCaseDir + "/" + self.list_box_2.GetString(nc[0])
            error = copyPolyMesh(sources)
            if error != "":
                print _(u"書き込みエラー発生")
                return
            #meshを貼り付け
            error = pasteMultiPolyMesh(toCopy)
            if error != "":
                self.errDialog(_(u"エラー"), error)
                return
            #regionPropertiesのコピー
            error = copyRegionProperties(selCaseDir, toCopyCaseDir)
            if error != "":
                mess = _(u"regionPropertiesがコピーできませんでした。\n新たにregionPopertiesを作成してください。")
                self.errDialog(_(u"エラー"), mess)
                return
            clearInternalBoundaryFieldMultiPolyMesh(toCopyCaseDir, toCopy,
                    sources)
            mess = _(u"regionのメッシュをコピーしました。")
            self.okDialog(_(u"meshの入れ替え"), mess)
            return

        #コピー元が1ヶのpolyMeshの場合
        source = selCaseDir + "/" + self.list_box_1.GetString(ns[0])
        toCopy = toCopyCaseDir + "/" + self.list_box_2.GetString(nc[0])
        if source == toCopy+"/polyMesh":
            mess = _(u"コピー元とコピー先が同じ設定になっています。")
            self.errDialog(_(u"エラー"), mess)
            return
        #cell数を取得
        nCellSource = getNCellPolyMesh(source)
        nCellToCopy = getNCellPolyMesh(toCopy+"/polyMesh")
        #clipBoardに書き込み
        error = copyPolyMesh([source])
        if error != "":
            print _(u"書き込みエラー発生")
            return
        #clipBoardの内容を貼り付け
        error = pastePolyMesh(toCopy, nCellSource)
        if error != "":
            print _(u"書き込みエラー発生")
            return
        #boundaryFieldとinternalFieldを整形
        if nCellSource != nCellToCopy and nCellSource != 0:
            #cell数が違う場合　（異なるmeshの場合）
            stat = self.isClearInternalFields()
            if stat == True:
                error = clearNullBoundaryFieldsInternalFields(toCopy)
            else:
                error = clearOnlyNullBoundaryFields(toCopy)
        else:
            #cell数が同じ場合　（同じメッシュの場合）
            error = clearOnlyNullBoundaryFields(toCopy)
        #現在のcaseのtimefolderからmeshを入れ替える場合
        if selCaseDir == currCaseDir:
            #現在caseのconstantにmeshをコピーする場合
            toCopyList = toCopy[len(currCaseDir)+1:].split("/")
            sourceList = source[len(currCaseDir)+1:].split("/")
            if toCopyList[0] == "constant" and sourceList[0] != "constant":
                title = _(u"timeフォルダ内のmeshクリア")
                message  = _(u"現在のcase「") + selCaseDir + _(u"」内の\n")
                message += _(u"  timeフォルダ内のpolyMeshをゴミ箱移動しますか？")
                stat = self.okCancelDialog(title, message)
                if stat == "OK":
                    mess = trashTimePolyMesh(selMeshDirs)
                    #現在caseのmeshを再取得
                    toCopyMeshDirs = getPolyMeshDirsInCase(toCopyCaseDir)
                    self.setDirsToListBox(self.list_box_2,
                                          toCopyCaseDir, toCopyMeshDirs)
                    #コピー元のmeshを再取得（現在caseと同じ)
                    selMeshDirs = []
                    for meshDir in toCopyMeshDirs:
                        selMeshDirs.append(meshDir+"/polyMesh")
                    self.setDirsToListBox(self.list_box_1, selCaseDir, selMeshDirs)
        #コピー先のmeshとcurrMeshをチェック
        self.checkCurrMesh(toCopyCaseDir, toCopy)
        #終了メッセージを表示
        title = _(u"メッシュの入れ替え")
        message = _(u"メッシュを入れ替えました。")
        if nCellSource == nCellToCopy:
            message += _(u"\n\n入れ替えたメッシュは、cell数が同じメッシュの為、ineternalFieldは、クリアしていません。")
        if nCellSource == 0:
            message += _(u"\ncell数不明の為、一部meshのみ入れ替え。\ninternalFieldは、クリアせず。")
        print message
        self.okDialog(title, message)

    #  checkCurrMesh
    #    toCopymeshとcurrMeshをチェックする。
    def checkCurrMesh(self, caseDir, copyMeshDir):
        #cpoyMeshDir内にboundaryが無ければ、戻る
        if len(glob.glob(copyMeshDir+"/polyMesh/boundary")) == 0:
            return
        #currTimeを取得
        currTime = getFoamContents.getCurrTimeFolder(caseDir)
        #regionを取得
        region = copyMeshDir.split("/")[-1]
        if region == "constant":
            region = "."
        else:
            try:
                a = float(region)
                region = "."
            except:
                pass
        #currMeshDirを取得
        relCurrMeshDir = getFoamContents.getCurrMeshDir(caseDir, currTime, 
                region, "boundary")
        currMeshDir = caseDir + "/" + relCurrMeshDir
        #  polyMeshを削除
        currMeshDir = "/".join(currMeshDir.split("/")[:-1])
        currMeshDir = os.path.abspath(currMeshDir)
        copyMeshDir = os.path.abspath(copyMeshDir)
        if currMeshDir != copyMeshDir:
            currMesh = currMeshDir[len(caseDir):]
            copyMesh = copyMeshDir[len(caseDir):]
            title = _(u"警告")
            mess = _(u"コピーしたmeshは、currMeshになっていない為、solverは認識しません。\n")
            mess += _(u"currMesh: ") + currMesh + "\n"
            mess += _(u"copyMesh: ") + copyMesh
            self.warningDialog(title, mess)

    #  isClearInternalFields
    #    internalFieldもclearするかどうか確認
    def isClearInternalFields(self):
        title = _(u"メッシュの入れ替え")
        mess = _(u"コピー元とコピー先のcell数が異なっています。\n")
        mess += _(u"コピー後のメッシュに合わせる為、")
        mess += _(u"internalFieldをクリアしますか？\n")
        mess += _(u"（nonuniform形式をuniform形式にクリアする。）")
        stat = self.yesNoDialog(title, mess)
        if stat == "YES":
            ans = True
        else:
            ans = False
        return ans


    #  setDirsToListBox
    #    dirをlistBoxにセット
    def setDirsToListBox(self, listBox, toCopyDir, toCopyMeshDirs):
        if listBox == self.list_box_2:
            #コピー先は、数字フォルダを追加
            data = []
            sTime = ""
            for meshDir in toCopyMeshDirs:
                relativeDir = meshDir[len(toCopyDir)+1:]
                dirList = relativeDir.split("/")
                folder = dirList[0]
                if len(dirList) == 1:
                    sTime = folder
                if sTime != folder:
                    data.append(folder)
                    sTime = folder
                data.append(relativeDir)
        else:
            #コピー元は、そのまま
            data = []
            for meshDir in toCopyMeshDirs:
                data.append(meshDir[len(toCopyDir)+1:])
        #listBoxのデータセット
        listBox.Clear()
        listBox.AppendItems(data)

    #
    #  openSourceFolder
    #    コピー元のfolderを開く
    def openSourceFolder(self, event):
        global selMeshDirs, selCaseDir
        nns = self.list_box_1.GetSelections()
        sourceDir = selCaseDir + "/" + self.list_box_1.GetString(nns[0])
        pyTreeFoam.openFolderNautilus(sourceDir)

    #
    #  openCurrFolder
    #    現在caseのfolderを開く
    def openCurrFolder(self, event):
        global toCopyMeshDirs, toCopyCaseDir, selCaseDir
        nnc = self.list_box_2.GetSelections()
        toCopyDir = toCopyCaseDir + "/" + self.list_box_2.GetString(nnc[0])
        pyTreeFoam.openFolderNautilus(toCopyDir)

    #
    #  initialize
    #    初期化
    def initialize(self):
        global selNote, newCase_loc, newCase_tuto, solver_tuto, currCaseDir
        global foamCase
        #初期値を設定
        selNote = 0
        newCase_loc = currCaseDir
        newCase_tuto = "yes"
        solver_tuto = "no"
        #fileを読み込み設定
        if len(glob.glob(createNewCase_data)) != 0:
            f=open(createNewCase_data); cont=f.read(); f.close()
            data = cont.split("\n")
            for line in data:
                comm = line.split()
                if len(comm) == 2:
                    if comm[0] == "selNote":
                        selNote = int(comm[1])
                    elif comm[0] == "newCase_location":
                        newCase_loc = comm[1]
                    elif comm[0] == "newCase_tutorials":
                        newCase_tuto = comm[1]
                    elif comm[0] == "solver_tutorials":
                        solver_tuto = comm[1]
        self.notebook_1.SetSelection(selNote)
        self.text_ctrl_1.SetValue(currCaseDir)
        if newCase_tuto == "yes":
            self.radio_btn_3.SetValue(True)
        else:
            self.radio_btn_4.SetValue(True)
        if solver_tuto == "yes":
            self.radio_btn_1.SetValue(True)
        else:
            self.radio_btn_2.SetValue(True)
        currSolver = pyTreeFoam.getSolver(currCaseDir)
        self.label_15.SetLabel(_(u"現在のsolver:")+currSolver)
        #if currSolver == "":
        if (len(glob.glob(currCaseDir+"/constant")) == 0
            and len(glob.glob(currCaseDir+"/system")) == 0):
            foamCase = False
            self.notebook_1_pane_2.Enable(False)
            self.notebook_1_pane_3.Enable(False)

    #
    #  ending
    #    終了処理
    def ending(self):
        global selNote, newCase_loc, newCase_tuto, solver_tuto
        selNote =  self.notebook_1.GetSelection()
        newCase_loc = self.text_ctrl_1.GetValue()
        if self.radio_btn_3.GetValue() == True:
            newCase_tuto = "yes"
        else:
            newCase_tuto = "no"
        if self.radio_btn_1.GetValue() == True:
            solver_tuto = "yes"
        else:
            solver_tuto = "no"
        cont  = "selNote " + str(selNote) + "\n"
        cont += "newCase_location " + newCase_loc + "\n"
        cont += "newCase_tutorials " + newCase_tuto + "\n"
        cont += "solver_tutorials " + solver_tuto + "\n"
        f=open(createNewCase_data, "w"); f.write(cont); f.close()
        

    def okDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_INFORMATION) 
        dlg.ShowModal()
        dlg.Destroy()

    def errDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_HAND) 
        dlg.ShowModal()
        dlg.Destroy()

    def warningDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_QUESTION) 
        dlg.ShowModal()
        dlg.Destroy()

    def okCancelDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                             wx.OK | wx.CANCEL | wx.ICON_INFORMATION) 
        ans = dlg.ShowModal()
        stat = "CANCEL"
        if ans == wx.ID_OK:
            stat = "OK"
        else:
            stat = "CANCEL"
        return stat

    def yesNoDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.YES | wx.NO | wx.ICON_INFORMATION) 
        res = dlg.ShowModal()
        if res == wx.ID_YES:
            ans = "YES"
        else:
            ans = "NO"
        dlg.Destroy()
        return ans


# end of class MyFrame

#
#  checkConstantFolder
#    constantフォルダをチェックし、無ければ追加する。
def checkConstantFolder(caseDir, meshDirs):
    if len(meshDirs) == 0:
        meshDirs += [caseDir + "/constant"]
    else:
        if meshDirs[0].split("/")[-1] != "constant":
            meshDirs = [caseDir + "/constant"] + meshDirs
    return meshDirs


#  addNullTimeFolder
#    meshのないtimeFolderを1ヶ追加
#    controlDictのdeltaT分進めたtimeFolder名を取得（存在しないtimeFolder）
def addNullTimeFolder(caseDir, meshDirs):
    nums = []
    for meshDir in meshDirs:
        num = meshDir[len(caseDir)+1:].split("/")[0]
        if num != "constant":
            nums.append(float(num))
    nums.sort()

    nullTime = "0"
    times = pyTreeFoam.getTimeFolders(caseDir)
    if len(times) > 0:
        nullTime = times[0]
    isSameNum = False
    for num in nums:
        if num == float(nullTime):
            isSameNum = True
            break
    if isSameNum == True:
        maxLoc = nums[-1]
        [deltaT] = getFoamContents.controlDict(caseDir+"/system", ["deltaT"])
        nullTime = str(maxLoc + float(deltaT))
    addTimeFolder = caseDir + "/" + nullTime
    meshDirs += [addTimeFolder]
    return meshDirs

#  copyRegionProperties
#    regionPropertiesのコピー
def copyRegionProperties(selCaseDir, toCopyCaseDir):
    copyFile = selCaseDir + "/constant/regionProperties"
    error = ""
    if len(glob.glob(copyFile)) == 0:
        error = "NG"
    else:
        comm = "cp " + copyFile + " " + toCopyCaseDir + "/constant"
        os.system(comm)
        error = ""
    return error

#  getPolyMeshDirsInCase
#    case内のpolyMesh/boundaryを探す
def getPolyMeshDirsInCase(caseDir):
    meshDirs = getConstantPolyMeshDirs(caseDir)
    meshDirs += getTimePolyMeshDirs(caseDir)
    return meshDirs

#
#  getConstantPolyMeshDirs
#    constantフォルダ内のmeshDirListを返す
def getConstantPolyMeshDirs(caseDir):
    dirs = glob.glob(caseDir + "/constant/polyMesh/boundary")
    dirs += glob.glob(caseDir + "/constant/*/polyMesh/boundary")
    meshDirs = []
    for name in dirs:
        #polyMesh/boundaryを削除
        nameList = name.split("/")[:-2]
        meshDirs.append("/".join(nameList))
    meshDirs.sort()
    return meshDirs

#
#  getTimePolyMeshDirs
#    timeFolder内のmeshDirListを返す
def getTimePolyMeshDirs(caseDir):
    dirs = glob.glob(caseDir + "/*/polyMesh/boundary")
    dirs += glob.glob(caseDir + "/*/*/polyMesh/boundary")
    aMeshDirs = []
    for name in dirs:
        nameList = name[len(caseDir)+1:].split("/")[:-2]
        try:
            a = float(nameList[0])
            aMeshDirs.append([a, nameList])
        except:
            pass
    aMeshDirs.sort()
    meshDirs = []
    for aMeshDir in aMeshDirs:
        meshDir = caseDir + "/" + "/".join(aMeshDir[1])
        meshDirs.append(meshDir)
    return meshDirs

#
#  getNCellPolyMesh
#    cell数を取得
#    owner, neighbourが存在しない場合は、「0」を戻す
def getNCellPolyMesh(meshDir):
    #owner側のcellNoを確認
    fileName = meshDir + "/owner"
    fileName = getFoamContents.checkFileName(fileName)
    if len(glob.glob(fileName))==0:
        return 0
    #f=open(fileName); cont=f.read(); f.close()
    [ascii, data] = getFoamContents.readFileSeparate(fileName)
    if getFoamContents.checkBinary(ascii) == "binary":
        numData = getFoamContents.structBinData(data[0])
        ownerMax = max(numData)
    else:
        sList = data[0][1].split()
        numData = []
        for n in sList:
            if len(n)>0:
                numData.append(int(n))
        ownerMax = max(numData)
    nMax = ownerMax
    #neighbour側のcellNoを確認
    fileName = meshDir + "/neighbour"
    fileName = getFoamContents.checkFileName(fileName)
    if len(glob.glob(fileName))==0:
        return 0
    #f=open(fileName); cont=f.read(); f.close()
    [ascii, data] = getFoamContents.readFileSeparate(fileName)
    if getFoamContents.checkBinary(ascii) == "binary":
        numData = getFoamContents.structBinData(data[0])
        neighbourMax = max(numData)
    else:
        sList = data[0][1].split()
        numData = []
        for n in sList:
            if len(n) > 0:
                numData.append(int(n))
        neighbourMax = max(numData)
    #最大のcellNoを取得
    if nMax < neighbourMax:
        nMax = neighbourMax
    return nMax

#
#  copyOtherCase
#    createNewCaseの為のcaseCopy
#       currDir:pasteDir
#       selDir:copyCase
def copyOtherCase(currDir, newCaseName, selDir):
    error = ""
    if len(selDir) == 0:
        error = _(u"dirが選択されていません。")
        print error
        return error

    if selDir[0] != "/":
        selDir = os.path.abspath(selDir.encode("utf-8"))
    if len(glob.glob(selDir)) == 0:
        error = _(u"選択した「") + selDir + _(u"」は、存在しません。")
        print error
        return error

    error = caseCopyToClipBoard(selDir)
    if error != "":
        return error

    f=open(clipBoard); contents=f.read(); f.close()
    lines = contents.decode("utf-8").split("\n")[:-1]
    copyCont = lines[0].split()
    error = pasteCase(copyCont[1:], lines[1:], currDir, newCaseName)
    return error

#
#  pasteCase
#    caseをpasteDirにpasteする
#       caseDirs:コピー元のcaseDir
#       copyDirs:コピー元のcopyするfolderとfile
#       pasteDir:コピー先のcaseDir
def pasteCase(caseDirs, copyDirs, pasteDir, newCaseName):
    newDirs = []
    for copyDir in copyDirs:
        for caseDir in caseDirs:
            if caseDir == copyDir[:len(caseDir)]:
                #case名のチェック
                newCaseDir = checkCopyDirFile(pasteDir, caseDir, newCaseName)
                if len(newCaseDir) != 0:
                    newDir = newCaseDir + copyDir[len(caseDir):]
                    newDirs.append(newDir)
                break
    if len(newDir) != 0:
        [error, mess] = pasteFoldersFilesAtGauge(copyDirs, newDirs)
        if error != "":
            print error + ":" + mess
            if error == "cancel":
                error = _(u"コピーをキャンセルしました。")
            else:
                error = mess
        return error
    else:
        error = _(u"コピーできません。")
        print error
        return error

#
#  pasteFoldersFilesAtGauge
#    folder, fileをコピー（ゲージを表示）
#       copyDirs:コピー元のfolders, files
#       newDirs:コピー先のfilders, files
def pasteFoldersFilesAtGauge(copyDirs, newDirs):
    gaugeDDialog.job = "copy"
    gaugeDDialog.title = _(u"folder, fileのコピー")
    gaugeDDialog.copyDirs = copyDirs
    gaugeDDialog.newDirs = newDirs
    dialog = gaugeDDialog.MyDialog(None)
    reult = dialog.ShowModal()
    ans = gaugeDDialog.status
    error = ""
    mess = ""
    if ans == "NG":
        #コピーをキャンセルした場合
        mess = _(u"コピーを中止しました")
        error = "cancel"
    if ans == "ERR":
        #エラーが発生した場合
        title = _(u"エラー")
        [error, mess] = gaugeDDialog.error
    return [error, mess]

#
#  checkCopyDirFile
#    コピー先に同じ名前があるかどうかチェック
def checkCopyDirFile(pasteDir, copyDir, newCaseName):
    #newDir = pasteDir + "/" + copyDir.split("/")[-1]
    newDir = pasteDir + "/" + newCaseName
    if os.path.isdir(copyDir) == True:
        if len(glob.glob(newDir)) == 0:
            return newDir
        else:
            flag = 0
            n = 0
            while (flag == 0 and n < 10):
                newDir_n = newDir + "_copy" + str(n)
                if len(glob.glob(newDir_n)) == 0:
                    flag = 1
                n += 1
            if flag == 1:
                return newDir_n
            else:
                return ""
    elif os.path.isfile(copyDir) == True:
        return newDir
    else:
        return ""


#  caseCopyToClipBoard
#    clipBoardにコピー
def caseCopyToClipBoard(caseDir):
    error = ""
    f = open(clipBoard, "w")
    cont = "caseCopy " + caseDir + "\n"
    f.write(cont)
    print "case copy--->", caseDir
    [copyDirs, copyFiles] = checkCaseDir_getContents(caseDir)
    if len(copyDirs) == 0:
        error = _(u"選択した「") + caseDir + _(u"」は、OpenFOAMのcaseDirではありません。")
        print error
        return error

    for copyDir in copyDirs:
        f.write(copyDir.encode("utf-8") + "\n")
    for copyFile in copyFiles:
        f.write(copyFile.encode("utf-8") + "\n")
    f.close()
    return error


#  checkCaseDir_getContents
#    caseのチェックとcopyすべきfolder、fileを取得
def checkCaseDir_getContents(caseDir):
    ans = [[],[]]
    noans = [[],[]]
    [folders, files] = pyTreeFoam.getFoldersFiles(caseDir)
    caseFolders = []
    numFolders = []
    for folder in folders:
        try:
            a = float(folder)
            numFolders.append([a, folder])
        except:
            if folder[:len("processor")] != "processor":
                caseFolders.append(folder)
    nFolder = 0
    for folder in caseFolders:
        if folder == "system":
            nFolder += 1
        if folder == "constant":
            nFolder += 1
    if nFolder != 2:
        return noans

    if len(numFolders) != 0:
        numFolders.sort()
        numIni = numFolders[0][1]
        caseFolders.append(numIni)
    for name in caseFolders:
        ans[0].append(caseDir + "/" + name)
    for name in files:
        ans[1].append(caseDir + "/" + name)
    return ans

#
#  pasteMultiPolyMesh
#    複数のpolyMeshをpaste
def pasteMultiPolyMesh(toCopy):
    global clipBoard
    f=open(clipBoard); contents=f.read(); f.close()
    lines = contents.decode("utf-8").split("\n")[:-1]
    sources = lines[1:]
    newDirs = []
    for source in sources:
        newDir = toCopy + "/" + "/".join(source.split("/")[-2:])
        newDirs.append(newDir)
    i = 0
    mess = ""
    while i<len(sources):
        if sources[i] == newDirs[i]:
            mess = _(u"コピー元とコピー先が同じ設定になっています。\nコピーできません。")
            break
        i += 1
    if mess == "":
        [error, mess] = pasteFoldersFilesAtGauge(sources, newDirs)
    return mess

#
#  pastePolyMesh
#    clipBoardの内容をpaste
def pastePolyMesh(toCopy, nCellSource):
    global clipBoard
    f=open(clipBoard); contents=f.read(); f.close()
    lines = contents.decode("utf-8").split("\n")[:-1]
    error = pasteMesh(lines[1:], [toCopy+"/polyMesh"], nCellSource)
    return error

#  pasteMesh
#    polyMeshのpaste
def pasteMesh(copyDirs, newDirs, nCellSource):
    error = ""
    for copyDir in copyDirs:
        if os.path.isfile(copyDir) == True:
            error = "error1"
            break
    if error != "":
        return error

    if (len(copyDirs) != len(newDirs)) or (len(copyDirs)==0 or len(newDirs)==0):
        error = "error2"
        return error

    #コピー元が一部meshのみの場合、paste先のmeshをクリアしない
    i=0
    for copyDir in copyDirs:
        if nCellSource != 0:
            os.system("rm -r " + newDirs[i])
        [error, mess] = pasteFoldersFilesAtGauge([copyDir], [newDirs[i]])
        i+=1
    return error

#
#  copyPolyMesh
#    polyMeshのコピー
def copyPolyMesh(sourceDirs):
    global clipBoard
    error = ""
    cont  = "foldersCopy\n"
    for source in sourceDirs:
        cont += source + "\n"
    #clipBoardにコピー
    f=open(clipBoard, "w"); f.write(cont); f.close()
    return error

#
#  trashTimePolyMesh
#    timeFolder内のmeshをゴミ箱に移動する
def trashTimePolyMesh(meshDirs):
    mess = ""
    delDirs = []
    for meshDir in meshDirs:
        dirList = meshDir.split("/")
        #constant以外のpolyMeshを削除の対照にする。
        if dirList[-2] != "constant" and dirList[-3] != "constant":
            delDirs.append(meshDir)
    for delDir in delDirs:
        try:
            #ファイルをゴミ箱へ移動
            obj = gio.File(delDir.encode("utf-8"))
            obj.trash()
        except:
            mess = _(u"「") + delDir + _(u"」フォルダが削除できませんでした。\n　継続しますか？")
            print mess
    return mess


#
#  copySolver
#    solverのコピー
def copySolver(tutoCaseDir, currCaseDir):
    error = ""
    cont  = "casesCopy " + tutoCaseDir + "\n"
    #constant内のfileをコピー
    checkDir = tutoCaseDir + "/constant"
    [folders, files] = pyTreeFoam.getFoldersFiles(checkDir)
    for fileName in files:
        cont += checkDir + "/" + fileName + "\n"
    #system内のfileをコピー
    checkDir = tutoCaseDir + "/system"
    [folders, files] = pyTreeFoam.getFoldersFiles(checkDir)
    for fileName in files:
        cont += checkDir + "/" + fileName + "\n"
    #fieldのコピー
    tutoTimeFolders = pyTreeFoam.getTimeFolders(tutoCaseDir)
    if len(tutoTimeFolders) != 0:
        checkDir = tutoCaseDir + "/" + tutoTimeFolders[0]
        [folders, tutoFields] = pyTreeFoam.getFoldersFiles(checkDir)
        currTimeFolders = pyTreeFoam.getTimeFolders(currCaseDir)
        checkDir = currCaseDir + "/" + currTimeFolders[0]
        [folders, currFields] = pyTreeFoam.getFoldersFiles(checkDir)
        fields = []
        for tutoField in tutoFields:
            flag = 0
            for currField in currFields:
                if tutoField == currField:
                    flag = 1
                    break
            if flag == 0:
                fields.append(tutoField)
        for field in fields:
            cont += tutoCaseDir + "/" + tutoTimeFolders[0] + "/" + field + "\n"
    #clipBoardにコピー
    f=open(clipBoard, "w"); f.write(cont); f.close()
    return error

#
#  pasteSolver
#    solverをpaste
def pasteSolver(currCaseDir):
    f=open(clipBoard); contents=f.read(); f.close()
    lines = contents.decode("utf-8").split("\n")[:-1]
    copyCont = lines[0].split()
    error = pasteSolverFiles(copyCont[1], lines[1:], currCaseDir)
    return error

#  pasteSolverFiles
def pasteSolverFiles(caseDir, copyDirs, pasteDir):
    newDirs = []
    error = ""
    for copyDir in copyDirs:
        newDir = pasteDir + copyDir[len(caseDir):]
        newDirs.append(newDir)
    if len(newDirs) != 0:
        [error, mess] = pasteFoldersFilesAtGauge(copyDirs, newDirs)
    return error

#
#  clearInternalBoundaryFieldMultiPolyMesh
#    複数polyMeshをコピーしたときのinternal, boundaryFieldクリア
def clearInternalBoundaryFieldMultiPolyMesh(caseDir, meshDir, sources):
    regionDirs = glob.glob(meshDir + "/*/polyMesh")
    regions = []
    for regionDir in regionDirs:
        region = regionDir.split("/")[-2]
        regions.append(region)
    currTime = getFoamContents.getCurrTimeFolder(caseDir)
    i = 0
    for region in regions:
        fieldDir = caseDir + "/" + currTime + "/" + region
        if len(glob.glob(fieldDir)) == 0:
            #timeFolderにregionがない場合、fieldをコピーする
            copyFieldParentToRegion(caseDir+"/"+currTime, region)
        fields = pyTreeFoam.getFieldNames(fieldDir)
        polyMeshDir = meshDir + "/" + region
        setFoamContents.clearNullBoundaryFields(fieldDir, polyMeshDir, fields)
        setFoamContents.clearInternalFields(fieldDir, polyMeshDir, fields)
        i += 1

#
#  copyFieldParentToRegion
#    timeFolderのfieldをregionにコピーする
def copyFieldParentToRegion(parentDir, region):
    comm = "mkdir " + parentDir + "/" + region
    os.system(comm)
    comm = "cp " + parentDir + "/* " + parentDir + "/" + region
    os.system(comm)

#
#  clearNullBoundaryFieldsInternalFields
#    メッシュ入れ替え時のnullpatchのみboundaryFieldとinternalFieldをクリアする
def clearNullBoundaryFieldsInternalFields(toCopyDir):
    global toCopyMeshDirs
    caseDir = toCopyCaseDir
    #polyMeshDirの取得
    polyMeshDir = toCopyDir
    #fieldDir, fieldsを取得
    time = getFoamContents.getCurrTimeFolder(caseDir)
    region = polyMeshDir.split("/")[-1]
    if region == "constant":
        region = "."
        fieldDir = caseDir + "/" + time
    else:
        try:
            a = float(region)
            region = "."
        except:
            pass
        fieldDir = caseDir + "/" + time + "/" + region
    fields = pyTreeFoam.getFieldNames(fieldDir)
    #currMeshDirの取得
    relativeCurrPolyMeshDir = getFoamContents.getCurrMeshDir(caseDir, time,
            region, "boundary")
    currPolyMeshDir = caseDir + "/" + relativeCurrPolyMeshDir
    #boundaruFieldのクリア
    setFoamContents.clearNullBoundaryAllFields(fieldDir, currPolyMeshDir)
    #internalFieldのクリア
    setFoamContents.clearInternalFields(fieldDir, currPolyMeshDir, fields)
    return ""

#
#  clearOnlyNullBoundaryFields
#    メッシュ入れ替え時のnullPatchのみboundaryfieldをクリアする
def clearOnlyNullBoundaryFields(toCopyDir):
    global toCopyMeshDirs
    caseDir = toCopyCaseDir
    #polyMeshDirの取得
    polyMeshDir = toCopyDir
    #fieldDir, fieldsを取得
    time = getFoamContents.getCurrTimeFolder(caseDir)
    region = polyMeshDir.split("/")[-1]
    if region == "constant":
        region = "."
        fieldDir = caseDir + "/" + time
    else:
        try:
            a = float(region)
            region = "."
        except:
            pass
        fieldDir = caseDir + "/" + time + "/" + region
    fields = pyTreeFoam.getFieldNames(fieldDir)
    #currMeshDirの取得
    relativeCurrPolyMeshDir = getFoamContents.getCurrMeshDir(caseDir, time,
            region, "boundary")
    currPolyMeshDir = caseDir + "/" + relativeCurrPolyMeshDir
    #boundaruFieldのクリア
    setFoamContents.clearNullBoundaryFields(fieldDir, currPolyMeshDir, fields)
    return ""


#
#  clearNullBoundaryFields
#    nullPatchのみboundaryFieldをクリアする。
#    clipBoard上のデータを読み込み、コピーしたfieldのみクリアする
def clearNullBoundaryFields(caseDir):
    #fieldを取得
    timeFolders = pyTreeFoam.getTimeFolders(caseDir)
    time = timeFolders[0]
    f=open(clipBoard); cont=f.read(); f.close()
    lines = cont.decode("utf-8").split("\n")[1:-1]
    fields = []
    fieldDirList = []
    for line in lines:
        flag = 0
        nums = line.split("/")
        #region無の場合
        try:
            a = float(nums[-2])
            fields.append(nums[-1])
            flag = 1
        except:
            flag = 0
        #region有りの場合
        if flag == 0:
            try:
                a = float(nums[-3])
                fields.append(nums[-1])
            except:
                dummy = nums[-3]
    if len(fields) == 0:
        return ""
    #polyMeshDirの取得
    polyMeshDirs = getConstantPolyMeshDirs(caseDir)
    for polyMeshDir in polyMeshDirs:
        region = polyMeshDir.split("/")[-1]
        if region != "constant":
            #region有りの場合
            #　fieldDirの取得
            fieldDir = caseDir + "/" + time + "/" + region
            #　boundaryFieldのクリア
            setFoamContents.clearNullBoundaryFields(fieldDir, polyMeshDir, fields)
        else:
            #regionなし
            fieldDir = caseDir + "/" + time
            setFoamContents.clearNullBoundaryFields(fieldDir, polyMeshDir, fields)
    return ""

#
#  initialize
#
def initialize():
    global currCaseDir, toCopyCaseDir
    currCaseDir = os.getcwd()
    toCopyCaseDir = currCaseDir

if __name__ == "__main__":
    import gettext
    treeFoamPath = os.environ["TreeFoamPath"]
    gettext.install("treefoam", treeFoamPath + "/data/locale")

    initialize()
    #app = wx.PySimpleApp(0)
    app = wx.App(0)
    #wx.InitAllImageHandlers()
    frame_1 = MyFrame(None, -1, "")
    app.SetTopWindow(frame_1)
    frame_1.Show()
    app.MainLoop()
