#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   createBlockAndSnappyDict.py
#
#       blockMesh、snappyHexMeshDict作成用のcsvFileの作成
#       csvFileからmeshDictを作成する。
#
#   13/10/20    新規作成
#      10/26    バグ修正
#      12/02    sectにregを追加（refinementRegionのみを追加）
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/01/05    blockMeshDict中の「convertToMeters」のチェック追加
#      02/04    modelSize取得（getmodelSize）時、patchのみ→全stlに変更
#      03/18    checkStlFileName:圧縮時のfileNameが取得できなかった事を修正
#      07/12    sectに「face」を追加
#      07/17    sectに「faceZone」を追加
#      07/18    allowFaceZoneを操作する様に修正
#      09/07    setCastellatedSnapFlagを追加（snappyHexMeshのflagをセット）
#   15/01/22    setGeometryData:バグ修正（"\n"が余分にｾｯﾄされていた）
#      09/18    sectに「wall」を追加
#      10/03    sectに「empty、symmetry」を追加。
#               addLayersのflagを「false」に固定。
#      11/11    readCsvData:snappy用のcsvファイルかどうかエラーチェック追加
#      12/12    getFoamContents.getKeywordPointerMemをgetKeywordPointerに変更
#               getKeywordPointerは、コメントが有っても検索できる。
#


import os
import csv
import glob
import math
import struct
import getFoamContents

import logFileCreater; logFileCreater.log()


No=[]; stl=[]; sect=[]; featureEdge=[]; minLevel=[]; maxLevel=[]
caseDir = ""

#
#  setCsvData
#    defaultのcsvファイル作成
def setCsvData():
    row = []
    row.append(["", "", "", "", "", ""])
    line = ["<blockMesh>", "", "x", "y", "z", _(u"備考")]
    row.append(line)
    line = ["", "cellSize", "0.002", "0.002", "0.002", _(u"blockMeshのcellSize")]
    row.append(line)
    line = ["", "overBlockSize", "5", "5", "5" ,_(u'"cells: stlのMinMax値を越えるcell数"')]
    row.append(line)
    line = ["", "", "", "", "", ""]
    row.append(line)
    line = ["<snappyHexMesh>", "", "", "", "", ""]
    row.append(line)
    line = ["", "mesh", "0.0", "0.0", "0.0", _(u"meshの位置(materialPoint)")]
    row.append(line)
    line = ["stlFile", '"sect\n(patch/wall/\nempty/symmetry(Plane)/\nfaceZone/face/\ncellZone/reg)"', '"featureEdge\ncellSize"',
            '"base\ncellSize"', '"fine\ncellSize"', _(u'"featureEdge: cellSizeを入力したstlのみ抽出。\nbase: surface, regionとも設定する。"')]
    row.append(line)
    return row

#
#  writeCsvData
#    csvデータの書き込み
def writeCsvData(caseDir, fileName, data):
    f=open(caseDir+"/"+fileName, "w")
    for line in data:
        cont = ",".join(line) + "\n"
        f.write(cont.encode("utf-8"))
    f.close()
    return 0

#
#  addStlFileName
#    stlFileを追加
def addStlFileName(stlDir, data):
    files = []
    stlFiles = glob.glob(stlDir + "/*.stl")
    for stlFile in stlFiles:
        name = stlFile.split("/")[-1].split(".")[0]
        files.append(name)
    files.sort()
    for name in files:
        line = [name, "", "", "", "", ""]
        data.append(line)
    return data

#
#  addBlockCellSize
#    blockMeshのcellSize（30分割)
def addBlockCellSize(stlDir, data):
    flag = 0
    xVals=[]; yVals=[]; zVals=[]
    i = 0
    cellSize = -1
    for row in data:
        if flag == 1:
            fileName = stlDir + "/" + row[0] + ".stl"
            [mins, maxs] = getMinMaxValueStl(fileName)
            xVals.append(mins[0])
            xVals.append(maxs[0])
            yVals.append(mins[1])
            yVals.append(maxs[1])
            zVals.append(mins[2])
            zVals.append(maxs[2])
        if row[0] == "stlFile":
            flag = 1
        if row[1] == "cellSize":
            cellSize = i
        if row[1] == "mesh":
            meshLoc = i
        i += 1
    if len(xVals) != 0:
        minx = min(xVals); maxx = max(xVals)
        miny = min(yVals); maxy = max(yVals)
        minz = min(zVals); maxz = max(zVals)
    else:
        minx = 0.0; maxx = 0.0
        miny = 0.0; maxy = 0.0
        minz = 0.0; maxz = 0.0
    #cellSizeの設定
    dxf = (maxx - minx) * 1.0 / 30.0
    dyf = (maxy - miny) * 1.0 / 30.0
    dzf = (maxz - minz) * 1.0 / 30.0
    dx = getSizeLog10(dxf)
    dy = getSizeLog10(dyf)
    dz = getSizeLog10(dzf)
    if cellSize >= 0:
        data[cellSize][2:5] = [str(dx), str(dy), str(dz)]
    #meshLocationの設定
    meshx = (maxx + minx)/2.0
    meshy = (maxy + miny)/2.0
    meshz = (maxz + minz)/2.0
    #meshx = getSizeLog10(meshx)
    #meshy = getSizeLog10(meshy)
    #meshz = getSizeLog10(meshz)
    data[meshLoc][2:5] = [str(meshx), str(meshy), str(meshz)]
    return data

#  getSizeLog10
def getSizeLog10(x):
    if x<=0.0:
        return 0.0
    lx = math.log10(x)
    if lx>0.0:
        llx = 10**float(int(lx) - 1)
    else:
        llx = 10**float(int(lx) - 2)
    xx = int(x/llx + 0.5) * llx
    return xx
#
#  addStlBlockSize
#    stl のblockサイズを取得
def addStlBlockSize(stlDir, data):
    flag = 0
    i = 0
    for row in data:
        if flag == 1:
            fileName = stlDir + "/" + row[0] + ".stl"
            [x, y, z] = getStlBlockSize(fileName)
            if not (x==0.0 and y==0.0 and z==0.0):
                data[i][5] = '"(' + ' '.join([str(x), str(y), str(z)]) + ')"'
        if row[0] == "stlFile":
            flag = 1
        i += 1
    return data

#  getStlBlockSize
def getStlBlockSize(fileName):
    minMax = getMinMaxValueStl(fileName)
    dx = minMax[1][0] - minMax[0][0]
    dy = minMax[1][1] - minMax[0][1]
    dz = minMax[1][2] - minMax[0][2]
    dx = getSizeLog10(dx)
    dy = getSizeLog10(dy)
    dz = getSizeLog10(dz)
    return [dx, dy, dz]

#
#  readCsvData
#    csvデータの読み込み
#    全てtextとして読み込む
def readCsvData(fileName):
    #tab区切りorカンマ区切りのチェック
    f=open(fileName); line=f.readline(); f.close()
    if len(line.split("\t")) != 1:
        #tab区切り
        f=open(fileName); tabCont=f.readlines(); f.close()
        contents = []
        for line in tabCont:
            contents.append(",".join(line.split("\t")))
        cont = csv.reader(contents)
    else:
        #カンマ区切り
        cont = csv.reader(file(fileName))
    #データの取り込み
    data = []
    otherData = []
    cellSize = []
    blockSize = []
    meshLoc = []
    readStl = 0
    for row in cont:
        flag = 0
        for val in row:
            if val !="":
                flag = 1
                break
        if flag == 1:
            if row[1] == "cellSize":
                cellSize = [row[2], row[3], row[4]]
            elif row[1] == "blockSize" or row[1] == "overBlockSize":
                blockSize = [row[2], row[3], row[4]]
            elif row[1] == "mesh":
                meshLoc = [row[2], row[3], row[4]]
            elif row[0] == "stlFile":
                readStl = 1
            else:
                if readStl == 1:
                    data.append(row)
    #空白cellに値をセット
    cellSize = setNullValue(cellSize)
    blockSize = setNullValue(blockSize)
    #その他データをセット
    otherData = [cellSize, blockSize, meshLoc]
    return [otherData, data]

#  setNullValue
#    空白cellに値をセット
def setNullValue(vals):
    if len(vals) == 0:
        return vals

    #空白のチェック
    if vals[0] == "":
        vals = [0, 0, 0]
    if vals[1] == "":
        vals[1] = vals[0]
    if vals[2] == "":
        vals[2] = vals[0]
    #数字のチェック
    i = 0
    while i<len(vals):
        try:
            a = float(vals[i])
        except:
            vals[i] = "0.0"
        i += 1
    return vals

#
#  getBaseCellSize
#    csvのblockCellSizeをチェックし、取得する
def getBaseCellSize(loc):
    size = []
    error = 0
    for val in loc:
        if val != "":
            try:
                size.append(float(val))
            except:
                error = 1
                size.append(0.0)
    if error == 0:
        return [0, size]
    else:
        print _(u"error: blockMeshのcellSizeの値が間違っています。")
        return [1, size]

#
#  getValueCsvData
#    csv データを各global変数に代入する
def getValueCsvData(cellSize, data):
    global No, stl, sect, featureEdge, minLevel, maxLevel
    error = 0
    for line in data:
        try:
            #No.append(line[0])
            stl.append(line[0])
            sect.append(line[1])
            featureEdge.append(line[2])
            minLevel.append(line[3])
            maxLevel.append(line[4])
        except:
            error = 1
            print _(u"error: csvファイルから値が取得できません。")
            break
    dAve = sum(cellSize)/len(cellSize)
    dMax = max(cellSize)
    dMin = min(cellSize)
    if error == 0:
        i = 0
        while i<len(featureEdge):
            #sect, minLevelの関係をチェック
            if (sect[i] == "wall" or sect[i] == "patch" or sect[i] == "cellZone" or
                sect[i] == "reg" or sect[i] == "faceZone" or sect[i] == "face"):
                if minLevel[i] == "":
                    error = 1
                    print _(u"error: patch/cellZoneに対し、cellSizeが設定されていません。\nbaseのcellSizeは、省略できません。")
                else:
                    try:
                        a = float(minLevel[i])
                        if a == 0:
                            error = 1
                            print _(u"error: cellSizeが「0」になっています。\ncellSize=0では、メッシュが作れません。")
                    except:
                        error = 1
                        print _(u"error: cellSizeは、数字を入力してください。\ncellSizeに文字が入力されています。")
            #cellSizeをlevelに変換
            if featureEdge[i] != "":
                level = calcCellLevel(dAve, featureEdge[i])
                featureEdge[i] = str(level)
            if minLevel[i] != "":
                level = calcCellLevel(dAve, minLevel[i])
                minLevel[i] = str(level)
            if maxLevel[i] != "":
                level = calcCellLevel(dAve, maxLevel[i])
                maxLevel[i] = str(level)
            else:
                #maxLevelが空白の場合、minLvelと同じ値にセット
                maxLevel[i] = minLevel[i]
            if error != 0:
                break
            i += 1
    return error

#  calcCellLevel
#    cell寸法をcellLevelに変換する
def calcCellLevel(dAve, strDx):
    level = 0
    try:
        a = dAve/float(strDx)
        if a > 1.0:
            level = int(math.log(a)/math.log(2) + 0.5)
        else:
            level = 0
    except:
        level = 0
    return level

#
#  checkStlFileName
#    stlFile有無チェック
def checkStlFileName(caseDir, stlDir, stl, sect, featureEdge):
    error = 0
    fileNames=[]
    i = 0
    for name in stl:
        if (sect[i] == "patch" or sect[i] == "wall" or sect[i] == "cellZone" or
            sect[i] == "reg" or sect[i] == "faceZone" or sect[i] == "face"):
            fileNames.append(name+".stl")
        if featureEdge[i] != "":
            fileNames.append(name+".eMesh")
        i += 1
    noFiles = []
    if len(fileNames) != 0:
        for name in fileNames:
            fileName = stlDir + "/" + name
            if len(glob.glob(fileName)) == 0:
                fileName = getFoamContents.checkFileName(fileName)
                if len(glob.glob(fileName)) == 0:
                    noFiles.append(name)
        if len(noFiles) != 0:
            error = 1 
    else:
        print _(u"error: patch/cellZone/regが未記入です。")
        error = 1
    if error == 0:
        #必要なfileをセット
        distDir = caseDir + "/constant/triSurface"
        if len(glob.glob(distDir)) == 0:
            comm = "mkdir " + distDir
            os.system(comm)
        for name in fileNames:
            sourceName = stlDir + "/" + name
            distName = distDir + "/" + name
            sourceName = getFoamContents.checkFileName(sourceName)
            distName = getFoamContents.checkFileName(distName)
            comm = "cp " + sourceName + " " + distName
            os.system(comm)
    return [error, noFiles]

#
#  checkNeededEMeshFileName
#    必要なeMeshファイルを取得する
def checkNeededEMeshFileName(caseDir, stlDir, stl, sect, featureEdge):
    eMeshFiles = []
    i = 0
    for name in stl:
        if featureEdge[i] != "":
            eMeshFiles.append(name+".stl")
        i += 1
    return eMeshFiles

#
#  readSnappyHexMeshDict
#    Dictファイルを読み込む
def readSnappyHexMeshDict(caseDir):
    fileName = caseDir + "/system/snappyHexMeshDict"
    if len(glob.glob(fileName)) == 0:
        cont = ""
    else:
        f=open(fileName); cont=f.read(); f.close()
    return cont

#
#  readBlockMeshDict
#
def readBlockMeshDict(caseDir):
    fileName = caseDir + "/system/blockMeshDict"
    if len(glob.glob(fileName)) == 0:
        fileName = caseDir + "/constant/polyMesh/blockMeshDict"
        if len(glob.glob(fileName)) == 0:
            cont = ""
            return

    f=open(fileName); cont=f.read(); f.close()
    return cont

#
#  writeSnappyHexMeshDict
#    snappyHexMeshを書き込む
def writeSnappyHexMeshDict(caseDir, snappyCont):
    fileName = caseDir + "/system/snappyHexMeshDict"
    f=open(fileName, "w"); f.write(snappyCont); f.close()
    return 0

#
#  writeBlockMeshDict
#    blockMeshを書き込む
def writeBlockMeshDict(caseDir, blockCont):
    fileName = caseDir + "/system/blockMeshDict"
    if len(glob.glob(fileName)) == 0:
        fileName = caseDir + "/constant/polyMesh/blockMeshDict"
    f=open(fileName, "w"); f.write(blockCont); f.close()
    return 0

#
#  setBlockPointData
#    blockMeshDictの座標作成
def setBlockPointData(blockCont, blockSize, baseCell):
    #pVer = getFoamContents.getKeywordPointerMem("vertices", blockCont, 0)
    pVer = getKeywordPointer("vertices", blockCont, 0)
    if pVer < 0:
        return blockCont

    [dummy, pEnd] = getFoamContents.getSmallPairMem(blockCont, pVer)
    minx=blockSize[0][0]; miny=blockSize[0][1]; minz=blockSize[0][2]
    maxx=blockSize[1][0]; maxy=blockSize[1][1]; maxz=blockSize[1][2]
    p0 = " ".join([str(minx), str(miny), str(minz)])
    p1 = " ".join([str(maxx), str(miny), str(minz)])
    p2 = " ".join([str(maxx), str(maxy), str(minz)])
    p3 = " ".join([str(minx), str(maxy), str(minz)])
    p4 = " ".join([str(minx), str(miny), str(maxz)])
    p5 = " ".join([str(maxx), str(miny), str(maxz)])
    p6 = " ".join([str(maxx), str(maxy), str(maxz)])
    p7 = " ".join([str(minx), str(maxy), str(maxz)])
    cont = "vertices\n" + 4*" " + "(\n"
    cont += 8*" " + "( " + p0 + " )\n"
    cont += 8*" " + "( " + p1 + " )\n"
    cont += 8*" " + "( " + p2 + " )\n"
    cont += 8*" " + "( " + p3 + " )\n"
    cont += 8*" " + "( " + p4 + " )\n"
    cont += 8*" " + "( " + p5 + " )\n"
    cont += 8*" " + "( " + p6 + " )\n"
    cont += 8*" " + "( " + p7 + " )\n"
    cont += 4*" " + ")"
    blockCont = blockCont[:pVer] + cont + blockCont[pEnd:]
    pBlo = getFoamContents.getKeywordPointerMem("blocks", blockCont, pVer)
    [dummy, pEnd] = getFoamContents.getSmallPairMem(blockCont, pBlo)
    [dx, dy, dz] = baseCell
    nx = int((maxx-minx)/dx + 0.5)
    ny = int((maxy-miny)/dy + 0.5)
    nz = int((maxz-minz)/dz + 0.5)
    cont = ("blocks\n" + 4*" " + "(\n"
            + 4*" " + "hex (0 1 2 3 4 5 6 7) ("
            + " ".join([str(nx), str(ny), str(nz)])
            + ") simpleGrading (1 1 1)\n"
            + 4*" " + ")")
    newCont = blockCont[:pBlo] + cont + blockCont[pEnd:]
    return newCont

#
#  setCastellatedSnapFlag
#    snappyHexMeshのflagをセット
def setCastellatedSnapFlag(snappyCont):
    newLine = "castellatedMesh true;"
    [snappyCont, p, stat] = getFoamContents.replace1lineMem(snappyCont, 0, newLine)
    newLine = "snap true;"
    [snappyCont, p, stat] = getFoamContents.replace1lineMem(snappyCont, 0, newLine)
    newLine = "addLayers false;"
    [snappyCont, p, stat] = getFoamContents.replace1lineMem(snappyCont, 0, newLine)
    return snappyCont

#
#  setGeometryData
#    geometry部を書き換え
def setGeometryData(snappyCont, stl, sect):
    #pGeom = getFoamContents.getKeywordPointerMem("geometry", snappyCont, 0)
    pGeom = getKeywordPointer("geometry", snappyCont, 0)
    if pGeom < 0:
        return snappyCont

    [dummy, pEnd] = getFoamContents.getKeywordContentsMem("geometry", snappyCont, pGeom)
    cont = "geometry\n" + 4*" " + "{\n"
    i = 0
    for name in stl:
        if (sect[i] == "wall" or sect[i] == "patch" or sect[i] == "cellZone" or
            sect[i] == "reg" or sect[i] == "faceZone" or sect[i] == "face" or
            sect[i] == "empty" or sect[i] == "symmetry" or sect[i] == "symmetryPlane"):
            cont += 8*" " + name + ".stl\n"
            cont += 8*" " + "{\n"
            cont += 12*" " + "type triSurfaceMesh;\n"
            cont += 12*" " + "name " + name + ";\n"
            cont += 8*" " + "}\n\n"
        i += 1
    #cont += 4*" " + "}\n"
    cont += 4*" " + "}"
    newCont = snappyCont[:pGeom] + cont + snappyCont[pEnd:]
    return newCont

#
#  setFeatureEdgeData
#    featureEdgeを書き換え
def setFeatureEdgeData(snappyCont, stl, sect, featureEdge):
    #pFt = getFoamContents.getKeywordPointerMem("features", snappyCont, 0)
    pFt = getKeywordPointer("features", snappyCont, 0)
    if pFt < 0:
        return snappyCont

    [dummy, pEnd] = getFoamContents.getSmallPairMem(snappyCont, pFt)
    cont = "features\n" + 8*" " + "(\n"
    i = 0
    for name in stl:
        if featureEdge[i] != "":
            cont += 8*" " + "{\n"
            cont += 12*" " + 'file "' + name + '.eMesh";\n'
            cont += 12*" " + "level " + featureEdge[i] + ";\n"
            cont += 8*" " + "}\n\n"
        i += 1
    cont += 8*" " + ")"
    newCont = snappyCont[:pFt] + cont + snappyCont[pEnd:]
    return newCont

#
#  setRefinementSurfaces
#    MinMaxレベル、cellZoneの設定
def setRefinementSurfaces(snappyCont, stl, sect, minLevel, maxLevel):
    #pRs = getFoamContents.getKeywordPointerMem("refinementSurfaces", snappyCont, 0)
    pRs = getKeywordPointer("refinementSurfaces", snappyCont, 0)
    if pRs < 0:
        return snappyCont

    [dummy, pEnd] = getFoamContents.getMiddlePairMem(snappyCont, pRs)
    cont = "refinementSurfaces\n" + 8*" " + "{\n"
    i = 0
    for name in stl:
        if sect[i] == "wall":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "regions\n"
            cont += 16*" " + "{\n"
            cont += 16*" " + "}\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "patch":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "regions\n"
            cont += 16*" " + "{\n"
            cont += 16*" " + "}\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 16*" " + "patchInfo\n"
            cont += 16*" " + "{\n"
            cont += 20*" " + "type patch;\n"
            cont += 20*" " + "inGroups (patches);\n"
            cont += 16*" " + "}\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "cellZone":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "cellZoneInside inside;\n"
            cont += 16*" " + "cellZone " + name + ";\n"
            cont += 16*" " + "faceZone " + name + ";\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "faceZone":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "faceZone " + name + ";\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "empty":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "regions\n"
            cont += 16*" " + "{\n"
            cont += 16*" " + "}\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 16*" " + "patchInfo\n"
            cont += 16*" " + "{\n"
            cont += 20*" " + "type empty;\n"
            cont += 20*" " + "inGroups (empty);\n"
            cont += 16*" " + "}\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "symmetry":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "regions\n"
            cont += 16*" " + "{\n"
            cont += 16*" " + "}\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 16*" " + "patchInfo\n"
            cont += 16*" " + "{\n"
            cont += 20*" " + "type symmetry;\n"
            cont += 20*" " + "inGroups (symmetry);\n"
            cont += 16*" " + "}\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "symmetryPlane":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "regions\n"
            cont += 16*" " + "{\n"
            cont += 16*" " + "}\n"
            cont += 16*" " + "level (" + minLevel[i] + " " + maxLevel[i] + ");\n"
            cont += 16*" " + "patchInfo\n"
            cont += 16*" " + "{\n"
            cont += 20*" " + "type symmetryPlane;\n"
            cont += 20*" " + "inGroups (symmetryPlane);\n"
            cont += 16*" " + "}\n"
            cont += 12*" " + "}\n\n"
        i += 1
    cont += 8*" " + "}"
    newCont = snappyCont[:pRs] + cont + snappyCont[pEnd:]
    return newCont

#
#  getKeywordPointer
#    contents内をpsからwordを検索する。
#    コメント文があっても検索できる。検索wordが見つからない場合は、「-1」を戻す
def getKeywordPointer(word, contents, ps):
    p = ps
    keyword = " "
    while keyword != "" and p < len(contents) and keyword != word:
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    if keyword == "":
        return -1
    return p - len(word) - 1

#
#  setRefinementRegions
#    regionのレベルの設定
def setRefinementRegions(snappyCont, stl, sect, minLevel, baseCellSize):
    pRs = getKeywordPointer("refinementRegions", snappyCont, 0)
    if pRs < 0:
        return snappyCont

    [dummy, pEnd] = getFoamContents.getMiddlePairMem(snappyCont, pRs)
    #"refinementRegions"の内容を入れ替え
    cont = "refinementRegions\n" + 8*" " + "{\n"
    i = 0
    for name in stl:
        if sect[i] == "cellZone" or sect[i] == "reg":
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "levels (( 1E5 " + minLevel[i] + " ));\n"
            cont += 16*" " + "mode inside;\n"
            cont += 12*" " + "}\n\n"
        elif sect[i] == "face":
            distance = baseCellSize[0]
            disStr = str(distance)
            cont += 12*" " + name + "\n"
            cont += 12*" " + "{\n"
            cont += 16*" " + "levels (( "+ disStr + " "  + minLevel[i] + " ));\n"
            cont += 16*" " + "mode distance;\n"
            cont += 12*" " + "}\n\n"
        i += 1
    cont += 8*" " + "}"
    newCont = snappyCont[:pRs] + cont + snappyCont[pEnd:]
    return newCont

#
#  checkFaceZone
#    faceZoneを作成する場合はallowFaceZZoneをtrueに設定
def checkFaceZone(snappyCont, sect):
    faceZone = False
    for item in sect:
        if item == "faceZone":
            faceZone = True
            break
    if faceZone == True:
        #pRs = getFoamContents.getKeywordPointerMem("allowFreeStandingZoneFaces",
        #            snappyCont, 0)
        pRs = getKeywordPointer("allowFreeStandingZoneFaces", snappyCont, 0)
        if pRs < 0:
            newCont = snappyCont
        else:
            [lineCont, pEnd, kind] = getFoamContents.get1lineMem(snappyCont, pRs)
            cont = "allowFreeStandingZoneFaces true;"
            newCont = snappyCont[:pRs] + cont + snappyCont[pEnd:]
    else:
        newCont = snappyCont
    return newCont

#
#  setLocationInMesh
#    mesh位置（materialPoint）の座標をセット
def setLocationInMesh(snappyCont, meshLoc):
    #pRs = getFoamContents.getKeywordPointerMem("locationInMesh", snappyCont, 0)
    pRs = getKeywordPointer("locationInMesh", snappyCont, 0)
    if pRs < 0:
        return snappyCont

    [lineCont, pEnd, kind] = getFoamContents.get1lineMem(snappyCont, pRs)
    cont = "locationInMesh (" + " ".join(meshLoc) + ");"
    newCont = snappyCont[:pRs] + cont + snappyCont[pEnd:]
    return newCont

#
#  getModelSize
#    modelのmin,max値を取得
def getModelSize(stlDir, stl, sect):
    fileNames=[]
    for name in stl:
        fileNames.append(name+".stl")
    xValues=[]; yValues=[]; zValues=[]
    for fileName in fileNames:
        [minxyz, maxxyz] = getMinMaxValueStl(stlDir+"/"+fileName)
        xValues.append(minxyz[0])
        xValues.append(maxxyz[0])
        yValues.append(minxyz[1])
        yValues.append(maxxyz[1])
        zValues.append(minxyz[2])
        zValues.append(maxxyz[2])
    minx = min(xValues); maxx = max(xValues)
    miny = min(yValues); maxy = max(yValues)
    minz = min(zValues); maxz = max(zValues)
    return [[minx, miny, minz], [maxx, maxy, maxz]]

#
#  getMinMaxValueStl
#    stlファイルのminMax値を取得
def getMinMaxValueStl(fileName):
    #fileFormatのチェック
    f=open(fileName); cont=f.readlines(10); f.close()
    if cont[0][:len("solid")] != "solid":
        #binaryの場合
        ans = readBinaryStl(fileName)
        return ans
    #asciiの場合
    f=open(fileName); cont=f.readlines(); f.close()
    xValues=[]; yValues=[]; zValues=[]
    for line in cont:
        if line.find("vertex") >= 0:
            values = line.split()
            xValues.append(float(values[1]))
            yValues.append(float(values[2]))
            zValues.append(float(values[3]))
    if len(xValues) != 0:
        minx = min(xValues); maxx = max(xValues)
        miny = min(yValues); maxy = max(yValues)
        minz = min(zValues); maxz = max(zValues)
    else:
        minx=0.0; maxx=0.0
        miny=0.0; maxy=0.0
        minz=0.0; maxz=0.0
    return [[minx, miny, minz], [maxx, maxy, maxz]]

#
#  readBinaryStl
#    binaryのstl読込
def readBinaryStl(fileName):
    f=open(fileName); cont=f.read(); f.close()
    #header部をスキップ
    i = 80
    #三角形の数
    num = struct.unpack("1i", cont[i:i+4])
    nTet = num[0]
    i += 4
    #座標を取得
    xVal=[]; yVal=[]; zVal=[]
    while i<len(cont):
        num = struct.unpack("12f", cont[i:i+48])
        #単位ベクトルはスキップ
        xVal += [num[3]] + [num[6]] + [num[9]]
        yVal += [num[4]] + [num[7]] + [num[10]]
        zVal += [num[5]] + [num[8]] + [num[11]]
        #最後の2byte分も加えてスキップ
        i += 48 + 2
    if len(xVal) != 0:
        minx = min(xVal); maxx = max(xVal)
        miny = min(yVal); maxy = max(yVal)
        minz = min(zVal); maxz = max(zVal)
    else:
        minx=0.0; maxx=0.0
        miny=0.0; maxy=0.0
        minz=0.0; maxz=0.0
    return [[minx, miny, minz], [maxx, maxy, maxz]]

#
#  getBlockSize
#    blockMeshのサイズを求める
def getBlockSize(modelSize, bSize, nAddCells):
    nCellx = float(nAddCells[0])
    nCelly = float(nAddCells[1])
    nCellz = float(nAddCells[2])
    [dx, dy, dz] = bSize
    minx = float(int((modelSize[0][0] - nCellx*dx)/dx - 0.5))*dx
    miny = float(int((modelSize[0][1] - nCelly*dy)/dy - 0.5))*dy
    minz = float(int((modelSize[0][2] - nCellz*dz)/dz - 0.5))*dz
    maxx = float(int((modelSize[1][0] + nCellx*dx)/dx + 0.5))*dx
    maxy = float(int((modelSize[1][1] + nCelly*dy)/dy + 0.5))*dy
    maxz = float(int((modelSize[1][2] + nCellz*dz)/dz + 0.5))*dz
    return [[minx, miny, minz], [maxx, maxy, maxz]]

#
#  checkBlockMeshMag
#    blockMeshDictの倍率（convertToMeters）のチェック修正
def checkBlockMeshMag(cont):
    p = getFoamContents.skipFoamFileMem(cont, 0)
    #p = getFoamContents.getKeywordPointerMem("convertToMeters", cont, p)
    p = getKeywordPointer("convertToMeters", cont, p)
    if p < 0:
        return cont

    [lineCont, pEnd, kind] = getFoamContents.get1lineMem(cont, p)
    if lineCont != "":
        changeCont = "convertToMeters 1;"
        newCont = cont[:p] + changeCont + cont[pEnd:]
    else:
        newCont = cont
    return newCont
    

#
#  createCsvFile
#  -------------
#    defaultのcsvファイル作成
def createCsvFile(caseDir, stlDir, fileName):
    data = setCsvData()
    data = addStlFileName(stlDir, data)
    data = addBlockCellSize(stlDir, data)
    data = addStlBlockSize(stlDir, data)
    #error = writeCsvData(caseDir, fileName, data)
    error = writeCsvData(stlDir, fileName, data)
    return error

#
#  checkStlEMeshFiles
#  ------------------
#    fileの有無チェック
def checkStlEMeshFiles(caseDir, stlDir, fileName):
    global stl, sect, featureEdge, minLevel, maxLevel
    stl=[]; sect=[]; featureEdge=[]; minLevel=[]; maxLevel=[]
    nonFiles = []
    [otherData, data] = readCsvData(fileName)
    if len(otherData[0]) == 0 or len(otherData[1]) == 0 or len(otherData[2]) == 0:
        error = 1
        return [error, nonFiles]
    [error, baseCellSize] = getBaseCellSize(otherData[0])
    if error == 0:
        error = getValueCsvData(baseCellSize, data)
    if error == 0:
        [error, nonFiles] = checkStlFileName(caseDir, stlDir, stl, sect,
                                featureEdge)
    return [error, nonFiles]

#
#  getNeededEMeshFiles
#  -------------------
#    featureEdgeを作成するstlファイル名を取得して返す
def getNeededEMeshFiles(caseDir, stlDir, fileName):
    global stl, sect, featureEdge, minLevel, maxLevel
    stl=[]; sect=[]; featureEdge=[]; minLevel=[]; maxLevel=[]
    nonFiles = []
    [otherData, data] = readCsvData(fileName)
    [error, baseCellSize] = getBaseCellSize(otherData[0])
    if error == 0:
        error = getValueCsvData(baseCellSize, data)
    if error == 0:
        eMeshFiles = checkNeededEMeshFileName(caseDir, stlDir, stl, sect,
                                featureEdge)
    return eMeshFiles


#
#  createMeshDict
#  --------------
#    csvファイルからblockMesh, snappyHexMeshDictを作成する
def createMeshDict(caseDir, stlDir, fileName):
    global stl, sect, featureEdge, minLevel, maxLevel
    stl=[]; sect=[]; featureEdge=[]; minLevel=[]; maxLevel=[]
    [otherData, data] = readCsvData(fileName)
    [blockCellSize, nAddCells, meshLoc] = otherData
    if len(blockCellSize) == 0 or len(nAddCells) == 0 or len(meshLoc) == 0:
        error = 1
        return error
    [error, baseCellSize] = getBaseCellSize(blockCellSize)
    if error == 0:
        error = getValueCsvData(baseCellSize, data)
    if error == 0:
        [error, nonFiles] = checkStlFileName(caseDir, stlDir, stl, sect,
                                featureEdge)
        if error != 0:
            print _(u"以下のファイルが存在しません")
            print nonFiles
    if error == 0:
        modelSize = getModelSize(stlDir, stl, sect)
        blockSize = getBlockSize(modelSize, baseCellSize, nAddCells)
        blockCont = readBlockMeshDict(caseDir)
        if blockCont == "":
            error = 1
            print _(u"blockMeshDictが存在しません。")
            return error
        blockCont = setBlockPointData(blockCont, blockSize, baseCellSize)
        blockCont = checkBlockMeshMag(blockCont)
        error = writeBlockMeshDict(caseDir, blockCont)
        snappyCont = readSnappyHexMeshDict(caseDir)
        if snappyCont == "":
            error = 1
            print _(u"snappyHexMeshDictが存在しません。")
            return error
        snappyCont = setCastellatedSnapFlag(snappyCont)
        snappyCont = setGeometryData(snappyCont, stl, sect)
        snappyCont = setFeatureEdgeData(snappyCont, stl, sect, featureEdge)
        snappyCont = setRefinementSurfaces(snappyCont, stl, sect, minLevel,
                        maxLevel)
        snappyCont = setRefinementRegions(snappyCont, stl, sect, minLevel, baseCellSize)
        snappyCont = checkFaceZone(snappyCont, sect)
        snappyCont = setLocationInMesh(snappyCont, meshLoc)
        error = writeSnappyHexMeshDict(caseDir, snappyCont)
    return error



#----------
#  メイン
#----------
if __name__ == "__main__":
    import gettext
    treeFoamPath = os.environ["TreeFoamPath"]
    gettext.install("treefoam", treeFoamPath + "/data/locale")

    caseDir = os.getcwd()
    fileName = "./helyxosReadData.csv"
    stlDir = caseDir + "/model"
    #error = createCsvFile(caseDir, stlDir, fileName)
    error = createMeshDict(caseDir, stlDir, fileName)

