#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   createCfMeshDict.py
#
#   15/10/01    新規作成
#      10/11    layerの設定に「maxFirstLayerThickness」を追加
#


import os
import glob
import math
import getFoamContents

#
#  setCsvData
#    defaultのcsvファイル作成
def setCsvData():
    row = []
    row.append(["", "", "", "", "", "", ""])
    line = ["<cfMesh>", "", "", "", "", "", ""]
    row.append(line)
    line = ["", "maxCellSize", "", "", "", "", _(u":最大のcellサイズ")]
    row.append(line)
    line = ["", "minCellSize", "", "", "", "", _(u":最小のcellサイズ")]
    row.append(line)
    line = ["", "featureAngle", "30", "", "", "", _(u":特徴線抽出用の角度")]
    row.append(line)
    line = ["stlFile", '"sect\n(patch/wall/\nempty/symmetry(Plane)/\nregBox/regSph/face)"', "cellSize", "nLayer", "ratio", "maxThickness", ""]
    row.append(line)
    return row

#
#  addMaxCellSize
#    maxCellSizeの設定（30分割）
def addMaxCellSize(stlDir, data):
    flag = 0
    xVals=[]; yVals=[]; zVals=[]
    i = 0
    maxCellSize = -1
    for row in data:
        if flag == 1:
            fileName = stlDir + "/" + row[0] + ".stl"
            [mins, maxs] = getMinMaxValueStl(fileName)
            xVals.append(mins[0])
            xVals.append(maxs[0])
            yVals.append(mins[1])
            yVals.append(maxs[1])
            zVals.append(mins[2])
            zVals.append(maxs[2])
        if row[0] == "stlFile":
            flag = 1
        if row[1] == "maxCellSize":
            maxCellSize = i
        i += 1
    if len(xVals) != 0:
        minx = min(xVals); maxx = max(xVals)
        miny = min(yVals); maxy = max(yVals)
        minz = min(zVals); maxz = max(zVals)
    else:
        minx = 0.0; maxx = 0.0
        miny = 0.0; maxy = 0.0
        minz = 0.0; maxz = 0.0
    #cellSizeの設定
    dxf = (maxx - minx) * 1.0 / 30.0
    dyf = (maxy - miny) * 1.0 / 30.0
    dzf = (maxz - minz) * 1.0 / 30.0
    dx = getSizeLog10(dxf)
    dy = getSizeLog10(dyf)
    dz = getSizeLog10(dzf)
    cellSize = max([dx, dy, dz])
    if maxCellSize >= 0:
        data[maxCellSize][2] = str(cellSize)
    return data

#
#  addStlFileName
#    stlFileを追加
def addStlFileName(stlDir, data):
    files = []
    stlFiles = glob.glob(stlDir + "/*.stl")
    for stlFile in stlFiles:
        name = stlFile.split("/")[-1].split(".")[0]
        #assy.stl以外を取得
        if name != "assy":
            files.append(name)
    files.sort()
    for name in files:
        line = [name, "", "", "", "", "", ""]
        data.append(line)
    return data

#
#  addStlBlockSize
#    stl のblockサイズを取得
def addStlBlockSize(stlDir, data):
    flag = 0
    i = 0
    for row in data:
        if flag == 1:
            fileName = stlDir + "/" + row[0] + ".stl"
            [x, y, z] = getStlBlockSize(fileName)
            if not (x==0.0 and y==0.0 and z==0.0):
                data[i][6] = '"(' + ' '.join([str(x), str(y), str(z)]) + ')"'
        if row[0] == "stlFile":
            flag = 1
        i += 1
    return data

#  getStlBlockSize
def getStlBlockSize(fileName):
    minMax = getMinMaxValueStl(fileName)
    dx = minMax[1][0] - minMax[0][0]
    dy = minMax[1][1] - minMax[0][1]
    dz = minMax[1][2] - minMax[0][2]
    dx = getSizeLog10(dx)
    dy = getSizeLog10(dy)
    dz = getSizeLog10(dz)
    return [dx, dy, dz]

#  getSizeLog10
def getSizeLog10(x):
    if x<=0.0:
        return 0.0
    lx = math.log10(x)
    if lx>0.0:
        llx = 10**float(int(lx) - 1)
    else:
        llx = 10**float(int(lx) - 2)
    xx = int(x/llx + 0.5) * llx
    return xx

#
#  getMinMaxValueStl
#    stlファイルのminMax値を取得
def getMinMaxValueStl(fileName):
    #fileFormatのチェック
    f=open(fileName); cont=f.readlines(10); f.close()
    if cont[0][:len("solid")] != "solid":
        #binaryの場合
        ans = readBinaryStl(fileName)
        return ans
    #asciiの場合
    f=open(fileName); cont=f.readlines(); f.close()
    xValues=[]; yValues=[]; zValues=[]
    for line in cont:
        if line.find("vertex") >= 0:
            values = line.split()
            xValues.append(float(values[1]))
            yValues.append(float(values[2]))
            zValues.append(float(values[3]))
    if len(xValues) != 0:
        minx = min(xValues); maxx = max(xValues)
        miny = min(yValues); maxy = max(yValues)
        minz = min(zValues); maxz = max(zValues)
    else:
        minx=0.0; maxx=0.0
        miny=0.0; maxy=0.0
        minz=0.0; maxz=0.0
    return [[minx, miny, minz], [maxx, maxy, maxz]]

#
#  writeCsvData
#    csvデータの書き込み
def writeCsvData(caseDir, fileName, data):
    f=open(caseDir+"/"+fileName, "w")
    for line in data:
        cont = ",".join(line) + "\n"
        f.write(cont.encode("utf-8"))
    f.close()
    return 0

#
#  separateContents
#
def separateContents(cont):
    #headerを取得
    p = getFoamContents.skipFoamFileMem(cont, 0)
    [lineCont, p, kind] = getFoamContents.get1lineMem(cont, p)
    ps = p - len(lineCont)
    header = cont[:ps]
    #footerを取得
    p = ps
    pe = p
    [lineCont, p, kind] = getFoamContents.get1lineMem(cont, p)
    while lineCont != "":
        pe = p
        [lineCont, p, kind] = getFoamContents.get1lineMem(cont, p)
    newCont = cont[ps:pe]
    footer = cont[pe:]
    return [header, newCont, footer]


#
#  setMaxCellSize
#    meshDictにmaxCellSizeの設定
def setMaxCellSize(maxCellSize, cont):
    newLine = "maxCellSize " + maxCellSize + ";"
    cont = setOrDeleteLine(cont, "maxCellSize", maxCellSize, newLine)
    return cont

#  setMinCellSize
#    meshDictにminCellSizeを設定
def setMinCellSize(minCellSize, cont):
    newLine = "minCellSize " + minCellSize + ";"
    cont = setOrDeleteLine(cont, "minCellSize", minCellSize, newLine)
    return cont

#  setSurfaceFile
#    meshDictにsurfaceFile名を設定
def setSurfaceFile(caseDir, stlDir, surfaceFile, cont):
    absFileName = stlDir + "/" + surfaceFile
    relFileName = os.path.relpath(absFileName, caseDir)
    surfaceFile = relFileName
    if len(relFileName) > len(absFileName):
        surfaceFile = absFileName
    newLine = 'surfaceFile "' + surfaceFile + '";'
    cont = setOrDeleteLine(cont, "surfaceFile", surfaceFile, newLine)
    return cont

#  setLocalRefinement
#    meshDictにlocalRefinementを設定
def setLocalRefinement(data, cont):
    newLine  = "localRefinement\n"
    newLine += "{\n"
    for row in data:
        patchName = row[0]
        sect = row[1]
        cellSize = row[2]
        if (sect == "wall" or sect == "patch" or sect == "empty" or
            sect == "symmetry" or sect == "symmetryPlane"):
            try:
                #cellSizeが実数値の場合
                a = float(cellSize)
                newLine += "    " + patchName + "\n"
                newLine += "    {\n"
                newLine += "        cellSize " + cellSize + ";\n"
                newLine += "    }\n"
            except:
                pass
    newLine += "}"
    cont = setOrDeleteLine(cont, "localRefinement", newLine, newLine)
    return cont

#  setRenameBoundary
#    patchNameとpatchTypeを設定
def setRenameBoundary(data, cont):
    value = ""
    for row in data:
        patchName = row[0]
        sect = row[1]
        if (sect == "wall" or sect == "patch" or sect == "empty" or
            sect == "symmetry" or sect == "symmetryPlane"):
            value += "        " + patchName + "\n"
            value += "        {\n"
            value += "            newName " + patchName + ";\n"
            value += "            type    " + sect + ";\n"
            value += "        }\n"
    newLine  = "renameBoundary\n"
    newLine += "{\n"
    newLine += "    newPatchNames\n"
    newLine += "    {\n"
    newLine += value
    newLine += "    }\n"
    newLine += "}"
    cont = setOrDeleteLine(cont, "localRefinement", newLine, newLine)
    return cont

#  setSurfaceMeshRefinement
#    表面のcellSizeを設定
def setSurfaceMeshRefinement(caseDir, stlDir, data, cont):
    absDir = stlDir
    relDir = os.path.relpath(absDir, caseDir)
    setDir = relDir
    if len(relDir) > len(absDir):
        setDir = absDir 
    value = ""
    for row in data:
        patchName = row[0]
        sect = row[1]
        cellSize = row[2]
        if sect == "face":
            surfaceFile = patchName + ".stl"
            value += "    " + patchName + "\n"
            value += "    {\n"
            value += '        surfaceFile "' + setDir + "/" + surfaceFile + '";\n'
            value += "        cellSize    " + cellSize + ";\n"
            value += "    }\n"
    newLine  = "surfaceMeshRefinement\n"
    newLine += "{\n"
    newLine += value
    newLine += "}"
    cont = setOrDeleteLine(cont, "surfaceMeshRefinement", newLine, newLine)
    return cont

#  setObjectRefinements
#    objectのcellSizeを設定
def setObjectRefinements(data, stlDir, cont):

    #  getRegionSize
    #    regionSizeを取得
    def getRegionSize(stlFile):
        [[minx, miny, minz], [maxx, maxy, maxz]] = getMinMaxValueStl(stlFile)
        dx = maxx - minx
        dy = maxy - miny
        dz = maxz - minz
        locx = (maxx + minx) / 2.0
        locy = (maxy + miny) / 2.0
        locz = (maxz + minz) / 2.0
        centre = [str(locx), str(locy), str(locz)]
        return [centre, str(dx), str(dy), str(dz)]

    value = ""
    for row in data:
        patchName = row[0]
        sect = row[1]
        cellSize = row[2]
        if sect == "regBox":
            stlFile = stlDir + "/" + patchName + ".stl"
            [centre, dx, dy, dz] = getRegionSize(stlFile)
            value += "    " + patchName + "\n"
            value += "    {\n"
            value += "        cellSize " + cellSize + ";\n"
            value += "        centre   (" + " ".join(centre) + ");\n"
            value += "        lengthX  " + dx + ";\n"
            value += "        lengthY  " + dy + ";\n"
            value += "        lengthZ  " + dz + ";\n"
            value += "        type     box;\n"
            value += "    }\n"
        if sect == "regSph":
            stlFile = stlDir + "/" + patchName + ".stl"
            [centre, dx, dy, dz] = getRegionSize(stlFile)
            radius = (float(dx) + float(dy) + float(dz)) / 3.0 / 2.0
            value += "    " + patchName + "\n"
            value += "    {\n"
            value += "        cellSize " + cellSize + ";\n"
            value += "        centre   (" + " ".join(centre) + ");\n"
            value += "        radius   " + str(radius) + ";"
            value += "        type     sphere;"
            value += "    }\n"
    newLine  = "objectRefinements\n"
    newLine += "{\n"
    newLine += value
    newLine += "}"
    cont = setOrDeleteLine(cont, "objectRefinements", newLine, newLine)
    return cont

#
#  setBoundaryLayers
#    layerの設定
def setBoundaryLayers(data, cont):
    value = ""
    for row in data:
        patchName = row[0]
        sect = row[1]
        nLayer = row[3]
        ratio = row[4]
        maxThickness = row[5]
        #ratioのチェック
        lineRa = "            thicknessRatio 1.0;\n"
        if ratio != "":
            try:
                ratioV = float(ratio)
                lineRa = "            thicknessRatio " + str(ratioV) + ";\n"
            except:
                pass
        #maxThicknessのチェック
        lineTh = ""
        if maxThickness != "":
            try:
                maxTh = float(maxThickness)
                lineTh = "            maxFirstLayerThickness " + str(maxTh) + ";\n"
            except:
                pass
        #layer設定文作成
        if nLayer != "":
            try:
                if int(nLayer) >= 1:
                    value += "        " + patchName + "\n"
                    value += "        {\n"
                    value += "            nLayers " + nLayer + ";\n"
                    value += lineRa
                    #value += "            thicknessRatio " + ratio + ";\n"
                    value += lineTh
                    value += "        }\n"
            except:
                pass
    newLine  = "boundaryLayers\n"
    newLine += "{\n"
    newLine += "    patchBoundaryLayers\n"
    newLine += "    {\n"
    newLine += value
    newLine += "    }\n"
    newLine += "}"
    cont = setOrDeleteLine(cont, "boundaryLayers", newLine, newLine)
    return cont

#
#  setOrDeleteLine
#    set or delete line
def setOrDeleteLine(cont, key, value, newLine):
    if value != "":
        [cont, p, keyword] = getFoamContents.replace1lineMem(cont, 0, newLine)
        #置き換えが成功したか？
        if keyword == "":
            if key == "surfaceFile" or key == "maxCellSize" or key == "minCellSize":
                #先頭行にnewLineを追加
                if newLine[-1] == "\n":
                    cont = newLine + cont
                else:
                    cont = newLine + "\n" + cont
            else:
                #最終行にnewLineを追加
                if cont[-1] == "\n":
                    cont += newLine
                else:
                    cont += "\n" + newLine
    else:
        #key行を削除
        [lineCont, p, kind] = getFoamContents.get1lineMem(cont, 0)
        [keyword, pp] = getFoamContents.getKeywordMem(lineCont, 0)
        ps = p
        while keyword != key and keyword != "":
            ps = p
            [lineCont, p, kind] = getFoamContents.get1lineMem(cont, p)
            [keyword, pp] = getFoamContents.getKeywordMem(lineCont, 0)
        if keyword == key:
            cont = cont[:ps] + cont[p:]                
    return cont


#
#  createCsvFile
#  -------------
#    defaultのcsvファイル作成
def createCsvFile(caseDir, stlDir, fileName):
    error = ""
    data = setCsvData()
    data = addStlFileName(stlDir, data)
    data = addMaxCellSize(stlDir, data)
    data = addStlBlockSize(stlDir, data)
    error = writeCsvData(stlDir, fileName, data)
    return error

#
#  createMeshDict
#  --------------
#    meshDictの作成
def createMeshDict(caseDir, stlDir, surfaceFile, otherData, data):
    error = ""
    maxCellSize = otherData[0]
    minCellSize = otherData[1]
    featureAngle = otherData[2]
    meshDict = caseDir + "/system/meshDict"
    f=open(meshDict); cont=f.read(); f.close()
    [header, cont, footer] = separateContents(cont)
    cont = setSurfaceFile(caseDir, stlDir, surfaceFile, cont)
    cont = setMaxCellSize(maxCellSize, cont)
    cont = setMinCellSize(minCellSize, cont)
    cont = setLocalRefinement(data, cont)
    cont = setRenameBoundary(data, cont)
    cont = setSurfaceMeshRefinement(caseDir, stlDir, data, cont)
    cont = setObjectRefinements(data, stlDir, cont)
    cont = setBoundaryLayers(data, cont)
    newCont = header + cont + footer
    f=open(meshDict, "w"); f.write(newCont); f.close()



