#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   fileOperationInRegionDialog.py
#
#       各region内のfile操作
#
#   13/01/06    新規作成
#      01/07    DBの場所（defaultの値）を変更
#      01/09    固体の材料選択を修正
#      01/12    DBの場所（defaultの値）を変更
#      01/30    全region削除ボタンを追加
#      02/21    材料設定をlistBox→listCtrlに変更
#      04/07    バグ修正（材料DB内のfile編集ボタンでエラー発生）
#      04/11    file名変更ボタン追加
#      05/01    csvFileで一括設定を追加
#      05/14    csvFile設定した後、表示が変わらないので、表示も変えるよう変更
#      05/19    setMaterialToRegionを修正（copy→delete後、copy）
#      05/20    solidの物性値設定をOF2.2.0に対応
#      05/21    OFversionチェックをinitialize内に統一
#      05/25    checkMaterialDBDirのチェック方法修正
#      05/29    regionを削除した場合の処理を追加、戻すボタンを追加
#      06/22    全region削除ボタン削除
#      07/14    DBフォルダ内にfileが無かった場合、fileをコピーする。
#      09/01    liveDVDで起動したとき、
#                   setMultiRegionでDBdirが設定できない、ゴミ箱が無いエラーを修正
#      09/20    変更ボタンの設定が削除になっていたのを修正
#      10/04    iniTimeの取得をcurrTimeに変更
#      10/15    国際化のため、修正
#      11/12    iniTime, constant, systemフォルダ直下をtree表示に変更
#      12/19    コピー、戻すボタンをfluid側、solid側各々に設置（昔に戻す）
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/01/05    各regionへのコピー時、無印を全regionへコピーを追加
#      04/07    editorでfileOpen時、binaryのみtempフォルダでOpenする様修正
#      05/01    popupMenuを追加、toolTipを追加
#      05/03    materialFileの読み込みにcodecsで読み込み（unucodeでエラー）
#      06/21    getCopyFileNameFromBox:「戻す」時、「_all_」を廃止
#      10/25    clearBoundaryFieldsを追加（fieldコピー時）
#               textBox内の選択項目を保持
#      11/07    radCsvData:pyTreeFoamから読み込む様に修正
#      11/24    setMaterialToListBox:候補材を全てｾｯﾄする様に修正（同名はｾｯﾄせず）
#      11/29    「配布」「戻す」で項目を選択表示させる
#   15/02/08    region選択時、fileListを再表示させる
#      02/09    ﾎﾟｯﾌﾟｱｯﾌﾟﾒﾆｭｰに「名前からregion名を削除」を追加
#      02/23    showRegionFileList:バグ修正（checkFileNameを追加）
#      03/02    fileコピー時、同名があるとｴﾗｰ発生を修正
#      07/13    clipBoardの場所をTreeFoam/dataに変更
#      07/17    dialogの起動方法修正
#      08/09    setMaterialToSelRegions:新規追加（材料設定ボタンを追加）
#   16/12/31    checkMaterialDBDir:OF-ext用に修正
#


# generated by wxGlade 0.6.5 on Fri Jan  4 17:54:48 2013

#------------コード生成後の変更内容-----------------------
#
#  ・アイコンのdirを変更（絶対path→相対path）
#       ./icons/***　形式に変更
#

import wx
import os
import sys
import glob
import gio
import csv
import codecs
import getFoamContents
import setFoamContents
import pyTreeFoam
import getOpenFolderDDialog
import inputTextDDialog
import gaugeDDialog
from getOpenFileDDialog import Show as getOpenFileDDialogShow

import logFileCreater; logFileCreater.log()

# begin wxGlade: extracode
# end wxGlade

OFversion = ""                          #OFのバージョン
solidProperties = ""                    #materialDBのフォルダ名
openPage = ""                           #初期でnoteBookを開くページ
#multiRegion_data = os.environ["TreeFoamPath"] + "/data/multiRegion_data"
multiRegion_data = os.environ["TreeFoamUserPath"] + "/data/multiRegion_data"
solidPhysicalDBdir = os.environ["HOME"] + "/OpenFOAM/multiRegionDB/solid/thermophysicalProperties"
iniDBdata = os.environ["TreeFoamPath"] + "/data/multiRegionDB/solid/thermophysicalProperties"
#clipBoard = os.environ["TreeFoamPath"] + "/data/clipBoard"
clipBoard = os.environ["TreeFoamUserPath"] + "/data/clipBoard"
toolTipTextOrg = ""
toolTipTextFluidSolid = ""
toolTipTextMaterial = ""
fluidRegions = []
solidRegions = []
iniTime = ""
timeFiles = []
timeFluidFiles = []
timeSolidFiles = []
constantFiles = []
constantFluidFiles = []
constantSolidFiles = []
solidPhysicalDB = []
constantSolidPhysical = []              #設定されている材料
constantSolidPhysicalReq = []           #材料設定の候補
systemFiles = []
systemFluidFiles = []
systemSolidFiles = []
iniTimeExpand = [0, 0]                  #iniTimeのtreeを広げるflag [fluid, solid]
constantExpand = [0, 0]                 #constant
systemExpand = [0, 0]                   #system
selectItems = [[],[],[]]                #[[iniTime], [cinstant], [system]]


def Show(page):
    global openPage
    openPage = page
    #materialDBDirをチェックし、dirが存在しないならば、作成する。
    checkMaterialDBDir()
    #初期化
    initialize()
    #dialog表示
    #app = wx.PySimpleApp(0)
    app = wx.App(0)
    #wx.InitAllImageHandlers()
    frame_1 = MyFrame(None, -1, "")
    app.SetTopWindow(frame_1)
    frame_1.Show()
    app.MainLoop()

class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):

        savePath = os.getcwd()
        treeFoamPath = os.getenv("TreeFoamPath")
        os.chdir(treeFoamPath)

        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)
        self.label_1 = wx.StaticText(self, wx.ID_ANY, _(u"各regionのfile操作"))
        self.notebook_1 = wx.Notebook(self, wx.ID_ANY, style=0)
        self.notebook_1_pane_1 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.tree_ctrl_1 = wx.TreeCtrl(self.notebook_1_pane_1, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_NO_LINES | wx.TR_MULTIPLE | wx.TR_HIDE_ROOT | wx.TR_MULTIPLE | wx.TR_DEFAULT_STYLE | wx.SUNKEN_BORDER)
        self.button_3 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"編集"))
        self.label_27 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file内容確認"))
        self.button_17 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"変更"))
        self.label_24 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file名変更"))
        self.button_1 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"削除"))
        self.label_2 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"選択fileを削除"))
        self.sizer_4_staticbox = wx.StaticBox(self.notebook_1_pane_1, wx.ID_ANY, _("file(region0)"))
        self.button_2 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"配布>>"))
        self.button_20 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"<<戻す"))
        self.button_23 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"配布>>"))
        self.button_26 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"<<戻す"))
        self.label_3 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _("file"))
        self.list_box_2 = wx.ListBox(self.notebook_1_pane_1, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_11 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"変更"))
        self.label_15 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file名変更"))
        self.button_4 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"削除"))
        self.label_4 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_5 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _("region"))
        self.list_box_3 = wx.ListBox(self.notebook_1_pane_1, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_5 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"編集"))
        self.label_6 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_8_staticbox = wx.StaticBox(self.notebook_1_pane_1, wx.ID_ANY, _(u"流体"))
        self.label_7 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _("file"))
        self.list_box_4 = wx.ListBox(self.notebook_1_pane_1, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_12 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"変更"))
        self.label_16 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file名変更"))
        self.button_6 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"削除"))
        self.label_8 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_9 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _("region"))
        self.list_box_5 = wx.ListBox(self.notebook_1_pane_1, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_7 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"編集"))
        self.label_10 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_9_staticbox = wx.StaticBox(self.notebook_1_pane_1, wx.ID_ANY, _(u"固体"))
        self.notebook_1_pane_2 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.tree_ctrl_c1 = wx.TreeCtrl(self.notebook_1_pane_2, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_NO_LINES | wx.TR_MULTIPLE | wx.TR_HIDE_ROOT | wx.TR_MULTIPLE | wx.TR_DEFAULT_STYLE | wx.SUNKEN_BORDER)
        self.button_9 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"編集"))
        self.label_28 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file内容確認"))
        self.button_24 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"変更"))
        self.label_25 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file名変更"))
        self.button_c1 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"削除"))
        self.label_2_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"選択fileを削除"))
        self.sizer_4_copy_staticbox = wx.StaticBox(self.notebook_1_pane_2, wx.ID_ANY, _("file(region0)"))
        self.button_c2 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"配布>>"))
        self.button_22 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"<<戻す"))
        self.button_28 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"配布>>"))
        self.button_29 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"<<戻す"))
        self.label_3_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _("file"))
        self.list_box_c2 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_13 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"変更"))
        self.label_18 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file名変更"))
        self.button_c4 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"削除"))
        self.label_4_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_5_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _("region"))
        self.list_box_c3 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_c5 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"編集"))
        self.label_6_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_8_copy_staticbox = wx.StaticBox(self.notebook_1_pane_2, wx.ID_ANY, _(u"流体"))
        self.label_7_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _("file"))
        self.list_box_c4 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_14 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"変更"))
        self.label_19 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file名変更"))
        self.button_c6 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"削除"))
        self.label_8_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_9_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _("region"))
        self.list_box_c5 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_c7 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"編集"))
        self.label_10_copy = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_39_staticbox = wx.StaticBox(self.notebook_1_pane_2, wx.ID_ANY, _(u"固体"))
        self.notebook_1_pane_3 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.tree_ctrl_s1 = wx.TreeCtrl(self.notebook_1_pane_3, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_NO_LINES | wx.TR_MULTIPLE | wx.TR_HIDE_ROOT | wx.TR_MULTIPLE | wx.TR_DEFAULT_STYLE | wx.SUNKEN_BORDER)
        self.button_21 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"編集"))
        self.label_29 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file内容確認"))
        self.button_27 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"変更"))
        self.label_26 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file名変更"))
        self.button_s1 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"削除"))
        self.label_2_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"選択fileを削除"))
        self.sizer_4_copy_1_staticbox = wx.StaticBox(self.notebook_1_pane_3, wx.ID_ANY, _("file(region0)"))
        self.button_s2 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"配布>>"))
        self.button_25 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"<<戻す"))
        self.button_30 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"配布>>"))
        self.button_31 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"<<戻す"))
        self.label_3_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _("file"))
        self.list_box_s2 = wx.ListBox(self.notebook_1_pane_3, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_15 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"変更"))
        self.label_20 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file名変更"))
        self.button_s4 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"削除"))
        self.label_4_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_5_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _("region"))
        self.list_box_s3 = wx.ListBox(self.notebook_1_pane_3, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_s5 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"編集"))
        self.label_6_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_8_copy_1_staticbox = wx.StaticBox(self.notebook_1_pane_3, wx.ID_ANY, _(u"流体"))
        self.label_7_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _("file"))
        self.list_box_s4 = wx.ListBox(self.notebook_1_pane_3, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_16 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"変更"))
        self.label_21 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file名変更"))
        self.button_s6 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"削除"))
        self.label_8_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"選択fileを削除"))
        self.label_9_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _("region"))
        self.list_box_s5 = wx.ListBox(self.notebook_1_pane_3, wx.ID_ANY, choices=[], style=wx.LB_MULTIPLE)
        self.button_s7 = wx.Button(self.notebook_1_pane_3, wx.ID_ANY, _(u"編集"))
        self.label_10_copy_1 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"file内容確認"))
        self.sizer_9_copy_1_staticbox = wx.StaticBox(self.notebook_1_pane_3, wx.ID_ANY, _(u"固体"))
        self.notebook_1_pane_4 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_12 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"solidThermophysicalPropertiesの設定"))
        self.label_17 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"DBの場所:"))
        self.text_ctrl_2 = wx.TextCtrl(self.notebook_1_pane_4, wx.ID_ANY, "")
        self.button_sm0 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"参照..."))
        self.sizer_26_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料DBの場所を設定"))
        self.label_11 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料設定方法：・「材料DB」から材料名を選択後、「設定候補」をダブルクリック。\n　　　　　　　　「新材料を設定」ボタンをクリックして設定。\n　　　　　　　・設定fileを準備し、「csvFileで設定」ボタンをクリックして一括設定。"))
        self.label_14 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料設定"))
        self.list_ctrl_1 = wx.ListCtrl(self.notebook_1_pane_4, wx.ID_ANY, style=wx.LC_REPORT | wx.SUNKEN_BORDER)
        self.button_18 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"csvFileで設定..."))
        self.button_19 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"設定をfileに出力"))
        self.label_23 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"各regionの材料設定\nをcsvで一括設定"))
        self.sizer_48_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料の一括設定"))
        self.button_sm1 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"編集"))
        self.button_sm2 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"名称変更"))
        self.button_sm3 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"DB登録"))
        self.label_22 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料名を選択後、\nボタンをクリック"))
        self.sizer_49_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料DB追加修正"))
        self.button_sm4 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"新材料を設定"))
        self.button_8 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"設定候補クリア"))
        self.sizer_50_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料設定"))
        self.bitmap_button_1 = wx.BitmapButton(self.notebook_1_pane_4, wx.ID_ANY, wx.Bitmap("./icons/backwardArrow.png", wx.BITMAP_TYPE_ANY))
        self.label_13 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料DB"))
        self.list_ctrl_2 = wx.ListCtrl(self.notebook_1_pane_4, wx.ID_ANY, style=wx.LC_REPORT | wx.SUNKEN_BORDER)
        self.button_sm5 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"編集"))
        self.button_sm6 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"folder開く"))
        self.sizer_53_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"材料DB編集"))
        self.sizer_22_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _(u"各regionの材料設定"))
        self.button_10 = wx.Button(self, wx.ID_ANY, _(u"閉じる"))

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.onCollapsed, self.tree_ctrl_1)
        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.onExpanded, self.tree_ctrl_1)
        self.Bind(wx.EVT_BUTTON, self.onEditTime, self.button_3)
        self.Bind(wx.EVT_BUTTON, self.onRenameTime, self.button_17)
        self.Bind(wx.EVT_BUTTON, self.onDeleteTime, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.onCopyFluidTime, self.button_2)
        self.Bind(wx.EVT_BUTTON, self.onReturnFluidTime, self.button_20)
        self.Bind(wx.EVT_BUTTON, self.onCopySolidTime, self.button_23)
        self.Bind(wx.EVT_BUTTON, self.onReturnSolidTime, self.button_26)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenTimeFluid, self.list_box_2)
        self.Bind(wx.EVT_BUTTON, self.onRenameTimeFluid, self.button_11)
        self.Bind(wx.EVT_BUTTON, self.onDeleteTimeFluid, self.button_4)
        self.Bind(wx.EVT_LISTBOX, self.onSelectTimeFluidRegion, self.list_box_3)
        self.Bind(wx.EVT_BUTTON, self.onEditTimeFluid, self.button_5)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenTimeSolid, self.list_box_4)
        self.Bind(wx.EVT_BUTTON, self.onRenameTimeSolid, self.button_12)
        self.Bind(wx.EVT_BUTTON, self.onDeleteTimeSolid, self.button_6)
        self.Bind(wx.EVT_LISTBOX, self.onSelectTimeSolidRegion, self.list_box_5)
        self.Bind(wx.EVT_BUTTON, self.onEditTimeSolid, self.button_7)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.onCollapsedConst, self.tree_ctrl_c1)
        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.onExpandedConst, self.tree_ctrl_c1)
        self.Bind(wx.EVT_BUTTON, self.onEditConstant, self.button_9)
        self.Bind(wx.EVT_BUTTON, self.onRenameConstant, self.button_24)
        self.Bind(wx.EVT_BUTTON, self.onDeleteConstant, self.button_c1)
        self.Bind(wx.EVT_BUTTON, self.onCopyFluidConstant, self.button_c2)
        self.Bind(wx.EVT_BUTTON, self.onReturnFluidConstant, self.button_22)
        self.Bind(wx.EVT_BUTTON, self.onCopySolidConstant, self.button_28)
        self.Bind(wx.EVT_BUTTON, self.onReturnSolidConstant, self.button_29)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenConstantFluid, self.list_box_c2)
        self.Bind(wx.EVT_BUTTON, self.onRenameConstantFluid, self.button_13)
        self.Bind(wx.EVT_BUTTON, self.onDeleteConstantFluid, self.button_c4)
        self.Bind(wx.EVT_LISTBOX, self.onSelectConstantFluidRegion, self.list_box_c3)
        self.Bind(wx.EVT_BUTTON, self.onEditConstantFluid, self.button_c5)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenConstantSolid, self.list_box_c4)
        self.Bind(wx.EVT_BUTTON, self.onRenameConstantSolid, self.button_14)
        self.Bind(wx.EVT_BUTTON, self.onDeleteConstantSolid, self.button_c6)
        self.Bind(wx.EVT_LISTBOX, self.onSelectConstantSolidRegion, self.list_box_c5)
        self.Bind(wx.EVT_BUTTON, self.onEditConstantSolid, self.button_c7)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.onCollapsedSys, self.tree_ctrl_s1)
        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.onExpanedSys, self.tree_ctrl_s1)
        self.Bind(wx.EVT_BUTTON, self.onEditSystem, self.button_21)
        self.Bind(wx.EVT_BUTTON, self.onRenameSystem, self.button_27)
        self.Bind(wx.EVT_BUTTON, self.onDeleteSystem, self.button_s1)
        self.Bind(wx.EVT_BUTTON, self.onCopyFluidSystem, self.button_s2)
        self.Bind(wx.EVT_BUTTON, self.onReturnFluidSystem, self.button_25)
        self.Bind(wx.EVT_BUTTON, self.onCopySolidSystem, self.button_30)
        self.Bind(wx.EVT_BUTTON, self.onReturnSolidSystem, self.button_31)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenSystemFluid, self.list_box_s2)
        self.Bind(wx.EVT_BUTTON, self.onRenameSystemFluid, self.button_15)
        self.Bind(wx.EVT_BUTTON, self.onDeleteSystemFluid, self.button_s4)
        self.Bind(wx.EVT_LISTBOX, self.onSelectSystemFluidRegion, self.list_box_s3)
        self.Bind(wx.EVT_BUTTON, self.onEditSystemFluid, self.button_s5)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onOpenSystemSolid, self.list_box_s4)
        self.Bind(wx.EVT_BUTTON, self.onRenameSystemSolid, self.button_16)
        self.Bind(wx.EVT_BUTTON, self.onDeleteSystemSolid, self.button_s6)
        self.Bind(wx.EVT_LISTBOX, self.onSelectSystemSolidRegion, self.list_box_s5)
        self.Bind(wx.EVT_BUTTON, self.onEditSystemSolid, self.button_s7)
        self.Bind(wx.EVT_BUTTON, self.onSetPhysicalDir, self.button_sm0)
        self.Bind(wx.EVT_BUTTON, self.onSetCSVFile, self.button_18)
        self.Bind(wx.EVT_BUTTON, self.onOutputCSVFile, self.button_19)
        self.Bind(wx.EVT_BUTTON, self.onEditRegionMaterial, self.button_sm1)
        self.Bind(wx.EVT_BUTTON, self.onRenameRegionMaterial, self.button_sm2)
        self.Bind(wx.EVT_BUTTON, self.onSaveMaterialDB, self.button_sm3)
        self.Bind(wx.EVT_BUTTON, self.onSetMaterialToRegion, self.button_sm4)
        self.Bind(wx.EVT_BUTTON, self.onClearMatReq, self.button_8)
        self.Bind(wx.EVT_BUTTON, self.onSetMaterialToRegions, self.bitmap_button_1)
        self.Bind(wx.EVT_BUTTON, self.onEditMaterialDB, self.button_sm5)
        self.Bind(wx.EVT_BUTTON, self.onOpenMaterialDBfolder, self.button_sm6)
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.onReloadData, self.notebook_1)
        self.Bind(wx.EVT_BUTTON, self.onClose, self.button_10)
        # end wxGlade

        self.list_ctrl_1.Bind(wx.EVT_LEFT_DCLICK, self.onCheckInListDclick)
        self.list_ctrl_2.Bind(wx.EVT_LEFT_DCLICK, self.onOpenMaterialDB)
        self.tree_ctrl_1.Bind(wx.EVT_LEFT_DCLICK, self.onOpenTime)
        self.tree_ctrl_c1.Bind(wx.EVT_LEFT_DCLICK, self.onOpenConstant)
        self.tree_ctrl_s1.Bind(wx.EVT_LEFT_DCLICK, self.onOpenSystem)

        #-------- popupMenu --------------------------
        menu = [_(u"編集"), _(u"名前変更"), _(u"削除")]
        menuOrg = [_(u"_"), _(u"編集"), _(u"名前変更"), _(u"名前からregion名を削除"),
                    _(u"削除"), _(u"_"), _(u"コピー"), _(u"貼り付け")]
        # time-all--------------
        menuA = ( [iniTime + _(u" folder開く")] + menuOrg
                    + [iniTime + _(u" folderへ貼り付け")] )
        self.popupMenuTimeAll = wx.Menu()
        i = 0
        for text in menuA:
            if text == "_":
                self.popupMenuTimeAll.AppendSeparator()
            else:
                item = self.popupMenuTimeAll.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuTimeAll, item)
            i += 1
        self.tree_ctrl_1.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuTimeAll)

        # time-fluid
        self.popupMenuTimeFluid = wx.Menu()
        i = 0
        for text in menu:
            if text == "-":
                self.popupMenuTimeFluid.AppendSeparator()
            else:
                item = self.popupMenuTimeFluid.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuTimeFluid, item)
            i += 1
        self.list_box_2.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuTimeFluid)

        # time-solid
        self.popupMenuTimeSolid = wx.Menu()
        i = 0
        for text in menu:
            item = self.popupMenuTimeSolid.Append(-1, text)
            self.Bind(wx.EVT_MENU, self.onPopupMenuTimeSolid, item)
            i += 1
        self.list_box_4.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuTimeSolid)

        # const-all------------
        menuA = ( [_(u"constant folder開く")] + menuOrg
                    + [_(u"constant folderへ貼り付け")] )
        self.popupMenuConstAll = wx.Menu()
        i = 0
        for text in menuA:
            if text == "_":
                self.popupMenuConstAll.AppendSeparator()
            else:
                item = self.popupMenuConstAll.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuConstAll, item)
            i += 1
        self.tree_ctrl_c1.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuConstAll)

        # const-fluid
        self.popupMenuConstFluid = wx.Menu()
        i = 0
        for text in menu:
            if text == "-":
                self.popupMenuConstFluid.AppendSeparator()
            else:
                item = self.popupMenuConstFluid.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuConstFluid, item)
            i += 1
        self.list_box_c2.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuConstFluid)

        # const-solid
        self.popupMenuConstSolid = wx.Menu()
        i = 0
        for text in menu:
            item = self.popupMenuConstSolid.Append(-1, text)
            self.Bind(wx.EVT_MENU, self.onPopupMenuConstSolid, item)
            i += 1
        self.list_box_c4.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuConstSolid)

        # system-all--------------------
        menuA = ( [_(u"system folder開く")] + menuOrg
                    + [_(u"system folderへ貼り付け")] )
        self.popupMenuSystemAll = wx.Menu()
        i = 0
        for text in menuA:
            if text == "_":
                self.popupMenuSystemAll.AppendSeparator()
            else:
                item = self.popupMenuSystemAll.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuSystemAll, item)
            i += 1
        self.tree_ctrl_s1.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuSystemAll)

        # system-fluid
        self.popupMenuSystemFluid = wx.Menu()
        i = 0
        for text in menu:
            if text == "-":
                self.popupMenuSystemFluid.AppendSeparator()
            else:
                item = self.popupMenuSystemFluid.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.onPopupMenuSystemFluid, item)
            i += 1
        self.list_box_s2.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuSystemFluid)

        # system-solid
        self.popupMenuSystemSolid = wx.Menu()
        i = 0
        for text in menu:
            item = self.popupMenuSystemSolid.Append(-1, text)
            self.Bind(wx.EVT_MENU, self.onPopupMenuSystemSolid, item)
            i += 1
        self.list_box_s4.Bind(wx.EVT_RIGHT_UP, self.onShowPopupMenuSystemSolid)

        #------toolTip表示設定-------------------------
        global toolTipTextOrg, toolTipTextFluidSolid
        toolTipTextOrg = _(u"ここをダブルクリックすると、\n　選択したfileを開きます。")
        toolTipTextFluidSolid = _(u"ここをダブルクリックすると、\n　選択したregion内の選択したfileを開きます。")
        self.tree_ctrl_1.Bind(wx.EVT_MOTION, self.showToolTipTimeOrg)
        self.list_box_2.Bind(wx.EVT_MOTION, self.showToolTipTimeFluid)
        self.list_box_4.Bind(wx.EVT_MOTION, self.showToolTipTimeSolid)
        self.tree_ctrl_c1.Bind(wx.EVT_MOTION, self.showToolTipConstantOrg)
        self.list_box_c2.Bind(wx.EVT_MOTION, self.showToolTipConstantFluid)
        self.list_box_c4.Bind(wx.EVT_MOTION, self.showToolTipConstantSolid)
        self.tree_ctrl_s1.Bind(wx.EVT_MOTION, self.showToolTipSystemOrg)
        self.list_box_s2.Bind(wx.EVT_MOTION, self.showToolTipSystemFluid)
        self.list_box_s4.Bind(wx.EVT_MOTION, self.showToolTipSystemSolid)
        self.list_ctrl_1.Bind(wx.EVT_MOTION, self.showToolTipMaterialRegion)
        self.list_ctrl_2.Bind(wx.EVT_MOTION, self.showToolTipMaterialDB)


        os.chdir(savePath)
        #regionがあるかどうかを確認する
        self.checkRegions()
        #dataをlistBoxにセット
        self.setFileNameMaterialToListBox()
        #setPage
        page = int(openPage)
        if 0<= page <= 3:
            self.notebook_1.SetSelection(page)
        else:
            self.notebook_1.SetSelection(1)


    # time
    def onShowPopupMenuTimeAll(self, event):
        pos = event.GetPosition()
        self.tree_ctrl_1.PopupMenu(self.popupMenuTimeAll, pos)

    def onShowPopupMenuTimeFluid(self, event):
        pos = event.GetPosition()
        self.list_box_2.PopupMenu(self.popupMenuTimeFluid, pos)

    def onShowPopupMenuTimeSolid(self, event):
        pos = event.GetPosition()
        self.list_box_4.PopupMenu(self.popupMenuTimeSolid, pos)

    #const
    def onShowPopupMenuConstAll(self, event):
        pos = event.GetPosition()
        self.tree_ctrl_c1.PopupMenu(self.popupMenuConstAll, pos)

    def onShowPopupMenuConstFluid(self, event):
        pos = event.GetPosition()
        self.list_box_c2.PopupMenu(self.popupMenuConstFluid, pos)

    def onShowPopupMenuConstSolid(self, event):
        pos = event.GetPosition()
        self.list_box_c4.PopupMenu(self.popupMenuConstSolid, pos)

    #system
    def onShowPopupMenuSystemAll(self, event):
        pos = event.GetPosition()
        self.tree_ctrl_s1.PopupMenu(self.popupMenuSystemAll, pos)

    def onShowPopupMenuSystemFluid(self, event):
        pos = event.GetPosition()
        self.list_box_s2.PopupMenu(self.popupMenuSystemFluid, pos)

    def onShowPopupMenuSystemSolid(self, event):
        pos = event.GetPosition()
        self.list_box_s4.PopupMenu(self.popupMenuSystemSolid, pos)


    def __set_properties(self):
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle(_(u"regionのファイル操作"))
        self.tree_ctrl_1.SetMinSize((220, 300))
        self.button_3.SetMinSize((40, 25))
        self.button_17.SetMinSize((40, 25))
        self.button_1.SetMinSize((40, 25))
        self.button_2.SetMinSize((55, 48))
        self.button_20.SetMinSize((55, 30))
        self.button_23.SetMinSize((55, 48))
        self.button_26.SetMinSize((55, 30))
        self.list_box_2.SetMinSize((150, 100))
        self.button_11.SetMinSize((40, 25))
        self.button_4.SetMinSize((40, 25))
        self.list_box_3.SetMinSize((150, 100))
        self.button_5.SetMinSize((40, 25))
        self.list_box_4.SetMinSize((150, 100))
        self.button_12.SetMinSize((40, 25))
        self.button_6.SetMinSize((40, 25))
        self.list_box_5.SetMinSize((150, 100))
        self.button_7.SetMinSize((40, 25))
        self.tree_ctrl_c1.SetMinSize((220, 300))
        self.button_9.SetMinSize((40, 25))
        self.button_24.SetMinSize((40, 25))
        self.button_c1.SetMinSize((40, 25))
        self.button_c2.SetMinSize((55, 48))
        self.button_22.SetMinSize((55, 30))
        self.button_28.SetMinSize((55, 48))
        self.button_29.SetMinSize((55, 30))
        self.list_box_c2.SetMinSize((150, 100))
        self.button_13.SetMinSize((40, 25))
        self.button_c4.SetMinSize((40, 25))
        self.list_box_c3.SetMinSize((150, 100))
        self.button_c5.SetMinSize((40, 25))
        self.list_box_c4.SetMinSize((150, 100))
        self.button_14.SetMinSize((40, 25))
        self.button_c6.SetMinSize((40, 25))
        self.list_box_c5.SetMinSize((150, 100))
        self.button_c7.SetMinSize((40, 25))
        self.tree_ctrl_s1.SetMinSize((220, 300))
        self.button_21.SetMinSize((40, 25))
        self.button_27.SetMinSize((40, 25))
        self.button_s1.SetMinSize((40, 25))
        self.button_s2.SetMinSize((55, 48))
        self.button_25.SetMinSize((55, 30))
        self.button_30.SetMinSize((55, 48))
        self.button_31.SetMinSize((55, 30))
        self.list_box_s2.SetMinSize((150, 100))
        self.button_15.SetMinSize((40, 25))
        self.button_s4.SetMinSize((40, 25))
        self.list_box_s3.SetMinSize((150, 100))
        self.button_s5.SetMinSize((40, 25))
        self.list_box_s4.SetMinSize((150, 100))
        self.button_16.SetMinSize((40, 25))
        self.button_s6.SetMinSize((40, 25))
        self.list_box_s5.SetMinSize((150, 100))
        self.button_s7.SetMinSize((40, 25))
        self.button_sm0.SetMinSize((60, 30))
        self.list_ctrl_1.SetMinSize((330, 130))
        self.button_18.SetMinSize((122, 25))
        self.button_19.SetMinSize((122, 25))
        self.button_sm1.SetMinSize((120, 25))
        self.button_sm2.SetMinSize((120, 25))
        self.button_sm3.SetMinSize((120, 25))
        self.button_sm4.SetMinSize((120, 25))
        self.button_8.SetMinSize((120, 25))
        self.bitmap_button_1.SetToolTipString(_(u"選択した材料を、選択しているregion（複数選択可）\nに設定します"))
        self.bitmap_button_1.SetSize(self.bitmap_button_1.GetBestSize())
        self.button_sm5.SetMinSize((98, 25))
        self.button_sm6.SetMinSize((98, 25))
        # end wxGlade

        #listCtrlの設定
        self.list_ctrl_1.InsertColumn(0, u"region", wx.LIST_FORMAT_LEFT, 140)
        self.list_ctrl_1.InsertColumn(1, _(u"現在の設定"), wx.LIST_FORMAT_LEFT, 135)
        self.list_ctrl_1.InsertColumn(2, _(u"設定候補"), wx.LIST_FORMAT_LEFT, 135)
        i = 0
        for name in solidRegions:
            self.list_ctrl_1.InsertStringItem(i, name)
            self.list_ctrl_1.SetStringItem(i, 1, constantSolidPhysical[i])
            self.list_ctrl_1.SetStringItem(i, 2, constantSolidPhysicalReq[i])
            i += 1

        self.list_ctrl_2.InsertColumn(0, u"material", wx.LIST_FORMAT_LEFT, 160)
        i = 0
        for name in solidPhysicalDB:
            self.list_ctrl_2.InsertStringItem(i, name)
            i += 1
        #iconの設定
        ilist = wx.ImageList(16,16)
        folderImg = wx.Bitmap("./icons/folder16.png", wx.BITMAP_TYPE_PNG)
        self.folderIdx = ilist.Add(folderImg)
        self.tree_ctrl_1.AssignImageList(ilist)
        ilistc = wx.ImageList(16,16)
        self.folderIdx = ilistc.Add(folderImg)
        self.tree_ctrl_c1.AssignImageList(ilistc)
        ilists = wx.ImageList(16,16)
        self.folderIdx = ilists.Add(folderImg)
        self.tree_ctrl_s1.AssignImageList(ilists)

    def __do_layout(self):
        # begin wxGlade: MyFrame.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_24 = wx.BoxSizer(wx.VERTICAL)
        sizer_25 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_21 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_22_staticbox.Lower()
        sizer_22 = wx.StaticBoxSizer(self.sizer_22_staticbox, wx.HORIZONTAL)
        sizer_46 = wx.BoxSizer(wx.VERTICAL)
        sizer_45 = wx.BoxSizer(wx.VERTICAL)
        sizer_51 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_53_staticbox.Lower()
        sizer_53 = wx.StaticBoxSizer(self.sizer_53_staticbox, wx.VERTICAL)
        sizer_52 = wx.BoxSizer(wx.VERTICAL)
        sizer_44 = wx.BoxSizer(wx.VERTICAL)
        sizer_23 = wx.BoxSizer(wx.VERTICAL)
        sizer_47 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_50_staticbox.Lower()
        sizer_50 = wx.StaticBoxSizer(self.sizer_50_staticbox, wx.VERTICAL)
        self.sizer_49_staticbox.Lower()
        sizer_49 = wx.StaticBoxSizer(self.sizer_49_staticbox, wx.VERTICAL)
        self.sizer_48_staticbox.Lower()
        sizer_48 = wx.StaticBoxSizer(self.sizer_48_staticbox, wx.VERTICAL)
        sizer_32 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_18 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_19 = wx.BoxSizer(wx.VERTICAL)
        sizer_20 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_26_staticbox.Lower()
        sizer_26 = wx.StaticBoxSizer(self.sizer_26_staticbox, wx.HORIZONTAL)
        sizer_3_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy_1 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_9_copy_1_staticbox.Lower()
        sizer_9_copy_1 = wx.StaticBoxSizer(self.sizer_9_copy_1_staticbox, wx.HORIZONTAL)
        sizer_16_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_17_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_14_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_34 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_31 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_8_copy_1_staticbox.Lower()
        sizer_8_copy_1 = wx.StaticBoxSizer(self.sizer_8_copy_1_staticbox, wx.HORIZONTAL)
        sizer_12_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_13_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_10_copy_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_33 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_30 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6_copy_1 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_4_copy_1_staticbox.Lower()
        sizer_4_copy_1 = wx.StaticBoxSizer(self.sizer_4_copy_1_staticbox, wx.VERTICAL)
        sizer_5_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_55 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_56 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy = wx.BoxSizer(wx.VERTICAL)
        self.sizer_39_staticbox.Lower()
        sizer_39 = wx.StaticBoxSizer(self.sizer_39_staticbox, wx.HORIZONTAL)
        sizer_41 = wx.BoxSizer(wx.VERTICAL)
        sizer_43 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_40 = wx.BoxSizer(wx.VERTICAL)
        sizer_36 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_29 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_8_copy_staticbox.Lower()
        sizer_8_copy = wx.StaticBoxSizer(self.sizer_8_copy_staticbox, wx.HORIZONTAL)
        sizer_12_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_13_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_10_copy = wx.BoxSizer(wx.VERTICAL)
        sizer_35 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_28 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6_copy = wx.BoxSizer(wx.VERTICAL)
        self.sizer_4_copy_staticbox.Lower()
        sizer_4_copy = wx.StaticBoxSizer(self.sizer_4_copy_staticbox, wx.VERTICAL)
        sizer_5_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_54 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_15 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_9_staticbox.Lower()
        sizer_9 = wx.StaticBoxSizer(self.sizer_9_staticbox, wx.HORIZONTAL)
        sizer_16 = wx.BoxSizer(wx.VERTICAL)
        sizer_17 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_14 = wx.BoxSizer(wx.VERTICAL)
        sizer_38 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_27 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_8_staticbox.Lower()
        sizer_8 = wx.StaticBoxSizer(self.sizer_8_staticbox, wx.HORIZONTAL)
        sizer_12 = wx.BoxSizer(wx.VERTICAL)
        sizer_13 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_10 = wx.BoxSizer(wx.VERTICAL)
        sizer_37 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_4_staticbox.Lower()
        sizer_4 = wx.StaticBoxSizer(self.sizer_4_staticbox, wx.VERTICAL)
        sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_42 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_11 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1.Add(self.label_1, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_4.Add((20, 10), 0, 0, 0)
        sizer_4.Add(self.tree_ctrl_1, 1, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 5)
        sizer_11.Add(self.button_3, 0, wx.LEFT, 5)
        sizer_11.Add(self.label_27, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4.Add(sizer_11, 0, wx.EXPAND, 0)
        sizer_42.Add(self.button_17, 0, wx.LEFT, 5)
        sizer_42.Add(self.label_24, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4.Add(sizer_42, 0, wx.EXPAND, 0)
        sizer_5.Add(self.button_1, 0, wx.LEFT, 5)
        sizer_5.Add(self.label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4.Add(sizer_5, 0, wx.EXPAND, 0)
        sizer_3.Add(sizer_4, 1, wx.ALL | wx.EXPAND, 5)
        sizer_6.Add((20, 60), 0, 0, 0)
        sizer_6.Add(self.button_2, 0, 0, 0)
        sizer_6.Add(self.button_20, 0, wx.TOP, 0)
        sizer_6.Add((20, 150), 0, 0, 0)
        sizer_6.Add(self.button_23, 0, 0, 0)
        sizer_6.Add(self.button_26, 0, 0, 0)
        sizer_3.Add(sizer_6, 0, wx.EXPAND, 0)
        sizer_10.Add(self.label_3, 0, wx.LEFT, 5)
        sizer_10.Add(self.list_box_2, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_2.Add(self.button_11, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_2.Add(self.label_15, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10.Add(sizer_2, 0, wx.EXPAND, 0)
        sizer_37.Add(self.button_4, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_37.Add(self.label_4, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10.Add(sizer_37, 0, wx.EXPAND, 0)
        sizer_8.Add(sizer_10, 1, wx.EXPAND, 0)
        sizer_12.Add(self.label_5, 0, wx.LEFT, 5)
        sizer_12.Add(self.list_box_3, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_13.Add(self.button_5, 0, wx.LEFT, 5)
        sizer_13.Add(self.label_6, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_12.Add(sizer_13, 0, wx.EXPAND, 0)
        sizer_12.Add((20, 25), 0, 0, 0)
        sizer_8.Add(sizer_12, 1, wx.EXPAND, 0)
        sizer_7.Add(sizer_8, 1, wx.ALL | wx.EXPAND, 5)
        sizer_14.Add(self.label_7, 0, wx.LEFT, 5)
        sizer_14.Add(self.list_box_4, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_27.Add(self.button_12, 0, wx.LEFT, 5)
        sizer_27.Add(self.label_16, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_14.Add(sizer_27, 0, wx.EXPAND, 0)
        sizer_38.Add(self.button_6, 0, wx.LEFT, 5)
        sizer_38.Add(self.label_8, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_14.Add(sizer_38, 0, wx.EXPAND, 0)
        sizer_9.Add(sizer_14, 1, wx.EXPAND, 0)
        sizer_16.Add(self.label_9, 0, wx.LEFT, 5)
        sizer_16.Add(self.list_box_5, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_17.Add(self.button_7, 0, wx.LEFT, 5)
        sizer_17.Add(self.label_10, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_16.Add(sizer_17, 0, wx.EXPAND, 0)
        sizer_16.Add((20, 25), 0, 0, 0)
        sizer_9.Add(sizer_16, 1, wx.EXPAND, 0)
        sizer_7.Add(sizer_9, 1, wx.ALL | wx.EXPAND, 5)
        sizer_3.Add(sizer_7, 2, wx.EXPAND, 0)
        self.notebook_1_pane_1.SetSizer(sizer_3)
        sizer_4_copy.Add((20, 10), 0, 0, 0)
        sizer_4_copy.Add(self.tree_ctrl_c1, 1, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 5)
        sizer_15.Add(self.button_9, 0, wx.LEFT, 5)
        sizer_15.Add(self.label_28, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy.Add(sizer_15, 0, wx.EXPAND, 0)
        sizer_54.Add(self.button_24, 0, wx.LEFT, 5)
        sizer_54.Add(self.label_25, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy.Add(sizer_54, 0, wx.EXPAND, 0)
        sizer_5_copy.Add(self.button_c1, 0, wx.LEFT, 5)
        sizer_5_copy.Add(self.label_2_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy.Add(sizer_5_copy, 0, wx.EXPAND, 0)
        sizer_3_copy.Add(sizer_4_copy, 1, wx.ALL | wx.EXPAND, 5)
        sizer_6_copy.Add((20, 60), 0, 0, 0)
        sizer_6_copy.Add(self.button_c2, 0, 0, 0)
        sizer_6_copy.Add(self.button_22, 0, 0, 0)
        sizer_6_copy.Add((20, 150), 0, 0, 0)
        sizer_6_copy.Add(self.button_28, 0, 0, 0)
        sizer_6_copy.Add(self.button_29, 0, 0, 0)
        sizer_3_copy.Add(sizer_6_copy, 0, wx.EXPAND, 0)
        sizer_10_copy.Add(self.label_3_copy, 0, wx.LEFT, 5)
        sizer_10_copy.Add(self.list_box_c2, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_28.Add(self.button_13, 0, wx.LEFT, 5)
        sizer_28.Add(self.label_18, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy.Add(sizer_28, 0, wx.EXPAND, 0)
        sizer_35.Add(self.button_c4, 0, wx.LEFT, 5)
        sizer_35.Add(self.label_4_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy.Add(sizer_35, 0, wx.EXPAND, 0)
        sizer_8_copy.Add(sizer_10_copy, 1, wx.EXPAND, 0)
        sizer_12_copy.Add(self.label_5_copy, 0, wx.LEFT, 5)
        sizer_12_copy.Add(self.list_box_c3, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_13_copy.Add(self.button_c5, 0, wx.LEFT, 5)
        sizer_13_copy.Add(self.label_6_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_12_copy.Add(sizer_13_copy, 0, wx.EXPAND, 0)
        sizer_12_copy.Add((20, 25), 0, 0, 0)
        sizer_8_copy.Add(sizer_12_copy, 1, wx.EXPAND, 0)
        sizer_7_copy.Add(sizer_8_copy, 1, wx.ALL | wx.EXPAND, 5)
        sizer_40.Add(self.label_7_copy, 0, wx.LEFT, 5)
        sizer_40.Add(self.list_box_c4, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_29.Add(self.button_14, 0, wx.LEFT, 5)
        sizer_29.Add(self.label_19, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40.Add(sizer_29, 0, wx.EXPAND, 0)
        sizer_36.Add(self.button_c6, 0, wx.LEFT, 5)
        sizer_36.Add(self.label_8_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_40.Add(sizer_36, 0, wx.EXPAND, 0)
        sizer_39.Add(sizer_40, 1, wx.EXPAND, 0)
        sizer_41.Add(self.label_9_copy, 0, wx.LEFT, 5)
        sizer_41.Add(self.list_box_c5, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_43.Add(self.button_c7, 0, wx.LEFT, 5)
        sizer_43.Add(self.label_10_copy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_41.Add(sizer_43, 0, wx.EXPAND, 0)
        sizer_41.Add((20, 25), 0, 0, 0)
        sizer_39.Add(sizer_41, 1, wx.EXPAND, 0)
        sizer_7_copy.Add(sizer_39, 1, wx.ALL | wx.EXPAND, 5)
        sizer_3_copy.Add(sizer_7_copy, 2, wx.EXPAND, 0)
        self.notebook_1_pane_2.SetSizer(sizer_3_copy)
        sizer_4_copy_1.Add((20, 10), 0, 0, 0)
        sizer_4_copy_1.Add(self.tree_ctrl_s1, 1, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 5)
        sizer_56.Add(self.button_21, 0, wx.LEFT, 5)
        sizer_56.Add(self.label_29, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy_1.Add(sizer_56, 0, wx.EXPAND, 0)
        sizer_55.Add(self.button_27, 0, wx.LEFT, 5)
        sizer_55.Add(self.label_26, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy_1.Add(sizer_55, 0, wx.EXPAND, 0)
        sizer_5_copy_1.Add(self.button_s1, 0, wx.LEFT, 5)
        sizer_5_copy_1.Add(self.label_2_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_4_copy_1.Add(sizer_5_copy_1, 0, wx.EXPAND, 0)
        sizer_3_copy_1.Add(sizer_4_copy_1, 1, wx.ALL | wx.EXPAND, 5)
        sizer_6_copy_1.Add((20, 60), 0, 0, 0)
        sizer_6_copy_1.Add(self.button_s2, 0, 0, 0)
        sizer_6_copy_1.Add(self.button_25, 0, 0, 0)
        sizer_6_copy_1.Add((20, 150), 0, 0, 0)
        sizer_6_copy_1.Add(self.button_30, 0, 0, 0)
        sizer_6_copy_1.Add(self.button_31, 0, 0, 0)
        sizer_3_copy_1.Add(sizer_6_copy_1, 0, wx.EXPAND, 0)
        sizer_10_copy_1.Add(self.label_3_copy_1, 0, wx.LEFT, 5)
        sizer_10_copy_1.Add(self.list_box_s2, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_30.Add(self.button_15, 0, wx.LEFT, 5)
        sizer_30.Add(self.label_20, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy_1.Add(sizer_30, 0, wx.EXPAND, 0)
        sizer_33.Add(self.button_s4, 0, wx.LEFT, 5)
        sizer_33.Add(self.label_4_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_10_copy_1.Add(sizer_33, 0, wx.EXPAND, 0)
        sizer_8_copy_1.Add(sizer_10_copy_1, 1, wx.EXPAND, 0)
        sizer_12_copy_1.Add(self.label_5_copy_1, 0, wx.LEFT, 5)
        sizer_12_copy_1.Add(self.list_box_s3, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_13_copy_1.Add(self.button_s5, 0, wx.LEFT, 5)
        sizer_13_copy_1.Add(self.label_6_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_12_copy_1.Add(sizer_13_copy_1, 0, wx.EXPAND, 0)
        sizer_12_copy_1.Add((20, 25), 0, 0, 0)
        sizer_8_copy_1.Add(sizer_12_copy_1, 1, wx.EXPAND, 0)
        sizer_7_copy_1.Add(sizer_8_copy_1, 1, wx.ALL | wx.EXPAND, 5)
        sizer_14_copy_1.Add(self.label_7_copy_1, 0, wx.LEFT, 5)
        sizer_14_copy_1.Add(self.list_box_s4, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_31.Add(self.button_16, 0, wx.LEFT, 5)
        sizer_31.Add(self.label_21, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_14_copy_1.Add(sizer_31, 0, wx.EXPAND, 0)
        sizer_34.Add(self.button_s6, 0, wx.LEFT, 5)
        sizer_34.Add(self.label_8_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_14_copy_1.Add(sizer_34, 0, wx.EXPAND, 0)
        sizer_9_copy_1.Add(sizer_14_copy_1, 1, wx.EXPAND, 0)
        sizer_16_copy_1.Add(self.label_9_copy_1, 0, wx.LEFT, 5)
        sizer_16_copy_1.Add(self.list_box_s5, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_17_copy_1.Add(self.button_s7, 0, wx.LEFT, 5)
        sizer_17_copy_1.Add(self.label_10_copy_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_16_copy_1.Add(sizer_17_copy_1, 0, wx.EXPAND, 0)
        sizer_16_copy_1.Add((20, 25), 0, 0, 0)
        sizer_9_copy_1.Add(sizer_16_copy_1, 1, wx.EXPAND, 0)
        sizer_7_copy_1.Add(sizer_9_copy_1, 1, wx.ALL | wx.EXPAND, 5)
        sizer_3_copy_1.Add(sizer_7_copy_1, 2, wx.EXPAND, 0)
        self.notebook_1_pane_3.SetSizer(sizer_3_copy_1)
        sizer_21.Add(self.label_12, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_26.Add(self.label_17, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_26.Add(self.text_ctrl_2, 1, 0, 0)
        sizer_26.Add(self.button_sm0, 0, wx.RIGHT, 10)
        sizer_21.Add(sizer_26, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 10)
        sizer_20.Add(self.label_11, 0, 0, 0)
        sizer_19.Add(sizer_20, 1, wx.ALIGN_RIGHT, 0)
        sizer_18.Add(sizer_19, 1, wx.EXPAND, 0)
        sizer_21.Add(sizer_18, 0, wx.EXPAND, 0)
        sizer_22.Add((20, 20), 0, 0, 0)
        sizer_32.Add(self.label_14, 0, wx.ALIGN_BOTTOM, 0)
        sizer_23.Add(sizer_32, 0, wx.EXPAND, 0)
        sizer_23.Add(self.list_ctrl_1, 1, wx.EXPAND, 0)
        sizer_48.Add(self.button_18, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_48.Add(self.button_19, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_48.Add(self.label_23, 0, 0, 0)
        sizer_47.Add(sizer_48, 1, wx.ALL | wx.EXPAND, 3)
        sizer_49.Add(self.button_sm1, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_49.Add(self.button_sm2, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_49.Add(self.button_sm3, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_49.Add(self.label_22, 0, 0, 0)
        sizer_47.Add(sizer_49, 1, wx.ALL | wx.EXPAND, 3)
        sizer_50.Add(self.button_sm4, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_50.Add(self.button_8, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_47.Add(sizer_50, 1, wx.ALL | wx.EXPAND, 3)
        sizer_23.Add(sizer_47, 0, wx.EXPAND, 0)
        sizer_22.Add(sizer_23, 2, wx.EXPAND, 0)
        sizer_44.Add((50, 80), 0, 0, 0)
        sizer_44.Add(self.bitmap_button_1, 0, wx.LEFT | wx.RIGHT, 8)
        sizer_22.Add(sizer_44, 0, wx.EXPAND, 0)
        sizer_45.Add(self.label_13, 0, wx.TOP, 5)
        sizer_45.Add(self.list_ctrl_2, 1, wx.EXPAND, 0)
        sizer_52.Add((20, 140), 0, 0, 0)
        sizer_51.Add(sizer_52, 0, wx.EXPAND, 0)
        sizer_53.Add(self.button_sm5, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_53.Add(self.button_sm6, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_51.Add(sizer_53, 1, wx.ALL | wx.EXPAND, 3)
        sizer_51.Add((20, 20), 0, 0, 0)
        sizer_45.Add(sizer_51, 0, wx.EXPAND, 0)
        sizer_22.Add(sizer_45, 1, wx.EXPAND, 0)
        sizer_46.Add((20, 20), 0, 0, 0)
        sizer_22.Add(sizer_46, 0, wx.EXPAND, 0)
        sizer_21.Add(sizer_22, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 10)
        self.notebook_1_pane_4.SetSizer(sizer_21)
        self.notebook_1.AddPage(self.notebook_1_pane_1, _("time"))
        self.notebook_1.AddPage(self.notebook_1_pane_2, _("constant"))
        self.notebook_1.AddPage(self.notebook_1_pane_3, _("system"))
        self.notebook_1.AddPage(self.notebook_1_pane_4, _(u"固体の材料設定"))
        sizer_1.Add(self.notebook_1, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 10)
        sizer_25.Add(self.button_10, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM, 10)
        sizer_24.Add(sizer_25, 1, wx.ALIGN_RIGHT, 0)
        sizer_1.Add(sizer_24, 0, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()
        # end wxGlade

        #page 0 のtitleを書き換え
        self.notebook_1.SetPageText(0 , iniTime)
        
    #------------Dclick時------------------
    def onCheckInListDclick(self, event):
        self.checkInListDclick(event)

    #------------pageChangeed--------------
    def onReloadData(self, event): # wxGlade: MyFrame.<event_handler>
        self.reloadData(event)

    #------------time--------------
    #time編集
    def onEditTime(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openOrgFile(event, iniTime)

    #time削除    
    def onDeleteTime(self, event):  # wxGlade: MyFrame.<event_handler>
        #fileBox = self.list_box_1
        treeBox = self.tree_ctrl_1
        self.deleteDirectFiles(treeBox, iniTime)

    #timeRename
    def onRenameTime(self, event):  # wxGlade: MyFrame.<event_handler>
        #fileBox = self.list_box_1
        treeBox = self.tree_ctrl_1
        self.renameDirectFile(treeBox, iniTime)

    #timeFluidコピー
    def onCopyFluidTime(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_1
        self.copyFiles(treeBox, "fluidRegions", iniTime)

    #timeFluid戻す
    def onReturnFluidTime(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_2
        regionBox = self.list_box_3
        self.returnFiles(fileBox, regionBox, "fluidRegions", iniTime)

    #timeFluidRegion選択
    def onSelectTimeFluidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_2
        regionBox = self.list_box_3
        self.showRegionFileList(regionBox, fileBox, "fluidRegions", iniTime)

    #timeSolidコピー
    def onCopySolidTime(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_1
        self.copyFiles(treeBox, "solidRegions", iniTime)

    #timeSolid戻す
    def onReturnSolidTime(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_4
        regionBox = self.list_box_5
        self.returnFiles(fileBox, regionBox, "solidRegions", iniTime)

    #timeSolidRegion選択
    def onSelectTimeSolidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_4
        regionBox = self.list_box_5
        self.showRegionFileList(regionBox, fileBox, "solidRegions", iniTime)

    #timeFluid削除
    def onDeleteTimeFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_2
        self.deleteFiles(fileBox, "fluidRegions", iniTime)

    #timeFluid編集
    def onEditTimeFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_2
        regionBox = self.list_box_3
        self.editFiles(fileBox, regionBox, iniTime)

    #timeSolid削除
    def onDeleteTimeSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_4
        self.deleteFiles(fileBox, "solidRegions", iniTime)

    #timeSolid編集
    def onEditTimeSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_4
        regionBox = self.list_box_5
        self.editFiles(fileBox, regionBox, iniTime)

    #------------constant--------------
    #constant編集
    def onEditConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openOrgFile(event, "constant")

    #constant削除
    def onDeleteConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.deleteDirectFiles(treeBox, "constant")

    #constantRename
    def onRenameConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.renameDirectFile(treeBox, "constant")

    #constantFluidコピー
    def onCopyFluidConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.copyFiles(treeBox, "fluidRegions", "constant")

    #constantFluid戻す
    def onReturnFluidConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c2
        regionBox = self.list_box_c3
        self.returnFiles(fileBox, regionBox, "fluidRegions", "constant")

    #constantFluidRegion選択
    def onSelectConstantFluidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c2
        regionBox = self.list_box_c3
        self.showRegionFileList(regionBox, fileBox, "fluidRegions", "constant")

    #constantSolidコピー（配布）
    def onCopySolidConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.copyFiles(treeBox, "solidRegions", "constant")

    #constantSolid戻す
    def onReturnSolidConstant(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c4
        regionBox = self.list_box_c5
        self.returnFiles(fileBox, regionBox, "solidRegions", "constant")

    #constantSolidRegion選択
    def onSelectConstantSolidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c4
        regionBox = self.list_box_c5
        self.showRegionFileList(regionBox, fileBox, "solidRegions", "constant")

    #constantFluid削除
    def onDeleteConstantFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c2
        self.deleteFiles(fileBox, "fluidRegions", "constant")

    #constantFluid編集
    def onEditConstantFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c2
        regionBox = self.list_box_c3
        self.editFiles(fileBox, regionBox, "constant")

    #constantSolid削除
    def onDeleteConstantSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c4
        self.deleteFiles(fileBox, "solidRegions", "constant")

    #constantSolid編集
    def onEditConstantSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c4
        regionBox = self.list_box_c5
        self.editFiles(fileBox, regionBox, "constant")

    #constant参照
    def onSetPhysicalDir(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setSolidPhysicalDBdir(event)

    #------------system--------------
    #system編集
    def onEditSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openOrgFile(event, "system")

    #system削除
    def onDeleteSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.deleteDirectFiles(treeBox, "system")

    #systemRename
    def onRenameSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.renameDirectFile(treeBox, "system")

    #systemFluidコピー
    def onCopyFluidSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.copyFiles(treeBox, "fluidRegions", "system")

    #systemFluid戻す
    def onReturnFluidSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s2
        regionBox = self.list_box_s3
        self.returnFiles(fileBox, regionBox, "fluidRegions", "system")

    #systemFluidRegion選択
    def onSelectSystemFluidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s2
        regionBox = self.list_box_s3
        self.showRegionFileList(regionBox, fileBox, "fluidRegions", "system")

    #systemSolidコピー
    def onCopySolidSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.copyFiles(treeBox, "solidRegions", "system")

    #systemSolid戻す
    def onReturnSolidSystem(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s4
        regionBox = self.list_box_s5
        self.returnFiles(fileBox, regionBox, "solidRegions", "system")

    #systemSolidRegion選択
    def onSelectSystemSolidRegion(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s4
        regionBox = self.list_box_s5
        self.showRegionFileList(regionBox, fileBox, "solidRegions", "system")

    #systemFluid削除
    def onDeleteSystemFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s2
        self.deleteFiles(fileBox, "fluidRegions", "system")

    #systemFluid編集
    def onEditSystemFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s2
        regionBox = self.list_box_s3
        self.editFiles(fileBox, regionBox, "system")

    #systemSolid削除
    def onDeleteSystemSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s4
        self.deleteFiles(fileBox, "solidRegions", "system")

    #systemSolid編集
    def onEditSystemSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s4
        regionBox = self.list_box_s5
        self.editFiles(fileBox, regionBox, "system")

    #------------close--------------
    #close
    def onClose(self, event):  # wxGlade: MyFrame.<event_handler>
        self.Destroy()

    #------------固体の材料設定--------------
    #regionMaterial編集
    def onEditRegionMaterial(self, event): # wxGlade: MyFrame.<event_handler>
        self.editMaterialFile("region")

    #regionMterial名変更
    def onRenameRegionMaterial(self, event): # wxGlade: MyFrame.<event_handler>
        self.renameRegionMaterial(event)

    #materialDB登録
    def onSaveMaterialDB(self, event): # wxGlade: MyFrame.<event_handler>
        self.saveMaterialDB(event)

    #候補材料をregionにセット
    def onSetMaterialToRegion(self, event): # wxGlade: MyFrame.<event_handler>
        self.setMaterialToRegion(event)

    def onEditMaterialDB(self, event): # wxGlade: MyFrame.<event_handler>
        self.editMaterialFile("DB")

    #materialDBを開く
    def onOpenMaterialDBfolder(self, event): # wxGlade: MyFrame.<event_handler>
        self.openMaterialDB(event)

    #materialDBの材料名をlistBoxにセット
    def onSetMaterialToListBox(self, event): # wxGlade: MyFrame.<event_handler>
        self.setMaterialToListBox(event)

    #候補材料のクリア
    def onClearMatReq(self, event): # wxGlade: MyFrame.<event_handler>
        self.clearMatReq(event)

    #regionFolderを開く
    def onOpenRegionFolder(self, event): # wxGlade: MyFrame.<event_handler>
        self.openRegionFolder("constant", self.list_box_sm1)

    #solidMaterialを開く
    def onOpenSolidMaterial(self, event): # wxGlade: MyFrame.<event_handler>
        self.openRegionFile("constant", self.list_box_sm2)

    #materialDB内のfileを開く
    def onOpenMaterialDB(self, event): # wxGlade: MyFrame.<event_handler>
        self.openMaterialDBfile()

    #材料をregionsに設定
    def onSetMaterialToRegions(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setMaterialToSelRegions(event)

    #------file open-----
    #time
    def onOpenTime(self, event): # wxGlade: MyFrame.<event_handler>
        self.openExpand(event, iniTime)

    #timeFluid
    def onOpenTimeFluid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openFluidFile(event, iniTime)

    #timeSold
    def onOpenTimeSolid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openSolidFile(event, iniTime)

    #constant
    def onOpenConstant(self, event): # wxGlade: MyFrame.<event_handler>
        self.openExpand(event, "constant")
        #self.openOrgFile(event, "constant")

    #constantFluid
    def onOpenConstantFluid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openFluidFile(event, "constant")

    #constantSolid
    def onOpenConstantSolid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openSolidFile(event, "constant")

    #system
    def onOpenSystem(self, event): # wxGlade: MyFrame.<event_handler>
        self.openExpand(event, "system")
        #self.openOrgFile(event, "system")

    #systemFluid
    def onOpenSystemFluid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openFluidFile(event, "system")

    #systemSolid
    def onOpenSystemSolid(self, event): # wxGlade: MyFrame.<event_handler>
        self.openSolidFile(event, "system")

    #----rename------
    #renameTimeFluid
    def onRenameTimeFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_2
        self.renameFiles(fileBox, "fluidRegions", iniTime)

    #renameTimeSolid
    def onRenameTimeSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_4
        #self.deleteFiles(fileBox, "solid", iniTime)
        self.renameFiles(fileBox, "solidRegions", iniTime)

    #renameConstantFluid
    def onRenameConstantFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c2
        self.renameFiles(fileBox, "fluidRegions", "constant")

    #renameConstantSolid
    def onRenameConstantSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_c4
        self.renameFiles(fileBox, "solidRegions", "constant")

    #renameSystemFluid
    def onRenameSystemFluid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s2
        self.renameFiles(fileBox, "fluidRegions", "system")

    #renameSystemSolid
    def onRenameSystemSolid(self, event):  # wxGlade: MyFrame.<event_handler>
        fileBox = self.list_box_s4
        self.renameFiles(fileBox, "solidRegions", "system")

    #csvFileで設定
    def onSetCSVFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setCSVFile(event)

    #fileに出力
    def onOutputCSVFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.outputCsvFile(event)

    #--------- treeExpand or collapsed -----------
    #iniTime
    def onCollapsed(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_1
        self.treeCollapsed(event, treeBox)

    def onExpanded(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_1
        self.treeExpanded(event, treeBox)

    #constant
    def onCollapsedConst(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.treeCollapsed(event, treeBox)

    def onExpandedConst(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_c1
        self.treeExpanded(event, treeBox)

    #system
    def onCollapsedSys(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.treeCollapsed(event, treeBox)

    def onExpanedSys(self, event):  # wxGlade: MyFrame.<event_handler>
        treeBox = self.tree_ctrl_s1
        self.treeExpanded(event, treeBox)

# end of class MyFrame


    #---- popupMenu command ----------------
    # time
    def onPopupMenuTimeAll(self, event):
        PPitem = self.popupMenuTimeAll.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == iniTime + _(u" folder開く"):
            caseDir = os.getcwd()
            pyTreeFoam.openFolderNautilus(caseDir + "/" + iniTime)
        elif text == _(u"編集"):
            self.openOrgFile(event, iniTime)
        elif text == _(u"名前変更"):
            self.renameDirectFile(self.tree_ctrl_1, iniTime)
        elif text == _(u"名前からregion名を削除"):
            self.deleteRegionName(self.tree_ctrl_1, iniTime)
        elif text == _(u"削除"):
            self.deleteDirectFiles(self.tree_ctrl_1, iniTime)
        elif text == _(u"コピー"):
            self.foldersCopy(self.tree_ctrl_1, iniTime)
        elif text == _(u"貼り付け"):
            self.foldersPaste(self.tree_ctrl_1, iniTime)
        elif text == iniTime + _(u" folderへ貼り付け"):
            self.foldersPasteOrgFolder(self.tree_ctrl_1, iniTime)

    def onPopupMenuTimeFluid(self, event):
        PPitem = self.popupMenuTimeFluid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_2
            regionBox = self.list_box_3
            self.editFiles(fileBox, regionBox, iniTime)
        elif text == _(u"名前変更"):
            fileBox = self.list_box_2
            self.renameFiles(fileBox, "fluidRegions", iniTime)
        elif text == _(u"削除"):
            fileBox = self.list_box_2
            self.deleteFiles(fileBox, "fluidRegions", iniTime)

    def onPopupMenuTimeSolid(self, event):
        PPitem = self.popupMenuTimeSolid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_4
            regionBox = self.list_box_5
            self.editFiles(fileBox, regionBox, iniTime)
        elif text == _(u"名前変更"):
            fileBox = self.list_box_4
            self.renameFiles(fileBox, "solidRegions", iniTime)
        elif text == _(u"削除"):
            fileBox = self.list_box_4
            self.deleteFiles(fileBox, "solidRegions", iniTime)

    # const
    def onPopupMenuConstAll(self, event):
        PPitem = self.popupMenuConstAll.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"constant folder開く"):
            caseDir = os.getcwd()
            pyTreeFoam.openFolderNautilus(caseDir + "/constant")
        elif text == _(u"編集"):
            self.openOrgFile(event, "constant")
        elif text == _(u"名前変更"):
            self.renameDirectFile(self.tree_ctrl_c1, "constant")
        elif text == _(u"名前からregion名を削除"):
            self.deleteRegionName(self.tree_ctrl_c1, "constant")
        elif text == _(u"削除"):
            self.deleteDirectFiles(self.tree_ctrl_c1, "constant")
        elif text == _(u"コピー"):
            self.foldersCopy(self.tree_ctrl_c1, "constant")
        elif text == _(u"貼り付け"):
            self.foldersPaste(self.tree_ctrl_c1, "constant")
        elif text == _(u"constant folderへ貼り付け"):
            self.foldersPasteOrgFolder(self.tree_ctrl_c1, "constant")

    def onPopupMenuConstFluid(self, event):
        PPitem = self.popupMenuConstFluid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_c2
            regionBox = self.list_box_c3
            self.editFiles(fileBox, regionBox, "constant")
        elif text == _(u"名前変更"):
            fileBox = self.list_box_c2
            self.renameFiles(fileBox, "fluidRegions", "constant")
        elif text == _(u"削除"):
            fileBox = self.list_box_c2
            self.deleteFiles(fileBox, "fluidRegions", "constant")

    def onPopupMenuConstSolid(self, event):
        PPitem = self.popupMenuConstSolid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_c4
            regionBox = self.list_box_c5
            self.editFiles(fileBox, regionBox, "constant")
        elif text == _(u"名前変更"):
            fileBox = self.list_box_c4
            self.renameFiles(fileBox, "solidRegions", "constant")
        elif text == _(u"削除"):
            fileBox = self.list_box_c4
            self.deleteFiles(fileBox, "solidRegions", "constant")

    # system
    def onPopupMenuSystemAll(self, event):
        PPitem = self.popupMenuSystemAll.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"system folder開く"):
            caseDir = os.getcwd()
            pyTreeFoam.openFolderNautilus(caseDir + "/system")
        elif text == _(u"編集"):
            self.openOrgFile(event, "system")
        elif text == _(u"名前変更"):
            self.renameDirectFile(self.tree_ctrl_s1, "system")
        elif text == _(u"名前からregion名を削除"):
            self.deleteRegionName(self.tree_ctrl_s1, "system")
        elif text == _(u"削除"):
            self.deleteDirectFiles(self.tree_ctrl_s1, "system")
        elif text == _(u"コピー"):
            self.foldersCopy(self.tree_ctrl_s1, "system")
        elif text == _(u"貼り付け"):
            self.foldersPaste(self.tree_ctrl_s1, "system")
        elif text == _(u"system folderへ貼り付け"):
            self.foldersPasteOrgFolder(self.tree_ctrl_s1, "system")

    def onPopupMenuSystemFluid(self, event):
        PPitem = self.popupMenuSystemFluid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_s2
            regionBox = self.list_box_s3
            self.editFiles(fileBox, regionBox, "system")
        elif text == _(u"名前変更"):
            fileBox = self.list_box_s2
            self.renameFiles(fileBox, "fluidRegions", "system")
        elif text == _(u"削除"):
            fileBox = self.list_box_s2
            self.deleteFiles(fileBox, "fluidRegions", "system")

    def onPopupMenuSystemSolid(self, event):
        PPitem = self.popupMenuSystemSolid.FindItemById(event.GetId())
        text = PPitem.GetText()
        if text == _(u"編集"):
            fileBox = self.list_box_s4
            regionBox = self.list_box_s5
            self.editFiles(fileBox, regionBox, "system")
        elif text == _(u"名前変更"):
            fileBox = self.list_box_s4
            self.renameFiles(fileBox, "solidRegions", "system")
        elif text == _(u"削除"):
            fileBox = self.list_box_s4
            self.deleteFiles(fileBox, "solidRegions", "system")

    #  showToolTipTimeOrg
    #    timeFolderのtoolTip
    def showToolTipTimeOrg(self, event):
        self.tree_ctrl_1.SetToolTip(wx.ToolTip(toolTipTextOrg))
    #  showToolTipTimeFluid
    def showToolTipTimeFluid(self, event):
        self.list_box_2.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))
    #  showToolTipTimeFluid
    def showToolTipTimeSolid(self, event):
        self.list_box_4.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))

    #  showToolTipConstantOrg
    #    constantFolderのtoolTip
    def showToolTipConstantOrg(self, event):
        self.tree_ctrl_c1.SetToolTip(wx.ToolTip(toolTipTextOrg))
    #  showToolTipConstantFluid
    def showToolTipConstantFluid(self, event):
        self.list_box_c2.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))
    #  showToolTipConstantFluid
    def showToolTipConstantSolid(self, event):
        self.list_box_c4.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))

    #  showToolTipSystemOrg
    #    systemFolderのtoolTip
    def showToolTipSystemOrg(self, event):
        self.tree_ctrl_s1.SetToolTip(wx.ToolTip(toolTipTextOrg))
    #  showToolTipSystemFluid
    def showToolTipSystemFluid(self, event):
        self.list_box_s2.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))
    #  showToolTipSystemFluid
    def showToolTipSystemSolid(self, event):
        self.list_box_s4.SetToolTip(wx.ToolTip(toolTipTextFluidSolid))

    #  showToolTipMaterialRegion
    #    materialのtoolTip
    def showToolTipMaterialRegion(self, event):
        global toolTipTextMaterial
        (x, y) = event.GetPositionTuple()
        col0 = self.list_ctrl_1.GetColumnWidth(0)
        col1 = self.list_ctrl_1.GetColumnWidth(1)
        col2 = self.list_ctrl_1.GetColumnWidth(2)
        if x < col0:
            toolTipTextMaterial = _(u"ここをダブルクリックすると、\n　regionのfolderを開きます。")
        elif x < col0 + col1:
            toolTipTextMaterial = _(u"ここをダブルクリックすると、\n　現在の材料設定ファイルを開きます。")
        else:
            toolTipTextMaterial = _(u"ダブルクリックすると、材料DB内の選択された材料に置き変わります。\n　この後、「新材料を設定」ボタンをクリックして材料設定します。")

        self.list_ctrl_1.SetToolTip(wx.ToolTip(toolTipTextMaterial))
    #  showToolTipMaterialDB
    def showToolTipMaterialDB(self, event):
        global toolTipTextMaterial
        toolTipTextMaterial = _(u"ここをダブルクリックすると、\n　材料DB内のファイルを開きます。")
        self.list_ctrl_2.SetToolTip(wx.ToolTip(toolTipTextMaterial))

    #  showRegionFileList
    #    region選択時、そのregion内のfileListを表示
    def showRegionFileList(self, regionBox, fileBox, fluidSolid, sourceFolder):
        global timeFluidFiles, timeSolidFiles, constantFluidFiles, constantSolidFiles
        global systemFluidFiles, systemSolidFiles
        nums = regionBox.GetSelections()
        if len(nums) != 1:
            return
        selRegion = regionBox.GetString(nums[0])
        regionDir = sourceFolder + "/" + selRegion
        [folders, allFiles] = pyTreeFoam.getFoldersFiles(regionDir)
        files = checkFileName(allFiles)
        if sourceFolder == iniTime and fluidSolid == "fluidRegions":
            timeFluidFiles = files
        elif sourceFolder == iniTime and fluidSolid == "solidRegions":
            timeSolidFiles = files
        elif sourceFolder == "constant" and fluidSolid == "fluidRegions":
            constantFluidFiles = files
        elif sourceFolder == "constant" and fluidSolid == "solidRegions":
            constantSolidFiles = files
        elif sourceFolder == "system" and fluidSolid == "fluidRegions":
            systemFluidFiles = files
        elif sourceFolder == "system" and fluidSolid == "solidRegions":
            systemSolidFiles = files

        nums = fileBox.GetSelections()
        selFileNames = []
        for n in nums:
            name = fileBox.GetString(n)
            selFileNames.append(name)
        fileBox.Clear()
        fileBox.AppendItems(files)
        for name in selFileNames:
            fileBox.SetStringSelection(name)            

    #  deleteRegionName
    #    file名からregion名を削除
    def deleteRegionName(self, treeBox, sourceFolder):
        global timeFiles, constantFiles, systemFiles
        items = treeBox.GetSelections()
        if len(items) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        #選択fileを取得
        renameFiles = []
        for item in items:
            name = self.getFileNameFromTree(treeBox, item)
            renameFiles.append(name)
        #rename（region名を削除）
        sourceDir = os.getcwd() + "/" + sourceFolder
        newNames = []
        flag = 0
        for name in renameFiles:
            oldName = sourceDir + "/" + name
            words = name.split(".")
            if len(words) == 1:
                fileName = words[0]
            elif len(words) > 1:
                flag = 1
                fileName = ".".join(words[:-1])
            else:
                fileName = ""
            if fileName != "":
                newNames.append(fileName)
                renName = sourceDir + "/" + fileName
                comm = "mv " + oldName + " " + renName
                os.system(comm)
        if flag == 0:
            return
        #treeを再作成
        [folders, files] = pyTreeFoam.getFoldersFiles(sourceDir)
        parFiles = checkFileName(files)
        [folders, files] = pyTreeFoam.getFoldersFiles(sourceDir + "/fluidRegions")
        parFluidFiles = checkFileName(files)
        [folders, files] = pyTreeFoam.getFoldersFiles(sourceDir + "/solidRegions")
        parSolidFiles = checkFileName(files)
        sourceTree = makeTreeData(parFluidFiles, parSolidFiles, parFiles)
        if sourceFolder == iniTime:
            timeFiles = sourceTree
            self.setTreeData(self.tree_ctrl_1, timeFiles)
        elif sourceFolder == "constant":
            constantFiles = sourceTree
            self.setTreeData(self.tree_ctrl_c1, constantFiles)
        elif sourceFolder == "system":
            systemFiles = sourceTree
            self.setTreeData(self.tree_ctrl_s1, systemFiles)
        #newNamesを選択する
        names = self.getTextBoxNames(sourceFolder)
        [nFolder, treeBox, fluidFilesTextBox, fluidRegionsTextBox,
            solidFilesTextBox, solidRegionsTextBox] = names
        self.setSelectItemsInTree(treeBox, newNames)
        title = _(u"ファイル名からregion名を削除")
        mess = _(u"ファイル名からregion名を削除しました。")
        self.okDialog(title, mess)


    #  foldersCopy
    #    コピー(popupMenu)
    def foldersCopy(self, treeBox, sourceFolder):
        items = treeBox.GetSelections()
        if len(items) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        names = []
        for item in items:
            names.append(self.getFileNameFromTree(treeBox, item))
        f = open(clipBoard, "w")
        f.write("foldersCopy\n")
        caseDir = os.getcwd()
        for name in names:
            fileName = caseDir + "/" + sourceFolder + "/" + name
            print "copy--->", fileName
            f.write(fileName.encode("utf-8") + "\n")
        f.close()

    #  foldersPaste
    #    貼り付け(popupMenu)
    def foldersPaste(self, treeBox, sourceFolder):
        items = treeBox.GetSelections()
        if len(items) == 0:
            title = _(u"エラー")
            mess = _(u"folderを選択してください。")
            self.errDialog(title, mess)
            return
        if len(items) != 1:
            title = _(u"エラー")
            mess = _(u"複数のfolderは選択できません。")
            self.errDialog(title, mess)
            return
        caseDir = os.getcwd()
        name = self.getFileNameFromTree(treeBox, items[0])
        pasteFolderName = caseDir + "/" + sourceFolder + "/" + name
        if os.path.isfile(pasteFolderName) == True:
            title = _(u"エラー")
            mess = _(u"folderを選択してください。")
            self.errDialog(title, mess)
            return
        f=open(clipBoard); lines=f.readlines(); f.close()
        sourceFiles = []
        newFolders = []
        for line in lines[1:]:
            if line[-1] == "\n":
                sourceFiles.append(line[:-1])
            else:
                sourceFiles.append(line)
            newFolders.append(pasteFolderName)
        newNames = self.checkNewNames(sourceFiles, newFolders)
        self.pasteFoldersFilesAtGauge(sourceFiles, newNames)
        self.getSelectItemsInTextBox(sourceFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(sourceFolder)      #selectItemsをセット
        self.selectNewNamesInTreeBox(treeBox, pasteFolderName, newNames)

    #  selectNewNamesInTreeBox
    #    newNamesを選択状態に設定
    def selectNewNamesInTreeBox(self, treeBox, pasteFolderName, newDirs):
        newNames = []
        pasteFolder = "/".join(pasteFolderName.split("/")[:-1])
        for newDir in newDirs:
            name = newDir[len(pasteFolder)+1:]
            words = name.split("/")
            if words[0] != "fluidRegions" and words[0] != "solidRegions":
                name = words[1]
            newNames.append(name)
        treeBox.UnselectAll()
        self.setSelectItemsInTree(treeBox, newNames)

    #  checkNewNames
    #    copy先に同名があるかどうか確認
    def checkNewNames(self, sourceFiles, newFolders):

        def isInName(names, name):
            flag = False
            for a in names:
                if a == name:
                    flag = True
                    break
            return flag

        newNames = []
        sourceNames = []
        for sourceFile in sourceFiles:
            sourceName = sourceFile.split("/")[-1]
            sourceNames.append(sourceName)
        for newFolder in newFolders:
            files = glob.glob(newFolder + "/*")
            names = []
            for name in files:
                names.append(name.split("/")[-1])
            for sourceName in sourceNames:
                if isInName(names, sourceName) == False:
                    newNames.append(newFolder + "/" + sourceName)
                else:
                    n = 0
                    newName = sourceName + "_copy" + str(n)
                    while isInName(names, newName) == True:
                        n += 1
                        newName = sourceName + "_copy" + str(n)
                    newNames.append(newFolder + "/" + newName)
        return newNames

    #  foldersPasteOrgFolder
    #    貼り付け（popupMenu）
    def foldersPasteOrgFolder(self, treeBox, sourceFolder):
        caseDir = os.getcwd()
        pasteFolderName = caseDir + "/" + sourceFolder
        if os.path.isfile(pasteFolderName) == True:
            title = _(u"エラー")
            mess = _(u"folderを選択してください。")
            self.errDialog(title, mess)
            return
        f=open(clipBoard); lines=f.readlines(); f.close()
        sourceFiles = []
        newFolders = []
        for line in lines[1:]:
            if line[-1] == "\n":
                sourceFiles.append(line[:-1])
            else:
                sourceFiles.append(line)
            newFolders.append(pasteFolderName)
        newNames = self.checkNewNames(sourceFiles, newFolders)
        self.pasteFoldersFilesAtGauge(sourceFiles, newNames)
        self.getSelectItemsInTextBox(sourceFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(sourceFolder)      #selectItemsをセット
        self.selectNewNamesInTreeBox(treeBox, pasteFolderName, newNames)
        

    #  pasteFoldersFilesAtGauge
    #
    def pasteFoldersFilesAtGauge(self, copyDirs, newDirs):
        gaugeDDialog.job = "copy"
        gaugeDDialog.title = _(u"folder, fileのコピー")
        gaugeDDialog.copyDirs = copyDirs
        gaugeDDialog.newDirs = newDirs
        dialog = gaugeDDialog.MyDialog(None)
        result = dialog.ShowModal()
        ans = gaugeDDialog.status
        if ans == "NG":
            #コピーをキャンセルした場合
            title = _(u"fileのコピー")
            mess = _(u"コピーを中止しました")
            self.okDialog(title, mess)
        if ans == "ERR":
            #エラーが発生した場合
            title = _(u"エラー")
            [error, mess] = gaugeDDialog.error
            self.errDialog(title, mess)
        return
        

    #  checkInListDclick
    #  Dclick時のevent確認（list_ctrl_1）
    def checkInListDclick(self, event):
        pos = event.GetPositionTuple()
        selectRegion = ""
        row = 0
        for item in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = item
                break
            row += 1
        colWidth = [self.list_ctrl_1.GetColumnWidth(0),
                    self.list_ctrl_1.GetColumnWidth(1),
                    self.list_ctrl_1.GetColumnWidth(2)]
        if pos[0] <= colWidth[0]:
            #regionFolderを開く
            self.openRegionNameFolder("constant", selectRegion)
        elif pos[0] <= colWidth[0] + colWidth[1]:
            #現在の設定材を開く
            self.openRegionNameFile("constant", selectRegion)
        else:
            #候補材を設定
            self.setMaterialToListBox(event)

    #  treeCollapsed
    #    treeItemを閉じる
    def treeCollapsed(self, event, treeBox):
        global iniTimeExpand, constantExpand, systemExpand
        #fluid or solid
        item = event.GetItem()
        name = treeBox.GetItemText(item)
        if name == "fluidRegions":
            n = 0
        else:
            n = 1
        #iniTime, constant or systemにセット
        if treeBox == self.tree_ctrl_1:
            iniTimeExpand[n] = 0
        elif treeBox == self.tree_ctrl_c1:
            constantExpand[n] = 0
        else:
            systemExpand[n] = 0

    #  treeExpanded
    #    treeItemを開く
    def treeExpanded(self, event, treeBox):
        global iniTimeExpand, constantExpand, systemExpand
        #fluid or solid
        item = event.GetItem()
        name = treeBox.GetItemText(item)
        if name == "fluidRegions":
            n = 0
        else:
            n = 1
        #iniTime, constant or systemにセット
        if treeBox == self.tree_ctrl_1:
            iniTimeExpand[n] = 1
        elif treeBox == self.tree_ctrl_c1:
            constantExpand[n] = 1
        else:
            systemExpand[n] = 1

    #  reloadData
    #    deta再読み込み（pageChangeの時）
    def reloadData(self, event):
        initialize()
        self.setFileNameMaterialToListBox()
        self.setFileNameToListBox()
        self.selectFirstRegion()

    #  selectFirstRegion
    #    最初のregionを選択
    def selectFirstRegion(self):
        self.list_box_3.Select(0)
        self.list_box_5.Select(0)
        self.list_box_c3.Select(0)
        self.list_box_c5.Select(0)
        self.list_box_s3.Select(0)
        self.list_box_s5.Select(0)

    #  openMaterialDBfile
    #    materialDB内のfileを開く
    def openMaterialDBfile(self):
        row = 0
        selectName = ""
        for name in solidPhysicalDB:
            if self.list_ctrl_2.IsSelected(row) == True:
                selectName = self.list_ctrl_2.GetItemText(row)
                break
            row += 1
        fileName = solidPhysicalDBdir + "/" + selectName
        pyTreeFoam.runEditor(fileName)

    #  openRegionFile
    #    region内のfileを開く
    def openRegionFile(self, folder, listBox):
        n = listBox.GetSelection()
        region = solidRegions[n]
        self.openRegionNameFile(folder, region)

    def openRegionNameFile(self, folder, region):
        #name = "solidThermophysicalProperties"
        name = solidPhysicalDBdir.split("/")[-1]
        fileName = os.getcwd() + "/" + folder + "/" + region + "/" + name
        pyTreeFoam.runEditor(fileName)

    #  openRegionFolder
    #    regionFolderを開く
    def openRegionFolder(self, folder, listBox):
        name = listBox.GetStringSelection()
        self.openRegionNameFolder(folder, name)

    def openRegionNameFolder(self, folder, name):    
        openRegionDir = os.getcwd() + "/" + folder + "/" + name
        pyTreeFoam.openFolderNautilus(openRegionDir)

    #  setMaterialToSelRegions
    #    材料を選択しているregionsに設定候補として設定する
    def setMaterialToSelRegions(self, event):
        #材料DBの選択行を検索
        material = ""
        row = 0
        for name in solidPhysicalDB:
            if self.list_ctrl_2.IsSelected(row) == True:
                material = self.list_ctrl_2.GetItemText(row)
                break
            row += 1
        if material == "":
            title = _(u"エラー")
            mess = _(u"材料DB内の材料名を選択していません。\n材料名を選択してください")
            self.errDialog(title, mess)
            return

        #選択行に材料を候補材として設定
        row = 0
        selectRegion = ""
        for region in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = region
                self.list_ctrl_1.SetStringItem(row, 2, material)
                constantSolidPhysicalReq[row] = material
            row += 1
        if selectRegion == "":
            title = _(u"エラー")
            mess = _(u"regionが選択されていません\n")
            mess += _(u"材料を設定するregionを選択（複数選択可）してください")
            self.errDialog(title, mess)
            return

    #  outputCsvFile
    #    csvFileを出力する
    def outputCsvFile(self, event):
        #file名の取得
        inputTextDDialog.title = _(u"file名の入力")
        inputTextDDialog.message = _(u"出力するcsvファイルのファイル名を入力してください。")
        inputTextDDialog.inputText = "regionMaterial.csv"
        dialog = inputTextDDialog.MyDialog(None)
        result = dialog.ShowModal()
        if inputTextDDialog.status == "OK":
            text = inputTextDDialog.inputText
            csvFileName = os.getcwd() + "/" + text
        else:
            return

        #出力データの作成
        cont  = "region,material\n"
        i = 0
        for region in solidRegions:
            material = constantSolidPhysical[i]
            cont += ",".join([region, material]) + "\n"
            i += 1
        f=open(csvFileName, "w")
        f.write(cont)
        f.close()
        title = _(u"csvFileの作成")
        message = _(u"現在の材料設定（region,material）を出力しました。")
        self.okDialog(title, message)

    #  setCSVFile
    #    csvFileを読み込み、各regionに一括して材料名をセット
    def setCSVFile(self, event):
        global solidRegions, constantSolidPhysicalReq, constantSolidPhysical
        global solidPhysicalDB
        title = _(u"fileの選択")
        message  = _(u"各regionの材料を設定したcsvfileを選択してください。\n")
        message += _(u"（csvFileは、regin名と材料名のセットで準備する。）")
        currDir = os.getcwd()
        fileFilter = "*.csv"
        [stat, csvFileName] = getOpenFileDDialogShow(title, message, 
                                currDir, fileFilter)
        if stat != "OK":
            return

        #csvFile読み込み
        dataFile = currDir + "/" + csvFileName
        data = readCsvData(dataFile)
        #エラーチェック
        error = self.csvErrorCheck(data, solidRegions, solidPhysicalDB)
        if error != "OK":
            return

        #読み込んだデータをconstantSolidPhysicalReqにセット
        constantSolidPhysicalReq = []
        row = 0
        for regName in solidRegions:
            flag = 0
            for line in data:
                if regName == line[0]:
                    constantSolidPhysicalReq.append(line[1])
                    flag = 1
                    break
            if flag == 0:
                constantSolidPhysicalReq.append("")
            row += 1
        #データをセット
        properties = solidProperties
        row = 0
        for regName in solidRegions:
            if constantSolidPhysicalReq[row] != "":
                fileName = solidPhysicalDBdir + "/" + constantSolidPhysicalReq[row]
                comm  = "cp " + fileName + " " + currDir + "/constant/" + regName
                comm += "/" + properties
                os.system(comm)
            row += 1
        initialize()
        self.setFileNameMaterialToListBox()
        title = _(u"データの一括セット")
        message = _(u"csvFile「") + csvFileName + _(u"」の材料データを各regionにセットしました。")
        self.okDialog(title, message)

    #  csvErrorCheck
    #    読み込んだcsvデータのエラーチェック
    def csvErrorCheck(self, data, regions, physicalDB):
        #未定義材料のチェック
        unDefineMat = []
        for regMat in data:
            sameFlag = 0
            for mat in physicalDB:
                if regMat[1] == mat:
                    sameFlag = 1
                    break
            if sameFlag == 0:
                unDefineMat.append(regMat[1])
        if len(unDefineMat) > 0:
            title = _(u"エラー")
            message  = _(u"csvFile内に、DBで定義されていない材料があります。\n")
            message += _(u"「")
            if len(unDefineMat) > 10:
                names = ",".join(unDefineMat[:10]) + "..."
            else:
                names = ",".join(unDefineMat)
            message += names + _(u"」が未定義です。")
            self.errDialog(title, message)
            return "error"
        #未定義regionチェック
        unDefine = []
        for regMat in data:
            sameFlag = 0
            for regName in regions:
                if regName == regMat[0]:
                    sameFlag = 1
                    break
            if sameFlag == 0:
                unDefine.append(regMat[0])
        if len(unDefine) > 0:
            title = _(u"エラー")
            message  = _(u"csvFile内に、存在しないregion名があります。\n")
            message += _(u"「")
            if len(unDefine) > 10:
                names = ",".join(unDefine[:10]) + "..."
            else:
                names = ",".join(unDefine)
            message += names + _(u"」が存在しません。")
            self.errDialog(title, message)
            return "error"
        #region洩れのチェック
        unDefine = []
        for regName in regions:
            sameFlag = 0
            for regMat in data:
                if regName == regMat[0]:
                    sameFlag = 1
                    break
            if sameFlag == 0:
                unDefine.append(regName)
        if len(unDefine) > 0:
            title = _(u"エラー")
            message  = _(u"以下のregionは、csvFile内に存在しません。\n")
            message += _(u"「")
            if len(unDefine) > 10:
                names = ",".join(unDefine[:10]) + "..."
            else:
                names = ",".join(unDefine)
            message += names + _(u"」は、材料が設定できません。")
            self.errDialog(title, message)
            return "error"
        return "OK"

    #  setMaterialToRegion
    #    候補材料をregionにセット
    def setMaterialToRegion(self, event):
        global constantSolidPhysical, constantSolidPhysicalReq
        nReg = len(solidRegions)
        setName = []
        i = 0
        while i<nReg:
            currMat = constantSolidPhysical[i]
            reqMat  = constantSolidPhysicalReq[i]
            """
            if currMat != reqMat:
                setName.append([i, reqMat])
            """
            setName.append([i, reqMat])
            i += 1
        currDir = os.getcwd()
        #OFversionチェック
        properties = solidProperties
        for name in setName:
            fileName = solidPhysicalDBdir + "/" + name[1]
            newFile = currDir + "/constant/"
            newFile += solidRegions[name[0]] + "/" + properties
            os.system("rm " + newFile)
            comm  = "cp " + fileName + " " + newFile
            os.system(comm)
        initialize()
        self.setFileNameMaterialToListBox()

    #  clearMatReq
    #    候補材料をクリア
    def clearMatReq(self, event):
        initialize()
        self.setFileNameMaterialToListBox()

    #  setMaterialToListBox
    #    材料を候補材としてlistBoxにセット
    def setMaterialToListBox(self, event):
        global constantSolidPhysicalReq
        #材料DBの選択行を検索
        material = ""
        row = 0
        for name in solidPhysicalDB:
            if self.list_ctrl_2.IsSelected(row) == True:
                material = self.list_ctrl_2.GetItemText(row)
                break
            row += 1

        #nMat = self.list_box_sm4.GetSelections()
        if material == "":
            title = _(u"エラー")
            mess = _(u"材料DB内の材料名を選択していません。\n材料名を選択後、クリックして材料を設定してください。")
            self.errDialog(title, mess)
            return
        #material = self.list_box_sm4.GetString(nMat[0])
        fileName = solidPhysicalDBdir + "/" + material
        self.checkPhysicalCont(fileName)
        #選択行を検索
        row = 0
        selectRegion = ""
        for region in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = region
                break
            row += 1
        #候補材を設定
        self.list_ctrl_1.SetStringItem(row, 2, material)
        constantSolidPhysicalReq[row] = material

    #  openMaterialDB
    #    materialDBを開く
    def openMaterialDB(self, event):
        openDir = solidPhysicalDBdir
        pyTreeFoam.openFolderNautilus(openDir)

    #  saveMaterialDB
    #    materialDB登録
    def saveMaterialDB(self, event):
        #選択行を検索
        row = 0
        nReg = 0
        selectRegion = ""
        for region in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = region
                nReg = row
                break
            row += 1
        #errorチェック
        if selectRegion == "":
            title = _(u"エラー")
            mess = _(u"材料名を選択してください。")
            self.errDialog(title, mess)
            return
        #材料を登録
        regName = self.list_ctrl_1.GetItemText(nReg)
        fileName  = os.getcwd() + "/constant/" + regName
        properties = solidProperties
        fileName += "/" + properties
        material = getMaterial(fileName)
        if material == "-":
            title = _(u"エラー")
            mess = _(u"材料名を設定してください。")
            self.errDialog(title, mess)
            return
        comm = "cp " + fileName + " " + solidPhysicalDBdir + "/" + material
        os.system(comm)
        initialize()
        self.setFileNameMaterialToListBox()        

    #  renameRegionMaterial
    #    regionMaterial名変更
    def renameRegionMaterial(self, event):
        #選択行を検索
        row = 0
        nReg = 0
        selectRegion = ""
        for region in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = region
                nReg = row
                break
            row += 1
        #errorチェック
        if selectRegion == "":
            title = _(u"エラー")
            mess = _(u"材料名を選択してください。")
            self.errDialog(title, mess)
            return
        #rename
        regName = self.list_ctrl_1.GetItemText(nReg)
        fileName  = os.getcwd() + "/constant/" + regName
        properties = solidProperties
        fileName += "/" + properties
        material = getMaterial(fileName)
        title = _(u"材料名の変更")
        message = _(u"新しい材料名を入力してください。")
        newMaterial = self.textDialog(title, message, material)
        if newMaterial == "":
            return
        setMaterial(fileName, newMaterial)
        initialize()
        self.setFileNameMaterialToListBox()        


    #  editMaterialFile
    #    材料fileを開く
    def editMaterialFile(self, folder):
        #選択行を検索
        row = 0
        nReg = 0
        selectRegion = ""
        for region in solidRegions:
            if self.list_ctrl_1.IsSelected(row) == True:
                selectRegion = region
                nReg = row
                break
            row += 1
        #処理
        if folder == "region":
            #errorチェック
            if selectRegion == "":
                title = _(u"エラー")
                mess = _(u"材料名を選択してください。")
                self.errDialog(title, mess)
                return
            #regionフォルダをひらく
            regName = self.list_ctrl_1.GetItemText(nReg)
            fileName  = os.getcwd() + "/constant/" + regName
            properties = solidProperties
            fileName += "/" + properties
        else:
            #材料DBファイルを開く
            n = 0
            selName = ""
            for name in solidPhysicalDB:
                if self.list_ctrl_2.IsSelected(n) == True:
                    selName = solidPhysicalDB[n]
                    break
                n += 1
            print n
            if selName == "":
                title = _(u"エラー")
                mess = _(u"材料名を選択してください。")
                self.errDialog(title, mess)
                return
            fileDir = solidPhysicalDBdir
            fileName = fileDir + "/" + selName
        pyTreeFoam.runEditor(fileName)

    #  renameFiles
    #    folder内のfile名を変更
    def renameFiles(self, fileBox, FluidSolid, renameFolder):
        nFiles = fileBox.GetSelections()
        if len(nFiles) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        if len(nFiles) != 1:
            title = _(u"エラー")
            mess = _(u"一度に複数のfile名を変更できません。\n  1ヶのfileを選択してください。")
            self.errDialog(title, mess)
            return

        if FluidSolid == "fluidRegions":
            regions = fluidRegions
        else:
            regions = solidRegions
        folders = []
        for region in regions:
            folder = os.getcwd() + "/" + renameFolder + "/" + region
            folders.append(folder)
        renFileName = fileBox.GetString(nFiles[0])
        title = _(u"file名の変更")
        mess = _(u"選択したfileのfile名を変更します。")
        newFileName = self.textDialog(title, mess, renFileName)
        if newFileName == "":
            return

        print _(u"file名を変更します。")
        for folder in folders:
            newNamePath = folder + "/" + newFileName
            renNamePath = folder + "/" + renFileName
            renNamePath = getFoamContents.checkFileName(renNamePath)
            if renNamePath[-3:] == ".gz":
                newNamePath += ".gz"
            comm = "mv " + renNamePath + " " + newNamePath
            print comm
            os.system(comm)
        self.getSelectItemsInTextBox(renameFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(renameFolder)      #selectItemsをセット

    #  openExpand
    #    folder or file open
    def openExpand(self, event, folder):
        if folder == iniTime:
            treeBox = self.tree_ctrl_1
        elif folder == "constant":
            treeBox = self.tree_ctrl_c1
        else:
            treeBox = self.tree_ctrl_s1
        items = treeBox.GetSelections()
        caseDir = os.getcwd()
        fileName = self.getFileNameFromTree(treeBox, items[0])
        if fileName == "fluidRegions" or fileName == "solidRegions":
            if treeBox.IsExpanded(items[0]) == True:
                treeBox.Collapse(items[0])
            else:
                treeBox.Expand(items[0])
        else:
            self.openOrgFile(event, folder)

    #  openOrgFile
    #    各folder内のfileを開く
    def openOrgFile(self, event, folder):
        if folder == iniTime:
            treeBox = self.tree_ctrl_1
        elif folder == "constant":
            treeBox = self.tree_ctrl_c1
        else:
            treeBox = self.tree_ctrl_s1
        items = treeBox.GetSelections()
        caseDir = os.getcwd()
        openFiles = []
        for item in items:
            fileName = self.getFileNameFromTree(treeBox, item)
            if not (fileName == "fluidRegions" or fileName == "solidRegions"):
                openFiles.append(caseDir + "/" + folder + "/" + fileName)
        fileNames = []
        for openFile in openFiles:
            fileName = getFoamContents.checkFileName(openFile)
            if os.path.isfile(fileName) == True:
                fileNames.append(fileName)
        fileName = " ".join(fileNames)
        if len(fileName) != 0:
            self.openEditFiles(fileName)


    #  openFluidFile
    #    fluid内のfileを開く
    def openFluidFile(self, event, folder):
        if folder == iniTime:
            fileBox = self.list_box_2
            regBox = self.list_box_3
        elif folder == "constant":
            fileBox = self.list_box_c2
            regBox = self.list_box_c3
        else:
            fileBox = self.list_box_s2
            regBox = self.list_box_s3
        self.openFiles(fileBox, regBox, folder)


    #openFiles
    #  fileをeditorでopen
    def openFiles(self, fileBox, regBox, folder):
        #file名取得
        nFile = fileBox.GetSelections()
        if len(nFile) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        names = []
        for n in nFile:
            names.append(fileBox.GetString(n))
        #region名取得
        nReg = regBox.GetSelections()
        if len(nReg) == 0:
            title = _(u"エラー")
            mess = _(u"regionを選択してください。")
            self.errDialog(title, mess)
            return
        regions = []
        for n in nReg:
            regions.append(regBox.GetString(n))
        #openFileを取得
        openFiles = []
        caseDir = os.getcwd()
        for region in regions:
            for name in names:
                openFile = "/".join([caseDir, folder, region, name])
                fileName = getFoamContents.checkFileName(openFile)
                openFiles.append(fileName)
        files = " ".join(openFiles)
        self.openEditFiles(files)

    #
    #  openEditFiles
    #    editorでfileをopenする
    def openEditFiles(self, files):
        #binary or size>2MB かどうかチェック
        if self.isBinaryFiles(files) == True:
            #binary or 2MB以上ファイルが含まれている場合
            title = _(u"fileの編集")
            mess = _(u"binary 又は 2MB以上のファイルが含まれている為、")
            mess += _(u"temporaryファイルを作ってファイルをopenします。\n")
            mess += _(u"  binary部（List <xxx> を含む行）は、編集できません。\n")
            mess += _(u"  編集内容は、editorを閉じた時に反映されます。\n")
            self.okDialog(title, mess)
            comm = "editFoamFile.py " + files + " &"
            os.system(comm)
        else:
            #2MB以下のasciiファイルの場合
            pyTreeFoam.runEditor(files)

    #
    #  isBinaryFiles
    #    binaryが存在するかどうか、fileSizeが2MBを超えるかどうかをチェック
    #    両方共満足するとbinary=Trueで戻る
    def isBinaryFiles(self, files):
        binary = False
        fileNames = files.split()
        for fileName in fileNames:
            cont = getFoamContents.readFile2000Bytes(fileName)
            if getFoamContents.checkBinary(cont) == "ascii":
                if os.path.getsize(fileName) > 2000000:
                    #2MBを超える場合は、binaryとして戻す
                    binary = True
                    break
                else:
                    #2MB以下はasciiとして戻す
                    binary = False
            else:
                binary = True
                break
        return binary

    #  openSolidFile
    #    Solid内のfileを開く
    def openSolidFile(self, event, folder):
        if folder == iniTime:
            fileBox = self.list_box_4
            regBox = self.list_box_5
        elif folder == "constant":
            fileBox = self.list_box_c4
            regBox = self.list_box_c5
        else:
            fileBox = self.list_box_s4
            regBox = self.list_box_s5
        self.openFiles(fileBox, regBox, folder)

    #  renameDirectFile
    #    folder直下のfileのrename
    def renameDirectFile(self, treeBox, renFolder):
        items = treeBox.GetSelections()
        if len(items) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        if len(items) != 1:
            title = _(u"エラー")
            mess = _(u"複数のfileは、選択できません。")
            self.errDialog(title, mess)
            return
        #name = fileBox.GetString(nFiles[0])
        name = self.getFileNameFromTree(treeBox, items[0])
        oldFileName = os.getcwd() + "/" + renFolder + "/" + name
        inputTextDDialog.title = _(u"file名の変更")
        inputTextDDialog.message = _(u"新しいファイル名を入力してください。")
        inputTextDDialog.inputText = name
        dialog = inputTextDDialog.MyDialog(None)
        result = dialog.ShowModal()
        if inputTextDDialog.status == "OK":
            name = inputTextDDialog.inputText
            fileName = os.getcwd() + "/" + renFolder + "/" + name
        else:
            return
        oldFileName = getFoamContents.checkFileName(oldFileName)
        if oldFileName[-3:] == ".gz":
            fileName += ".gz"
        comm = "mv " + oldFileName + " " + fileName
        os.system(comm)
        self.getSelectItemsInTextBox(renFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(renFolder)      #selectItemsをセット


    #  deleteDirectFiles
    #    folder直下のfile削除
    def deleteDirectFiles(self, treeBox, delFolder):
        items = treeBox.GetSelections()
        if len(items) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        title = _(u"file削除")
        mess = _(u"fileをゴミ箱に移動します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        delFiles = []
        for item in items:
            name = self.getFileNameFromTree(treeBox, item)
            delFiles.append(name)
        folder = os.getcwd() + "/" + delFolder
        for name in delFiles:
            delName = folder + "/" + name
            delName = getFoamContents.checkFileName(delName)
            obj = gio.File(delName.encode("utf-8"))
            obj.trash()
        self.getSelectItemsInTextBox(delFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(delFolder)      #selectItemsをセット


    #  deleteFiles
    #    file削除
    def deleteFiles(self, fileBox, FluidSolid, deleteFolder):
        nFiles = fileBox.GetSelections()
        if len(nFiles) == 0:
            title = _(u"エラー")
            mess = _(u"fileを選択してください。")
            self.errDialog(title, mess)
            return
        title = _(u"file削除")
        mess = _(u"fileをゴミ箱に移動します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return

        if FluidSolid == "fluidRegions":
            regions = fluidRegions
        else:
            regions = solidRegions
        folders = []
        for region in regions:
            folder = os.getcwd() + "/" + deleteFolder + "/" + region
            folders.append(folder)
        deleteFiles = []
        for n in nFiles:
            name = fileBox.GetString(n)
            deleteFiles.append(name)
        for folder in folders:
            for name in deleteFiles:
                delName = folder + "/" + name
                delName = getFoamContents.checkFileName(delName)
                #ゴミ箱に移動できなかった場合、削除する
                try:
                    obj = gio.File(delName.encode("utf-8"))
                    obj.trash()
                except:
                    comm = "rm " + delName
                    print comm
                    os.system(comm)
        self.getSelectItemsInTextBox(deleteFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(deleteFolder)      #selectItemsをセット


    #  copyFiles
    #    fileコピー
    def copyFiles(self, treeBox, FluidSolid, copyFolder):
        #nFiles = fileBox.GetSelections()
        items = treeBox.GetSelections()
        if len(items) <= 1:
            if len(items) == 1:
                name = treeBox.GetItemText(items[0])
                if name == "root":
                    title = _(u"エラー")
                    mess = _(u"fileを選択してください。")
                    self.errDialog(title, mess)
                    return
            else:
                title = _(u"エラー")
                mess = _(u"fileを選択してください。")
                self.errDialog(title, mess)
                return

        
        caseDir = os.getcwd()
        copyFiles = []
        toCopyFiles = []
        for item in items:
            fileName = self.getFileNameFromTree(treeBox, item)
            names = fileName.split("/")
            if len(names) == 2:
                regName = names[1].split(".")
                if len(regName) == 2:
                    region = regName[1]
                    name = regName[0]
                else:
                    region = ""
                    name = regName[0]
            else:
                #folder直下のfileを選択（全regionに配布）
                region = "."
                name = names[0]
                if FluidSolid == "fluidRegions":
                    regions = fluidRegions
                else:
                    regions = solidRegions
                #regions = fluidRegions + solidRegions
                for regName in regions:
                    copyFiles.append(caseDir + "/" + copyFolder + "/" + fileName)
                    toCopyFiles.append(caseDir + "/" + copyFolder + "/" + regName + "/" + name)
            if region == "_all_" or region == "":
                #全てのregionに配布
                if names[0] == "fluidRegions":
                    regions = fluidRegions
                else:
                    regions = solidRegions
                for regName in regions:
                    copyFiles.append(caseDir + "/" + copyFolder + "/" + fileName)
                    toCopyFiles.append(caseDir + "/" + copyFolder + "/" + regName + "/" + name)
            elif region != ".":
                #指定されたregionに配布
                copyFiles.append(caseDir + "/" + copyFolder + "/" + fileName)
                toCopyFiles.append(caseDir + "/" + copyFolder + "/" + region + "/" + name)

        #コピー開始
        badFiles = []
        i = 0
        while i<len(copyFiles):
            copyFile = copyFiles[i]
            pasteFile = toCopyFiles[i]
            pasteFolder = "/".join(pasteFile.split("/")[:-1])
            if len(glob.glob(pasteFolder)) > 0:
                comm = "cp " + copyFile + " " + pasteFile
                os.system(comm)
            else:
                badFiles.append(copyFile[len(caseDir):])
            i += 1
        if copyFolder == iniTime:
            #timeFolderの場合、コーピーしたfieldのboundaryFieldのクリア
            clearBoundaryFields(caseDir, iniTime, toCopyFiles)
        self.getSelectItemsInTextBox(copyFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(copyFolder)      #selectItemsをセット
        self.setSelectItemsToTextBox(copyFolder, FluidSolid)
        if len(badFiles) == 0:
            title = _(u"fileの配布")
            mess = _(u"regionにファイルを配布しました。")
            self.okDialog(title, mess)
        else:
            title = _(u"fileの配布")
            mess = _(u"以下のfileがコピーできていません。\n")
            i = 0
            while i<len(badFiles) and i<5:
                mess += badFiles[i] + "\n"
                i += 1
            if i < len(badFiles):
                mess += "    ..."
            print mess
            self.warningDialog(title, mess)

    #  setSelectItemsToTextBox
    #    copy先（配布先）の項目を選択表示させる
    def setSelectItemsToTextBox(self, copyFolder, FluidSolid):
        global selectItems

        if copyFolder == "iniTime":
            nFolder = 0
            if FluidSolid == "fluidRegions":
                self.listBox = self.list_box_2
            else:
                self.listBox = self.list_box_4
        elif copyFolder == "constant":
            nFolder = 1
            if FluidSolid == "fluidRegions":
                self.listBox = self.list_box_c2
            else:
                self.listBox = self.list_box_c4
        elif copyFolder == "system":
            nFolder = 2
            if FluidSolid == "fluidRegions":
                self.listBox = self.list_box_s2
            else:
                self.listBox = self.list_box_s4
        else:
            return

        dirs = selectItems[nFolder][0]
        names = []
        for name in dirs:
            names.append(name.split("/")[-1])
        self.listBox.DeselectAll()
        for name in names:
            if len(name.split(".")) > 0:
                name = name.split(".")[0]
            self.listBox.SetStringSelection(name)

    #  getFileNameFromTree
    #    treeからfileNameを取得
    def getFileNameFromTree(self, treeBox, item):
        name = treeBox.GetItemText(item)
        if name == "root":
            return ""
        fileName = name
        while name != "root":
            item = treeBox.GetItemParent(item)
            name = treeBox.GetItemText(item)
            if name != "root":
                fileName = name + "/" + fileName
        return fileName

    #  returnFiles
    #    fileを戻す
    def returnFiles(self, fileBox, regionBox, FluidSolid, toCopyFolder):
        nFiles = fileBox.GetSelections()
        if len(nFiles) == 0:
            title = _(u"エラー")
            mess = _(u"戻すfileを選択してください。")
            self.errDialog(title, mess)
            return
        #file名を取得
        if FluidSolid == "fluidRegions":
            [copyNames, pasteNames] = self.getCopyFileNameFromBox(
                                "fluidRegions", fileBox, regionBox)
        else:
            [copyNames, pasteNames] = self.getCopyFileNameFromBox(
                                "solidRegions", fileBox, regionBox)
        #dirを取得
        caseDir = os.getcwd()
        i = 0
        while i<len(copyNames):
            copyName = caseDir + "/" + toCopyFolder + "/" + copyNames[i]
            pasteName = caseDir + "/" + toCopyFolder + "/" + pasteNames[i]
            comm = "cp " + copyName + " " + pasteName
            os.system(comm)
            i += 1
        self.getSelectItemsInTextBox(toCopyFolder)      #textBoxのselectItemsを取得
        initialize()
        self.setFileNameToListBox()
        self.setSelectItemsInTextBox(toCopyFolder)      #selectItemsをセット
        self.setSelectItemsToTreeBox(toCopyFolder, pasteNames)
        title = _(u"fileを戻す")
        mess = _(u"region内のファイルをfolder直下に戻しました。")
        self.okDialog(title, mess)

    #  setSelectItemsToTreeBox
    #    copy先（戻す先）の項目を選択表示させる
    def setSelectItemsToTreeBox(self, toCopyFolder, pasteNames):
        if toCopyFolder == "constant":
            self.treeBox = self.tree_ctrl_c1
        elif toCopyFolder == "system":
            self.treeBox = self.tree_ctrl_s1
        else:
            try:
                a = float(toCopyFolder)
                self.treeBox = self.tree_ctrl_1
            except:                
                return

        self.treeBox.UnselectAll()
        for dirName in pasteNames:
            words = dirName.split("/")
            if len(words) == 2:
                #親を検索（fluidRegions or solidRegions）
                parentItem = self.treeBox.GetRootItem()
                item, cookie = self.treeBox.GetFirstChild(parentItem)
                while item:
                    text = self.treeBox.GetItemText(item)
                    if text == words[0]:
                        break
                    item, cookie = self.treeBox.GetNextChild(parentItem, cookie)
                if text == words[0]:
                    #子を検索
                    parentItem = item
                    item, cookie = self.treeBox.GetFirstChild(parentItem)
                    while item:
                        text = self.treeBox.GetItemText(item)
                        if text == words[1]:
                            #選択
                            self.treeBox.SelectItem(item, True)
                            self.treeBox.Expand(item)
                            break
                        item, cookie = self.treeBox.GetNextChild(parentItem, cookie)

    #  getCopyFileNameFromBox
    #    copyするfile名を取得
    def getCopyFileNameFromBox(self, folder, fileBox, regionBox):
        fluidNames = []
        toCopyFluidNames = []
        regFlag = 0
        #    fluidRegion名取得
        regionNo = regionBox.GetSelections()
        fluidRegions = []
        for region in regionNo:
            name = regionBox.GetString(region)
            fluidRegions.append(name)
        if len(fluidRegions) == 0:
            regFlag = 1
            fluidRegions += [regionBox.GetString(0)]
        #    region毎のfile名取得
        for fluidRegion in fluidRegions:
            fluidFiles = []
            regionNo = fileBox.GetSelections()
            for No in regionNo:
                fluidFiles.append(fileBox.GetString(No))
            for fluidFile in fluidFiles:
                fluidName = fluidRegion + "/" + fluidFile
                fluidNames.append(fluidName)
                if regFlag == 0:
                    regName = fluidRegion
                    toCopyName = folder + "/" + fluidFile + "." + regName
                else:
                    #regName = "_all_"
                    toCopyName = folder + "/" + fluidFile
                toCopyFluidNames.append(toCopyName)
        sourceFiles = fluidNames
        toCopyFiles = toCopyFluidNames
        return [sourceFiles, toCopyFiles]


    #  editFiles
    #    file編集
    def editFiles(self, fileBox, regionBox, editFolder):
        self.openFiles(fileBox, regionBox, editFolder)

    #
    #  checkPhysicalCont
    #    thermophysicalProperties内のmaterialをチェック
    def checkPhysicalCont(self, fileName):
        f=codecs.open(fileName, "r", "utf-8"); cont=f.read(); f.close()
        p = cont.find("material")
        if p < 0:
            p = getFoamContents.skipFoamFileMem(cont, 0)
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
            i = p
            while i >= 0 and cont[i] != "\n":
                i -= 1
            addLine = "material    " + fileName.split("/")[-1] + ";\n\n"
            newCont = cont[:i] + addLine + cont[i+1:]
            f=codecs.open(fileName, "w", "utf-8"); f.write(newCont); f.close()
        else:
            ps = p
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
            material = fileName.split("/")[-1]
            if keyword != material:
                pe = getFoamContents.skipUntilCRMem(cont, ps)
                addLine = "material    " + material + ";"
                newCont = cont[:ps] + addLine + cont[pe:]
                f=codecs.open(fileName, "w", "utf-8"); f.write(newCont); f.close()

    #
    #  setSelectItemsInTextBox
    #    各textBoxのselectItemsを取得
    def setSelectItemsInTextBox(self, sourceFolder):
        global selectItems

        #  setSelectString
        #    wordsを選択する
        def setSelectString(textBox, words):
            for word in words:
                textBox.SetStringSelection(word)

        #処理-----------
        names = self.getTextBoxNames(sourceFolder)
        if len(names) == 0:
            return
        #textBox名を取得
        [nFolder, treeBox, fluidFilesTextBox, fluidRegionsTextBox,
            solidFilesTextBox, solidRegionsTextBox] = names
        #treeをセット
        selNames = selectItems[nFolder][0]
        self.setSelectItemsInTree(treeBox, selNames)
        #fluidFilesをセット
        words = selectItems[nFolder][1]
        setSelectString(fluidFilesTextBox, words)
        #fluidRegionsをセット
        words = selectItems[nFolder][2]
        setSelectString(fluidRegionsTextBox, words)
        #solidFilesをセット
        words = selectItems[nFolder][3]
        setSelectString(solidFilesTextBox, words)
        #solidRegions
        words = selectItems[nFolder][4]
        setSelectString(solidRegionsTextBox, words)


    #  setSelectItemsInTree
    #    Tree内のitemを選択する
    def setSelectItemsInTree(self, treeBox, selNames):
        for name in selNames:
            words = name.split("/")
            if len(words) == 1:
                parentItem = treeBox.GetRootItem()
                #親itemから子のtextを取得し、選択
                item, cookie = treeBox.GetFirstChild(parentItem)
                while item:
                    text = treeBox.GetItemText(item)
                    if text == words[0]:
                        #選択
                        treeBox.SelectItem(item, True)
                        break
                    item, cookie = treeBox.GetNextChild(parentItem, cookie)
            elif len(words) == 2:
                #親を検索
                parentItem = treeBox.GetRootItem()
                item, cookie = treeBox.GetFirstChild(parentItem)
                while item:
                    text = treeBox.GetItemText(item)
                    if text == words[0]:
                        break
                    item, cookie = treeBox.GetNextChild(parentItem, cookie)
                if text == words[0]:
                    #子を検索
                    parentItem = item
                    item, cookie = treeBox.GetFirstChild(parentItem)
                    while item:
                        text = treeBox.GetItemText(item)
                        if text == words[1]:
                            #選択
                            treeBox.SelectItem(item, True)
                            break
                        item, cookie = treeBox.GetNextChild(parentItem, cookie)

    #
    #  getSelectItemsInTextBox
    #    各textBoxのselectItemsを取得
    def getSelectItemsInTextBox(self, sourceFolder):
        global selectItems

        #  getSelectString
        #    listBoxの選択項目を取得
        def getSelectString(textBox):
            ans = []
            nItems = textBox.GetSelections()
            for n in nItems:
                text = textBox.GetString(n)
                ans.append(text)
            return ans

        #処理-----------------
        names = self.getTextBoxNames(sourceFolder)
        if len(names) == 0:
            return
        #textBox名を取得
        [nFolder, treeBox, fluidFilesTextBox, fluidRegionsTextBox,
            solidFilesTextBox, solidRegionsTextBox] = names
        #保存先をクリア
        selectItems[nFolder] = []
        #treeを取得
        items = treeBox.GetSelections()
        ans = []
        for item in items:
            parentItem = treeBox.GetItemParent(item)
            if parentItem == treeBox.GetRootItem():
                text = treeBox.GetItemText(item)
            else:
                folder = treeBox.GetItemText(parentItem)
                text = folder + "/" + treeBox.GetItemText(item)
            ans.append(text)
        selectItems[nFolder].append(ans)
        #fluidFilesを取得
        ans = getSelectString(fluidFilesTextBox)
        selectItems[nFolder].append(ans)
        #fluidRegionsを取得
        ans = getSelectString(fluidRegionsTextBox)
        selectItems[nFolder].append(ans)
        #solidFilesを取得
        ans = getSelectString(solidFilesTextBox)
        selectItems[nFolder].append(ans)
        #solidRegionsを取得
        ans = getSelectString(solidRegionsTextBox)
        selectItems[nFolder].append(ans)

    #  getTextBoxNames
    def getTextBoxNames(self, sourceFolder):
        if sourceFolder == iniTime:
            nFolder = 0
            treeBox = self.tree_ctrl_1
            fluidFilesTextBox = self.list_box_2
            fluidRegionsTextBox = self.list_box_3
            solidFilesTextBox = self.list_box_4
            solidRegionsTextBox = self.list_box_5
        elif sourceFolder == "constant":
            nFolder = 1
            treeBox = self.tree_ctrl_c1
            fluidFilesTextBox = self.list_box_c2
            fluidRegionsTextBox = self.list_box_c3
            solidFilesTextBox = self.list_box_c4
            solidRegionsTextBox = self.list_box_c5
        elif sourceFolder == "system":
            nFolder = 2
            treeBox = self.tree_ctrl_s1
            fluidFilesTextBox = self.list_box_s2
            fluidRegionsTextBox = self.list_box_s3
            solidFilesTextBox = self.list_box_s4
            solidRegionsTextBox = self.list_box_s5
        else:
            return []
        return [nFolder, treeBox, fluidFilesTextBox, fluidRegionsTextBox,
                solidFilesTextBox, solidRegionsTextBox]

    #
    #  setSolidPhysicalDBdir
    #    solidPhysicalDBのdirをセット
    def setSolidPhysicalDBdir(self, event):
        global solidPhysicalDBdir
        title = _(u"固体用の物性値DBフォルダのセット")
        mess = _(u"固体の物性値データベースの保存先フォルダを設定します。")
        root = os.environ["HOME"]
        currDir = os.getcwd()
        selDir = solidPhysicalDBdir.encode("utf-8")
        [stat, selDir] = getOpenFolderDDialog.Show(title, mess, root, currDir, selDir)
        if stat == "OK":
            solidPhysicalDBdir = selDir
            self.text_ctrl_2.SetValue(selDir)
            setSolidDirMultiRegion_data(selDir)
            solidPhysicalDB = getSolidPhysicalDB(solidPhysicalDBdir)
            self.list_box_sm4.Clear()
            self.list_box_sm4.AppendItems(solidPhysicalDB)


    def setFileNameMaterialToListBox(self):
        #setFileName
        self.setFileNameToListBox()
        #constantSolidPhysical
        i = 0
        for name in solidRegions:
            self.list_ctrl_1.SetStringItem(i, 0, name)
            self.list_ctrl_1.SetStringItem(i, 1, constantSolidPhysical[i])
            self.list_ctrl_1.SetStringItem(i, 2, constantSolidPhysicalReq[i])
            i += 1
        #solidPhysicalDB
        self.list_ctrl_2.DeleteAllItems()
        i = 0
        for name in solidPhysicalDB:
            self.list_ctrl_2.InsertStringItem(i, name)
            i += 1
        #solidPhysicalDBdir
        self.text_ctrl_2.SetValue(solidPhysicalDBdir)

    def setFileNameToListBox(self):
        #timeFile
        self.setTreeData(self.tree_ctrl_1, timeFiles)
        self.list_box_2.Clear()
        self.list_box_2.AppendItems(timeFluidFiles)
        self.list_box_3.Clear()
        self.list_box_3.AppendItems(fluidRegions)
        self.list_box_4.Clear()
        self.list_box_4.AppendItems(timeSolidFiles)
        self.list_box_5.Clear()
        self.list_box_5.AppendItems(solidRegions)
        #constant
        self.setTreeData(self.tree_ctrl_c1, constantFiles)
        self.list_box_c2.Clear()
        self.list_box_c2.AppendItems(constantFluidFiles)
        self.list_box_c3.Clear()
        self.list_box_c3.AppendItems(fluidRegions)
        self.list_box_c4.Clear()
        self.list_box_c4.AppendItems(constantSolidFiles)
        self.list_box_c5.Clear()
        self.list_box_c5.AppendItems(solidRegions)
        #system
        self.setTreeData(self.tree_ctrl_s1, systemFiles)
        self.list_box_s2.Clear()
        self.list_box_s2.AppendItems(systemFluidFiles)
        self.list_box_s3.Clear()
        self.list_box_s3.AppendItems(fluidRegions)
        self.list_box_s4.Clear()
        self.list_box_s4.AppendItems(systemSolidFiles)
        self.list_box_s5.Clear()
        self.list_box_s5.AppendItems(solidRegions)
        #solidMaterial
        i = 0
        for name in solidRegions:
            self.list_ctrl_1.SetStringItem(i, 0, name)
            self.list_ctrl_1.SetStringItem(i, 1, constantSolidPhysical[i])
            self.list_ctrl_1.SetStringItem(i, 2, constantSolidPhysicalReq[i])
            i += 1

    #  setTreeData
    #    treeDataをセット
    def setTreeData(self, treeCtrl, treeData):
        #dataSet
        rootName = treeData[0]
        items = treeData[1]
        treeCtrl.DeleteAllItems()
        root = treeCtrl.AddRoot(rootName)
        self.AddTreeNodes(treeCtrl, root, items)
        #treeExpand
        if treeCtrl == self.tree_ctrl_1:
            if iniTimeExpand[0] == 1:
                self.expandItem(treeCtrl, "fluidRegions")
            if iniTimeExpand[1] == 1:
                self.expandItem(treeCtrl, "solidRegions")
        if treeCtrl == self.tree_ctrl_c1:
            if constantExpand[0] == 1:
                self.expandItem(treeCtrl, "fluidRegions")
            if constantExpand[1] == 1:
                self.expandItem(treeCtrl, "solidRegions")
        else:
            if systemExpand[0] == 1:
                self.expandItem(treeCtrl, "fluidRegions")
            if systemExpand[1] == 1:
                self.expandItem(treeCtrl, "solidRegions")
        #treeColour
        self.setBackGroundColour(treeCtrl)

    # AddTreeNode
    def AddTreeNodes(self, treeCtrl, parentItem, items):
        for item in items:
            if type(item) == str or type(item) == unicode:
                newItem = treeCtrl.AppendItem(parentItem, item)
                if item == "." or item == "fluidRegions" or item == "solidRegions":
                    treeCtrl.SetItemImage(newItem, self.folderIdx,
                        wx.TreeItemIcon_Normal)
            else:
                newItem = treeCtrl.AppendItem(parentItem, item[0])
                self.AddTreeNodes(treeCtrl, newItem, item[1])
                if item[0] == "." or item[0] == "fluidRegions" or item[0] == "solidRegions":
                    treeCtrl.SetItemImage(newItem, self.folderIdx,
                        wx.TreeItemIcon_Normal)

    #  expandItem
    def expandItem(self, treeCtrl, name):
        rootItem = treeCtrl.GetRootItem()
        items = self.getChildrenItems(rootItem)
        for item in items:
            itemName = treeCtrl.GetItemText(item)
            if name == itemName:
                treeCtrl.Expand(item)
                break

    #  getChildrenItems
    #    親itemから子itemを取得
    def getChildrenItems(self, parent):
        items = []
        #firstChild
        item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
        while item:
            items.append(item)
            #nextChild
            item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
        return items

    #  setBackGroundColour
    #    fluid solidのbackGroundColourをセット
    def setBackGroundColour(self, treeCtrl):
        rootItem = treeCtrl.GetRootItem()
        items = self.getChildrenItems(rootItem)
        for item in items:
            itemName = treeCtrl.GetItemText(item)
            if itemName == "fluidRegions" or itemName == "solidRegions":
                colourItems = self.getChildrenItems(item)
                for colourItem in colourItems:
                    #treeCtrl.SetItemBackgroundColour(colourItem, wx.Colour(238, 238, 235))
                    #treeCtrl.SetItemBackgroundColour(colourItem, wx.Colour(246, 244, 242))
                    treeCtrl.SetItemBackgroundColour(colourItem, wx.Colour(252, 252, 252))

    def checkRegions(self):
        error = 0
        if len(fluidRegions) == 0 or len(solidRegions) == 0:
            error = 1
        else:
            if len(fluidRegions) != 0:
                region = os.getcwd() + "/constant/" + fluidRegions[0]
                if len(glob.glob(region)) == 0:
                    error = 1
            if len(solidRegions) != 0:
                region = os.getcwd() + "/constant/" + solidRegions[0]
                if len(glob.glob(region)) == 0:
                    error = 1
        if error == 1:
            title = _(u"エラー")
            mess = _(u"このcaseには、regionがありません。\n解析caseを確認してください。")
            self.errDialog(title, mess)


    def errDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_HAND) 
        dlg.ShowModal()
        dlg.Destroy()

    def warningDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_QUESTION) 
        dlg.ShowModal()
        dlg.Destroy()

    def okDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_INFORMATION) 
        dlg.ShowModal()
        dlg.Destroy()

    def okCancelDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                             wx.OK | wx.CANCEL | wx.ICON_INFORMATION) 
        ans = dlg.ShowModal()
        stat = ""
        if ans == wx.ID_OK:
            stat = "OK"
        else:
            stat = "CANCEL"
        return stat

    def textDialog(self, title, message, defText):
        dlg = wx.TextEntryDialog(self, message, title, defaultValue=defText)
        ansBtn = dlg.ShowModal()
        ansTxt = dlg.GetValue()
        if ansBtn == wx.ID_OK:
            text = ansTxt
        else:
            text = ""
        return text

#
#  clearBoundaryFields
#    boundaryFieldのクリア
def clearBoundaryFields(caseDir, timeFolder, copyFiles):
    for copyFile in copyFiles:
        words = copyFile.split("/")
        regName = words[-2]
        fieldDir = "/".join(words[:-1])
        fields = [words[-1]]
        relativePolyMeshDir = getFoamContents.getCurrMeshDir(caseDir, timeFolder,
            regName, "boundary")
        polyMeshDir = caseDir + "/" + relativePolyMeshDir
        setFoamContents.clearNullBoundaryFields(fieldDir, polyMeshDir, fields)


#
#  readCsvData
#    csvデータの読み込み
def readCsvData(fileName):
    data = pyTreeFoam.readCsvData(fileName)
    return data[1:]

#
#  checkMaterialDBDir
#    materialDBDirをチェックし、無ければ作成する。
#　　defaultの場所は、「~/OpenFOAM/multiRegionDB/solid」
def checkMaterialDBDir():
    global solidPhysicalDBdir, iniDBdata, OFversion, solidProperties
    #OFversion取得
    config = pyTreeFoam.readConfigTreeFoam()
    OFversion = config[0]
    #solidProperties取得
    if OFversion >= "2.2.0" and OFversion[:3] != "ext":
        solidProperties = "thermophysicalProperties"
    else:
        solidProperties = "solidThermophysicalProperties"
    #現在の設定場所を確認する
    fileName = multiRegion_data
    setDir = ""
    if len(glob.glob(fileName)) != 0:
        f=open(fileName); cont=f.readlines(); f.close()
        for line in cont:
            if line.split()[0] == "solidPhysicalDBdir":
                setDir = line.split()[1]
                break

    iniDBdata = os.environ["TreeFoamPath"] + "/data/multiRegionDB/solid/" + solidProperties
    
    setFolder = setDir.split("/")[-1]
    if (len(glob.glob(setDir)) == 0 or setFolder != solidProperties):
        #materialDBDirが無い場合、defaultの場所に設定する
        homeDir = os.environ["HOME"]
        DBdir = homeDir + "/OpenFOAM"
        if len(glob.glob(DBdir)) == 0:
            os.system("mkdir " + DBdir)
        DBdir += "/multiRegionDB"
        if len(glob.glob(DBdir)) == 0:
            os.system("mkdir " + DBdir)
        DBdir += "/solid"
        if len(glob.glob(DBdir)) == 0:
            os.system("mkdir " + DBdir)
        properties = solidProperties
        #iniDBdata = os.environ["TreeFoamPath"] + "/data/multiRegionDB/solid/" + properties
        DBdir += "/" + properties
        if len(glob.glob(DBdir)) == 0:
            os.system("mkdir " + DBdir)
        #setSolidDirMultiRegion_data(DBdir)
        setDir = DBdir

    if len(glob.glob(setDir + "/*")) == 0:
        sourceDir = iniDBdata
        comm = "cp " + sourceDir + "/* " + setDir + " > /dev/null"
        os.system(comm)
        #書き込めなかった場合、treeFoam内のdirを設定
        if len(glob.glob(setDir + "/*")) == 0:
            setDir = iniDBdata
    setSolidDirMultiRegion_data(setDir)

#
#  initialize
#
def initialize():
    global fluidRegions, solidRegions
    global iniTime, timeFiles, timeFluidFiles, timeSolidFiles
    global constantFiles, constantFluidFiles, constantSolidFiles
    global systemFiles, systemFluidFiles, systemSolidFiles
    global constantSolidPhysical, solidPhysicalDBdir, solidPhysicalDB
    global constantSolidPhysicalReq
    global OFversion, solidProperties
    caseDir = os.getcwd()
    #regionProperties有無チェック
    fileName = caseDir + "/constant/regionProperties"
    if len(glob.glob(fileName)) == 0:
        print _(u"regionPropertiesが存在しません")
        return
    #regionの取得
    [fluidRegions, solidRegions] = getRegionName(caseDir)
    if len(fluidRegions) == 0 or len(solidRegions) == 0:
        return
    #timeFolder取得
    iniTime = getFoamContents.getCurrTimeFolder(caseDir)
    #timeFile取得
    timeDir = caseDir + "/" + iniTime
    checkFluidSolidFolder(timeDir)
    parFiles = pyTreeFoam.getFieldNames(timeDir)
    parFluidFiles = pyTreeFoam.getFieldNames(timeDir + "/fluidRegions")
    parSolidFiles = pyTreeFoam.getFieldNames(timeDir + "/solidRegions")
    timeFiles = makeTreeData(parFluidFiles, parSolidFiles, parFiles)
    timeFluidFiles = pyTreeFoam.getFieldNames(timeDir + "/" + fluidRegions[0])
    timeSolidFiles = pyTreeFoam.getFieldNames(timeDir + "/" + solidRegions[0])
    #constantFile取得
    constDir = caseDir + "/constant"
    checkFluidSolidFolder(constDir)
    [folders, files] = pyTreeFoam.getFoldersFiles(constDir)
    parFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(constDir + "/fluidRegions")
    parFluidFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(constDir + "/solidRegions")
    parSolidFiles = checkFileName(files)
    constantFiles = makeTreeData(parFluidFiles, parSolidFiles, parFiles)
    [folders, files] = pyTreeFoam.getFoldersFiles(constDir + "/" + fluidRegions[0])
    constantFluidFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(constDir + "/" + solidRegions[0])
    constantSolidFiles = checkFileName(files)
    #systemFile取得
    sysDir = caseDir + "/system"
    checkFluidSolidFolder(sysDir)
    [folders, files] = pyTreeFoam.getFoldersFiles(sysDir)
    parFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(sysDir + "/fluidRegions")
    parFluidFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(sysDir + "/solidRegions")
    parSolidFiles = checkFileName(files)
    systemFiles = makeTreeData(parFluidFiles, parSolidFiles, parFiles)
    [folders, files] = pyTreeFoam.getFoldersFiles(sysDir + "/" + fluidRegions[0])
    systemFluidFiles = checkFileName(files)
    [folders, files] = pyTreeFoam.getFoldersFiles(sysDir + "/" + solidRegions[0])
    systemSolidFiles = checkFileName(files)
    #DBdata取得
    solidPhysicalDBdir = getSolidDirMultiRegion_data()
    solidPhysicalDB = getSolidPhysicalDB(solidPhysicalDBdir)
    #各regionのsolidPhysicalFile取得
    constantSolidPhysical = []
    constantSolidPhysicalReq = []
    properties = solidPhysicalDBdir.split("/")[-1]
    for solidRegion in solidRegions:
        fileName = caseDir+"/constant/"+solidRegion+"/"+properties
        if len(glob.glob(fileName)) != 0:
            material = getMaterial(fileName)
        else:
            material = ""
        constantSolidPhysical.append(material)
        constantSolidPhysicalReq.append(material)

#  checkFluidSolidFolder
#    fluid, solid folderをチェックし、無ければ作成
def checkFluidSolidFolder(checkDir):
    if len(glob.glob(checkDir + "/fluidRegions")) == 0:
        comm = "mkdir " + checkDir + "/fluidRegions"
        os.system(comm)
    if len(glob.glob(checkDir + "/solidRegions")) == 0:
        comm = "mkdir " + checkDir + "/solidRegions"
        os.system(comm)

#  makeTreeData
#    TreeDataを作成する
def makeTreeData(fluids, solids, files):
    treeData = []
    treeData.append("root")
    items = []
    items.append(".")
    fluid = ["fluidRegions", fluids]
    solid = ["solidRegions", solids]
    items.append(fluid)
    items.append(solid)
    for name in files:
        items.append(name)
    treeData.append(items)
    return treeData

def getSolidPhysicalDB(solidPhysicalDBdir):
    DBdir = solidPhysicalDBdir
    if len(glob.glob(DBdir)) != 0:
        [folders, fileNames] = pyTreeFoam.getFoldersFiles(DBdir)
        if len(fileNames) == 0:
            #fileが無かった場合、fileをコピーする。
            setDBFilesToDBDir(DBdir)
        files = []
        for fileName in fileNames:
            files.append(fileName.split("/")[-1])
        solidPhysicalDB = checkFileName(files)
        files = []
        for fileName in solidPhysicalDB:
            fileDir = DBdir + "/" + fileName
            if isSolidThermophysicalProperties(fileDir) == True:
                files.append(fileName)
        solidPhysicalDB = checkFileName(files)
        return solidPhysicalDB


def setDBFilesToDBDir(DbDir):
    global iniDBdata
    folder = DbDir.split("/")[-1]
    iniDBdataDir = "/".join(iniDBdata.split("/")[:-1]) + "/" + folder
    comm = "cp " + iniDBdataDir + "/* " + DbDir
    print comm
    os.system(comm)
    
    

def isSolidThermophysicalProperties(fileName):
    f=open(fileName); cont=f.read(); f.close()
    word = fileName.split("/")[-2]
    p = cont.find(word)
    if p>=0:
        return True
    else:
        return False


def getSolidDirMultiRegion_data():
    global solidProperties
    fileName = multiRegion_data
    DBdir = solidPhysicalDBdir
    try:
        f=open(fileName); cont=f.readlines(); f.close()
        for line in cont:
            p = line.find("solidPhysicalDBdir")
            if p>=0:
                DBdir = line.split()[1]
                break
        if len(glob.glob(DBdir)) == 0:
            DBdir = solidPhysicalDBdir
    except:
        DBdir = solidPhysicalDBdir
    #OFversionに合わせDBdirを設定
    wdir = DBdir.split("/")[:-1]
    DBdir = "/".join(wdir) + "/" + solidProperties
    return DBdir

def setSolidDirMultiRegion_data(selDir):
    fileName = multiRegion_data
    if len(glob.glob(fileName)) != 0:
        f=open(fileName); cont=f.readlines(); f.close()
        i = 0
        while i<len(cont):
            p = cont[i].find("solidPhysicalDBdir")
            if p>= 0:
                cont[i] = "solidPhysicalDBdir " + selDir + "\n"
                break
            i += 1
    else:
        cont = ["solidPhysicalDBdir " + selDir + "\n"]
    f=open(fileName, "w")
    for line in cont:
        f.write(line)
    f.close();


def getMaterial(fileName):
    f=open(fileName); cont=f.read(); f.close()
    p = getFoamContents.getKeywordPointerMem("material", cont, 0)
    if p<0:
        material = "-"
    else:
        while p<len(cont) and cont[p] != " ":
            p += 1
        [material, p] = getFoamContents.getKeywordMem(cont, p)
    return material

def setMaterial(fileName, material):
    f=codecs.open(fileName,"r","utf-8"); cont=f.read(); f.close()
    p = cont.find("material")
    if p>=0:
        newCont = changeMaterial(cont, p, material)
    else:
        newCont = insertMaterial(cont, material)
    f=codecs.open(fileName, "w", "utf-8"); f.write(newCont); f.close()

def changeMaterial(cont, p, material):
    ps = p
    [keyword, p] = getFoamContents.getKeywordMem(cont, p)
    [keyword, p] = getFoamContents.getKeywordMem(cont, p)
    pe = getFoamContents.skipUntilCRMem(cont, ps)
    addLine = "material    " + material + ";"
    newCont = cont[:ps] + addLine + cont[pe:]
    return newCont

def insertMaterial(cont, material):
    p = getFoamContents.skipFoamFileMem(cont, 0)
    [keyword, p] = getFoamContents.getKeywordMem(cont, p)
    i = p
    while i >= 0 and cont[i] != "\n":
        i -= 1
    addLine = "\nmaterial    " + material + ";\n\n"
    newCont = cont[:i] + addLine + cont[i+1:]
    return newCont

def getRegionName(caseDir):
    [fluidRegions, solidRegions] = pyTreeFoam.getZonesInRegionProperties(caseDir)
    return [fluidRegions, solidRegions]

def checkFileName(files):
    ans = []
    for name in files:
        if name[0] != "." and name[-1] != "~" and name != "README":
            ans.append(name)
    return ans 


if __name__ == "__main__":
    import gettext
    treeFoamPath = os.environ["TreeFoamPath"]
    gettext.install("treefoam", treeFoamPath + "/data/locale")

    #materialDBDirをチェックし、dirが存在しないならば、作成する。
    checkMaterialDBDir()
    #初期化
    initialize()
    #dialog表示
    #app = wx.PySimpleApp(0)
    app = wx.App(0)
    #wx.InitAllImageHandlers()
    frame_1 = MyFrame(None, -1, "")
    app.SetTopWindow(frame_1)
    frame_1.Show()
    app.MainLoop()
