#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   makeRunTopoSet
#
#       topoSetDictを作り、topoSetを実行する
#       カレントcaseより実行
#
#       ＜使い方＞
#       makeSetOrZone <caseDir>
#           caseDir     :case directory
#
#       12/09/25    新規作成
#          12/16    makeCombiContentsを追加
#       13/01/22    setSourceName, setSourceNameWqを修正（「;」が取得できない）
#          01/27    regionToCellを修正
#          02/03    cellToFace:cellSet(Zone)からcell表面のfaceSet(Zone)を抽出を追加
#          08/11    setSourceName:nameがstrかどうかをチェック。strでない場合(list)は、
#                   空文を返す。
#       14/12/06    makeCombiContents:newAddsSetを修正
#          12/07    getSourceInfo:sourceNameがlistの場合、「""」を戻す
#       15/07/01    topoSetDictの内容をOF-2.4.0に合わせる
#          07/02    actionの検索方法を変更、resultTypeに「sets」と「zones」を追加
#          07/03    resultType「sets」「zones」にadd, delete, clear, invert, remove actionを追加
#          10/10    faceSetSourceList:「regionToFace」を追加（OF-2.4.0対応）
#


import sys
import os
import glob
import pyTreeFoam
import getFoamContents

topoSetDB = []      #文字列のDB


#---------set--------
#  resultType:cellSetのsourceリスト
#     [<sourceType>, <検索word>, <sourceName置き換えword>]
def cellSetSourceList():
    sets = [["cellSet", "cellToCell", "set"], ["faceSet", "faceToCell", "set"],
            ["pointSet", "pointToCell", "set"], ["cellZone", "zoneToCell", "name"],
            ["faceZone", "faceZoneToCell", "name"],
            ["surface", "surfaceToCell", "file"], ["label", "labelToCell", ""],
            ["shape", "shapeToCel", ""], ["box", "boxToCell", ""],
            ["cylinder", "cylinderToCell", ""], ["sphere", "sphereToCell", ""],
            ["rotatedBox", "rotatedBoxToCell", ""],
            ["targetVolume", "targetVolumeToCell", ""],
            ["nearest", "nearestToCell", ""], ["field", "fieldToCell", ""],
            ["region", "regionToCell", "set"]]
    return sets

#  resultType:faceSetのsourceリスト
def faceSetSourceList():
    sets = [["cellSet", "cellToFace", "set"], ["faceSet", "faceToFace", "set"],
            ["pointSet", "pointToFace", "set"],
            ["faceZone", "zoneToFace", "name"], ["label", "labelToFace", ""],
            ["box", "boxToFace", ""],
            ["patch", "patchToFace", "name"], ["normal", "normalToFace", ""],
            ["boundary", "boundaryToFace", ""],
            ["region", "regionToFace", "set"]]
    return sets

#  resultType:pointSetのsourceリスト
def pointSetSourceList():
    if pyTreeFoam.readConfigTreeFoam()[0] < "2.1":
        sets = [["cellSet", "cellToPoint", "set"], ["faceSet", "faceToPoint", "set"],
                ["pointSet", "pointToPoint", "set"],
                ["cellZone", "zoneToPoint", "name"],
                ["surface", "surfaceToPoint", "file"],
                ["label", "labelToPoint", ""], ["box", "boxToPoint", ""],
                ["nearest", "nearestToPoint", ""]]
    else:
        sets = [["cellSet", "cellToPoint", "set"], ["faceSet", "faceToPoint", "set"],
                ["pointSet", "pointToPoint", "set"],
                ["pointZone", "zoneToPoint", "name"],
                ["surface", "surfaceToPoint", "file"],
                ["label", "labelToPoint", ""], ["box", "boxToPoint", ""],
                ["nearest", "nearestToPoint", ""]]
    return sets

#  resultType:setsのsourceリスト
def setsSourceList():
    sets = [["cellSet", "setToCellZone", "set"], ["faceSet", "setToFaceZone", "faceSet"],
            ["pointSet", "setToPointZone", "set"],
            ["cellZone", "zoneToCell", "name"], ["faceZone", "zoneToFace", "name"],
            ["pointZone", "zoneToPoint", "name"]]
    return sets

#----- zone -------
#  resultType:cellZoneのsourceリスト
def cellZoneSetSourceList():
    sets = [["cellSet", "setToCellZone", "set"]]
    return sets

#  resultType:faceZoneのsourceリスト
def faceZoneSetSourceList():
    sets = [["faceSet", "setToFaceZone", "faceSet"],
            ["searchableSurface", "searchableSurfaceToFaceZone", ""],
            ["sets", "setsToFaceZone", "cellSet", "faceSet"]]
    return sets

#  resultType:pointZoneのsourceリスト
def pointZoneSetSourceList():
    sets = [["pointSet", "setToPointZone", "set"]]
    return sets

#  resultType:zonesのsourceリスト
def zonesSourceList():
    sets = [["cellSet", "setToCellZone", "set"], ["faceSet", "setToFaceZone", "faceSet"],
            ["pointSet", "setToPointZone", "set"]]
    return sets


def getCommFromDB(comm):
    comms = topoSetDB
    #p = comms.find("    source "+comm)
    p = findCommand(comm)
    if p<0:
        return ""

    p = getBforeLineHead(comms, p)
    while comms[p:p+6] == "    //":
        p = getBforeLineHead(comms, p)
    ps = getNextLineHead(comms, p)
    [dummy, p, kind] = getFoamContents.get1lineMem(comms, ps)
    [dummy, p, kind] = getFoamContents.get1lineMem(comms, p)
    commCont = comms[ps:p]
    return commCont

#  findCommand
#    " source " と <comm>の文字列を検索
def findCommand(comm):
    comms = topoSetDB

    def skipSp(p):
        while comms[p] == " ":
            p += 1
        return p

    p = 0
    flag = 0
    while 0 <= p and p < len(comms):
        p = comms.find(" source ", p)
        if p >= 0:
            p += len(" source ")
            p = skipSp(p)
            if comms[p:p+len(comm)] == comm:
                flag = 1
                break
    if p >= len(comms):
        p = -1
    return p


def getBforeLineHead(cont, p):
    p -= 1
    while cont[p] != "\n" and p >= 0:
        p -= 1
    p -= 1
    while cont[p] != "\n" and p >= 0:
        p -= 1
    p += 1
    return p

def getNextLineHead(cont, p):
    p += 1
    while cont[p] != "\n" and p<=len(cont):
        p += 1
    p += 1
    return p




#-----------------------

#  setOption
#    optionの内容を設定
def setOption(cont, optionCont):
    newCont = setSourceName(cont, "option", optionCont)
    return newCont

def setSourceName(cont, comm, name):
    if type(name) == list:
        return ""

    lines = cont.split("\n")
    i=0
    for line in lines:
        [keyword, p] = getFoamContents.getKeywordMem(line, 0)
        if keyword == comm:
            sp = p
            [keyword, ep] = getFoamContents.getKeywordMem(line, sp)
            #次の1行を追加（";"が取得出来ないため）13/01/22
            ep = ep-1
            #次のif文を削除　12/01/22
            #if ep < len(line):
            #    ep = ep-1
            #newline = line[:p] + " " + name + line[ep-1:]
            #keywordの内容により処理変更
            if keyword[0] == '"':
                newline = line[:p] + ' "' + name + '" ' + line[ep:]
            else:
                newline = line[:p] + " " + name + line[ep:]
            lines[i] = newline
            break
        i+=1
    newCont = "\n".join(lines)
    return newCont

#---------------------
def add4Spaces(cont):
    lines = cont.split("\n")
    i=0
    for line in lines:
        lines[i] = "    " + line
        i+=1
    cont = "\n".join(lines)
    return cont


#
#  makeCombiContents
#    組み合わせActionの内容を作成
def makeCombiContents(sourceType, sourceName, action, resultType, resultName):

    #rename　SetとZoneの名称変更
    if action == "renameSetZone":
        if sourceType.find("Zone") >= 0:
            if sourceType == "cellZone":
                #cont  = makeContents(sourceType, sourceName, "new",
                #        "cellSet", resultName)
                cont  = makeContents(sourceType, sourceName, "new",
                        "cellSet", "temp")
                #cont += makeContents("cellSet", resultName, "new",
                #        "cellZoneSet", resultName)
                cont += makeContents("cellSet", "temp", "new",
                        "cellZoneSet", resultName)
                cont += makeContents("cellSet", sourceName, "remove",
                        "cellZoneSet", sourceName)
                cont += makeContents("cellSet", "temp", "remove",
                        "cellZoneSet", "temp")
            else:
                cont  = makeContents(sourceType, sourceName, "new",
                        "faceSet", resultName)
                cont += makeContents("faceSet", sourceName, "new",
                        "faceZoneSet", resultName)
                cont += makeContents("faceSet", sourceName, "remove",
                        "faceZoneSet", sourceName)
        elif sourceType.find("Set") >= 0:
            cont  = makeContents(sourceType, sourceName, "new",
                        sourceType, resultName)
            cont += makeContents(sourceType, sourceName, "remove",
                        sourceType, sourceName)
        else:
            cont = u""
        return cont

    #newAdds　cellSet(faceSet)を複数加え、新しい名称をつける
    elif action == "newAddsSet":
        if sourceType == "faceSet":
            #newAddFace
            if type(sourceName) == str:
                #one sourceName
                cont  = makeContents(sourceType, sourceName, "new",
                            "faceSet", resultName)
                return cont
            else:
                #multi sourceName
                cont  = makeContents(sourceType, sourceName[0], "new",
                            "faceSet", resultName)
                i=1
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "add",
                            "faceSet", resultName)
                    i+=1
            return cont

        elif sourceType == "cellSet":
            #newAddCell
            if type(sourceName) == str:
                #one sourceName
                cont  = makeContents(sourceType, sourceName, "new",
                            "cellSet", resultName)
                return cont
            else:
                #multi sourceName
                cont  = makeContents(sourceType, sourceName[0], "new",
                            "cellSet", resultName)
                i=1
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "add",
                            "cellSet", resultName)
                    i+=1
            return cont
        else:
            return ""

    #newInvert　cellSetの反転をnewNameでつくる
    elif action == "newInvertSet":
        if sourceType == "cellSet":
            cont  = makeContents(sourceType, sourceName, "new",
                        "cellSet", resultName)
            cont += makeContents(sourceType, resultName, "invert",
                        "cellSet", resultName)
        else:
            cont - ""
        return cont

    #newCellToFace  cellSet（Zone）からcell表面のfaceSet（Zone）を作る
    elif action == "newCellToFace":
        if sourceType == "cellSet":
            cont = makeContentsCellToFace(sourceType, sourceName, action,
                    resultType, resultName)
        elif sourceType == "cellZone":
            cont = makeContents(sourceType, sourceName, "new", "cellSet",
                    sourceName)
            cont += makeContentsCellToFace("cellSet", sourceName, action,
                    resultType, resultName)
            cont += makeContents("faceSet", resultName, "new", "faceZoneSet",
                    resultName)
        return cont
    #newZonesToSets:cellZone(faceZone)からcellSet(faceSet)を作る
    elif action == "newZonesToSets":
        if sourceType == "cellZone":
            if type(sourceName) != list:
                #one sourceName(string)
                cont = makeContents(sourceType, sourceName, "new", "cellSet",
                        sourceName)
            else:
                #multi sourceName(list)
                cont = ""
                i = 0
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "new", "cellSet",
                            sourceName[i])
                    i += 1
            return cont
        elif sourceType == "faceZone":
            if type(sourceName) != list:
                #one sourceName(string)
                cont = makeContents(sourceType, sourceName, "new", "faceSet",
                        sourceName)
            else:
                #multi sourceName(list)
                cont = ""
                i = 0
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "new", "faceSet",
                            sourceName[i])
                    i += 1
            return cont
    else:
        return ""


def makeContentsCellToFace(sourceType, sourceName, action, resultType, resultName):
    Tcont  = makeContents(sourceType, sourceName, "new", "faceSet", "temp")
    cont = setOption(Tcont, "both")
    Tcont = makeContents(sourceType, sourceName, "new", "faceSet", resultName)
    cont += setOption(Tcont, "all")
    cont += makeContents("faceSet", "temp", "delete", "faceSet", resultName)
    cont += makeContents(sourceType, sourceName, "remove", "faceSet", "temp")
    return cont

#
#  makeSetsZonesContents
#    sets or zonesの繰り返しのcontentsを作成
def makeSetsZonesContents(sourceType, sourceName, action, resultType, resultName):
    contents = ""
    if action == "new":
        #action new
        contents = makeNewSetsZonesContents(sourceType, sourceName, action,
                        resultType, resultName)
    elif action == "add" or action == "delete":
        #action add, delete
        contents = makeAddDeleteSetsZonesContents(sourceType, sourceName, action,
                        resultType, resultName)
    elif action == "remove" or action == "clear" or action == "invert":
        #action remove, clear, invert
        contents = makeRemoveSetsZonesContents(sourceType, sourceName, action,
                        resultType, resultName)
    return contents

#
#  makeNewSetsZonesContents
#    action new
def makeNewSetsZonesContents(sourceType, sourceName, action, resultType, resultName):
    #resultNameを取得（sourceNameと同じ）
    if type(sourceName) == str or type(sourceName) == unicode:
        names = [sourceName]
    else:
        names = sourceName
    #resultTypeを取得
    resultType = changeResultTypeFromSourceType(sourceType)
    #内容を取得
    footer = makeContentsFooter()
    contents = ""
    for name in names:
        contU = makeContentsHeader(action, sourceType, resultType, name)
        cont = getSourceInfo(sourceType, name, action, resultType)
        if cont != "":
            cont = add4Spaces(cont)
        contents += contU + cont + footer
    return contents

#
#  makeAddDeleteSetsZonesContents
#    action add, delete
def makeAddDeleteSetsZonesContents(sourceType, sourceName, action, resultType, resultName):
    #resultNameを取得（sourceNameと同じ）
    if type(sourceName) == str or type(sourceName) == unicode:
        names = [sourceName]
    else:
        names = sourceName
    #resultTypeを取得
    resultType = getResultTypeFromSourceType(sourceType, resultType)
    #内容を取得
    footer = makeContentsFooter()
    contents = ""
    for name in names:
        contU = makeContentsHeader(action, sourceType, resultType, resultName)
        cont = getSourceInfo(sourceType, name, action, resultType)
        if cont != "":
            cont = add4Spaces(cont)
        contents += contU + cont + footer
    return contents


#  changeResultTypeFromSourceType
#    sourceTypeからresultTypeを特定する（反対のtypeを取得）
def changeResultTypeFromSourceType(sourceType):
    name = ""
    if sourceType[-3:] == "Set":
        name = sourceType[:-3] + "ZoneSet"
    elif sourceType[-4:] == "Zone":
        name = sourceType[:-4] + "Set"
    return name

#  getResultTypeFromSourceType
#    sourceTypeからresultTypeを特定する（同じtypeを取得）
def getResultTypeFromSourceType(sourceType, resultType):
    name = ""
    if resultType == "sets":
        if sourceType[-3:] == "Set":
            name = sourceType
        elif sourceType[-4:] == "Zone":
            name = sourceType[:-4] + "Set"
    elif resultType == "zones":
        if sourceType[-3:] == "Set":
            name = sourceType[:-3] + "ZoneSet"
        elif sourceType[:-4] == "Zone":
            name = sourceType + "Set"
    return name

#
#  makeRemoveSetsZonesContents
#    action remove, clear, invert
def makeRemoveSetsZonesContents(sourceType, sourceName, action, resultType, resultName):
    #nameを取得
    cellNames = []
    faceNames = []
    pointNames = []
    for name in resultName:
        words = name.split(":")
        if words[0] == "c":
            cellNames.append(words[1])
        elif words[0] == "f":
            faceNames.append(words[1])
        elif words[0] == "p":
            pointNames.append(words[1])
    #内容を取得
    contents = ""
    footer = makeContentsFooter()
    if resultType == "sets":
        nType = "Set"
    elif resultType == "zones":
        nType = "ZoneSet"
    #cellNameを作成
    if len(cellNames) > 0:
        for name in cellNames:
            contU = makeContentsHeader(action, sourceType, "cell"+nType, name)
            contents += contU + footer
    #faceNameを作成
    if len(faceNames) > 0:
        for name in faceNames:
            contU = makeContentsHeader(action, sourceType, "face"+nType, name)
            contents += contU + footer
    #pointNameを作成
    if len(pointNames) > 0:
        for name in pointNames:
            contU = makeContentsHeader(action, sourceType, "point"+nType, name)
            contents += contU + footer
    return contents

#
#  makeContents
#    処理内容を作成
def makeContents(sourceType, sourceName, action, resultType, resultName):
    contU = makeContentsHeader(action, sourceType, resultType, resultName)
    cont = getSourceInfo(sourceType, sourceName, action, resultType)
    if cont != "":
        cont = add4Spaces(cont)
    footer = makeContentsFooter()
    contents = contU + cont + footer
    return contents

#  makeContentsHeader
#    処理内容のheader部作成
def makeContentsHeader(action, sourceType, resultType, resultName):
    contU  = "\n    // " + action + " To " + resultType + "\n"
    contU += "    {\n"
    contU += "        name    " + resultName + ";\n"
    contU += "        type    " + resultType + ";\n"
    contU += "        action  " + action + ";\n"
    return contU    

#  makeContentsFooter
#    処理のfooterを作成
def makeContentsFooter():
    cont = "\n    }\n"
    return cont

#
#  getSourceInfo
#  -----------------
#    sourceのoptionを取得
def getSourceInfo(sourceType, sourceName, action, resultType):

    #  source内容を取得
    #    <DB内容の例>
    #       // Copy elements from cellSet
    #       source cellToCell;
    #       sourceInfo
    #       {
    #           set c1;
    #       }
    def searchNameGetCont(resultSourceList):
        cont = ""
        for sourceList in resultSourceList:
            if sourceList[0] == sourceType:
                searchName = sourceList[1]
                keyName = sourceList[2]
                cont = getCommFromDB(searchName)
                if keyName != "":
                    cont = setSourceName(cont, keyName, sourceName)
                break
        return cont

    #  sourceType:setsの内容を取得
    def searchSetsCFNameGetCont():
        cont = getCommFromDB("sets")
        if len(sourceName) == 2:
            for name in sourceName:
                setName = name.split(":")
                if len(setName) == 2:
                    if setName[0] == "c":
                        cont = setSourceName(cont, "cellSet", setName[1])
                    elif setName[0] == "f":
                        cont = setSourceName(cont, "faceSet", setName[1])
        return cont

    #no source action?
    if action == "clear" or action == "invert" or action == "remove":
        return ""

    #combind or sets action?
    cont = ""
    if type(sourceName) == list:
        if sourceType == "sets":
            cont = searchSetsCFNameGetCont()
        return cont

    #sourceの内容を取得
    cont = ""
    if resultType == "cellSet":
        cont = searchNameGetCont(cellSetSourceList())
    elif resultType == "faceSet":
        cont = searchNameGetCont(faceSetSourceList())
    elif resultType == "pointSet":
        cont = searchNameGetCont(pointSetSourceList())
    elif resultType == "cellZoneSet":
        cont = searchNameGetCont(cellZoneSetSourceList())
    elif resultType == "faceZoneSet":
        cont = searchNameGetCont(faceZoneSetSourceList())
    elif resultType == "pointZoneSet":
        cont = searchNameGetCont(pointZoneSetSourceList())
    elif resultType == "sets":
        cont = searchNameGetCont(setsSourceList())
    elif resultType == "zones":
        cont = searchNameGetCont(zonesSourceList())
    return cont


#  makeHeader
#    topoSetDictのheader作成
def makeHeader():
    header = """/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\    /   O peration     | Version:  2.1.0                                 |
|   \\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\/     M anipulation  |                                                 |
\\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       dictionary;
    object      topoSetDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
"""
    return header

#  makeFooter
#
def makeFooter():
    footer = """// ************************************************************************* //"""
    return footer

#
#  addTopoSetDict
#    topoSetDictに項目を追加して保存
def addTopoSetDict(caseDir, cont):
    fileName = caseDir + "/system/topoSetDict"
    f=open(fileName)
    topoSetDict = f.read()
    f.close()
    p = getFoamContents.skipFoamFileMem(topoSetDict, 0)
    p = getFoamContents.getKeywordPointerMem("actions", topoSetDict, p)
    sp = p
    [actions, p] = getFoamContents.getSmallPairMem(topoSetDict, p)
    ep = p
    actions += topoSetDict[sp+1:ep-1] + cont
    newDict = topoSetDict[:sp] + "actions\n(" + actions + ")" + topoSetDict[ep:]
    f=open(fileName, "w")
    f.write(newDict)
    f.close()


#
#  saveTopoSetDict
#    topoSetDictをsystemフォルダに保存
def saveTopoSetDict(caseDir, topoSetDict):
    fileName = caseDir + "/system/topoSetDict"
    f=open(fileName, "w")
    f.write(topoSetDict)
    f.close()

#
#  runTopoSet
#    topoSetの実行
def runTopoSet():
    comm = "topoSet"
    pyTreeFoam.runOFUtility(comm)


#-------topoSetDB作成-----------------------
#
#  getTopoSetDB
#    topoSetのコマンドを文字列で取得
def getTopoSetCommandDB():
    comm = "echo $FOAM_APP"
    [stat, cont] = pyTreeFoam.runOFUtilityReturnStat(comm)
    #fileName = os.getenv("TreeFoamPath") + "/temp/temp.txt"
    fileName = os.getenv("TreeFoamUserPath") + "/temp/temp.txt"
    f=open(fileName); cont=f.read(); f.close()
    appDir = cont[:-1]
    topoSetDict = appDir + "/utilities/mesh/manipulation/topoSet/topoSetDict"
    if len(glob.glob(topoSetDict)) == 0:
        print "does not exist topoSet in " + appDir + "."
        return ""
    f=open(topoSetDict); lines=f.readlines(); f.close()
    topoSetCont = []
    for line in lines:
        if len(line)>2:
            if line[:2] == "//":
                topoSetCont.append(line[2:])
    #コマンド取得
    cellSet = getSubCommand("cellSet", topoSetCont)
    faceSet = getSubCommand("faceSet", topoSetCont)
    pointSet = getSubCommand("pointSet", topoSetCont)
    cellZoneSet = getSubCommand("cellZoneSet", topoSetCont)
    faceZoneSet = getSubCommand("faceZoneSet", topoSetCont)
    pointZoneSet = getSubCommand("pointZoneSet", topoSetCont)
    return cellSet + faceSet+ pointSet+ cellZoneSet+ faceZoneSet+ pointZoneSet

#  getSubCommand
#    subCommandを取得
def getSubCommand(comm, cont):
    i = 0
    while i<len(cont):
        line = cont[i]
        if len(line) > len(" "+comm):
            if line[:len(" "+comm)] == " "+comm:
                i += 1
                break
        i += 1
    #commを取得
    comms = []
    while i<len(cont):
        line = cont[i]
        if len(line) > 2:
            if line[1] == " ":
                comms.append(line)
            else:
                if ((line.find("cell") >= 0 and line.find("cell") < 3)
                    or (line.find("face") >= 0 and line.find("face") < 3)
                    or (line.find("point") >= 0 and line.find("point") < 3)):
                    break
        i += 1
    return "".join(comms) 

#DB作成
topoSetDB = getTopoSetCommandDB()
#-------------------------------------------------



#
#  main
#
if __name__ == "__main__":
    caseDir = sys.argv[1]
    os.chdir(caseDir)

    topoSetDict = makeHeader()
    topoSetDict += "\naction\n(\n"
    sourceName = "fineZone"
    sourceType = "cellSet"
    action = "add"
    resultName = "fineZone-1"
    resultType = "cellZone"

