#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   loadRegionsCheckFieldsFields.py
#
#   multiRegion用
#       全regionを読み込み、全fieldをチェックする
#
#   14/06/29    新規作成
#   16/09/08    paraviewの読み込みにPVFoamReaderを追加（paraview5対応）
#

import glob
import os

try: paraview.simple
except: from paraview.simple import *

paraview.simple._DisableFirstRenderCameraReset()

#
#  timeFolder取得
#
def getTimeFolders(caseDir):
    dirs = glob.glob(caseDir + "/*")
    folders = []
    for folder in dirs:
        folders.append(folder.split("/")[-1])
    time=[]
    for name in folders:
        try:
            a=float(name)
            time.append([a, name])
        except:
            a=0
    time.sort()
    timeFolders=[]
    for name in time:
        timeFolders.append(name[1])
    return timeFolders

#
#  region名取得
#
def getRegName(fileName):
    rightW = fileName.split("{")
    if len(rightW) == 2:
        midW = rightW[1].split("}")
        if len(midW) == 2:
            regName = midW[0]
        else:
            regName = ""
    else:
        regName = ""
    return regName

#
#  field名取得
#
def getFieldNames(wdir):
    fields = []
    files = glob.glob(wdir+"/*")
    for name in files:
        if os.path.isfile(name) == True:
            fields.append(name.split("/")[-1])
    fields.sort()
    return fields

#regionのみの空file名を取得
fileNames = glob.glob("*.OpenFOAM") + glob.glob("*.foam")
foamNames = []
for name in fileNames:
    regName = getRegName(name)
    if regName != "":
        foamNames.append(name)
foamNames.sort()

#paraViewが表示しているTimeを取得
caseDir = os.getcwd()
timeFolders = getTimeFolders(caseDir)
timeFolder = timeFolders[0]
view = GetActiveView()
viewTime = view.ViewTime
print "viewTime=", viewTime
for time in timeFolders:
    if float(time) == viewTime:
        timeFolder = time
        break

#region毎に設定
for name in foamNames:
    #表示Time内のfield名を取得
    regName = getRegName(name)
    checkFields = getFieldNames("./" + timeFolder + "/" + regName)
    print regName, checkFields, " were checked."
    if len(checkFields) == 0:
        checkFields = ["p"]

    #regionファイル読み込み
    try:
        openfoam = PV4FoamReader(FileName = name)       #paraview ver-4
    except:
        openfoam = PVFoamReader(FileName = name)        #paraview ver-5
    openfoam.UiRefresh = 0
    #pipeLineの表示名をfile名に変更
    RenameSource(name)
    #取得したfield名にチェックする
    openfoam.VolumeFields = checkFields

    openfoam.MeshParts = ['internalMesh']
    DataRepresentation0 = Show()
    DataRepresentation0.ExtractedBlockIndex = 2
print "end"

