#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   module of pyMehsOperation.py
#
#       13/04/21    新規作成（pyOFLauncherから抜きだし）
#          09/22    バグ修正
#          09/24    モデルサイズをbinaryに対応
#          10/15    国際化のため修正
#          10/27    mesh.unvを任意の場所に変更。magFileの場所をconstantに変更
#          12/21    stdout、stderrの設定（import logFileCreater）を追加
#
#

import os
import glob
import getFoamContents
import setFoamContents
import pyTreeFoam
import OKDialog
import MakePatchDialog
import ScaleOKDialog
import struct

import logFileCreater; logFileCreater.log()


#ideasUnvToFoam
def ideasUnvToFoam(modelDir):
    files = glob.glob(modelDir + "/mesh.unv")
    if len(files) == 0:
        title = _(u"ideasUnvToFoam 変換エラー")
        mess = _(u"「mesh.unv」ファイルがありません。")
        img = "error"
        OKDialog.Show(title, img, mess)
        return
    command = 'ideasUnvToFoam ' + modelDir + '/mesh.unv 2> ' + modelDir + '/error.txt | tee ' + modelDir + '/meshconv.map'
    pyTreeFoam.runOFUtility(command)
    n=[]
    #f = open("./model/error.txt")
    f = open(modelDir + "/error.txt")
    for line in f.readlines():
        n.append(line)
    f.close
    if len(n) != 0:
        mess =_(u"\n変換エラーが発生しました。\n  エラーの内容は、「error.txt」を確認してください。\n")
        print mess
        title = _(u"ideasUnvToFoam 変換エラー")
        img = "error"
        OKDialog.Show(title, img, mess)
    else:
        clearInternalBoundary()
        setMagnification(modelDir, "1.0")           #モデルの倍率をセット

#unv2gmshToFoam
def unv2gmshToFoam(modelDir):
    files = glob.glob(modelDir + "/mesh.unv")
    if len(files) == 0:
        title = _("unv2gmshToFoam 変換エラー")
        mess = _(u"「mesh.unv」ファイルがありません。")
        img = "error"
        OKDialog.Show(title, img, mess)
        return
    unvFile = " " + modelDir + '/mesh.unv'
    gmshFile = " " + modelDir + '/meshconv'
    gmshLogFile = " " + modelDir + '/meshconv.map.map'
    command = "rm ./constant/polyMesh/sets/*"       #setsフォルダ内をクリア
    os.system(command)
    command = 'python $TreeFoamPath/bin/unv2gmsh.py' + unvFile + gmshFile
    #os.system(command)
    pyTreeFoam.runCommand(command)
    command = 'gmshToFoam' + gmshFile + '.msh | tee ' + gmshLogFile
    pyTreeFoam.runOFUtility(command)
    print _(u"unv2gmshToFoam メッシュ変換は、完了しました。")
    print _(u"<unv2gmshToFoamSmooth>を実行します。")
    command = "unv2gmshToFoamSmooth " + modelDir
    pyTreeFoam.runOFUtility(command)
    n = []      #エラーチェック
    f = open(modelDir + "/error.txt")
    for line in f.readlines():
        n.append(line)
    f.close
    if len(n) != 0:
        eFlag = 1
        title = _(u"unv2gmshToFoam変換\n")
        message = _(u"変換エラーが発生しました。")
        for line in n:
            message = message + line
        img = "error"
        OKDialog.Show(title, img, message)
    else:
        clearInternalBoundary()
        setMagnification(modelDir, "1.0")           #モデルの倍率をセット

def clearInternalBoundary():
    print _(u"internalFieldをクリアします。")
    #変数を設定
    caseDir = os.getcwd()
    currTime = getFoamContents.getCurrTimeFolder(caseDir)
    fieldDir = caseDir + "/" + currTime
    fields = pyTreeFoam.getFieldNames(caseDir + "/" + currTime)
    boundaryDir = getFoamContents.getCurrMeshDir(caseDir, currTime, ".", "boundary")
    polyMeshDir = "/".join(boundaryDir.split("/")[:-1])
    #internalFieldをクリア
    setFoamContents.clearInternalFields(fieldDir, polyMeshDir, fields)
    print _(u"空のboundaryFieldをクリアします。")
    #空のboundaryFieldをクリアする
    setFoamContents.clearNullBoundaryFields(fieldDir, polyMeshDir, fields)

#
#  changeMagScale
#    スケールの変更
def changeMagScale(modelDir):
    #モデルの倍率をセット
    makeMagFile(modelDir, "1.0")
    message = _(u"unv2gmshToFoam の処理が終了しました。\n  引き続きモデルのスケールを変更しますか？")
    XYZ = getModelSize()
    modelSize = str(XYZ[0]) + ' x ' + str(XYZ[1]) + ' x ' + str(XYZ[2])
    print "modelSize =", modelSize        
    ScaleOKDialog.Show(message, modelSize)


#makePatch（内部patch作成）
def makePatch():
    patches = getInternalPatchName()
    MakePatchDialog.Show(patches)

def getInternalPatchName():
    ans = []
    files = glob.glob("./constant/polyMesh/sets/*")
    faceSets = []
    for name in files:
        fileName = name.split("/")[-1]
        if (fileName[-1] != "~"
          and fileName != "faceZone_Default"
          and fileName != "Default"):
            fileClass = getFoamFileClass("./constant/polyMesh/sets", fileName)
            if fileClass == "faceSet":
                faceSets.append(fileName)
    return faceSets


def setMagnification(modelDir, mag):
    makeMagFile(modelDir, mag)

#magFile作成
def makeMagFile(modelDir, mag):
    wdir = os.getcwd()
    magFile = wdir + "/constant/magnification"
    cont = u"#\n#  model scale\n#\n\n" + mag + "\n"
    f=open(magFile, "w")
    f.write(cont)
    f.close()


# モデルの大きさをセット
def scaleMesh():
    XYZ = getModelSize()
    modelSize = str(XYZ[0]) + ' x ' + str(XYZ[1]) + ' x ' + str(XYZ[2])
    message = _(u"モデルのスケールを変更します。")
    ScaleOKDialog.Show(message, modelSize)


# モデルのサイズを取得
def getModelSize():
    caseDir = os.getcwd()
    fileName = caseDir + "/constant/polyMesh/points"
    [ascii, data] = getFoamContents.readMeshSeparate(fileName)
    form = getFoamContents.checkBinary(ascii)
    if form == "ascii":
        #ascii dataの場合
        cont = data[0][1]
        p = 0
        xyz = " "
        flag = 0
        while xyz != "" and p<len(cont):
            [xyz, p] = getFoamContents.getSmallPairMem(cont, p)
            if xyz != "":
                xyzList = xyz.split()
                x = float(xyzList[0])
                y = float(xyzList[1])
                z = float(xyzList[2])
                if flag == 0:
                    maxX = x; maxY = y; maxZ = z
                    minX = x; minY = y; minZ = z
                    flag = 1
                if maxX < x:
                    maxX = x
                if maxY < y:
                    maxY = y
                if maxZ < z:
                    maxZ = z
                if minX > x:
                    minX = x
                if minY > y:
                    minY = y
                if minZ > z:
                    minZ = z
        lx = abs(maxX - minX)
        ly = abs(maxY - minY)
        lz = abs(maxZ - minZ)
    else:
        #binary dataの場合
        convData = data[0]
        [nNum, nByte, valType] = getFoamContents.getDataFormat(convData[0][0])
        numData = getFoamContents.structBinData(convData)
        i = 0
        maxx = numData[0]; maxy = numData[1]; maxz = numData[2]
        minx = numData[0]; miny = numData[1]; minz = numData[2]
        while i<len(numData):
            x = numData[i]; y = numData[i+1]; z = numData[i+2]
            if maxx < x:
                maxx = x
            if maxy < y:
                maxy = y
            if maxz < z:
                maxz = z
            if minx > x:
                minx = x
            if miny > y:
                miny = y
            if minz > z:
                minz = z
            i += 3
        lx = abs(maxx - minx)
        ly = abs(maxy - miny)
        lz = abs(maxz - minz)
    return [lx, ly, lz]

