#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   pyTreeFoam.py
#
#    module of treefoam.py
#
#       11/04/18    新規作成
#       12/01/01    replaceBoundaryFieldを修正（ｺﾒﾝﾄ文に対応）
#          01/04    deletePatch（パッチ削除）を追加
#          01/08    changePatchName（パッチ名の変更）を追加
#          01/15    delpatchを修正（最初のpatch削除ができなかった）
#          04/04    replacePatchDataを修正（patchデータの最初の行が"\n"の場合削除）
#          07/20    changePatchNameを修正（最初のpatchNameの場合、変更できなかった）
#          09/12    readConfigTreeFoamを追加
#          09/17    compressSPCRを修正（listが1行の場合ｴﾗｰが発生）
#          09/22    gridEditorの非表示設定関係のmodule追加
#          10/29    FFlistの取得を追加
#          11/07    空patch追加のルーチン（addPtchAtBoundaryField）を追加
#       13/01/11    changePatchNameを修正（patch名がpatchTypeと同じ名前の場合ｴﾗｰ）
#          03/04    makeDefineDataを修正（patch内容にpatch名が含まれるとｴﾗｰ）
#          03/16    regionPropertiesの内容取得を追加
#          04/14    checkMultiRegionのバグ修正
#          04/18    makeNameListFile:cellZoneNamesFの高速検索に変更
#          04/19    addPatchAtBoundaryField:バグ取り（fieldが無い時、ｴﾗｰ発生）
#          05/20    getZonesInRegionProperties:OF-2.2.0に対応
#          06/10    makeDefineData:nonuniform形式を非表示にしたので、元々あれば追加。
#                   isMatchName:patternチェックを追加
#          06/26    changePatchNameInBoundaryFieldのﾊﾞｸﾞ修正（";"文字たまに欠落）
#          07/11    checkMultiRegion:regionをcurrTimeFolderから探す様に修正
#          09/07    fieldの圧縮ファイルに対応する為、fieldの読込を修正
#          09/18    checkMultiRegionの検索方法を変更
#          09/19    delPatchBoundaryFieldのバグ（削除後、\nが1ヶ無くなる）
#          10/04    getSaveDisplayFieldsFileNameのバグ（設定したdirがない場合あり）
#          10/15    国際化のため修正
#          11/15    modelフォルダ場所が変更できるようにtextBox追加
#          12/21    stdout、stderrの設定（import logFileCreater）を追加
#          12/28    openFolderNautilus:windowをwmctrlで制御
#       14/06/04    deleteNullPatchesを追加
#          07/10    checkMultiRegion:regNamesをsortして戻す様に修正
#          08/23    IsOldPatchNameInBoundary:ｴﾗｰﾒｯｾｰｼﾞを修正
#          09/22    addPatchAtBoundaryField:バグ修正（「\n」を追加）
#          10/01    isMatchName:reﾓｼﾞｭｰﾙでmatchするかどうかを確認
#          10/23    replacePatchData:patchContがnonuniform形式の場合、ｴﾗｰ発生を修正
#          11/07    readCsvData:追加
#       15/02/04    selectNeedFiles:追加
#          02/14    selectNeedFolders:追加
#          07/13    convAbsDir:環境変数が使えるように修正。tempFileの場所を変更
#       16/03/03    readSshfsDataを追加
#          05/26    readSshfsDataを修正。（コメント文の削除を追加）
#          08/28    getTreeFoamVersionを追加
#          10/31    getTreeFoamServerResult:追加
#          12/31    getContentsInRegionProperties:OF-ext用に修正
#

import os
import re
import glob
import dircache
import string
import csv
import getFoamContents
import setFoamContents
from editBoundaryConditionsDialog import readConfigData as readGridEditorConfig

import logFileCreater; logFileCreater.log()

#
#  selectNeedFolders
#    folderを要・不要に分ける
#    timeFolderの場合は、minTimeのみ要、それ以外を不要
def selectNeedFolders(names):
    unneedFolders = ["processor.*", ".*\.log.*", "log\..*"]
    [unmatchNames, unneed] = selectNeedNames(unneedFolders, names)
    numFolders = []
    need = []
    for name in unmatchNames:
        try:
            a = float(name)
            numFolders.append([a, name])
        except:
            need.append(name)
    if len(numFolders) > 0:
        numFolders.sort()
        need.append(numFolders[0][1])
        i = 1
        while i<len(numFolders):
            [a, name] = numFolders[i]
            unneed.append(name)
            i += 1
    return [need, unneed]

#
#  selectNeedFiles
#    fileを要、不要に分ける
def selectNeedFiles(names):
    unneedFiles = ["log\..*|.*\.log", ".*~",
        ".*\.bak|.*\.png|.*\.obj|.*\.jpg|.*\.OpenFOAM|.*\.foam|.*\.blockMesh",
        "test.*|.*test", "run|plotWatcher", "Allrun.*|Allclean.*|README.*"]
    return selectNeedNames(unneedFiles, names)

#  selectNeedNames
def selectNeedNames(unneedNames, names):

    def isMatch(name):
        flag = False
        for pattern in unneedNames:
            if isMatchName(name, pattern) == True:
                flag = True
                break
        return flag

    need = []
    unneed = []
    for name in names:
        if isMatch(name) == True:
            unneed.append(name)
        else:
            need.append(name)
    return [need, unneed]

#
#  readCsvData
#    csvデータの読み込み a☓bのtableDataを読み込む
def readCsvData(fileName):
    #tab区切りorカンマ区切りのチェック
    #  1行読み込み
    f=open(fileName); line=f.readline(); f.close()
    if len(line.split("\t")) != 1:
        #tab区切り
        f=open(fileName); tabCont=f.readlines(); f.close()
        contents = []
        for line in tabCont:
            contents.append(",".join(line.split("\t")))
        cont = csv.reader(contents)
    else:
        #カンマ区切り
        cont = csv.reader(file(fileName))
    data = []
    for row in cont:
        if len(row) > 0:
            data.append(row)
    return data

#
#  isMatchName
#    nameがpatternと一致するかどうかをチェック
#    一致：Trueが戻る
def isMatchName(name, pattern):
    if pattern[0] == '"':
        pattern = pattern[1:-1]
    ans = False
    obj = re.match(pattern, name)   #正規表現でチェック
    if obj != None:
        ans = True
    return ans

#
#  checkMultiRegion
#    multiRegionのcaseかどうかをチェック
#    戻り値：[<bool>, location, regionNames]
#      location:polyMeshの親folder名を返す
def checkMultiRegion(caseDir):
    regions = []
    #polyMesh/boundaryの場所を取得
    saveCurrDir = os.getcwd()
    os.chdir(caseDir)
    regMeshDirs = glob.glob("*/*/polyMesh/boundary")
    os.chdir(saveCurrDir)
    #timeFolderを確認
    currTime = getFoamContents.getCurrTimeFolder(caseDir)
    timeMeshes = []
    location = ""
    for meshDir in regMeshDirs:
        time = meshDir.split("/")[0]
        try:
            a = float(time)
            timeMeshes.append(meshDir)
        except:
            pass
    if len(timeMeshes) != 0:
        #locationを取得
        timeMeshes.reverse()
        for timeMesh in timeMeshes:
            time = timeMesh.split("/")[0]
            if time <= currTime:
                location = time
        #regionを取得
        for timeMesh in timeMeshes:
            if timeMesh.split("/")[0] == location:
                regions.append(timeMesh.split("/")[1])
    #constantを確認
    if len(regions) == 0:
        for meshDir in regMeshDirs:
            location = "constant"
            if meshDir.split("/")[0] == location:
                regions.append(meshDir.split("/")[1])
    #戻り値を設定
    ans = False
    regNames = []
    if len(regions) != 0:
        ans = True
        regNames = regions
        regNames.sort()
        print _(u"folder「") + location + _(u"」内に "), regNames, _(u"のregionがあります。")
    return [ans, location, regNames]

#
#  getZonesInRegionProperties
#    regionPropertiesのzone名を取得
#    戻り値：[zones]
def getZonesInRegionProperties(caseDir):
    [fluidRegions, solidRegions] = getContentsInRegionProperties(caseDir)
    zones = [fluidRegions, solidRegions]
    return zones

#
#  getContentsInRegionProperties
#    regionPropertiesの内容を取得
#    戻り値：[[fluidRegions], [solidRegions]]
def getContentsInRegionProperties(caseDir):
    fluidRegions = []
    solidRegions = []
    fileName = caseDir + "/constant/regionProperties"
    if len(glob.glob(fileName)) == 0:
        return [fluidRegions, solidRegions]

    f=open(fileName); cont=f.read(); f.close()
    #OFversionチェック
    config = readConfigTreeFoam()
    OFversion = config[0]
    if OFversion >= "2.2.0" and OFversion[:3] != "ext":
        #version 2.2.0以降
        p = getFoamContents.skipFoamFileMem(cont, 0)
        #regionsを取得
        [keyword, p] = getFoamContents.getKeywordMem(cont, p)
        while (keyword != "" and keyword != "regions"):
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
        if keyword == "regions":
            [contents, p] = getFoamContents.getSmallPairMem(cont, p)
            #fluidを取得
            p = 0
            [keyword, p] = getFoamContents.getKeywordMem(contents, p)
            while (keyword != "" and keyword != "fluid"):
                [keyword, p] = getFoamContents.getKeywordMem(contents, p)
            if keyword == "fluid":
                [regionsCont, p] = getFoamContents.getSmallPairMem(contents, p)
                fluidRegions = regionsCont.split()
            #solidを取得
            p = 0
            [keyword, p] = getFoamContents.getKeywordMem(contents, p)
            while (keyword != "" and keyword != "solid"):
                [keyword, p] = getFoamContents.getKeywordMem(contents, p)
            if keyword == "solid":
                [regionsCont, p] = getFoamContents.getSmallPairMem(contents, p)
                solidRegions = regionsCont.split()
    else:
        #fluidRegionsを検索
        p=0
        ps=getFoamContents.skipFoamFileMem(cont, p)
        keyword = " "
        while (keyword != "fluidRegionNames") and (keyword != "") and (p<len(cont)):
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
        if keyword == "fluidRegionNames":
            [dummy, p] = getFoamContents.getSmallPairMem(cont, p)
            fluidRegions = dummy.split()
        else:
            fluidRegions = ""
        #solidRegionsを検索
        p=ps
        keyword = " "
        while (keyword != "solidRegionNames") and (keyword != "") and (p<len(cont)):
            [keyword, p] = getFoamContents.getKeywordMem(cont, p)
        if keyword == "solidRegionNames":
            [dummy, p] = getFoamContents.getSmallPairMem(cont, p)
            solidRegions = dummy.split()
        else:
            solidRegions = ""

    return [fluidRegions, solidRegions]


#
#  addPatchAtBoundaryField
#    boundaryに追加されたnewPatchNameをboundaryFieldに追加し、zeroGradientを書き込む
def addPatchAtBoundaryField(fieldDir, newPatchName):
    [folders, fields] = getFoldersFiles(fieldDir)
    #work = [fields[0]]
    for field in fields:
        fileName = fieldDir + "/" + field
        #f=open(fileName); contents=f.read(); f.close()
        [a1, a2, a3, nMaxLines] = readGridEditorConfig()
        contents = getFoamContents.readField(fileName, nMaxLines)
        p = getFoamContents.getKeywordPointerMem("boundaryField", contents, 0)
        sp = p
        [bCont, p] = getFoamContents.getKeywordContentsMem("boundaryField",
                            contents, p)
        ep = p
        #bCont  = "boundaryField\n{" + bCont
        bCont  = "boundaryField\n{\n" + bCont
        bCont += "    " + newPatchName + "\n"
        bCont += "    {\n"
        bCont += "        type        zeroGradient;\n"
        bCont += "    }\n"
        bCont += "}"
        newContents = contents[:sp] + bCont + contents[ep:]
        #f=open(fileName, "w"); f.write(newContents); f.close()
        getFoamContents.writeField(fileName, newContents)

#
#  changePatchName
#    patch名の変更
#
def changePatchName(boundaryDir, dataDir, fields, oldPatchName, newPatchName):
    #文字種をチェック（使える文字かどうか）
    error = checkPatchName(newPatchName)
    if error[0] != "OK":
        return error
    #oldPatchNameがboundaryFieldにあるかどうかチェック
    error = IsOldPatchNameInBoundary(boundaryDir, dataDir, fields, oldPatchName)
    if error[0] != "OK":
        return error
    error = changePatchNameInBoundary(boundaryDir, oldPatchName, newPatchName)
    if error[0] != "OK":
        return error
    error = changePatchNameInBoundaryField(dataDir, fields, oldPatchName, newPatchName)
    return error

#  checkPatchName
def checkPatchName(newPatchName):
    error = ["OK", ""]
    flag = 0
    for chara in newPatchName:
        if getFoamContents.checkChara(chara) == False:
            flag = 1
            break
    if flag == 0:
        if chara == "/":
            flag = 1
        elif chara == "\\":
            flag = 1
        elif chara == "'" or chara == '"':
            flag = 1
    if flag == 1:
        error = ["NG", _(u"使えない文字「") + chara + _(u"」が使われています")]
    return error

#  IsOldPatchNameInBoundary
def IsOldPatchNameInBoundary(boundaryDir, dataDir, fields, oldPatchName):
    error = ["OK", ""]
    #boundaryのチェック
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #boundary中にoldPatchNameがあるかチェック
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
        while (keyword != "") and (keyword != oldPatchName):
            [keyword, p] = getFoamContents.getKeywordMem(contents, p)
        if keyword == "":
            error = ["NG", _(u"boundary中に「") + oldPatchName + _(u"」が存在しません")]
            return error
    else:
        error = ["NG", fileName+_(u"が存在しません。")]
        return error
    #boundaryFieldのチェック
    for field in fields:
        fileName = dataDir + "/" + field
        #f=open(fileName); contents=f.read(); f.close()
        [a1, a2, a3, nMaxLines] = readGridEditorConfig()
        contents = getFoamContents.readField(fileName, nMaxLines)
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #boundaryFieldを取得
        [boundaryContents, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        #boundaryField中のoldPatchNameがあるかチェック
        [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, 0)
        while (keyword != "") and (keyword != oldPatchName):
            [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, p)
        if keyword == "":
            error = ["NG", _(u"field ") + field + _(u" のboundaryField中に「") + oldPatchName + _(u"」が存在しません")]
            return error
    return error

#  changePatchNameInBoundary
def changePatchNameInBoundary(boundaryDir, oldPatchName, newPatchName):
    error = ["OK", ""]
    newContents = []
    fileName = boundaryDir + "/boundary"
    f=open(fileName); contents=f.read(); f.close()
    #FoamFileをskip
    p = getFoamContents.skipFoamFileMem(contents, 0)
    #oldPatchName検索
    oldP = p
    [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    if keyword[0] <= "9":
        oldP = p
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    if keyword == oldPatchName:
        newContents = (contents[:oldP] +
                        "(\n    " +
                        newPatchName +
                        "\n" +
                        contents[p:])
    [dummy, p] = getFoamContents.getMiddlePairMem(contents, p)
    while (keyword != "") and (keyword != oldPatchName):
        oldP = p
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
        if keyword == oldPatchName:
            newContents = (contents[:oldP] +
                            #"    }\n    " +
                            "\n    " +
                            newPatchName +
                            "\n" +
                            contents[p:])
        else:
            [dummy, p] = getFoamContents.getMiddlePairMem(contents, p)
    if keyword != "":
        f=open(fileName, "w"); f.write(newContents); f.close()
    else:
        error = ["NG", _(u"「") + oldPatchName + _(u"」がboundary中に存在しません")]
    return error

#  changePatchNameInBoundaryField
def changePatchNameInBoundaryField(dataDir, fields, oldPatchName, newPatchName):
    error = ["OK", ""]
    for field in fields:
        newContents = []
        fileName = dataDir + "/" + field
        #f=open(fileName); contents=f.read(); f.close()
        [a1, a2, a3, nMaxLines] = readGridEditorConfig()
        contents = getFoamContents.readField(fileName, nMaxLines)
        #pointerをboundaryFieldまで進める
        p = getFoamContents.getKeywordPointerMem("boundaryField", contents, 0)    
        newContents = contents[:p] + "boundaryField\n{\n"
        #boundaryFieldを取得
        [boundaryCont, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        p = 0
        lineCont = " "
        while lineCont != "":
            #1行を取得
            [lineCont, p, lineType] = getFoamContents.get1lineMem(boundaryCont, p)
            if lineType == "keyword":
                #lineTypeが{}の場合
                [patchName, pp] = getFoamContents.getKeywordMem(lineCont, 0)
                if patchName == oldPatchName:
                    newLine = newPatchName + lineCont[len(patchName):]
                    newContents += "    " + newLine + "\n"
                else:
                    newContents += "    " + lineCont + "\n"
            else:
                if lineCont != "":
                    newContents += "    " + lineCont + "\n"
        newContents += "}\n"
        newContents += setFoamContents.makeFooter()
        #f=open(fileName, "w"); f.write(newContents); f.close()
        getFoamContents.writeField(fileName, newContents)
    return error

#
#  deleteNullPatches
#    case内の全ての空patchを削除
def deleteNullPatches(caseDir):
    #fieldDir、field名を取得
    currTime = getFoamContents.getCurrTimeFolder(caseDir)
    fieldDir = caseDir + "/" + currTime
    fields = getFieldNames(fieldDir)
    #polyMeshDirを取得
    relDir = getFoamContents.getCurrMeshDir(caseDir, currTime, ".", "boundary")
    polyMeshDir = caseDir + "/" + relDir
    #nulPatch名を取得
    patchConts = getFoamContents.getPatchCont(polyMeshDir)
    nullPatches = []
    for patchCont in patchConts:
        patchName = patchCont[0]
        nFaces = patchCont[1]
        if nFaces == 0:
            nullPatches.append(patchName)
    #nullPatchを削除
    error = ["OK", ""]
    for nullPatch in nullPatches:
        error = deletePatch(polyMeshDir, fieldDir, fields, nullPatch)
        if error[0] != "OK":
            break
    return error 

#
#  deletePatch
#    patchを削除
def deletePatch(boundaryDir, dataDir, fields, delPatch):
    error = delPatchBoundaryFile(boundaryDir, delPatch)
    if error[0] == "OK":
        error = delPatchBoundaryField(dataDir, fields, delPatch)
    return error

#  delPatchBoundaryFile
#    boundary file 中のpatchを削除
def delPatchBoundaryFile(boundaryDir, delPatch):
    error = ["OK", ""]
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #patch数を減らす
        [patchN, p] = getFoamContents.getKeywordMem(contents, p)
        stP = p - (len(patchN)+1)           #ポインタをkeywordの前に戻す
        newContents = contents[:stP]        #ポインタまでを書き込み
        n = int(patchN) -1
        newContents += str(n) + "\n("       #patch数を書き込み
        p = stP + len(patchN)               #ポインタをpatch数に設定
        #patchを削除
        [boundaryCont, p] = getFoamContents.getSmallPairMem(contents, p)
        p=0; flag=0
        patchName = ["", ""]
        while (patchName[0] != delPatch) and (p < len(boundaryCont)) and (flag == 0):
            pbak = p
            [patchName, p] = getFoamContents.getNextKeywordAndContentsMem(boundaryCont, p)
            if patchName[0] == delPatch:
                newContents += boundaryCont[:pbak] + boundaryCont[p:]
                flag = 1
        if flag == 1:
            newContents += ")\n\n//************************************************************************* //"
            f=open(fileName, "w"); f.write(newContents); f.close()
        else:
            print len(delPatch), " '"+ delPatch+"'"
            error = ["NG", _(u"「") + delPatch + _(u"」のpatchがboundary中に存在しません")]
    else:
        error = ["NG", fileName + _(u"が存在しません。")]
    return error

#  delPatchBoundaryField
#    boundaryField中のpatchを削除
def delPatchBoundaryField(dataDir, fields, delPatch):
    error = ["OK", ""]
    for field in fields:
        fileName = dataDir + "/" + field
        #f=open(fileName); contents=f.read(); f.close()
        [a1, a2, a3, nMaxLines] = readGridEditorConfig()
        contents = getFoamContents.readField(fileName, nMaxLines)
        p = getFoamContents.getKeywordPointerMem("boundaryField", contents, 0)    
        newContents = contents[:p]
        [boundaryContents, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        newContents += "boundaryField\n{\n"
        p=0; flag = 0
        patchName = ["", ""]
        while (patchName[0] != delPatch) and (p < len(boundaryContents)) and (flag == 0):
            pbak = p
            [patchName, p] = getFoamContents.getNextKeywordAndContentsMem(boundaryContents, p)
            if patchName[0] == delPatch:
                newContents += boundaryContents[:pbak] + boundaryContents[p:]
                flag = 1
        if flag == 1:
            newContents += "}\n\n// ************************************************************************* //"
            #f=open(fileName, "w"); f.write(newContents); f.close()
            getFoamContents.writeField(fileName, newContents)
        else:
            error = ["NG", 
                     _(u"「") + delPatch + _(u"」のpatchが「") + field
                     + _(u"」のboundaryField中に存在しません")]
    return error

#
#  setBoundaryData
#    boundary（patchの定義）を保存する
def setBoundaryData(boundaryDir, patches, defineData):
    #print defineData
    error = ["OK",""]
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        newDefine = makeDefineData(patches, defineData, contents)
        #print newDefine    
        repDefine = replaceDefineData(patches, newDefine, contents)
        #print repDefine
        if repDefine != "":
            f=open(fileName,"w"); f.write(repDefine); f.close()
        else:
            error = ["NG", fileName+_(u"のデータが取得できません。")]
    else:
        error = ["NG", fileName+_(u"が存在しません。")]
    return error


def replaceDefineData(patches, newDefine, contents):
    #boundaryから（）内データを取得
    p=0
    [dummy, p] = getFoamContents.getKeywordContentsMem("FoamFile", contents, p)
    [defineFile, p] = getFoamContents.getSmallPairMem(contents, p)
    if defineFile != "":
        #ポインタを（）内データの頭と後ろに設定しておく
        p = p - len(defineFile)
        pe = p + len(defineFile) - 1
        #patch毎に内容を入替え
        for patch in patches:
            defineFile = replaceKeywordData(patch, newDefine, defineFile)
        repData = contents[:p] + newDefine + contents[pe:]
    else:
        repData = ""
    return repData


def replaceKeywordData(patch, newDefine, defineFile):
    patch = patch.split()[0]
    [orgCon, p] = getFoamContents.getKeywordContentsMem(patch, defineFile, 0)
    if orgCon != "":
        [newCon, n] = getFoamContents.getKeywordContentsMem(patch, newDefine, 0)
        p = p - len(orgCon)
        repData = defineFile[:p] + newCon + defineFile[p+len(orgCon)-1:]
    else:
        repData = defineFile
    return repData


def makeDefineData(patches, defineData, contents):
    #patchデータを整形
    define = makeBoundaryField(patches, defineData)
    #各patch毎にdefineデータを作成
    newDefine = ""
    pp = 0
    pc = 0
    for patchSP in patches:
        patch = patchSP.split()[0]
        [definePatch, pp] = getFoamContents.getKeywordContentsMem(
                        patch, define, pp)
        [defineFile, pc] = getFoamContents.getKeywordContentsMem(
                        patch, contents, pc)
        definePatch = reformTabContents(definePatch)
        defineFile  = reformTabContents(defineFile)
        #nFaces, startFaceを追加。
        addLine = ""
        if definePatch != "" and defineFile != "":
            lines = defineFile.split("\n")
            for line in lines:
                if string.find(line, "nFaces")>=0:
                    addLine += line+"\n"
                if string.find(line, "startFace")>=0:
                    addLine += line+"\n"
            lines = defineFile.split(";")
            for line in lines:
                words = line.split()
                if len(words) >= 3:
                    if words[1] == "nonuniform" and words[2][:len("List")] == "List":
                        addLine += line + ";\n"
            newDefine += patchSP + "\n    {\n"
            newDefine += definePatch + addLine + "    }\n"
    return newDefine


#  reformTabContents
#    行頭に空白、行中のtab追加
#    tab:8,20で設定
def reformTabContents(value):
    tab1=8; tab2=20
    #各行に分割
    lines = value.split("\n")
    #各行のnul行を削除
    words = []
    for line in lines:
        ll = line.split()
        if len(ll) > 0:
            if ll[0] != "":
                words.append(line)
    #各行にtab追加
    i=0
    line=""
    while i<len(words):
        #行を単語に分割
        wds = words[i].split()
        if len(wds) > 1:
            #2単語以上の場合、行頭に空白(tab1個)追加
            wds[0] = tab1*" " + wds[0]
            if len(wds[0])<tab2-1:
                #第1単語がtab2より短い場合、tab2位置まで空白追加
                wds[0] += (tab2 - len(wds[0]))*" "
            else:
                #長い場合は、空白1ヶを追加
                wds[0] += " "
            #1行を作成
            line += wds[0] + string.join(wds[1:], " ") + "\n"
        elif len(wds) == 1:
            #行内に1単語のみの場合
            line += tab1*" " + wds[0] + "\n"
        i+=1
    return line

#
#  setFieldsPatchesName
#    boundaryConditionをfileにセット
#    「getFieldsPatchesName」と反対の処理
#     文字が多すぎる場合、
def setFieldsPatchesName(dataDir, fields, patches, patchData, intData):
    [a1, a2, a3, nMaxLines] = readGridEditorConfig()
    i=0
    mess = ""
    while i<len(fields):
        #fieldをオープン
        field = fields[i]
        fileName = dataDir + "/" + field
        #f=open(fileName); contents=f.read(); f.close()
        contents.getFoamContents.readField(fileName, nMaxLines)
        #internalFieldを作成、置き換え
        contents = makeReplaceInternalField(fields, intData[i], contents)
        #patchDataを元に戻す（"..."がある場合）
        patchData[i] = remakeBoundaryField(contents, patches, patchData[i])
        #patchDataから「boundaryField」を作成
        newBoundary = makeBoundaryField(patches, patchData[i])
        #「boundaryField」を置き換える
        newContents = replaceBoundaryField(newBoundary, contents)
        if newContents != "":
            getFoamContents.writeField(fileName, newContents)
            #f=open(fileName,"w"); f.write(newContents); f.close()
        else:
            print "error:",field,_(u"が置き換えできませんでした。")
            mess += field + ","
        i+=1
    if mess == "":
        error = ["OK", ""]
    else:
        error = ["NG", mess[:-1]+_(u"が書き込めません。")]
    return error


def remakeBoundaryField(contents, patches, patchData):
    i=0
    while i<len(patchData):
        if patchData[i][-4:-1]=="..." or patchData[i][-3:]=="...":
            patchData[i] = replacePatchData(contents, patches[i], patchData[i])
        i+=1
    return patchData


def replacePatchData(contents, patch, data):
    p=getFoamContents.skipFoamFileMem(contents, 0)
    [dummy, p] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", contents, p)
    #patchにスペースを追加してあるので、スペースを詰める
    #[patchSp, p] = getFoamContents.getKeywordMem(patch, 0)
    [chara, p] = getFoamContents.skipNoCharaMem(patch, 0)
    patchSp = patch[p:]
    #元のデータからpatchデータを取得
    [patchCont, p] = getFoamContents.getKeywordContentsMem(patchSp, dummy, 0)
    if len(patchCont) == 0:
        #patchContが取得できなかった場合
        #"patch.*"の様な変数で取得
        [patchCont, p] = getKeywordContentsMemVariablePatchName(patchSp, dummy, 0)
    #最後のスペースを詰める
    i=len(patchCont)-1
    while i>0 and patchCont[i] ==" ":
        i-=1
    patchCont = patchCont[:i+1]
    #最初の1行目が改行コードのみの場合は、その行を削除する
    if patchCont[0] == "\n":
        patchCont = patchCont[1:]
    return patchCont


#  getKeywordContentsMemVariablePatchName
#    patch名が変数になっている場合のpatchContの取得
def getKeywordContentsMemVariablePatchName(patchSp, contents, p):
    [[keyword, patchCont], p] = getFoamContents.getNextKeywordAndContentsMem(contents, p)
    flag = 0
    matchName = ""
    while keyword != "" and p<=len(contents) and flag == 0:
        if keyword[0] == '"':
            pattern = keyword[1:-1]
            #patternの一致をチェック
            if isMatchName(patchSp, pattern) == True:
                flag = 1
                break
        [[keyword, patchCont], p] = getFoamContents.getNextKeywordAndContentsMem(contents, p)
    if flag == 0:
        patchCont = ""
    return [patchCont, p]

def makeReplaceInternalField(fields, intData, contents):
    p=getFoamContents.skipFoamFileMem(contents, 0)
    [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    while keyword != "internalField" and keyword != "":
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    if keyword != "":
        p = p - (len("internalField")+1)
        line = contents[p:p+100]
        n = string.find(line, " uniform ")
        if n >= 0:
            m = string.find(line, "\n")
            contents = contents[:p] + intData + contents[p+m+1:]
        else:
            n = string.find(intData, " uniform ")
            if n>= 0:
                [dummy, m] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", contents, p)
                m = m - len(dummy)
                contents = contents[:p] + intData + "\nboundaryField\n{\n" + contents[m:]
    return contents 


def makeBoundaryField(patches, patchData):
    boundary = ""
    i=0
    lenPD=len(patchData)
    while i<lenPD:
        #patchDataが空文の場合、作成しない。
        if (patchData[i] != ""):
            patch = patches[i]
            boundary += patch + "\n    {\n"
            boundary += patchData[i]
            boundary += "    }\n"
        i+=1
    return boundary

def replaceBoundaryField(newBoundary, strings):
    #boundaryのポインタを取得
    p=0
    pp=getFoamContents.getKeywordPointerMem(
                        "boundaryField", strings, p)
    #boundaryの内容を取得
    [boundary, p] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", strings, pp)
    #ポインタをboundaryの頭に戻す
    if boundary != "":
        p = pp
        strings = (strings[:p]
                    + "boundaryField\n{\n"
                    + newBoundary
                    + "}\n\n// ************************************************************************* //")
    else:
        strings = ""
    return strings

#
#  getFieldsPatchesName
#    各Fieldのpatch名とboundaryConditionを取得
#    戻り値：[[fields], [patchs], [boundaryConditions]]
def getFieldsPatchesName(editDir, selectedFields):
    wdir = editDir + "/" + selectedFields[0][0]
    files = selectedFields[0][1:]
    [files, patches, patchData] = getFieldsPatchesName_sub(wdir, files)
    return [files, patches, patchData]

#
#  getAllFieldsPatchesName
#    各Fieldのpatch名とboundaryConditionを取得
#    戻り値：[[fields], [patches], [boundaryConditions]]
def getAllFieldsPatchesName(wdir):
    [folders, files] = getFoldersFiles(wdir)
    [files, patches, patchData] = getFieldsPatchesName_sub(wdir, files)
    return [files, patches, patchData]

#  getFieldsPatcesName_sub
def getFieldsPatchesName_sub(wdir, files):
    [a1, a2, a3, nMaxLines] = readGridEditorConfig()
    fieldsPatch = []
    patchData = []
    for name in files:
        patchesCont = []
        fileName = wdir + "/" + name
        #f=open(fileName); contents=f.read(); f.close()
        contents = getFoamContents.readField(fileName, nMaxLines)
        p=0
        [boundary, p] = getFoamContents.getKeywordContentsMem(
                            "boundaryField", contents, p)
        [patchCont, p] = getFoamContents.getNextKeywordAndContentsMem(
                            boundary, 0)
        while patchCont[0] != "":
            patchesCont.append(patchCont)
            [patchCont,p]=getFoamContents.getNextKeywordAndContentsMem(
                                boundary, p)
        fieldsPatch.append(patchesCont)
    patches = []
    for patchC in fieldsPatch[0]:
        patches.append(patchC[0])
    for fieldPatch in fieldsPatch:
        col=[]
        for patch in patches:
            col.append("")
        for fpatch in fieldPatch:
            flag = 0; i=0
            for patch in patches:
                if fpatch[0] == patch:
                    col[i]=compressSPCR(fpatch[1])
                    flag = 1
                    break
                i+=1
            if flag == 0:
                patches.append(fpatch[0])
                col.append(compressSPCR(fpatch[1]))
        patchData.append(col)
    return [files, patches, patchData]

def compressSPCR(contents):
    lines = contents.split("\n")
    ans = []
    for line in lines:
        if line != "":
            wordList = line.split()
            ans.append(string.join(wordList, " "))
    ansR = string.join(ans, "\n")
    #len(ansR)が0の場合を追加 12/09/17
    if len(ansR) > 0:
        if ansR[-1] == "\n":
            ansR = string.join(ans, "\n")[:-1]
    else:
        ansR = "\n"
    return ansR

#
#  remakeFolderConjugate
#    folderの位置をconjugate用に配置する
def remakeFolderConjugate(fluidZone, solidZone):
    caseDir = os.getcwd()
    regDir = caseDir + "/regCase"
    wdir = caseDir.split("/")[1:-1]
    timeFolders = getTimeFolders(caseDir)
    timeFolder = timeFolders[0]

    #流体側のcellZoneを処理
    fluidName = fluidZone[0]
    print _(u"流体側のzellZone「"), fluidName, _(u"」を整形中...")
    #  meshを移動
    sourceDir = regDir + "/" + timeFolder + "/" + fluidName + "/polyMesh"
    toDir = regDir + "/constant/polyMesh"
    moveFile(sourceDir, toDir)
    #　fieldを移動
    sourceDir = regDir + "/" + timeFolder + "/" + fluidName
    toDir = regDir + "/" + timeFolder
    moveFile(sourceDir, toDir)
    #  不要なfolderを削除
    os.system("rm -r " + regDir + "/" + timeFolder + "/" + fluidName + "> /dev/null")
    os.system("rm -r " + regDir + "/constant/" + fluidName + " > /dev/null")
    os.system("rm -r " + regDir + "/system/" + fluidName + " > /dev/null")

    #固体側のcellZoneを処理
    i=1
    for solidName in solidZone:
        print _(u"固体側「"), solidName, _(u"」を整形中...")
        #folder作成
        toDir = caseDir + "/" + solidName
        os.system("mkdir " + toDir)
        #constantをコピー
        sourceDir = regDir + "/"+ timeFolder + "/" + solidName + "/polyMesh"
        toDir = caseDir + "/" + solidName + "/constant"
        os.system("mkdir " + toDir)
        toDir += "/polyMesh"
        os.system("mkdir " + toDir)
        moveFile(sourceDir, toDir)
        os.system("rm -r " + sourceDir + " > /dev/null")
        os.system("rm -r " + regDir + "/constant/" + solidName)
        #source = caseDir + "/" + solidName + "/constant"
        source = "../../" + solidName + "/constant"
        toLink = "./regCase/constant/solid" + str(i)
        os.system("ln -s " + source + " " + toLink)
        #timeFolderをコピー
        sourceDir = regDir + "/" + timeFolder + "/" + solidName
        toDir = caseDir + "/" + solidName + "/" + timeFolder
        os.system("mkdir " + toDir)
        moveFile(sourceDir, toDir)
        os.system("rm -r " + sourceDir)
        #source = caseDir + "/" + solidName + "/" + timeFolder
        source = "../../" + solidName + "/" + timeFolder
        toLink = "./regCase/" + timeFolder + "/solid" + str(i)
        os.system("ln -s  " + source + " " + toLink)
        #systemをコピー
        sourceDir = regDir + "/system"
        toDir = caseDir + "/" + solidName + "/system"
        os.system("mkdir " + toDir)
        os.system("cp "+ sourceDir+"/controlDict " + toDir + " > /dev/null")
        os.system("cp "+ sourceDir+"/fv* " + toDir + " > /dev/null")
        os.system("rm -r " + regDir + "/system/" + solidName)
        #source = caseDir + "/" + solidName + "/system"
        source = "../../" + solidName + "/system"
        toLink = "./regCase/system/solid" + str(i)
        os.system("ln -s " + source + " " + toLink)

        i=i+1

    print _(u"メッシュをconjugate用に分割しました。")


def moveFile(sourceDir, toDir):
    comm = "cp " + sourceDir + "/* " + toDir + "> /dev/null"
    os.system(comm)
    comm = "rm -r " + sourceDir + "/* > /dev/null"
    os.system(comm)

#
#  setEditFolders
#    編集するtimeフォルダ内のファイルとフォルダをセット
#    （FFlistを取得）
def setEditFolders(editDir):
    FFlist = []
    [folders, files] = getFoldersFiles(editDir)
    fields = getFieldNames(editDir)
    # カレントフォルダをFFlistにセット
    folderFiles = setFolderFiles(".",fields)
    FFlist.append(folderFiles)
    # フォルダをFFlistにセット
    i = 0
    while i < len(folders):
        # 各フォルダをFFlistにセット
        wdir = editDir + "/" + folders[i]
        fields = getFieldNames(wdir)
        folderFiles = setFolderFiles(folders[i],fields)
        FFlist.append(folderFiles)
        i = i + 1
    return FFlist


def setFolderFiles(folder, files):
    fflist = []
    fflist.append(folder)
    for name in files:
        fflist.append(name)
    return fflist

#
#  makeNameListFile
#    nameListファイル作成
def makeNameListFile():
    cont = "nameList\n\n"
    # field名取得
    cont = cont + "<Field>"
    wdir = os.getcwd()
    times = getTimeFolders(wdir)
    items = getFoamContents.controlDict(wdir+"/system", ["startFrom"])
    if items[0] == "startTime":
        allFiles = getFoldersFiles(wdir + "/" + times[0])
        cont = cont + " startTime " + times[0] + "\n"
    elif items[0] == "latestTime":
        allFiles = getFoldersFiles(wdir + "/" + times[-1])
        cont = cont + " latestTime " + times[-1] + "\n"
    fields = allFiles[1]
    fields = delBackupFile(fields)
    fields.sort()
    cont = makeNextCont(fields, cont)
    # setsファイル取得
    files = glob.glob(wdir + "/constant/polyMesh/sets/*")
    sets = []
    for name in files:
        sets.append(name.split("/")[-1])
    sets = delBackupFile(sets)
    sets.sort()
    cont = cont + "\n<sets>\n"
    cont = makeNextCont(sets, cont)
    # cellZoneName取得
    #cellZones = getFoamContents.cellZoneNames()
    cellZones = getFoamContents.cellZoneNamesF(wdir)
    cellZones = delBackupFile(cellZones)
    cellZones.sort()
    cont = cont + "\n<cellZones>\n"
    cont = makeNextCont(cellZones, cont)
    # field sets cellZone のリスト作成
    fileName = "nameList"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def delBackupFile(files):
    names = []
    for name in files:
        if name[-1] != "~":
            names.append(name)
    return names

def makeNextCont(items, cont):
    for item in items:
        cont = cont + item + "\n"
    return cont

#
#  openFolderNautilus
#    folderをnautilusでopenする
#    ubuntu以外でも使えるように裏で起動
def openFolderNautilus(folder):
    if isWmctrl() == True:
        os.system(("nautilus " + folder.encode("utf-8")) + " &")
        name = folder.split("/")[-1]
        winId = waitUntilShowWindow(name)
        os.system("wmctrl -i -a " + winId)
    else:
        os.system(("nautilus " + folder.encode("utf-8")) + " &")

def isWmctrl():
    pathes = os.getenv("PATH").split(":")
    ans = False
    for path in pathes:
        if len(glob.glob(path+"/wmctrl")) != 0:
            ans = True
            break
    return ans

def waitUntilShowWindow(folderName):
    #tempFile = os.getenv("TreeFoamPath") + "/temp/temp.wmctrl"
    tempFile = os.getenv("TreeFoamUserPath") + "/temp/temp.wmctrl"
    comm = "wmctrl -l > " + tempFile
    flag = 0
    while flag == 0:
        os.system(comm)
        f=open(tempFile); cont=f.readlines(); f.close()
        for line in cont:
            winName = line.split()[-1].decode("utf-8")
            if winName == folderName:
                winId = line.split()[0]
                flag = 1
    return winId

#
#  getTimeFolders
#    caseDir内のtimeFolderを取得
def getTimeFolders(caseDir):
    dirs = glob.glob(caseDir + "/*")
    folders = []
    for folder in dirs:
        folders.append(folder.split("/")[-1])
    time=[]
    for name in folders:
        try:
            a=float(name)
            time.append([a, name])
        except:
            a=0
    time.sort()
    timeFolders=[]
    for name in time:
        timeFolders.append(name[1])
    return timeFolders

#
#  getFeildNames
#    dir内のfield名を返す
#    field名は、非圧縮のfield名に戻す
def getFieldNames(wdir):
    [folders, files] = getFoldersFiles(wdir)
    fields = []
    for field in files:
        #非圧縮のfileNameを取得
        field = getFoamContents.getOrgFileName(field)
        #field名を取得
        if (field[-1] != "~" and            #バックアップfileと
             field[0] != "."):              #隠しfileは、除く
            fields.append(field)
    #同名を削除
    ans = []
    fields.sort()
    name = ""
    for field in fields:
        if name != field:
            ans.append(field)
        name = field
    return ans

#
#  getFoldersFiles
#    dir内のfolder名とfile名を返す（名前のみ返す。pathではない）
#    戻り値：[[folders],[files]]
def getFoldersFiles(wdir):      #wdirは、絶対path
    folders = []
    files = []
    dirFiles = glob.glob(wdir + "/*")
    for name in dirFiles:
        if os.path.isdir(name) == True:
            folders.append(name.split("/")[-1])
        elif os.path.isfile(name) == True:
            files.append(name.split("/")[-1])
    folders.sort()
    files.sort()
    return [folders, files]

#
#  runOFUtilityReturnStat
#    OpenFOAMのutilityを実行しその結果を返す
#       OK or ERROR
def runOFUtilityReturnStat(comm):
    #tempFile = os.environ["TreeFoamPath"] + "/temp/temp.txt"
    #errFile = os.environ["TreeFoamPath"] + "/temp/tempError.txt"
    tempFile = os.environ["TreeFoamUserPath"] + "/temp/temp.txt"
    errFile = os.environ["TreeFoamUserPath"] + "/temp/tempError.txt"
    os.system("rm -f " + errFile + " " + tempFile)         #fileを削除
    bashrc = getBashrcFoam()
    logFile = os.getenv("LogFile")
    if os.path.isfile(logFile) == False:
        #そのまま出力
        cont = ""
        cont += ". " + bashrc + "; "
        cont += comm + " 2>" + errFile + " | tee " + tempFile
    else:
        #logFile追加
        if comm[-1] != "&":
            cont = ""
            cont += ". " + bashrc + " 2> " + errFile + " 1>> " + logFile
            cont += " 2>> " + logFile + "; "
            cont += comm + " 2>" + errFile + " | tee " + tempFile + " >> "
            cont += logFile + " 2>> " + logFile
        else:
            cont = ""
            cont += ". " + bashrc + " 2> " + errFile + " 1>> " + logFile
            cont += " 2>> " + logFile + "; "
            cont += comm[:-1] + " 2>" + errFile + " | tee " + tempFile + " 1>> "
            cont += logFile + " 2>> " + logFile + " &"

    os.system(cont)
    if os.path.getsize(errFile) == 0:
        stat = "OK"
    else:
        stat = "ERROR"
        f=open(errFile); cont=f.read(); f.close()
        print cont
    return [stat, cont]

#
#  runOFUtility
#    OpenFOAMのutilityを実行する
def runOFUtility(comm):
    logFile = os.getenv("LogFile")
    bashrc = getBashrcFoam()
    if os.path.isfile(logFile) == False:
        cont = ""
        cont += ". " + bashrc + "; "
        cont += comm
    else:
        if comm[-1] != "&":
            cont = ""
            cont += ". " + bashrc + " 1>> " + logFile + " 2>> " + logFile + "; "
            cont += comm + " 1>> " + logFile + " 2>> " + logFile
        else:
            cont = ""
            cont += ". " + bashrc + " 1>> " + logFile + " 2>> " + logFile + "; "
            cont += comm[:-1] + " 1>> " + logFile + " 2>> " + logFile + " &"
    os.system(cont)
    return

#
#  runCommandErrorCheck
#    commandを実行し、その結果を返す。
def runCommandErrorCheck(comm):
    #errFile = os.environ["TreeFoamPath"] + "/temp/tempError.txt"
    errFile = os.environ["TreeFoamUserPath"] + "/temp/tempError.txt"
    os.system("rm -f " + errFile)         #fileを削除
    cont = comm + " 2> " + errFile + " 1> /dev/null"
    os.system(cont)
    if len(glob.glob(errFile)) == 0:
        stat = "OK"
    else:
        if os.path.getsize(errFile) == 0:
            stat = "OK"
        else:
            stat = "ERROR"
            f=open(errFile); cont=f.read(); f.close()
            print cont
    return [stat, cont]

#
#  runCommand
#    commandを実行
def runCommand(comm):
    logFile = os.getenv("LogFile")
    if os.path.isfile(logFile) == False:
        cont = comm
    else:
        if comm[-1] != "&":
            cont = comm + " 1>> " + logFile + " 2>> " + logFile
        else:
            cont = comm[:-1] + " 1>> " + logFile + " 2>> " + logFile + " &"
    os.system(cont)
    return

#
#  readSshfsData
#    sshfs_dataの読み込み
#    戻り値のlist内容
#       0:HostName
#       1:User
#       2:HostDir
#       3:MountPoint
#       4:login
#       5:setEnviron
def readSshfsData():
    treeFoamUserPath = os.getenv("TreeFoamUserPath")
    fileName = treeFoamUserPath + "/data/sshfs_data"
    f=open(fileName); lines=f.readlines(); f.close()
    lines = deleteBashComment(lines)
    ans = ["", "", "", "", "", ""]
    for i in range(len(lines)):
        line = lines[i]
        words = line.split()
        if len(words) > 1:
            if line[:len("HostName")] == "HostName":
                ans[0] = words[1]
            elif line[:len("User")] == "User":
                ans[1] = words[1]
            elif line[:len("HostDir")] == "HostDir":
                ans[2] = words[1]
            elif line[:len("MountPoint")] == "MountPoint":
                ans[3] = words[1]
        if line[:len("login")] == "login":
            words = lines[i+1].split()
            ans[4] = " ".join(words)
        if line[:len("setEnviron")] == "setEnviron":
            n = i + 1
            while lines[n][0] >= " " and n < len(lines):
                words = lines[n].split()
                ans[5] += " ".join(words) + "\n"
                n += 1
    return ans

#
#  getTreeFoamServerResult
#    TreeFoamServerのコマンドを実行
#    実行結果fileは、$treefoamuserpath/temp/<resultFile> に入る。
def getTreeFoamServerResult(comm, resultFile):
    data = readSshfsData()
    hostName = data[0]
    userName = data[1]
    hostDir = data[2]
    mountPoint = data[3]
    #コマンド実行
    writeFile = "~/TreeFoamServer/tmp/" + resultFile
    comms = '"' + comm + ' > ' + writeFile + '"'
    commLine = "ssh " + userName + "@" + hostName + " " + comms
    print commLine
    [stat, cont] = runCommandErrorCheck(commLine)
    if stat != "OK":
        return "ERROR"

    #結果の転送
    sourceFile = userName + "@" + hostName + ":~/TreeFoamServer/tmp/" + resultFile
    toCopyFile = os.getenv("TreeFoamUserPath") + "/temp/" + resultFile
    comms = "scp -Cr " + sourceFile + " " + toCopyFile
    [stat, cont] = runCommandErrorCheck(comms)
    if stat != "OK":
        return "ERROR"
    return "OK"    

#
#  deleteBashComment
#    コメント（「#」から文末まで）を削除する
def deleteBashComment(lines):
    newLines = []
    for line in lines:
        if line[0] != "#":
            n = line.find("#")
            if n < 0:
                newLines.append(line)
            else:
                newLine = line[:n] + "\n"
                newLines.append(newLine)
    return newLines

#
#  writeSshfsData
#    sshfs_dataに書き込み
def writeSshfsData(data):
    treeFoamUserPath = os.getenv("TreeFoamUserPath")
    fileName = treeFoamUserPath + "/data/sshfs_data"
    f=open(fileName); lines=f.readlines(); f.close()
    flag = [0, 0, 0, 0, 0, 0]
    for i in range(len(lines)):
        line = lines[i]
        words = line.split()
        if len(words) > 0:
            if words[0] == "HostName":
                lines[i] = "HostName    " + data[0] + "\n"
                flag[0] = 1
            elif words[0] == "User":
                lines[i] = "User        " + data[1] + "\n"
                flag[1] = 1
            elif words[0] == "HostDir":
                lines[i] = "HostDir     " + data[2] + "\n"
                flag[2] = 1
            elif words[0] == "MountPoint":
                lines[i] = "MountPoint  " + data[3] + "\n"
                flag[3] = 1
    for i in range(len(flag)):
        if flag[i] == 0:
            if i == 0:
                lines.append("HostName    " + data[0] + "\n")
            elif i == 1:
                lines.append("User        " + data[1] + "\n")
            elif i == 2:
                lines.append("HostDir     " + data[2] + "\n")
            elif i == 3:
                lines.append("MountPoint  " + data[3] + "\n")
    f=open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#
#  readConfigTreeFoam
#    configTreeFoamの読み込み
#    戻り値のlist内容
#       0:OFversion         OpenFOAMのバージョン
#       1:bashrcFoam        環境設定ファイル
#       2:rootDir           rootDir
#       3:workDir           解析用case
#       4:paraFoam          paraFoam起動用ファイル
#       5:salomeMeca        salomeMeca起動用ファイル
#       6:CAD               CAD起動用ファイル
#       7:editor            editor起動ファイル
#       8:HelyxOS           OpenFOAMのGUI
#       9:language          使用言語
#      10:logFile           logFile作成有無
#
def readConfigTreeFoam():
    #configTreeFoamの読み込み
    treeFoamUserPath = os.environ["TreeFoamUserPath"]
    if treeFoamUserPath == "":
        print _(u"環境変数$TreeFoamPathが設定されていないので、隠しファイル~/.treeFoamInstallFolderから、TreeFoamPathを取得します。")
        homeFolder = os.environ["HOME"]
        f = open(homeFolder + "/.treeFoamInstallFolder")
        treeFoamUserPath = f.readline()
        f.close()
    f = open(treeFoamUserPath + "/configTreeFoam")
    configContents = f.readlines()
    f.close()
    #configDataの取得（11項目）
    ans = ["", "", "", "", "", "", "", "", "", "", ""]
    for line in configContents:
        word = line.split()
        if (len(word) > 0):
            if (word[0] == "OFversion"):
                if len(word) >1:
                    ans[0] = word[1]
                else:
                    ans[0] = "XXXX"
            elif (word[0] == "bashrcFOAM"):
                if len(word) >1:
                    ans[1] = convAbsDir(word[1])
            elif (word[0] == "rootDir"):
                if len(word) >1:
                    ans[2] = convAbsDir(word[1])
            elif (word[0] == "workDir"):
                if len(word) >1:
                    ans[3] = convAbsDir(word[1])
            elif (word[0] == "paraFoam"):
                if len(word) >1:
                    ans[4] = convAbsDir(" ".join(word[1:]))
            elif (word[0] == "salomeMeca"):
                if len(word) >1:
                    ans[5] = convAbsDir(" ".join(word[1:]))
            elif (word[0] == "CAD"):
                if len(word) >1:
                    ans[6] = convAbsDir(" ".join(word[1:]))
            elif (word[0] == "editor"):
                if len(word) >1:
                    ans[7] = convAbsDir(" ".join(word[1:]))
            elif (word[0] == "helyxos"):
                if len(word) >1:
                    ans[8] = convAbsDir(" ".join(word[1:]))
            elif (word[0] == "language"):
                if len(word) >1:
                    #ans[9] = convAbsDir(" ".join(word[1:]))
                    ans[9] = word[1]
            elif (word[0] == "logFile"):
                if len(word) >1:
                    ans[10] = word[1]
    return ans

#  絶対pathに変換する
def convAbsDir(wdir):
    work = wdir.split()
    work0 = os.path.expanduser(work[0])     #~を変換
    work0 = os.path.expandvars(work0)       #環境変数を変換
    if len(work) > 1:
        #option付き
        ans = work0 + " " + " ".join(work[1:])
    else:
        #pathのみ
        ans = work0
    return ans

#
#  getTreeFoamVersion
#    TreeFoamのバージョンを取得
def getTreeFoamVersion():
    version = ""
    verFile = os.getenv("TreeFoamPath") + "/TreeFoamVersion"
    f = open(verFile)
    lines = f.readlines()
    f.close()
    for line in lines:
        words = line.split()
        if len(words) >= 2:
            if words[0] == "currentTreeFoamVersion":
                version = words[1]
                break
    return version

#
# getOFversion
#   OpenFOAMのバージョン取得
#   現在の環境設定から取得する
def getOFversion():
    comm = "getOFversionWriteConfig.py > /dev/null"
    runOFUtility(comm)
    ans = readConfigTreeFoam()
    ver = ans[0]
    return ver

#
# getBashrcFoam
#   外からOpenFOAMの環境設定ファイルを読み込む時に使用
#   環境変数$TreeFoamPathから取得する
def getBashrcFoam():
    ans = readConfigTreeFoam()
    envOpenFOAMFix = ans[1]
    return envOpenFOAMFix

#
#  getEditorFromConfigFile
#    使用するeditor名をconfigFileから取得
def getEditorFromConfigFile():
    ans = readConfigTreeFoam()
    editor = ans[7]
    return editor

#
#  runEditor
#    editorの起動（通常：裏で動かす）
def runEditor(openFiles):
    editor = getEditorFromConfigFile()
    print editor, _(u" & で"), len(openFiles.split()), _(u"個のファイルを開きます")
    comm = editor
    comm += " " + openFiles
    comm += " &"
    os.system(comm)                         #editor起動

#
#  runEditorWait
#    closeするまで待つeditorを起動
def runEditorWait(openFiles):
    editor = getEditorFromConfigFile()
    print editor, _(u"で"), len(openFiles.split()), _(u"個のファイルを開きます")
    comm = editor
    comm += " " + openFiles
    os.system(comm)                         #editor起動

#------------------ TreeDataの取得関係-------------------------------------------

#
#  getNumberResult
#    計算結果folderの数を取得
def getNumberResult(selectDir):
    folders = getFoldersOnlyCurrDir(selectDir)
    ans = []
    for folder in folders:
        try:
            a = float(folder.split("/")[-1])
            ans.append(a)
        except:
            a = 1.0
    nRst = len(ans)
    if nRst < 2:
        for folder in folders:
            if folder.split("/")[-1] == "processor0":
                subFolders = getFoldersOnlyCurrDir(selectDir + "/processor0")
                ans = []
                for subFolder in subFolders:
                    try:
                        a = float(subFolder.split("/")[-1])
                        ans.append(a)
                    except:
                        a = 1.0
                nRst = len(ans)
                break
    ansR = []
    if nRst != 0:
        ansR.append(" " + str(nRst))
        ansR.append(" " + str(min(ans)))
        if min(ans) == max(ans):
            ansR.append("")
        else:
            ansR.append(" " + str(max(ans)))
    return ansR

#  getSolver
#    solverを取得
def getSolver(selectDir):
    fileName = selectDir + "/system/controlDict"
    solver = ""
    if os.path.isfile(fileName) == True:
        f=open(fileName)
        for line in f.readlines():
            lineList = line.split()
            if len(lineList) >= 2:
                if lineList[0] == "application":
                    solver = lineList[1]
                    if solver[-1] == ";":
                        solver = solver[:-1]
                    break
        f.close()
    return solver

#
#  checkFolder
#    folderのチェック
def checkFolder(folderDir):
    ans = ""
    if os.path.isdir(folderDir) == True:
        ans = "folder"
        folderName = folderDir.split("/")[-1]
        try:
            a=float(folderName)
            #数字フォルダ
            ans = "hide"
        except:
            if (folderName == "system" or
            folderName == "constant" or
            #folderName == "model" or
            folderName == "solve.log.analyzed" or
            folderName[0] == "."):
                #禁止フォルダ
                ans = "hide"
            else:
                controlDict = folderDir + "/system/controlDict"
                if os.path.isfile(controlDict)==True:
                    #caseフォルダ
                    ans = "case"
                elif len(glob.glob(folderDir+"/*.hdf")) != 0:
                    ans = "salome" 
    else:
        #ファイル
        ans = "file"
    return ans

#
#  getTreeDirOnlylRootChildren
#    HDの内容を直接読み込み
def getTreeDirOnlylRootChildren(rootDir):
    folderDir = []
    currDir = rootDir
    #rootのフォルダ取得
    folders = getFoldersOnlyCurrDir(rootDir)
    for folder in folders:
        folderDir.append(folder)
    #treeデータ作成
    folderDir.sort()
    i=0
    rootName = "root"
    treeData = [rootName,[]]
    while (i<len(folderDir)):
        folderDir[i] = rootName + folderDir[i][len(rootDir):]
        folderDir[i] = folderDir[i].split("/")
        i=i+1
    folderDir = checkTreeData(folderDir)
    for folder in folderDir:
        setTreeDataFolder(0, treeData, folder)
    treeData[0] = rootDir
    return treeData

#
#  getTreeDirUntilCurrCase
#    currCaseまでのtreeデータ取得
#    （HD内容を直接読み取り）
def getTreeDirUntilCurrCase(rootDir, selCaseDir):
    folderDir = []
    currDir = rootDir
    #rootのフォルダ取得
    folders = getFoldersOnlyCurrDir(rootDir)
    if len(folders) == 0:
        print _(u"rootDirがありません。\n　homeDirをrootDirに設定しました。")
        rootDir = os.environ["HOME"]
        currDir = rootDir
        folders = getFoldersOnlyCurrDir(rootDir)
    for folder in folders:
        folderDir.append(folder.decode("utf-8"))
        subFolders = getFoldersOnlyCurrDir(folder)
        for subFolder in subFolders:
            folderDir.append(subFolder.decode("utf-8"))
    #selCase系列のフォルダ取得
    selCaseNames = selCaseDir[len(rootDir)+1:].split("/")
    for name in selCaseNames:
        currDir = currDir + "/" + name
        folders = getFoldersOnlyCurrDir(currDir)
        for folder in folders:
            subFolders = getFoldersOnlyCurrDir(folder)
            for subFolder in subFolders:
                folderDir.append(subFolder)
    #treeデータ作成
    folderDir.sort()
    i=0
    rootName = "root"
    treeData = [rootName,[]]
    while (i<len(folderDir)):
        folderDir[i] = rootName + folderDir[i][len(rootDir):]
        folderDir[i] = folderDir[i].split("/")
        i=i+1
    folderDir = checkTreeData(folderDir)
    for folder in folderDir:
        setTreeDataFolder(0, treeData, folder)
    treeData[0] = rootDir
    return treeData


#
#　getFoldersOnlyCurrDir
#  　currDirのみのFolderを取得する
#
def getFoldersOnlyCurrDir(currDir):
    folders = []
    try:
        allItem = os.listdir(currDir)
        for item in allItem:
            newFolder = currDir + "/" + item
            if os.path.isdir(newFolder) == True:
                folders.append(newFolder)
    except:
        folders=[]
    return folders

#
#  禁止folderを削除
#   数字、system、constant、model、solve.log.analyzed、OpenFOAM
#   隠しフォルダ
def checkTreeData(folderDir):
    ans = []
    for directory in folderDir:
        flag = 0
        for folder in directory:
            try:
                a=float(folder)
                flag = 1
                break
            except:
                a=folder

            if folder == "":
                flag = 1
            else:
                if (folder == "system" or
                  folder == "constant" or
                  #folder == "model" or
                  folder == "solve.log.analyzed" or
                  folder[0] == "."):
                    flag = 1
                    break
        if flag == 0:
            ans.append(directory)
    return ans

def setTreeDataFolder(nest,tree, folder):
    nest = nest + 1
    if nest > 50:
        print _(u"フォルダのネスティングが深すぎます。")
        return
    #親folderまでskipする。
    parentDir = folder[nest]
    ii=0
    while ii<len(tree[1]):
        if type(tree[1][ii]) == list:
            if tree[1][ii][0] == parentDir:
                setTreeDataFolder(nest, tree[1][ii], folder)
                break
        else:
            if tree[1][ii] == parentDir:
                tree[1][ii] = [parentDir, []]
                setTreeDataFolder(nest, tree[1][ii], folder)
                break
        ii=ii+1
    #親folderの位置ならchildを追加する。
    if nest == len(folder)-1:
        parent = folder[-2]
        child = folder[-1]
        tree[1].append(child)
#^^^^^^^^^^^^^^^^^^ TreeDataの取得 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#------------------ GridEditorの非表示設定関係 ----------------------------------
#
# getSaveDisplayFieldsFileName
#    非表示設定ファイル名(directory)の取得
def getSaveDisplayFieldsFileName(dataDir):
    from treefoam import displayFieldsFile
    dataDirList = dataDir.split("/")
    caseDir = ""
    subFolder = 0
    try:
        a = float(dataDirList[-1])
        caseDir = string.join(dataDirList[:-1], "/")
    except:
        caseDir = string.join(dataDirList[:-2], "/")
        subFolder = 1
    timeFolders = getTimeFolders(caseDir)
    saveDir = caseDir + "/" + timeFolders[0]
    if subFolder == 1:
        saveDir += "/" + dataDirList[-1]
        #dirがあるか確認（timeFolders[0]内にregionがあるか確認）
        if len(glob.glob(saveDir)) == 0:
            #無ければ、dataDirに設定
            saveDir = dataDir
    #fileName = saveDir + "/" + treefoam.displayFieldsFile
    fileName = saveDir + "/" + displayFieldsFile
    return fileName

#  getDisplayFields
#    表示設定ファイルから表示field名を取得
def getDisplayFields(fileName):
    f=open(fileName)
    contents = f.read()
    f.close()
    fields = contents.split()
    return fields

#  saveDisplayFieldsIfFile
#    非表示設定ファイルが存在するならば表示field名を書き込む
def saveDisplayFieldsIfFile(colNames, dataDir):
    fileName = getSaveDisplayFieldsFileName(dataDir)
    if len(glob.glob(fileName)) != 0:
        saveDisplayFields(colNames, dataDir)

#  saveDisplayFields
#    表示設定ファイルに表示field名を書き込む
def saveDisplayFields(colNames, dataDir):
    fileName = getSaveDisplayFieldsFileName(dataDir)
    f=open(fileName, "w")
    for name in colNames:
        line = name + "\n"
        f.write(line)
    f.close()

