#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  setFoamContents
#      Foam形式のデータを作成する
#
#       10/03/04    新規作成
#       13/04/01    remakeTergetBoundaryFieldsのバグ修正。
#                   boundaryFieldが書き換わらない時がある。読み込みと書き込みのfを変更
#          04/06    clearNullBoundaryFieldsを追加（空patchのクリア）
#          06/01    setSurfaceFeatureExtractDictを追加
#          07/13    setFieldsDictのdefaultの内容を修正（boxToCellを追加）
#          10/15    国際化のため修正
#          12/21    stdout、stderrの設定（import logFileCreater）を追加
#       14/11/04    makeMapFieldsDict追加
#          12/05    decomposeParDictを修正
#       15/10/02    clearNullBoundaryAllFields, clearNullBoundaryFields,
#                   setNullBoundaryField, clearBoundaryFields, createBoundaryを修正
#                   （empty, symmetryの場合を追加）
#          11/30    setNullBoundaryField:バグ修正（Bcontの内容に空白と"\n"を追加）
#                   makeDecomposeParDictDefault:作成方法を変更。（fileから読み込む。）
#          12/2     makeMapFieldsDict:作成方法を変更（fileから読み込む）
#       16/12/31    makeRegionProperties:OF-ext用に修正
#                   getOFversionFolder:verFolder取得方法を修正
#       17/02/01    makeNullBoundaryFields:mass→messに訂正（バグ修正）
#

import os
import glob
import re
import getFoamContents
import pyTreeFoam
from editBoundaryConditionsDialog import readConfigData as readGridEditorConfig

import logFileCreater; logFileCreater.log()


workdir = ""
Pname = []

#
#  makeMapFieldsDict
#    mapFieldsDict作成
def makeMapFieldsDict():
    verFolderName = getOFversionFolder()
    fileName = verFolderName + "/system/mapFieldsDict"
    f=open(fileName); cont=f.read(); f.close()
    return cont

#
#  makeRegionProperties
#    regionPropertiesを作成・保存
def makeRegionProperties(caseDir, fluidRegions, solidRegions):
    config = pyTreeFoam.readConfigTreeFoam()
    OFversion = config[0]
    if OFversion >= "2.2.0" and OFversion[:3] != "ext":
        #version 2.2.0以降の場合
        #foamFile作成
        foamFile = makeFoamHeaderVersion(OFversion)
        foamFile += makeFoamFile("2.2", "ascii", "dictionary",
                    "constant", "regionProperties")
        footer = makeFooter()

        #regionPropertiesの内容作成
        contents = "regions\n(\n"
        contents += "    fluid       ( "
        for name in fluidRegions:
            contents += name + " "
        contents += ")\n"

        contents += "    solid       ( "
        for name in solidRegions:
            contents += name + " "
        contents += ")\n);\n"
    else:
        #version 2.1.1以前の場合
        #foamFile作成
        foamFile = makeFoamHeaderVersion(OFversion)
        foamFile += makeFoamFile("2.0", "ascii", "dictionary",
                    "constant", "regionProperties")
        footer = makeFooter()

        #regionPropertiesの内容作成
        contents = "fluidRegionNames ( "
        for name in fluidRegions:
            contents += name + " "
        contents += ");\n\n"
        contents += "solidRegionNames ( "
        for name in solidRegions:
            contents += name + " "
        contents += ");\n"

    #書き込み
    contentsDict = foamFile + contents + footer
    fileName = caseDir + "/constant/regionProperties"
    f = open(fileName, "w")
    f.write(contentsDict)
    f.close()


#
#  setSurfaceFeatureExtractDict
#    defaultのsurfaceFeatureExtractDictを作成する
def setSurfaceFeatureExtractDict():
    cont = (
"""xxxx.stl         //stl file. if many files, copy this area at each stl file.
{
    // How to obtain raw features (extractFromFile || extractFromSurface)
    extractionMethod    extractFromSurface;

    extractFromSurfaceCoeffs
    {
        // Mark edges whose adjacent surface normals are at an angle less
        // than includedAngle as features
        // - 0  : selects no edges
        // - 180: selects all edges
        includedAngle   150;
    }

    subsetFeatures
    {
        // Keep nonManifold edges (edges with >2 connected faces)
        nonManifoldEdges       no;

        // Keep open edges (edges with 1 connected face)
        openEdges       yes;
    }

    // Write options

    //   Write features to obj format for postprocessing
    writeObj                yes;
}
"""
        )
    return cont

#
#  setMachinesDefault
#    defaultのmachinesを作成する
#
def setMachinesDefault():
    wdir = os.getcwd()
    cont = makeMachinesDefault()
    fileName = "./machines"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def makeMachinesDefault():
    cont = """#  discript subDomain name and number of CPUs.
#  subDomain name is discripted host name in host of '/etc/hosts'.
#
# Example:
#localhost cpu=4
#host1 cpu=2
#host2

localhost cpu=4
"""
    return cont


#
#  setDecomposeParDictDefault
#    defaultのdecomposeParDictを作成する
#
def setDecomposeParDictDefault():
    wdir = os.getcwd()
    cont = makeDecomposeParDictDefault()
    fileName = wdir + "/system/decomposeParDict"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def makeDecomposeParDictDefault():
    verFolderName = getOFversionFolder() + "/system/decomposeParDict"
    f=open(verFolderName); cont=f.read(); f.close()
    return cont
    
#
#  getOFversionFolder
#    TreeFoam内のOFDictフォルダ名を取得
def getOFversionFolder():
    verDir = os.getenv("TreeFoamPath") + "/data/OFDict"
    folders = glob.glob(verDir + "/*")
    vers = []
    for folder in folders:
        name = folder.split("/")[-1]
        vers.append("-".join(name.split("-")[1:]))
    vers.sort()
    OFver = pyTreeFoam.getOFversion()
    currVer = vers[0]
    for ver in vers:
        if ver <= OFver:
            currVer = ver
        else:
            break
    ans = verDir + "/OF-" + currVer
    return ans

#
#  setFieldsDictDefault
#    defaultのsetFieldsDictを作成する
#
def setFieldsDictDefault():
    wdir = os.getcwd()
    files = glob.glob(wdir + "/system")
    if len(files) != 0:
        #setFieldsDictの作成
        cont = makeSetFieldsDictDefault()
        fileName = "./system/setFieldsDict"
        f=open(fileName,"w")
        f.write(cont)
        f.close
    else:
        print _(u"systemフォルダがないので、「setFieldsDict」が作れません。")

def makeSetFieldsDictDefault():
    cont = makeFoamHeader()
    location = "system"
    foamFile = makeFoamFile("2.0", "ascii", "dictionary", location, "setFieldsDict" )
    cont = cont + foamFile
    a = (
'''defaultFieldValues
(
    volScalarFieldValue xxxxx 0                 // scalar
    volVectorFieldValue yyyyy (0 0 0)           // vector
);

regions
(
    cellToCell
    {
        set cellset_name1;                     //cellSetName
        fieldValues
        (
            volScalarFieldValue xxxxx 10
            volVcalarFieldValue yyyyy (1 1 1)
        );
    }

    boxToCell
    {
        box (0 0 0) (1 1 1);                     //cellSetName
        fieldValues
        (
            volScalarFieldValue xxxxx 1
            volVcalarFieldValue yyyyy (2 2 2)
        );
    }
);

// ************************************************************************* //
'''
        )
    cont = cont + a
    return cont


#
#  controlDict
#    controlDictの内容を書き換える
#    items: [[item, value], [item, value], ...]
def controlDict(wdir, items):
    fileName = wdir + "/controlDict"
    f = open(fileName)
    contents = f.readlines()
    f.close()
    for item in items:
        n=0
        while n < len(contents):
            try:
                i = contents[n].index(item[0])
                if i == 0:
                    line = exchangeValue(item[1], contents[n])
                    contents[n] = line
                n=n+1
            except:
                n=n+1
    f = open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()
    
def exchangeValue(value, cont):
    a = cont.split()
    i = cont.index(a[1])
    line = cont[:i] + value + ";" + cont[i+len(a[1]):]
    return line


#
# makeFoamHeaderVersion
#
def makeFoamHeaderVersion(version):
    a=(
"""/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\\    /   O peration     | Version:  """ + version + """                                 |
|   \\\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
"""
      )
    return a
    
#
# makeFoamHeader
#
def makeFoamHeader():
    a=(
"""/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\\    /   O peration     | Version:  1.6.x                                 |
|   \\\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
"""
      )
    return a

#
#  makeFoamFile
#
def makeFoamFile(version, format, className, location, objectName):
    a=  "FoamFile\n"
    a=a+"{\n"
    a=a+"    version     " + version + ";\n"
    a=a+"    format      " + format + ";\n"
    a=a+"    class       " + className + ";\n"
    a=a+'    location    "' + location + '";\n'
    a=a+"    object      " + objectName + ";\n"
    a=a+"}\n"
    a=a+"// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n"
    a=a+"\n"
    return a

# makeCellZonesMiddle
def makeCellZonesMiddle():
    a=  "    type cellZone;\n"
    a=a+"cellLabels      List<label>\n"
    return a

# makeFaceZonesMiddle
def makeFaceZonesMiddle():
    a=  "    type faceZone;\n"
    a=a+"faceLabels      List<label>\n"
    return a

# makeFooter
def makeFooter():
    a="\n//************************************************************************* //\n"
    return a

#
#  clearNullBoundaryAllFields
#
#     空白patchのみのboundaryFieldのクリア
#       filedDir:       各fieldのDir　 例(.../barCut/0)
#       polyMeshDir:    polyMeshのDir　例(.../barCut/constant)
def clearNullBoundaryAllFields(fieldDir, polyMeshDir):
    fields = pyTreeFoam.getFieldNames(fieldDir)
    if len(fields) == 0:
        return

    # polyMeshのdirをチェック
    words = polyMeshDir.split("/")
    if words[-1] == "polyMesh":
        polyMeshDir = "/".join(words[:-1])
    # patch name の取得
    PnameList = getFoamContents.PatchNameFromBoundary(polyMeshDir)
    if PnameList != None:
        Pname = []
        Ptype = []
        for name in PnameList:
            Pname.append(name[0])
            Ptype.append(name[1])
        # boundary field の作成
        #zero = "type    zeroGradient;"
    else:
        return

    #field毎に確認修正
    print _(u"以下の空patchを「zeroGradient, empty, symmetry」で定義します。")
    changePatches = []
    for field in fields:
        [dummy, patches, Bcont] = pyTreeFoam.getFieldsPatchesName_sub(
                                    fieldDir, [field])
        #空patchのチェック
        setFlag = 0
        for newPatch in Pname:
            for oldPatch in patches:
                same = "no"
                if newPatch == oldPatch:
                    same = "yes"
                    break
            if same == "no":
                setFlag = 1
                changePatches.append(newPatch)
                print "  " + field + ": " + newPatch
        if setFlag == 0:
            pass
        else:
            [dummy, newBcont] = setNullBoundaryField([field], Pname, patches,
                                    Bcont, Ptype)
            error = makeNullBoundaryFields(fieldDir, [field], Pname, newBcont)
    if len(changePatches) > 0:
        print _(u"「zeroGradient, empty, symmetry」で定義しました。")
    else:
        print _(u"  空patchはありません。")

#
#  clearNullBoundaryFields
#
#     空白patchのみのboundaryFieldのクリア（Flistのfieldが同じpatch名の場合）
#       filedDir:       各fieldのDir　 例(.../barCut/0)
#       polyMeshDir:    polyMeshのDir　例(.../barCut/constant)
#       Flist:          クリアするFieldのみのlist [field1, field2, ...]
def clearNullBoundaryFields(fieldDir, polyMeshDir, Flist):
    if len(Flist) == 0:
        return
    # polyMeshのdirをチェック
    words = polyMeshDir.split("/")
    if words[-1] == "polyMesh":
        polyMeshDir = "/".join(words[:-1])
    # patch name の取得
    PnameList = getFoamContents.PatchNameFromBoundary(polyMeshDir)
    if PnameList != None:
        Pname = []
        Ptype = []
        for name in PnameList:
            Pname.append(name[0])
            Ptype.append(name[1])
        # boundary field の作成
        zero = "type    zeroGradient;"
        [dummy, patches, Bcont] = pyTreeFoam.getFieldsPatchesName_sub(
                                    fieldDir, Flist)
        #空patchのチェック
        setFlag = 0
        print _(u"以下の空patchを「zeroGradient, empty, symmetry」で定義します。")
        for newPatch in Pname:
            for oldPatch in patches:
                same = "no"
                if newPatch == oldPatch:
                    same = "yes"
                    break
            if same == "no":
                setFlag = 1
                print "  " + newPatch
        if setFlag == 0:
            print _(u"  空patchはありません。")
        else:
            [dummy, newBcont] = setNullBoundaryField(Flist, Pname, patches,
                                    Bcont, Ptype)
            error = makeNullBoundaryFields(fieldDir, Flist, Pname, newBcont)

def setNullBoundaryField(fields, newPatches, patches, Bcont, Ptype):
    newBcontents = []
    f=0
    for field in fields:
        newBcont = []
        p = 0
        for newPatch in newPatches:
            same = 0
            i=0
            for patch in patches:
                if newPatch == patch:
                    value = pyTreeFoam.reformTabContents(Bcont[f][i] + "\n")
                    newBcont.append(value)
                    same = 1
                    break
                i += 1
            if same == 0:
                value = getBoundaryContFromPatchType(Ptype[p])
                #valueを整形する。
                value = pyTreeFoam.reformTabContents(value + "\n")  #追加
                newBcont.append(value)
            p += 1
        newBcontents.append(newBcont)
        f += 1
    return [fields, newBcontents]

def makeNullBoundaryFields(fieldDir, fields, patches, Bconts):
    #patchNameを整形（空白追加）
    patchesSp = []
    for patch in patches:
        patchesSp.append("    " + patch)
    #各fieldのboundaryFieldを作成
    [a1, a2, a3, nMaxLines] = readGridEditorConfig()
    i=0
    mess = ""
    while i<len(fields):
        #fieldをオープン
        fileName = fieldDir + "/" + fields[i]
        contents = getFoamContents.readField(fileName, nMaxLines)
        #patchDataから「boundaryField」を作成
        newBoundary = pyTreeFoam.makeBoundaryField(patchesSp, Bconts[i])
        #「boundaryField」を置き換える
        newContents = pyTreeFoam.replaceBoundaryField(newBoundary, contents)
        if newContents != "":
            getFoamContents.writeField(fileName, newContents)
        else:
            print "error:", fields[i], _(u"が置き換えできませんでした。")
            mess += fields[i] + ","
        i += 1
    if mess == "":
        error = ["OK", ""]
    else:
        error = ["NG", mess[:-1] + _(u"が書き込めません。")]
    return error

#
#  clearBoundaryFields
#
#     boundaryFieldのクリア
#       filedDir:       各fieldのDir
#       polyMeshDir:    polyMeshのDir
#       Flist:          クリアするFieldのみのlist [field1, field2, ...]
def clearBoundaryFields(fieldDir, polyMeshDir, Flist):
    # patch name の取得
    PnameList = getFoamContents.PatchNameFromBoundary(polyMeshDir)
    if PnameList != None:
        Pname = []
        Ptype = []
        for name in PnameList:
            Pname.append(name[0])
            Ptype.append(name[1])
        # boundary field の作成
        cont = createBoundary(Pname, Ptype)
        # field の作成
        remakeBoundaryFields(fieldDir, Flist, cont)

#
#  clearInternalFields
#
def clearInternalFields(fieldDir, polyMeshDir, Flist):
    contAns = []
    if len(Flist) != 0:
        print _(u"internal field をクリアします...")
        [a1, a2, a3, nMaxLines] = readGridEditorConfig()
        for name in Flist:
            print "    " + name + "..."
            #fileName = getFoamContents.checkFileName(fieldDir + "/" + name)
            fileName = fieldDir + "/" + name
            cont = getFoamContents.readField(fileName, nMaxLines)
            p = getFoamContents.skipFoamFileMem(cont, 0)
            n = cont[p:].find("internalField")
            if n >= 0:
                ps = p + n
                n = cont[ps:].find(";")
                if n >= 0:
                    pe = ps + n + 1
                    line = cont[ps:pe]
                    newLine = clearInternalField(line)
                    newCont = cont[:ps] + newLine + cont[pe:]
                    getFoamContents.writeField(fileName, newCont)
        print _(u"internal field をクリアしました。")

def clearInternalField(line):
    [dummy, ps] = getFoamContents.getKeywordMem(line, 0)
    [valForm, p] = getFoamContents.getKeywordMem(line, ps)
    if valForm == "nonuniform":
        valType = line[p:p+100]
        if valType.find("scalar") >= 0:
            val = "0"
        elif valType.find("vector") >= 0:
            val = "(0 0 0)"
        elif valType.find("symmTensor") >= 0:
            val = "(0 0 0 0 0 0)"
        elif valType.find("tensor") >= 0:
            val = "(0 0 0 0 0 0 0 0 0)"
        else:
            val = "0"
        newLine = line[:ps] + " uniform " + val + ";\n"
        line = newLine
    return line


def createBoundary(Pname,Ptype):
    cont = ""
    i = 0
    for name in Pname:
      Bcont = getBoundaryContFromPatchType(Ptype[i])
      cont = cont + "    " + name + "\n"
      cont = cont + "    {\n"
      cont = cont + "        " + Bcont + "\n"
      cont = cont + "    }\n"
      i += 1
    cont = "{\n" + cont + "}\n"
    return cont

def getBoundaryContFromPatchType(pType):
    if (pType == "empty;" or pType == "symmetry;" or pType == "symmetryPlane;" or
        pType == "cyclic;" or pType == "cyclicAMI;" or pType == "cyclicACMI;" or
        pType == "cyclicSlip;" or pType == "wedge;"):
        pCont = "type    " + pType
    else:
        pCont = "type    zeroGradient;"
    return pCont

def remakeBoundaryFields(fieldDir, Flist, cont):
    fieldNames = []
    i=0
    while i < len(Flist):
        fieldNames.append(Flist[i])
        i=i+1
    remakeTergetBoundaryFields(fieldDir, fieldNames, cont)

def remakeTergetBoundaryFields(fieldDir, fieldNames, cont):
    print fieldDir, _(u"内の各Fieldをクリアします。")
    [a1, a2, a3, nMaxLines] = readGridEditorConfig()
    for name in fieldNames:
        print "    " + name + "..."
        #fileName = getFoamContents.checkFileName(fieldDir + "/" + name)
        fileName = fieldDir + "/" + name
        fileCont = getFoamContents.readField(fileName, nMaxLines)
        ps = getFoamContents.getKeywordPointerMem("boundaryField", fileCont, 0)
        [boundaryCont, p] = getFoamContents.getKeywordContentsMem(
                            "boundaryField", fileCont, ps)
        if len(boundaryCont) != "":
            contAll = fileCont[:ps] + "boundaryField\n" + cont + fileCont[p:]
        else:
            print _(u"      baoundaryFieldがありません。追加しました。")
            contAll = fileCont + "boundaryField\n" + cont
            contAll += "// ************************************************************************* //\n"
        getFoamContents.writeField(fileName, contAll)
    print _(u"boundaryFieldをクリアしました。")


