#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   setHelyxosPath.py
#
#       HelyxOSのcaseDirの設定やversionを設定する
#
#       12/11/21    新規作成
#       13/10/15    国際化のため修正
#          12/21    stdout、stderrの設定（import logFileCreater）を追加
#       14/05/14    HelyxOSのversion取得方法を変更
#

import sys
import os
import pyTreeFoam

import logFileCreater; logFileCreater.log()


#
#  getHelyxVersion
#    HelyxOSのバージョン取得
def getHelyxVersion():
    #環境変数から取得
    version = os.getenv("HelyxOSversion")
    if version != None:
        return version

    #起動ファイルから取得
    version = ""
    config = pyTreeFoam.readConfigTreeFoam()
    fileName = config[8]
    f=open(fileName); cont=f.readlines(); f.close()
    for line in cont:
        words = line.split()
        if len(words) > 1:
            if words[1][:len("HelyxOSversion=")] == "HelyxOSversion=":
                temp = words[1][len("HelyxOSversion="):]
                version = temp
                return version

    #設定ファイルから取得
    fileName = os.environ["HOME"]+"/.java/.userPrefs/eu/engys/helyx/prefs.xml"
    try:
        f=open(fileName); cont=f.readlines(); f.close()
    except:
        print fileName
        print _(u"-->設定ファイルが開けません")
        return version

    for line in cont:
        p = line.find('MAP_XML_VERSION=')
        if p>=0:
            p += len('MAP_XML_VERSION=')
            wlist = line[p:].split('"')
            version = wlist[1]
            break
    return version

#
#  setHelyxPath
#    HelyxOSのpath設定
def setHelyxPath(caseDir, setFile):
    fileName = setFile
    print fileName
    try:
        f=open(fileName); cont = f.readlines(); f.close()
    except:
        print _(u"-->設定ファイルが開けません")
        return
    i = 0
    flag = 0
    for line in cont:
        p = line.find('last.open.dir" value=')
        if p>=0:
            p += len('last.open.dir" value=')
            wlist = line[p:].split('"')
            wlist[1] = caseDir
            lastline = '"'.join(wlist)
            newline = line[:p] + lastline
            cont[i] = newline
            flag = 1
            break
        i += 1
    if flag == 1:
        f=open(fileName, "w")
        for line in cont:
            f.write(line)
        f.close()


if __name__ == "__main__":
    import gettext
    treeFoamPath = os.environ["TreeFoamPath"]
    gettext.install("treefoam", treeFoamPath + "/data/locale")

    caseDir = sys.argv[1]
    setFile = sys.argv[2]
    setHelyxPath(caseDir, setFile)


