#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   treefoam.py
#
#       OpenFOAM用のランチャ
#
#       programed by s.fujii
#       11/04/15    新規作成（ver 1.0）
#          05/08    editBoundary(grid)を追加（ver 1.1）
#          08/17    項目のダブルクリックでフォルダopen、plotWatcher追加（ver1.2）
#       12/01/01    ﾊﾞｸﾞ修正
#                   reloadで数字ﾌｫﾙﾀﾞが存在しない場合、ｴﾗｰが発生していた
#                   gridEditor:boundaryFieldにｺﾒﾝﾄ文がある時、正常に書き込めなかった
#                   フォルダの追加削除を可能にした
#       12/01/03    gridEditorでpatchの削除を可能にした
#          01/06    ﾊﾞｸﾞ修正（ﾌｫﾙﾀﾞ名変更時、修正しない時にｴﾗｰが発生）
#          01/08    gridEditorでのpatch名の変更を可能にした
#          01/15    ﾊﾞｸﾞ修正（最初のpatch削除できなかった）。helpを表示
#          01/17    ﾊﾞｸﾞ修正（patch削除時ｺﾞﾐが残る）、ﾓﾃﾞﾙ倍率の保存方法修正
#          01/21    gridEditorｺﾏﾝﾄﾞ追加。これに伴いﾗﾝﾁｬを修正（ver1.21）
#          01/22    caseOpen, cntDictOpenボタン追加
#          02/05    windowClose時に終了処理追加
#          02/12    gridEditor:patch変更、削除時にglobal変数設定（ﾊﾞｸﾞ修正）
#          02/13    folder削除時のﾊﾞｸﾞ修正(削除するfolderが解析Caseの場合ｴﾗｰ発生)
#          02/18    treefoam,gridEditorにﾒﾆｭｰｷｰとｼｮｰﾄｶｯﾄkey追加
#          02/19    folderのｺﾋﾟｰ、貼付けを追加（ver1.22)
#          02/25    gridEditor:cellData読込みをtimerEventで実施
#          02/26    windowSizeを保存する様に変更。ﾌｫﾙﾀﾞ削除時のﾊﾞｸﾞ修正
#          03/10    iconNameを変数に変更
#          03/25    gridEditor直接起動メニューとアイコンを追加
#          04/08    フォントサイズをsystemFontのサイズに設定。copy時にgaugeを使用。
#                   colWidthを保存。gridEditor:save時にgaugeを使用。
#          04/22    gridEditor:fieldの表示設定追加（ver1.23)
#          04/23    gridEditor:読込、保存時のｴﾗｰに対応
#          05/01    editorの設定追加、HD残り容量表示、表示を縞模様表示に、並列計算追加
#                   gridEditorの起動方法変更
#          05/13    folderCopyを修正（同じfolderを同じ場所に10回以上copyは禁止）
#          05/20    folder削除を2回繰り返すとｴﾗｰ発生していた事を修正。
#                   rootDirは削除できない様に修正。
#                   configTreeFoamの修正用ｺﾏﾝﾄﾞ追加（ver1.24）
#          05/21    BOLD表示を削除（文字長さが変わる為）
#          06/02    openGridEditorDialogを修正（.displayFieldの読込み）
#          06/09    「gridEditor」ｺﾏﾝﾄﾞを修正
#          06/24    caseコピーを追加。folder複数選択可能に設定。deleteをゴミ箱へ変更。
#          07/15    popupﾒﾆｭｰを充実（ver1.25）
#          07/21    gridEDitor:第1番目のpatch名変更時のｴﾗｰ対応
#          08/05    gridEditor:最大cell内行数を変更
#          08/11    gridEditor:patch名変更・削除時、cell変更チェック追加
#          08/14    gridEditor:field名表示部修正（field削除時、表示のみ削除できなかった）
#          08/16    gridEditor:初期化時にcell(0,0),(1,0)のｸﾘｱを追加
#          08/25    gridEditor:close時に「保存して終了」を追加
#          09/11    MakePatchDialog.py:OF-2.1.0以降createBafflesが変わった為、これに対応
#          09/13    popupﾒﾆｭｰを変更（ver1.26）
#          09/17    pyTreeFoamのバグ修正
#          09/23    gridEditorのバグ修正,importするmoduleを整理(segmentation fault対策）
#                   HelyxOS,結果folder削除ﾎﾞﾀﾝを追加（ver1.30）
#          10/13    結果フォルダの削除を修正（選択caseと解析caseが違っている時確認する）
#          10/14    toolTip表示を追加
#
#
#
#-----------code生成後の修正項目----------------
#  ・以下の2行をコメントアウトする。
#       self.tree_ctrl_1 = wx.TreeCtrl(...
#       sizer_1.Fit(self)
#
#   ・「size=treeSize」を追加
#       wx.Frame.__init__(self, size=treeSize, *args, **kwds)
#
#   ・アイコンのDirを書き直す（絶対path→相対path[./icons/***.png]）
#
#


# generated by wxGlade 0.6.3 on Sun Feb 20 15:26:34 2011

import wx
import wx.gizmos
import os
import os.path
import glob
import codecs
import shutil
import string
import gio

import pyTreeFoam
import meshOperationDialog
import setDataInFieldsDialog
import getFoamContents
import selectFieldsDialog
import selectFieldsForEditDialog
import gaugeDDialog
import runParallelDialog
import openGridEditorDialog
import changeConfigTreeFoamDDialog
import inputTextDDialog
import selectFolderFilesDialog
#from runHelyxosDialog import helyxosShow

version = "1.30-120924"
treeFoamPath = ""
OFversionFix = "1.7.0"
rootDirFix = ""
selCaseDirFix = ""
solveCaseFix = ""
paraFoamFix = ""
salomeMecaFix = ""
envOpenFOAMFix = ""
cadFix = ""
helyxosFix = ""
gridNo = []                 #gridのNo、Dirを一時保存
nTreeFoam = ""              #TreeFoamの起動数
treeSize = (800, 500)       #windowSize
fontSize = 10               #font size
colWidth = [300,            #Tree width
            200,            #solver width
            40,             #nR width
            60,             #st width
            60]             #ed width


rootDir = ""
selCaseDir = ""
selectTreeDir = ""
treeData = []
selCase = []
configFile = "configTreeFoam"
editName = ""
childrenList = []
timeFolders = ["0","1"]
clipBoard = ""
displayFieldsFile = ".displayField"     #表示させるfield名を保存するﾌｧｲﾙ名
editor = "gedit --new-window"           #使用するeditor（デフォルトの設定）
machinesFile = "machines"               #並列計算させる時のnode名記述ﾌｧｲﾙ
folderMsg = "no"                        #folderErrorMsgを禁止
toolTipText = ""                        #tree_ctrlのtoolTip表示内容

changeConfigTreeFoamID = wx.NewId()
changeRootID = wx.NewId()
reloadID = wx.NewId()
exitID = wx.NewId()
usageID = wx.NewId()
versionID = wx.NewId()
newCaseID = wx.NewId()
setCurrCaseID = wx.NewId()
gridEditorID = wx.NewId()
editFieldID = wx.NewId()
editPropertiesID = wx.NewId()
editDictID = wx.NewId()
editMeshID = wx.NewId()
renameCaseID = wx.NewId()
runSolverID = wx.NewId()
plotWatcherID = wx.NewId()
parallelSolverID = wx.NewId()
runFoamTermID = wx.NewId()
runParaFoamID = wx.NewId()
runFoamViewerID = wx.NewId()
runSalomeID = wx.NewId()
runCADID = wx.NewId()
dataSetClearID = wx.NewId()
FolderOpenID = wx.NewId()
copyFolderID = wx.NewId()
pasteFolderID = wx.NewId()
addFolderID = wx.NewId()
delFolderID = wx.NewId()
caseCopyID = wx.NewId()
casePasteID = wx.NewId()


# begin wxGlade: extracode
# end wxGlade

class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        global rootDir, selCaseDir, envOpenFOAMFix, fontSize
        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        #wx.Frame.__init__(self, *args, **kwds)
        wx.Frame.__init__(self, size=treeSize, *args, **kwds)
        self.panel_2 = wx.Panel(self, -1)
        self.panel_1 = wx.Panel(self, -1)
        
        # Menu Bar
        self.frame_1_menubar = wx.MenuBar()
        self.menuFile = wx.Menu()
        self.menuChangeConfigTreeFoam = wx.MenuItem(self.menuFile, changeConfigTreeFoamID, u"configTreeFoamの変更(&T)", u"TreeFoamの設定を変更します", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuChangeConfigTreeFoam)
        self.menuChangeRoot = wx.MenuItem(self.menuFile, changeRootID, u"rootの変更(&C)", u"rootDirectoryの変更", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuChangeRoot)
        self.menuReload = wx.MenuItem(self.menuFile, reloadID, u"再読み込み(&R)", u"treeDataを再読み込みします", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuReload)
        self.menuNewCase = wx.MenuItem(self.menuFile, newCaseID, u"新規case作成(&N)", u"tutorialからコピーして新しいcaseを作成します", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuNewCase)
        self.menuSetCurrCase = wx.MenuItem(self.menuFile, setCurrCaseID, u"解析Caseとして設定(&S)", u"選択しているcaseを解析Caseとして設定します", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuSetCurrCase)
        self.menuExit = wx.MenuItem(self.menuFile, exitID, u"閉じる(&Q)", "", wx.ITEM_NORMAL)
        self.menuFile.AppendItem(self.menuExit)
        self.frame_1_menubar.Append(self.menuFile, u"ファイル(&F)")
        self.menuEdit = wx.Menu()
        self.menuFolderOpen = wx.MenuItem(self.menuEdit, FolderOpenID, u"開く(&O)", u"解析Caseを開く", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuFolderOpen)
        self.menuGridEditor = wx.MenuItem(self.menuEdit, gridEditorID, u"gridEditor起動(&G)", u"gridEditoを起動します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuGridEditor)
        self.menuEditField = wx.MenuItem(self.menuEdit, editFieldID, u"Field編集(&F)", u"解析CaseのFieldを編集します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuEditField)
        self.menuEditProperties = wx.MenuItem(self.menuEdit, editPropertiesID, u"Properties編集(&P)", u"解析Caseのpropertiesを編集します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuEditProperties)
        self.menuEditDict = wx.MenuItem(self.menuEdit, editDictID, u"Dictionary編集(&D)", u"解析CaseのDictを編集します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuEditDict)
        self.menuEditMesh = wx.MenuItem(self.menuEdit, editMeshID, u"Mesh編集(&M)", u"解析Caseのmeshを編集（変換、scale変更等）します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuEditMesh)
        self.menuDataSetClear = wx.MenuItem(self.menuEdit, dataSetClearID, u"FieldDataSet又はclear(&S)", u"Fieldにデータセット、又はクリア", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuDataSetClear)
        self.menuEdit.AppendSeparator()
        self.menuCaseCopy = wx.MenuItem(self.menuEdit, caseCopyID, u"caseのコピー(&A)\tCtrl+A", u"caseデータをコピーする", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuCaseCopy)
        self.menuCopyFolder = wx.MenuItem(self.menuEdit, copyFolderID, u"フォルダコピー(&C)\tCtrl+C", u"フォルダをコピーします", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuCopyFolder)
        self.menuPasteFolder = wx.MenuItem(self.menuEdit, pasteFolderID, u"貼付け(&P)\tCtrl+V", u"フォルダやcaseの貼付け", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuPasteFolder)
        self.menuEdit.AppendSeparator()
        self.menuRenameCase = wx.MenuItem(self.menuEdit, renameCaseID, u"フォルダ名変更(&R)", u"解析Caseの名称を変更します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuRenameCase)
        self.menuAddFolder = wx.MenuItem(self.menuEdit, addFolderID, u"新しいフォルダ追加(&A)", u"新しいフォルダを追加します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuAddFolder)
        self.menuDelFolder = wx.MenuItem(self.menuEdit, delFolderID, u"フォルダ削除(&E)", u"folderを削除します", wx.ITEM_NORMAL)
        self.menuEdit.AppendItem(self.menuDelFolder)
        self.frame_1_menubar.Append(self.menuEdit, u"編集(&E)")
        self.menuSolve = wx.Menu()
        self.menuRunSolver = wx.MenuItem(self.menuSolve, runSolverID, u"計算開始(&G)", u"activeCaseの計算を開始します", wx.ITEM_NORMAL)
        self.menuSolve.AppendItem(self.menuRunSolver)
        self.menuPlotWatcher = wx.MenuItem(self.menuSolve, plotWatcherID, u"plotWatcher起動(&W)", u"plotWatcherを起動します。", wx.ITEM_NORMAL)
        self.menuSolve.AppendItem(self.menuPlotWatcher)
        self.menuParallelSolver = wx.MenuItem(self.menuSolve, parallelSolverID, u"並列計算(&P)", u"並列計算に関する設定を行います", wx.ITEM_NORMAL)
        self.menuSolve.AppendItem(self.menuParallelSolver)
        self.frame_1_menubar.Append(self.menuSolve, u"計算(&C)")
        self.menuTool = wx.Menu()
        self.menuRunCAD = wx.MenuItem(self.menuTool, runCADID, u"CADの起動(&C)", u"CADを起動します", wx.ITEM_NORMAL)
        self.menuTool.AppendItem(self.menuRunCAD)
        self.menuRunFoamTerm = wx.MenuItem(self.menuTool, runFoamTermID, u"FOAM端末の起動(&T)", u"FOAM端末を起動します", wx.ITEM_NORMAL)
        self.menuTool.AppendItem(self.menuRunFoamTerm)
        self.menuRunParaFoam = wx.MenuItem(self.menuTool, runParaFoamID, u"paraFoamの起動(&P)", u"paraFoamを起動します", wx.ITEM_NORMAL)
        self.menuTool.AppendItem(self.menuRunParaFoam)
        self.menuRunFoamViewer = wx.MenuItem(self.menuTool, runFoamViewerID, u"viewerの起動(&V)", u"viewerを起動します", wx.ITEM_NORMAL)
        self.menuTool.AppendItem(self.menuRunFoamViewer)
        self.menuRunSalome = wx.MenuItem(self.menuTool, runSalomeID, u"SalomeMecaの起動(&S)", u"SalomeMecaを起動します", wx.ITEM_NORMAL)
        self.menuTool.AppendItem(self.menuRunSalome)
        self.frame_1_menubar.Append(self.menuTool, u"ツール(&T)")
        self.menuHelp = wx.Menu()
        self.menuUsage = wx.MenuItem(self.menuHelp, usageID, u"使い方(&U)", u"使い方の表示", wx.ITEM_NORMAL)
        self.menuHelp.AppendItem(self.menuUsage)
        self.menuVersion = wx.MenuItem(self.menuHelp, versionID, u"バージョン表示(&V)", "", wx.ITEM_NORMAL)
        self.menuHelp.AppendItem(self.menuVersion)
        self.frame_1_menubar.Append(self.menuHelp, u"ヘルプ(&H)")
        self.SetMenuBar(self.frame_1_menubar)
        # Menu Bar end
        self.frame_1_statusbar = self.CreateStatusBar(1, wx.ST_SIZEGRIP)
        self.static_line_7 = wx.StaticLine(self, -1)
        self.bitmap_button_12 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/preferences-system.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_1 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/changeRoot.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_2 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/reload.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_3 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/newCase.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_4 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/setCase.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_16 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/exit.png", wx.BITMAP_TYPE_ANY))
        self.static_line_4 = wx.StaticLine(self, -1, style=wx.LI_VERTICAL)
        self.combo_box_1 = wx.ComboBox(self, -1, choices=["startTime", "latestTime"], style=wx.CB_DROPDOWN)
        self.bitmap_button_9 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/gridEditor.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_5 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/editFields.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_19 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/dataSetClear.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_6 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/editProperties-cons.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_7 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/editDict-sys.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_8 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/editMesh.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_18 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/helyxos-F2.png", wx.BITMAP_TYPE_ANY))
        self.static_line_5 = wx.StaticLine(self, -1, style=wx.LI_VERTICAL)
        self.bitmap_button_21 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/deleteResult.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_13 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/runSolver.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_14 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/plotWatcher.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_15 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/parallel.png", wx.BITMAP_TYPE_ANY))
        self.static_line_1 = wx.StaticLine(self, -1, style=wx.LI_VERTICAL)
        self.bitmap_button_20 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/runCAD.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_10 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/runFoamTerm.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_11 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/pqAppIcon22.png", wx.BITMAP_TYPE_ANY))
        self.bitmap_button_17 = wx.BitmapButton(self, -1, wx.Bitmap("./icons/salome_default.png", wx.BITMAP_TYPE_ANY))
        self.static_line_6 = wx.StaticLine(self, -1)
        self.label_1 = wx.StaticText(self, -1, u"    OpenFoam環境:", style=wx.ALIGN_RIGHT)
        self.label_6 = wx.StaticText(self, -1, "label_6")
        self.label_2 = wx.StaticText(self, -1, "  case directory:", style=wx.ALIGN_RIGHT)
        self.label_3 = wx.StaticText(self.panel_1, -1, "label_3")
        self.label_4 = wx.StaticText(self, -1, u" 現在の解析case名:", style=wx.ALIGN_RIGHT)
        self.button_1 = wx.Button(self.panel_2, -1, "open")
        self.label_5 = wx.StaticText(self.panel_2, -1, "label_5")
        self.button_2 = wx.Button(self.panel_2, -1, "open")
        self.label_7 = wx.StaticText(self.panel_2, -1, "label_7")
        #self.tree_ctrl_1 = wx.TreeCtrl(self, -1, style=wx.TR_HAS_BUTTONS|wx.TR_NO_LINES|wx.TR_MULTIPLE|wx.TR_MULTIPLE|wx.TR_DEFAULT_STYLE|wx.SUNKEN_BORDER)

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_MENU, self.selChangeConfigTreeFoam, self.menuChangeConfigTreeFoam)
        self.Bind(wx.EVT_MENU, self.selChangeRoot, self.menuChangeRoot)
        self.Bind(wx.EVT_MENU, self.selReload, self.menuReload)
        self.Bind(wx.EVT_MENU, self.selNewCase, self.menuNewCase)
        self.Bind(wx.EVT_MENU, self.selSetCurrCase, self.menuSetCurrCase)
        self.Bind(wx.EVT_MENU, self.selExit, self.menuExit)
        self.Bind(wx.EVT_MENU, self.selFolderOpen, self.menuFolderOpen)
        self.Bind(wx.EVT_MENU, self.selGridEditor, self.menuGridEditor)
        self.Bind(wx.EVT_MENU, self.selEditField, self.menuEditField)
        self.Bind(wx.EVT_MENU, self.selEditProperties, self.menuEditProperties)
        self.Bind(wx.EVT_MENU, self.selEditDict, self.menuEditDict)
        self.Bind(wx.EVT_MENU, self.selEditMesh, self.menuEditMesh)
        self.Bind(wx.EVT_MENU, self.selDataSetClear, self.menuDataSetClear)
        self.Bind(wx.EVT_MENU, self.selCaseCopy, self.menuCaseCopy)
        self.Bind(wx.EVT_MENU, self.selCopyFolder, self.menuCopyFolder)
        self.Bind(wx.EVT_MENU, self.selPasteFolder, self.menuPasteFolder)
        self.Bind(wx.EVT_MENU, self.selRenameCase, self.menuRenameCase)
        self.Bind(wx.EVT_MENU, self.selAddFolder, self.menuAddFolder)
        self.Bind(wx.EVT_MENU, self.selDeleteFolder, self.menuDelFolder)
        self.Bind(wx.EVT_MENU, self.selRunSolver, self.menuRunSolver)
        self.Bind(wx.EVT_MENU, self.selPlotWatcher, self.menuPlotWatcher)
        self.Bind(wx.EVT_MENU, self.selParallelSolver, self.menuParallelSolver)
        self.Bind(wx.EVT_MENU, self.selRunCAD, self.menuRunCAD)
        self.Bind(wx.EVT_MENU, self.selRunFoamTerm, self.menuRunFoamTerm)
        self.Bind(wx.EVT_MENU, self.selRunParaFoam, self.menuRunParaFoam)
        self.Bind(wx.EVT_MENU, self.selRunFoamViewer, self.menuRunFoamViewer)
        self.Bind(wx.EVT_MENU, self.selRunSalome, self.menuRunSalome)
        self.Bind(wx.EVT_MENU, self.selUsage, self.menuUsage)
        self.Bind(wx.EVT_MENU, self.selVersion, self.menuVersion)
        self.Bind(wx.EVT_BUTTON, self.onChangeConfigTreeFoam, self.bitmap_button_12)
        self.Bind(wx.EVT_BUTTON, self.onChangeRoot, self.bitmap_button_1)
        self.Bind(wx.EVT_BUTTON, self.onReload, self.bitmap_button_2)
        self.Bind(wx.EVT_BUTTON, self.onNewCase, self.bitmap_button_3)
        self.Bind(wx.EVT_BUTTON, self.onActiveCase, self.bitmap_button_4)
        self.Bind(wx.EVT_BUTTON, self.onExit, self.bitmap_button_16)
        self.Bind(wx.EVT_BUTTON, self.onGridEditor, self.bitmap_button_9)
        self.Bind(wx.EVT_BUTTON, self.onEditField, self.bitmap_button_5)
        self.Bind(wx.EVT_BUTTON, self.onDataSetClear, self.bitmap_button_19)
        self.Bind(wx.EVT_BUTTON, self.onEditProperties, self.bitmap_button_6)
        self.Bind(wx.EVT_BUTTON, self.onEditDict, self.bitmap_button_7)
        self.Bind(wx.EVT_BUTTON, self.onEditMesh, self.bitmap_button_8)
        self.Bind(wx.EVT_BUTTON, self.onHelyxos, self.bitmap_button_18)
        self.Bind(wx.EVT_BUTTON, self.onDeleteResult, self.bitmap_button_21)
        self.Bind(wx.EVT_BUTTON, self.onRunSolver, self.bitmap_button_13)
        self.Bind(wx.EVT_BUTTON, self.onPlotWatcher, self.bitmap_button_14)
        self.Bind(wx.EVT_BUTTON, self.onParallel, self.bitmap_button_15)
        self.Bind(wx.EVT_BUTTON, self.onRunCAD, self.bitmap_button_20)
        self.Bind(wx.EVT_BUTTON, self.onRunFoamTerm, self.bitmap_button_10)
        self.Bind(wx.EVT_BUTTON, self.onRunParaFoam, self.bitmap_button_11)
        self.Bind(wx.EVT_BUTTON, self.onRunSalome, self.bitmap_button_17)
        self.Bind(wx.EVT_BUTTON, self.onCaseOpen, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.onCntDictOpen, self.button_2)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onItemActivated, self.tree_ctrl_1)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.onItemCollapsed, self.tree_ctrl_1)
        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.onItemExpanded, self.tree_ctrl_1)
        # end wxGlade
        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.onSelChanged, self.tree_ctrl_1)

        #self.Bind(wx.EVT_TREE_KEY_DOWN, self.onKeyEvent, self.tree_ctrl_1)
        #self.Bind(wx.EVT_LEFT_DCLICK, self.onItemActivated)
        #self.Bind(wx.EVT_LEFT_DCLICK, self.onCaseOpen, self.tree_ctrl_1)


        #----------------panel内のlabel（caseName, solverName）
        item = self.tree_ctrl_1.GetSelection()
        self.changeSolveCase(item)

        #----------------PopupMenu-------------
        self.popupmenu = wx.Menu()
        i=0
        for text in self.setPopupMenuItems():
            if text == "-":
                self.popupmenu.AppendSeparator()
            else:
                mmi = wx.MenuItem(self.popupmenu, i, text)
                #mmi.SetBitmap(wx.Bitmap("./icons/gridEditor.png"))
                #item = self.popupmenu.Append(-1, text)
                item = self.popupmenu.AppendItem(mmi)
                self.Bind(wx.EVT_MENU, self.OnPopupItemSelected, item)
            i+=1

        self.popupmenuSolver = wx.Menu()
        for text in self.setPopupMenuItemsSolver():
            if text == "-":
                self.popupmenuSolver.AppendSeparator()
            else:
                item = self.popupmenuSolver.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.OnPopupItemSolverSelected, item)

        self.popupmenuResult = wx.Menu()
        for text in self.setPopupMenuItemsResult():
            if text == "-":
                self.popupmenuResult.AppendSeparator()
            else:
                item = self.popupmenuResult.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.OnPopupItemResultSelected, item)

        self.popupmenuNotSolveCase = wx.Menu()
        for text in self.setPopupMenuItemsNotSolveCase():
            if text == "-":
                self.popupmenuNotSolveCase.AppendSeparator()
            else:
                item = self.popupmenuNotSolveCase.Append(-1, text)
                self.Bind(wx.EVT_MENU, self.OnPopupItemNotSolveCaseSelected, item)

        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnShowPopup, self.tree_ctrl_1)

        #---------backgroundColourを設定-------------
        self.setBackgroundColour()

        #---------TreeCtrlのtoolTip表示設定---------
        self.tree_ctrl_1.SetToolTip(wx.ToolTip(toolTipText))
        self.tree_ctrl_1.Bind(wx.EVT_MOTION, self.onShowToolTip)

        #---------Dragを設定------------------------
        self.tree_ctrl_1.Bind(wx.EVT_TREE_BEGIN_DRAG, self.onSetDrag)
        #self.tree_ctrl_1.Bind(wx.EVT_LEFT_UP, self.onDragCheck)
        #    LEFT_UPでeventは、発生しないので、SEL_CHANGEDで代用

    def onTest(self, event):
        print "-----------"


    def __set_properties(self):
        global envOpenFOAMFix, fontSize
        fontSize = self.label_1.GetFont().GetPointSize()        #fontSizeを取得
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle("TreeFoam")
        self.frame_1_statusbar.SetStatusWidths([-1])
        # statusbar fields
        frame_1_statusbar_fields = ["frame_1_statusbar"]
        for i in range(len(frame_1_statusbar_fields)):
            self.frame_1_statusbar.SetStatusText(frame_1_statusbar_fields[i], i)
        self.bitmap_button_12.SetMinSize((34, 34))
        self.bitmap_button_12.SetToolTipString(u"configTreeFoamの設定")
        self.bitmap_button_1.SetMinSize((34, 34))
        self.bitmap_button_1.SetToolTipString(u"rootDirの変更")
        self.bitmap_button_2.SetMinSize((34, 34))
        self.bitmap_button_2.SetToolTipString(u"Dirの再読込")
        self.bitmap_button_3.SetMinSize((34, 34))
        self.bitmap_button_3.SetToolTipString(u"新規にcaseを作成")
        self.bitmap_button_4.SetMinSize((34, 34))
        self.bitmap_button_4.SetToolTipString(u"現在のcaseを解析caseとして設定")
        self.bitmap_button_16.SetMinSize((34, 34))
        self.bitmap_button_16.SetToolTipString(u"閉じる")
        self.combo_box_1.SetMinSize((130, 28))
        self.combo_box_1.SetSelection(-1)
        self.bitmap_button_9.SetMinSize((34, 34))
        self.bitmap_button_9.SetToolTipString(u"gridEditor起動")
        self.bitmap_button_5.SetMinSize((34, 34))
        self.bitmap_button_5.SetToolTipString(u"Fieldの編集")
        self.bitmap_button_19.SetMinSize((34, 34))
        self.bitmap_button_19.SetToolTipString(u"Fieldにデータをセット、又はクリアします")
        self.bitmap_button_6.SetMinSize((34, 34))
        self.bitmap_button_6.SetToolTipString(u"Propertiesの編集")
        self.bitmap_button_7.SetMinSize((34, 34))
        self.bitmap_button_7.SetToolTipString(u"Dict（system）の編集")
        self.bitmap_button_8.SetMinSize((34, 34))
        self.bitmap_button_8.SetToolTipString(u"meshの編集")
        self.bitmap_button_18.SetMinSize((34, 34))
        self.bitmap_button_18.SetToolTipString(u"HelyxOSによる編集")
        self.bitmap_button_21.SetMinSize((34, 34))
        self.bitmap_button_21.SetToolTipString(u"計算結果のクリア")
        self.bitmap_button_13.SetMinSize((34, 34))
        self.bitmap_button_13.SetToolTipString(u"solverを起動")
        self.bitmap_button_14.SetMinSize((34, 34))
        self.bitmap_button_14.SetToolTipString(u"plotWatcherの起動・停止")
        self.bitmap_button_15.SetMinSize((34, 34))
        self.bitmap_button_15.SetToolTipString(u"並列処理")
        self.bitmap_button_20.SetMinSize((34, 34))
        self.bitmap_button_20.SetToolTipString(u"CADの起動")
        self.bitmap_button_10.SetMinSize((34, 34))
        self.bitmap_button_10.SetToolTipString(u"FOAM端末の起動")
        self.bitmap_button_11.SetMinSize((34, 34))
        self.bitmap_button_11.SetToolTipString(u"paraFoamの起動")
        self.bitmap_button_17.SetMinSize((34, 34))
        self.bitmap_button_17.SetToolTipString(u"SalomeMecaの起動")
        self.panel_1.SetBackgroundColour(wx.Colour(233, 255, 240))
        self.button_1.SetMinSize((35, 21))
        self.button_1.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "VL ゴシック"))
        self.button_2.SetMinSize((35, 21))
        self.button_2.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "VL ゴシック"))
        self.panel_2.SetBackgroundColour(wx.Colour(233, 255, 240))
        # end wxGlade
        self.SetTitle("TreeFoam_" + version + " (" + nTreeFoam + ")")
        self.clearStatus()
        self.readConfig()
        self.getConfig()
        self.setSolverInLabel("")
        getTreeDirUntilCurrCase(rootDir, selCaseDir)
        self.tree_ctrl_1 = wx.gizmos.TreeListCtrl(self, -1,
            style=
                wx.TR_DEFAULT_STYLE|
                wx.TR_FULL_ROW_HIGHLIGHT|
                wx.TR_HAS_BUTTONS|
                wx.TR_MULTIPLE)
        self.tree_ctrl_1.AddColumn("Tree")
        self.tree_ctrl_1.AddColumn("solver")
        self.tree_ctrl_1.AddColumn("nR")
        self.tree_ctrl_1.AddColumn("st")
        self.tree_ctrl_1.AddColumn("ed")
        self.tree_ctrl_1.SetMainColumn(0)
        self.tree_ctrl_1.SetColumnWidth(0, colWidth[0])
        self.tree_ctrl_1.SetColumnWidth(1, colWidth[1])
        self.tree_ctrl_1.SetColumnWidth(2, colWidth[2])
        self.tree_ctrl_1.SetColumnWidth(3, colWidth[3])
        self.tree_ctrl_1.SetColumnWidth(4, colWidth[4])
        self.tree_ctrl_1.SetMinSize((700, 50))
        self.tree_ctrl_1.SetBackgroundColour(wx.Colour(229, 229, 229))

        #treeCtrlへimageセット
        ilist = wx.ImageList(16,16)
        self.fldridx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_FOLDER, wx.ART_OTHER, (16,16)))
        self.fldropenidx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_OTHER, (16,16)))
        self.fileidx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, (16,16)))
        self.OFidx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, (16,16)))
        self.OFsolveridx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_OTHER, (16,16)))
        self.SolveCaseidx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_TICK_MARK, wx.ART_OTHER, (16,16)))
        self.salomeidx = ilist.Add(
            wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_OTHER, (16,16)))
        self.tree_ctrl_1.AssignImageList(ilist)
        #treeDataのセット
        root = self.tree_ctrl_1.AddRoot(treeData[0])
        rootName = self.tree_ctrl_1.GetItemText(root)
        data = self.setValue()
        self.AddTreeNodes(root, data)
        self.tree_ctrl_1.SetItemImage(root, self.fldridx,
            wx.TreeItemIcon_Normal)
        caseDir = selCase
        self.setInitialImage(caseDir)
        self.setSelectCase(caseDir)
        #OpenFOAM環境設定ファイル表示
        self.label_6.SetLabel(" " + envOpenFOAMFix)



    def __do_layout(self):
        # begin wxGlade: MyFrame.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_9 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_8 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_12 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_11 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1.Add(self.static_line_7, 0, wx.BOTTOM|wx.EXPAND, 4)
        sizer_11.Add(self.bitmap_button_12, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_1, 0, wx.BOTTOM, 0)
        sizer_11.Add(self.bitmap_button_2, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_3, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_4, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_16, 0, 0, 0)
        sizer_11.Add(self.static_line_4, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 4)
        sizer_11.Add(self.combo_box_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_11.Add(self.bitmap_button_9, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_5, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_19, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_6, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_7, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_8, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_18, 0, 0, 0)
        sizer_11.Add(self.static_line_5, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 4)
        sizer_11.Add(self.bitmap_button_21, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_13, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_14, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_15, 0, 0, 0)
        sizer_11.Add(self.static_line_1, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 4)
        sizer_11.Add(self.bitmap_button_20, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_10, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_11, 0, 0, 0)
        sizer_11.Add(self.bitmap_button_17, 0, 0, 0)
        sizer_1.Add(sizer_11, 0, wx.EXPAND, 0)
        sizer_1.Add(self.static_line_6, 0, wx.TOP|wx.BOTTOM|wx.EXPAND, 4)
        sizer_12.Add(self.label_1, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_12.Add(self.label_6, 0, 0, 0)
        sizer_1.Add(sizer_12, 0, wx.EXPAND, 0)
        sizer_4.Add(self.label_2, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_8.Add(self.label_3, 0, 0, 0)
        self.panel_1.SetSizer(sizer_8)
        sizer_6.Add(self.panel_1, 1, wx.EXPAND, 0)
        sizer_4.Add(sizer_6, 1, wx.EXPAND, 0)
        sizer_4.Add((5, 20), 0, 0, 0)
        sizer_1.Add(sizer_4, 0, wx.EXPAND, 0)
        sizer_5.Add(self.label_4, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_9.Add(self.button_1, 0, 0, 0)
        sizer_9.Add(self.label_5, 0, wx.LEFT, 5)
        sizer_9.Add((20, 20), 0, 0, 0)
        sizer_3.Add(self.button_2, 0, 0, 0)
        sizer_3.Add(self.label_7, 0, wx.LEFT, 5)
        sizer_3.Add((10, 20), 0, 0, 0)
        sizer_9.Add(sizer_3, 1, wx.EXPAND|wx.ALIGN_RIGHT|wx.SHAPED, 0)
        self.panel_2.SetSizer(sizer_9)
        sizer_7.Add(self.panel_2, 1, wx.EXPAND, 0)
        sizer_5.Add(sizer_7, 1, wx.EXPAND, 0)
        sizer_5.Add((5, 20), 0, 0, 0)
        sizer_1.Add(sizer_5, 0, wx.EXPAND, 0)
        sizer_1.Add((500, 5), 0, 0, 0)
        sizer_2.Add((5, 300), 0, 0, 0)
        sizer_2.Add(self.tree_ctrl_1, 1, wx.RIGHT|wx.EXPAND, 0)
        sizer_2.Add((5, 100), 0, 0, 0)
        sizer_1.Add(sizer_2, 1, wx.EXPAND, 0)
        sizer_1.Add((100, 5), 0, 0, 0)
        self.SetSizer(sizer_1)
        #sizer_1.Fit(self)
        self.Layout()
        # end wxGlade

    #------menu bar ----------------------

    #configTreeFoamの変更
    def selChangeConfigTreeFoam(self, event): # wxGlade: MyFrame.<event_handler>
        self.onChangeConfigTreeFoam(event)
        event.Skip()

    #rootDirの変更
    def selChangeRoot(self, event): # wxGlade: MyFrame.<event_handler>
        self.writeStatus(u"rootDirを変更中...")
        root = self.tree_ctrl_1.GetRootItem()
        editName = self.tree_ctrl_1.GetItemText(root)
        self.changeRoot(editName)
        self.clearStatus()
        event.Skip()

    #終了
    def selExit(self, event): # wxGlade: MyFrame.<event_handler>
        treeSize = self.GetSize()
        colWidth = [self.tree_ctrl_1.GetColumnWidth(0),
                    self.tree_ctrl_1.GetColumnWidth(1),
                    self.tree_ctrl_1.GetColumnWidth(2),
                    self.tree_ctrl_1.GetColumnWidth(3),
                    self.tree_ctrl_1.GetColumnWidth(4)]
        try:
            self.writeConfig()
            endingTreeFoam(treeSize, colWidth)
        except:
            title = "エラー"
            message = "書込エラーが発生しました。\n  強制終了します。"
            self.errDialog(title, message)
        #frame_1.Destroy()
        self.Destroy()
        event.Skip()

    #reload
    def selReload(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix, selCaseDirFix
        self.writeStatus(u"tree data を読み込み中...")
        #currentCaseを保存
        item = self.tree_ctrl_1.GetSelection()
        selCaseDir = self.getDirectory(item)
        #再読込
        self.reloadItems()
        #caseName, solverをlabelにセット
        caseItem = self.getItem(solveCaseFix)
        self.changeSolveCase(caseItem)
        #currentCaseを設定
        self.setSelectCase(self.convDirToList(selCaseDir))
        self.clearStatus()
        event.Skip()

    #help
    def selUsage(self, event): # wxGlade: MyFrame.<event_handler>
        print "ヘルプを表示します。"
        treeFoamDir = getTreeFoamInstallDir()
        #firefox終了時エラーが出るので、firefoxのエラー表示をキャンセル
        command = "firefox " + treeFoamDir + "/help/TreeFoam-manual.pdf 2> null &"
        os.system(command)
        event.Skip()

    #version表示
    def selVersion(self, event): # wxGlade: MyFrame.<event_handler>
        global version
        print "versionを表示します。"
        title = u"version表示"
        message = u"TreeFoamのバージョンを表示します。\n\n"
        message += u"  TreeFoam " + version
        message += "\n\n\n                programed by s.fujii  "
        self.okDialog(title, message)
        event.Skip()

    #newCase
    def selNewCase(self, event): # wxGlade: MyFrame.<event_handler>
        print "Event handler `selNewCase' not implemented"
        event.Skip()

    #currentCase
    def selSetCurrCase(self, event): # wxGlade: MyFrame.<event_handler>
        item = self.tree_ctrl_1.GetSelection()
        self.changeSolveCase(item)
        event.Skip()

    #gridEditor起動
    def selGridEditor(self, event): # wxGlade: MyFrame.<event_handler>
        self.onGridEditor(event)
        event.Skip()

    #field編集
    def selEditField(self, event): # wxGlade: MyFrame.<event_handler>
        global nTreeFoam
        if self.checkFolderMsg(solveCaseFix) == "case":
            os.chdir(solveCaseFix)
            Num = self.combo_box_1.GetSelection()
            NumFolder = self.combo_box_1.GetString(Num)
            editDir = solveCaseFix + "/" + NumFolder
            FFlist = setEditFolders(editDir)
            #frame_1.Destroy()
            job = "editField"
            title = "boundary Field の編集"
            img = "boundary"
            mess = u"boundary Field を編集します"
            selectFieldsForEditDialog.Show(nTreeFoam, job, title, img, mess, FFlist, editDir)
        event.Skip()

    #properties編集
    def selEditProperties(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkFolderMsg(solveCaseFix) == "case":
            self.editPropertiesFiles()
        event.Skip()

    #Dict編集
    def selEditDict(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkFolderMsg(solveCaseFix) == "case":
            self.editDict()
        event.Skip()

    #mesh編集
    def selEditMesh(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkFolderMsg(solveCaseFix) == "case":
            os.chdir(solveCaseFix)
            meshOperationDialog.Show()
        event.Skip()

    #フォルダ名変更
    def selRenameCase(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkMultiSelection(u"フォルダ名変更")==False:
            self.renameFolder()
        event.Skip()

    #フォルダ追加
    def selAddFolder(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkMultiSelection(u"フォルダ追加")==False:
            self.insertFolder()
        event.Skip()

    #フォルダ削除
    def selDeleteFolder(self, event): # wxGlade: MyFrame.<event_handler>
        self.deleteFolders()
        event.Skip()

    #runSolver
    def selRunSolver(self, event): # wxGlade: MyFrame.<event_handler>
        self.runSolver(solveCaseFix)
        event.Skip()

    #plotWatcher
    def selPlotWatcher(self, event): # wxGlade: MyFrame.<event_handler>
        self.onPlotWatcher(event)
        event.Skip()

    #並列計算
    def selParallelSolver(self, event): # wxGlade: MyFrame.<event_handler>
        self.onParallel(event)
        event.Skip()

    #FOAM端末
    def selRunFoamTerm(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix
        self.runFoamTermDir(solveCaseFix)
        event.Skip()

    #raraFoam
    def selRunParaFoam(self, event): # wxGlade: MyFrame.<event_handler>
        self.onRunParaFoam(event)
        event.Skip()

    #foamViewer
    def selRunFoamViewer(self, event): # wxGlade: MyFrame.<event_handler>
        print "Event handler `selRunFoamViewer' not implemented"
        event.Skip()

    #runSlomeMeca
    def selRunSalome(self, event): # wxGlade: MyFrame.<event_handler>
        self.runSalomeMeca(solveCaseFix)
        event.Skip()

    #folderOpen
    def selFolderOpen(self, event): # wxGlade: MyFrame.<event_handler>
        self.onFolderOpen(event)
        event.Skip()

    #dataSetClear
    def selDataSetClear(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkFolderMsg(solveCaseFix) == "case":
            os.chdir(solveCaseFix)
            num = self.combo_box_1.GetSelection()
            timeFolder = self.combo_box_1.GetString(num)
            setDataInFieldsDialog.Show(solveCaseFix, timeFolder)
        event.Skip()

    #runCAD
    def selRunCAD(self, event): # wxGlade: MyFrame.<event_handler>
        self.runCAD(solveCaseFix)
        event.Skip()

    #folderCopy
    def selCopyFolder(self, event): # wxGlade: MyFrame.<event_handler>
        self.foldersCopy()
        event.Skip()

    #folderCasePase
    def selPasteFolder(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkMultiSelection(u"貼付け")==False:
            self.foldersCasesPaste()
        event.Skip()

    #caseCopy
    def selCaseCopy(self, event): # wxGlade: MyFrame.<event_handler>
        self.casesCopy()
        event.Skip()

    #-----------------------------------------------------


    #setSolverInLabel
    def setSolverInLabel(self, solver):
        #最大文字数
        maxChara = 30
        label = solver
        i=len(solver)
        while i<maxChara:
            label += " "
            i+=1
        self.label_7.SetLabel(label)

    # checkFolderMesg
    def checkFolderMsg(self, directory):
        global folderMsg
        ans = checkFolder(directory)
        if folderMsg == "yes":
            if ans != "case":
                title = u"警告"
                message = u"OpenFOAMのCaseフォルダではありません。"
                self.warningDialog(title, message)
        return ans

    # AddTreeNode
    def AddTreeNodes(self, parentItem, items):
        for item in items:
            #if type(item) == str:
            if type(item) == str or type(item) == unicode:
                newItem = self.tree_ctrl_1.AppendItem(parentItem, item)
                self.tree_ctrl_1.SetItemImage(newItem, self.fldridx,
                    wx.TreeItemIcon_Normal)
            else:
                newItem = self.tree_ctrl_1.AppendItem(parentItem, item[0])
                self.AddTreeNodes(newItem, item[1])
                self.tree_ctrl_1.SetItemImage(newItem, self.fldridx,
                    wx.TreeItemIcon_Normal)

    #
    # setValue
    #   treeのデータ構造
    #    ['root', [
    #        '0', 
    #        ['constant', [
    #             ['polyMesh', [
    #                'sets',
    #                'sets.bak',
    #                'sss']]]], 
    #        'model',
    #        'solve.log.analyzed', 
    #        ['system', [
    #            'test']]]]
    #
    def setValue(self):
        global treeData
        data = treeData[1]
        return data

    # setSelectCase
    #  treeDataからcurrentCaseをselectする
    def setSelectCase(self, dataDir):
        i=0
        while (i<len(dataDir)):
            if type(dataDir[i]) == str:
                dataDir[i] = dataDir[i].decode("utf-8")
            i=i+1
        parent = self.tree_ctrl_1.GetRootItem()
        #name = self.tree_ctrl_1.GetItemText(parent)
        i=1
        while (i<len(dataDir)):
            self.tree_ctrl_1.Expand(parent)
            #全chilrenをチェック
            items = self.getChildrenItems(parent)
            itemNames = self.getChildrenList(parent)
            ii=0
            while ii<len(itemNames):
                if type(itemNames[ii]) == str:
                    itemNames[ii] = itemNames[ii].decode("utf-8")
                ii=ii+1
            flag = 0
            ii=0
            while ii<len(itemNames):
                if itemNames[ii] == dataDir[i]:
                    parent = items[ii]
                    flag = 1
                    break
                ii=ii+1
            if flag == 0:
                print dataDir, "のDirがありません。"
                break
            i=i+1
        self.tree_ctrl_1.SelectItem(parent, parent, True)
        #item = self.tree_ctrl_1.GetSelection()
        #self.tree_ctrl_1.EnsureVisible(item)
        #self.tree_ctrl_1.ScrollTo(item)


    # setBackgroundColour
    #   treeのbackgroundColourを設定（treeを縞模様に設定）
    def setBackgroundColour(self):
        global solveCaseFix
        caseDirList = self.convDirToList(solveCaseFix)
        caseDir = string.join(caseDirList, "/")
        parent = self.tree_ctrl_1.GetRootItem()
        parentName = self.tree_ctrl_1.GetItemText(parent)
        setDir = parentName
        n=0
        n = self.checkColour(n, parent, setDir, caseDir)

    def checkColour(self, n, parent, setDir, caseDir):
        #parentのchildren[]を取得
        children = self.getChildrenList(parent)
        #全childrenをチェック
        item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
        for child in children:
            if (n % 2) == 0:
                #self.tree_ctrl_1.SetItemBackgroundColour(item, wx.Colour(235, 235, 235))
                self.tree_ctrl_1.SetItemBackgroundColour(item, wx.Colour(238, 238, 235))
            else:
                self.tree_ctrl_1.SetItemBackgroundColour(item, "litegray")
            newDir = setDir + "/" + child
            if newDir == caseDir:
                self.tree_ctrl_1.SetItemBackgroundColour(item, "white")
            if self.tree_ctrl_1.IsExpanded(item) == True:
                n += 1
                n = self.checkColour(n, item, newDir, caseDir)
            n += 1
            item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
        n += 1
        return n



    # setInitialImage
    #  TreeDataにイメージデータを設定
    def setInitialImage(self, caseDir):
        global solveCaseFix
        parent = self.tree_ctrl_1.GetRootItem()
        parentName = self.tree_ctrl_1.GetItemText(parent)
        setDir = parentName
        for folder in caseDir[1:]:
            #parentのchildren[]を取得
            children = self.getChildrenList(parent)
            #全childrenをチェック
            item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
            for child in children:
                checkDir = setDir + "/" + child
                checkResult = checkFolder(checkDir)
                if (checkResult == "case"):
                    itemOF = item
                    solver = getSolver(checkDir)
                    nRst = getNumberResult(checkDir)
                    self.setItemOFImage(itemOF, solver, nRst)
                elif (checkResult == "salome"):
                    self.tree_ctrl_1.SetItemImage(item, self.salomeidx, 0,
                        wx.TreeItemIcon_Normal)
                item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
            item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
            for child in children:
                parentName = ""
                if type(folder) == str:
                    folder = folder.decode("utf-8")
                if folder == child:
                    parent = item
                    parentName = self.tree_ctrl_1.GetItemText(parent)
                    setDir = setDir + "/" + parentName
                    break
                item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
            if parentName == "":
                break

    # setChildrenImage
    #  childrenにイメージをセット
    def setChildrenImage(self, parent):
        childrenItems = self.getChildrenItems(parent)
        for childItem in childrenItems:
            checkDir = self.getDirectory(childItem)
            if (checkFolder(checkDir) == "case"):
                solver = getSolver(checkDir)
                nRst = getNumberResult(checkDir)
                self.setItemOFImage(childItem, solver, nRst)                
            elif (checkFolder(checkDir) == "salome"):
                self.tree_ctrl_1.SetItemImage(childItem, self.salomeidx, 0,
                    wx.TreeItemIcon_Normal)

    # setChildrenImageSolveCase
    #  childrenに解析caseがあればimageをセット
    def setChildrenImageSolveCase(self, parent):
        childrenItems = self.getChildrenItems(parent)
        for childItem in childrenItems:
            checkDir = self.getDirectory(childItem)
            if checkDir == solveCaseFix:
                self.tree_ctrl_1.SetItemImage(childItem, self.SolveCaseidx, 0,
                    wx.TreeItemIcon_Normal)

    # setItemOFImage
    #  OpenFoamのImageをセット
    def setItemOFImage(self, itemOF, solver, nRst):
        self.tree_ctrl_1.SetItemImage(itemOF, self.OFidx, 0,
            wx.TreeItemIcon_Normal)
        self.tree_ctrl_1.SetItemText(itemOF, solver, 1)
        self.tree_ctrl_1.SetItemImage(itemOF, self.OFsolveridx,1,
            wx.TreeItemIcon_Normal)
        if len(nRst)>0:
            self.tree_ctrl_1.SetItemText(itemOF, nRst[0], 2)
            self.tree_ctrl_1.SetItemText(itemOF, nRst[1], 3)
            self.tree_ctrl_1.SetItemText(itemOF, nRst[2], 4)

    # getItem
    #  directoryからTreeItemを取得
    def getItem(self, directory):
        global rootDir
        treeDir = directory[len(rootDir)+1:].split("/")
        parent = self.tree_ctrl_1.GetRootItem()
        for name in treeDir:
            childrenItem = self.getChildrenItems(parent)
            flag = 0
            for item in childrenItem:
                child = self.tree_ctrl_1.GetItemText(item)
                if name == child:
                    flag = 1
                    break
            if flag == 1:
                parent = item
            else:
                ans = ""
                break
        ans = item
        return ans

    # getDirectory
    #  TreeDataからdirectoryを取得
    def getDirectory(self, item):
        name = self.tree_ctrl_1.GetItemText(item)
        selectDir = ""
        while (name != ""):
            selectDir = "/" + name + selectDir
            item = self.tree_ctrl_1.GetItemParent(item)
            name = self.tree_ctrl_1.GetItemText(item)
        selectDir = selectDir[1:]
        return selectDir

    # getChildrenList
    #  親itemから子のtextを取得
    def getChildrenList(self, parent):
        children = []
        #firstChild
        item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
        while item:
            children.append(self.tree_ctrl_1.GetItemText(item, 0))
            #nextChild
            item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
        return children

    #
    #  getChildrenItems
    #    親itemから子itemを取得
    def getChildrenItems(self, parent):
        items = []
        #firstChild
        item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
        while item:
            items.append(item)
            #nextChild
            item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
        return items


    #
    #  addGrandChildren
    #    親itemから孫itemを追加
    def addGrandChildren(self, parent):
        childrenItems = self.getChildrenItems(parent)
        for childItem in childrenItems:
            directory = self.getDirectory(childItem)
            folders = getFoldersOnlyCurrDir(directory)
            folders.sort()
            for folder in folders:
                itemText = folder.split("/")[-1]
                check = checkFolder(folder)
                if (check == "folder"):
                    newItem = self.tree_ctrl_1.AppendItem(childItem, itemText)
                    self.tree_ctrl_1.SetItemImage(newItem, self.fldridx,
                        wx.TreeItemIcon_Normal)
                elif (check == "case"):
                    newItem = self.tree_ctrl_1.AppendItem(childItem, itemText)
                    solver = getSolver(folder)
                    nRst = getNumberResult(folder)
                    self.setItemOFImage(newItem, solver, nRst)
                elif (check == "salome"):
                    newItem = self.tree_ctrl_1.AppendItem(childItem, itemText)
                    self.tree_ctrl_1.SetItemImage(newItem, self.salomeidx,
                        wx.TreeItemIcon_Normal)


    #
    #  delGrandChildrenItem
    #    孫itemの削除
    def delGrandChildrenItem(self, parent):
        children = self.getChildrenItems(parent)
        for childItem in children:
            self.tree_ctrl_1.DeleteChildren(childItem)

    def collapseChildren(self, parent):
        children = self.getChildrenItems(parent)
        for childItem in children:
            self.tree_ctrl_1.Collapse(childItem)

    def reloadItems(self):
        global rootDir, selCaseDir, treeData
        #treeをクリア
        self.writeStatus(u"tree data を読み込み中...")
        self.tree_ctrl_1.DeleteAllItems()
        #rootのdirを取得
        self.getConfig()
        selCaseDir = rootDir
        getTreeDirOnlylRootChildren(rootDir)
        root = self.tree_ctrl_1.AddRoot(treeData[0])
        rootName = self.tree_ctrl_1.GetItemText(root)
        #treeDataの作成
        data = treeData[1]
        self.AddTreeNodes(root, data)
        self.tree_ctrl_1.SetItemImage(root, self.fldridx,
            wx.TreeItemIcon_Normal)
        #selCaseの設定
        self.getConfig()
        caseDir = self.convDirToList(selCaseDir)
        self.setInitialImage(caseDir)
        self.setSelectCase(caseDir)
        self.setSolveCase()
        item = self.tree_ctrl_1.GetSelection()
        self.tree_ctrl_1.ScrollTo(item)
        item = self.getItem(solveCaseFix)
        self.tree_ctrl_1.SetItemImage(item, self.SolveCaseidx, 0,
            wx.TreeItemIcon_Normal)
        #編集するtimeFolderを設定
        self.setEditTimeFolder(solveCaseFix)
        #solverを設定
        #solver = getSolver(selCaseDir)
        #self.label_7.SetLabel(solver)

    def convDirToList(self, dataDir):
        global rootDir
        ans = []
        dataNames = dataDir[len(rootDir)+1:].split("/")
        ans.append(rootDir)
        for name in dataNames:
            ans.append(name)
        return ans

    def setSolveCase(self):
        global solveCaseFix, rootDir
        saveItem =  self.tree_ctrl_1.GetSelection()
        caseDir = self.convDirToList(solveCaseFix)
        self.setSelectCase(caseDir)
        item = self.tree_ctrl_1.GetSelection()
        self.tree_ctrl_1.SetItemTextColour(item, "Red")
        #self.tree_ctrl_1.SetItemBold(item, True)
        self.tree_ctrl_1.SelectItem(saveItem, saveItem, True)    

    #
    #  changeSolveCase
    #    解析caseを変更
    def changeSolveCase(self, item):
        global solveCaseFix, rootDir, timeFolders, folderMsg
        #folderMsgErrorの表示を禁止
        folderMsg = "no"
        #変更前の解析Caseのfont色を元に戻す
        preItem = self.getItem(solveCaseFix)
        self.tree_ctrl_1.SetItemTextColour(preItem, "#4c4c4c")      #system色（黒）
        #変更前のiconを元に戻す
        preDir = self.getDirectory(preItem)
        if checkFolder(preDir) == "case":
            self.tree_ctrl_1.SetItemImage(preItem, self.OFidx,0,
                wx.TreeItemIcon_Normal)
        elif checkFolder(preDir) == "salome":
            self.tree_ctrl_1.SetItemImage(preItem, self.salomeidx,0,
                wx.TreeItemIcon_Normal)
        else:
            self.tree_ctrl_1.SetItemImage(preItem, self.fldridx,0,
                wx.TreeItemIcon_Normal)
        #解析Caseのfontを変更
        solveDir = self.getDirectory(item)
        self.setSelectCase(self.convDirToList(solveDir))
        self.tree_ctrl_1.SetItemTextColour(item, "Red")  #ff0000
        #self.tree_ctrl_1.SetItemTextColour(item, "#ff0000")  #ff0000
        #解析Caseのiconを設定
        self.tree_ctrl_1.SetItemImage(item, self.SolveCaseidx,0,
            wx.TreeItemIcon_Normal)
        #lebelにDir, case, solverをセット, backGroundColourを設定
        solveCaseFix = solveDir
        self.setLabelDataInFrame()
        #時間フォルダの取得、設定
        self.setEditTimeFolder(solveCaseFix)
        #解析CaseのbackGroundColourを設定（縞模様）
        self.setBackgroundColour()
        #folderMsgErrorの表示を許可
        folderMsg = "yes"

    #
    #  setLabelDataInFrame
    #    OpenFam環境設定ファイル、CaseDir、Case名、solver名を表示
    #    backGroundColourを設定
    def setLabelDataInFrame(self):
        global envOpenFOAMFix, solveCaseFix
        #labelを設定
        caseName = solveCaseFix.split("/")[-1]
        caseDir = solveCaseFix[:-len(caseName)-1]
        solver = getSolver(solveCaseFix)
        maxCharaLength = 85
        if len(caseDir) > maxCharaLength:
            caseDir = "..." + caseDir[-(maxCharaLength-3):]
        self.label_3.SetLabel(" " + caseDir)
        self.label_5.SetLabel(caseName)
        self.setSolverInLabel(solver)
        self.label_6.SetLabel(envOpenFOAMFix)
        #labelのbackGroundColorを設定
        if self.checkFolderMsg(solveCaseFix) == "case":
            #lightGreenに設定
            self.panel_1.SetBackgroundColour(wx.Colour(233, 255, 240))
            self.panel_2.SetBackgroundColour(wx.Colour(233, 255, 240))
        elif self.checkFolderMsg(solveCaseFix) == "salome":
            #lightYellowに設定
            self.panel_1.SetBackgroundColour(wx.Colour(244, 244, 147))
            self.panel_2.SetBackgroundColour(wx.Colour(244, 244, 147))
        else:
            #pinkに設定
            self.panel_1.SetBackgroundColour(wx.Colour(255, 233, 240))
            self.panel_2.SetBackgroundColour(wx.Colour(255, 233, 240))


    #
    #  setEditTimeFolder
    #    編集するtimeFolderの設定
    def setEditTimeFolder(self, solveCaseFix):
        timeFolders = getResultTimeFolders(solveCaseFix)
        self.combo_box_1.Clear()
        for text in timeFolders:
            self.combo_box_1.Append(text)
        wdir = solveCaseFix+"/system"
        startFrom = getFoamContents.controlDict(wdir, ["startFrom"])
        if startFrom != None:
            if startFrom[0] == "startTime":
                self.combo_box_1.SetSelection(0)
            elif startFrom[0] == "latestTime":
                self.combo_box_1.SetSelection(len(timeFolders)-1)
        else:
            self.combo_box_1.SetSelection(0)

    #--------メニューボタン-------------------

    #configTreeFoamの変更
    def onChangeConfigTreeFoam(self, event): # wxGlade: MyFrame.<event_handler>
        print "configTreeFoamを変更します"
        self.changeConfigTreeFoam(event)
        event.Skip()

    #rootDirの変更
    def onChangeRoot(self, event): # wxGlade: MyFrame.<event_handler>
        self.selChangeRoot(event)
        event.Skip()

    #reload
    def onReload(self, event): # wxGlade: MyFrame.<event_handler>
        self.selReload(event)
        event.Skip()

    #newCase
    def onNewCase(self, event): # wxGlade: MyFrame.<event_handler>
        print "Event handler `onNewCase' not implemented"
        event.Skip()

    #currentCase
    def onActiveCase(self, event): # wxGlade: MyFrame.<event_handler>
        self.selSetCurrCase(event)
        event.Skip()

    #gridEditor起動
    def onGridEditor(self, event): # wxGlade: MyFrame.<event_handler>
        if self.checkFolderMsg(solveCaseFix) == "case":
            os.chdir(solveCaseFix)
            Num = self.combo_box_1.GetSelection()
            NumFolder = self.combo_box_1.GetString(Num)
            editDir = solveCaseFix + "/" + NumFolder
            self.runGridEditor(editDir)
        event.Skip()

    #field編集
    def onEditField(self, event): # wxGlade: MyFrame.<event_handler>
        self.selEditField(event)
        event.Skip()

    #properties編集
    def onEditProperties(self, event): # wxGlade: MyFrame.<event_handler>
        self.selEditProperties(event)
        event.Skip()

    #Dict編集
    def onEditDict(self, event): # wxGlade: MyFrame.<event_handler>
        self.selEditDict(event)
        event.Skip()

    #mesh編集
    def onEditMesh(self, event): # wxGlade: MyFrame.<event_handler>
        self.selEditMesh(event)
        event.Skip()

    #HelyxOS
    def onHelyxos(self, event): # wxGlade: MyFrame.<event_handler>
        print "HelyxOSを起動中..."
        self.runHelyxos()
        event.Skip()

    #deleteResult
    def onDeleteResult(self, event): # wxGlade: MyFrame.<event_handler>
        print "結果フォルダを削除します。"
        self.deleteResultFolders()
        self.reloadItems()
        event.Skip()

    #runSlomeMeca
    def onRunSalome(self, event): # wxGlade: MyFrame.<event_handler>
        self.selRunSalome(event)
        event.Skip()

    #case変更
    def onChangeCase(self, event): # wxGlade: MyFrame.<event_handler>
        print "Event handler `onChangeCase' not implemented"
        event.Skip()

    #runSolver
    def onRunSolver(self, event): # wxGlade: MyFrame.<event_handler>
        self.selRunSolver(event)
        event.Skip()

    #plotWatcher
    def onPlotWatcher(self, event): # wxGlade: MyFrame.<event_handler>
        self.runPlotWatcher()

    #並列計算
    def onParallel(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix, envOpenFOAMFix, folderMsg
        if self.checkFolderMsg(solveCaseFix) == "case":
            runParallelDialog.Show(solveCaseFix)
        event.Skip()

    #FOAM端末
    def onRunFoamTerm(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix
        self.runFoamTermDir(solveCaseFix)
        event.Skip()

    #paraFoam
    def onRunParaFoam(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix
        self.runParaFoamDir(solveCaseFix)
        event.Skip()

    #runViewer
    def onRunViewer(self, event): # wxGlade: MyFrame.<event_handler>
        print "Event handler `onRunViewer' not implemented"
        event.Skip()

    #終了
    def onExit(self, event): # wxGlade: MyFrame.<event_handler>
        self.selExit(event)

    #folderOpen
    def onFolderOpen(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix
        os.system(("nautilus " + solveCaseFix).encode("utf-8"))
        event.Skip()

    #dataSetClear
    def onDataSetClear(self, event): # wxGlade: MyFrame.<event_handler>
        self.selDataSetClear(event)
        event.Skip()

    #runCAD
    def onRunCAD(self, event): # wxGlade: MyFrame.<event_handler>
        self.selRunCAD(event)
        event.Skip()

    #caseOpen
    def onCaseOpen(self, event): # wxGlade: MyFrame.<event_handler>
        self.onFolderOpen(event)
        event.Skip()

    #controlDictOpen
    def onCntDictOpen(self, event): # wxGlade: MyFrame.<event_handler>
        global solveCaseFix
        if self.checkFolderMsg(solveCaseFix) == "case":
            openFiles = solveCaseFix + "/system/controlDict "
            openFiles += solveCaseFix + "/system/fvSchemes "
            openFiles += solveCaseFix + "/system/fvSolution"
            pyTreeFoam.runEditor(openFiles)
        event.Skip()

    #toolTip表示
    def onShowToolTip(self, event):
        self.showHideToolTip(event)
        event.Skip()

    #drag操作
    def onSetDrag(self, event):
        self.setDragFolders(event)
        event.Skip()

    #keyEvent
    def onKeyEvent(self, event):
        code = event.GetKeyCode()
        print "code=", code
        event.Skip()


    #--------event menu ---------------------
    #folder選択時（ダブルクリック時）
    def onItemActivated(self, event): # wxGlade: MyFrame.<event_handler>
        global folderMsg
        pos = event.GetPoint()
        colWidth = [self.tree_ctrl_1.GetColumnWidth(0),
                    self.tree_ctrl_1.GetColumnWidth(1),
                    self.tree_ctrl_1.GetColumnWidth(2),
                    self.tree_ctrl_1.GetColumnWidth(3),
                    self.tree_ctrl_1.GetColumnWidth(4)]
        folderMsg = "no"
        item = self.tree_ctrl_1.GetSelection()
        selectDir = self.getDirectory(item)
        name = self.tree_ctrl_1.GetItemText(item)
        i=0
        itemP = item
        while name != "":
            itemP = self.tree_ctrl_1.GetItemParent(itemP)
            name = self.tree_ctrl_1.GetItemText(itemP)
            i += 1
        if pos[0] <= (i+1)*16:
            #folderIcon Dクリック時:解析caseとして設定
            self.changeSolveCase(item)
        elif pos[0] <= colWidth[0]:
            #folder Dクリック時:folderOpen
            self.onItemOpen(event)
        elif pos[0] <= colWidth[0] + 16:
            #solverIcon Dクリック時:解析caseとして設定
            if self.checkFolderMsg(selectDir) == "case":
                self.changeSolveCase(item)
        elif pos[0] <= colWidth[0] + colWidth[1]:
            #solverName Dクリック時:controlDictをopen
            if self.checkFolderMsg(selectDir) == "case":
                if selectDir != solveCaseFix:
                    if self.isSolveCase(selectDir) == "OK":
                        self.changeSolveCase(item)
                        self.onCntDictOpen(event)
                else:
                    self.onCntDictOpen(event)
        elif pos[0] <= colWidth[0]+colWidth[1]+colWidth[2]+colWidth[3]+colWidth[4]:
            #nR Dクリック時:paraFoam起動
            if self.checkFolderMsg(selectDir) == "case":
                if selectDir != solveCaseFix:
                    if self.isSolveCase(selectDir) == "OK":
                        self.changeSolveCase(item)
                        self.onRunParaFoam(event)
                else:
                    self.onRunParaFoam(event)
        event.Skip()



    def isSolveCase(self, selectDir):
        title = u"解析Caseの確認"
        message = u"選択したcaseは、解析caseとして設定されていません。\n  解析caseとして設定しますか？"
        ans = self.okCancelDialog(title, message)
        return ans
        

    #folderTree縮小時
    def onItemCollapsed(self, event): # wxGlade: MyFrame.<event_handler>
        collItem = event.GetItem()
        rootItem = self.tree_ctrl_1.GetRootItem()
        selItem = self.tree_ctrl_1.GetSelection()
        if collItem == rootItem:
            self.tree_ctrl_1.SelectItem(rootItem, rootItem, True)
        if selItem == rootItem:
            parent = rootItem
        else:
            parent = self.tree_ctrl_1.GetItemParent(selItem)
        while parent != rootItem:
            if parent == collItem:
                self.tree_ctrl_1.SelectItem(parent, parent, True)
                break
            parent = self.tree_ctrl_1.GetItemParent(parent)
#        self.tree_ctrl_1.ScrollTo(collItem)
        #孫item削除
        self.delGrandChildrenItem(collItem)
        self.clearStatus()
        self.setBackgroundColour()
        event.Skip()

    #folderTree展開時
    def onItemExpanded(self, event): # wxGlade: MyFrame.<event_handler>
        self.writeStatus(u"childrenItemsを展開中...")
        parent = event.GetItem()
        #parentの子itemのイメージをセット
        self.setChildrenImage(parent)
        #parentの子itemに解析caseあればimageをセット
        self.setChildrenImageSolveCase(parent)
        #parentの孫itemを追加（孫が
        self.addGrandChildren(parent)
        #子itemを折りたたむ
        self.collapseChildren(parent)
        self.clearStatus()
        self.setBackgroundColour()
        event.Skip()


    #folder選択時
    def onSelChanged(self, event):
        global selectTreeDir
        item = self.tree_ctrl_1.GetSelection()
        selectDir = self.getDirectory(item)
        selectTreeDir = selectDir
        if (checkFolder(selectDir) == "case"):
            solver = getSolver(selectDir)
            nRst = getNumberResult(selectDir)
            self.setItemOFImage(item, solver, nRst)
            if selectDir == solveCaseFix:
                self.setSolverInLabel(solver)
        if selectDir == solveCaseFix:
            self.tree_ctrl_1.SetItemImage(item, self.SolveCaseidx, 0,
                wx.TreeItemIcon_Normal)
        self.saveSelCaseDirToConfig(selectDir)            #selectDirを保存
        self.clearStatus()
        event.Skip()


    def onItemOpen(self, event):
        print "フォルダをopenします。"
        item = self.tree_ctrl_1.GetSelection()
        directory = self.getDirectory(item)
        os.system(("nautilus " + directory).encode("utf-8"))

    #windowClose時
    def onCloseWindow(self, event):
        treeSize = self.GetSize()
        colWidth = [self.tree_ctrl_1.GetColumnWidth(0),
                    self.tree_ctrl_1.GetColumnWidth(1),
                    self.tree_ctrl_1.GetColumnWidth(2),
                    self.tree_ctrl_1.GetColumnWidth(3),
                    self.tree_ctrl_1.GetColumnWidth(4)]
        try:
            self.writeConfig()
            endingTreeFoam(treeSize, colWidth)
        except:
            title = "エラー"
            message = "書込エラーが発生しました。\n  強制終了します。"
            self.errDialog(title, message)
        self.Destroy()

    #toolTipの表示
    def showHideToolTip(self, event):
        global toolTipText, selCaseDirFix
        (x,y)=event.GetPositionTuple()
        (item, a, flag) = self.tree_ctrl_1.HitTest((x,y))
        currDir = self.getDirectory(item)
        if currDir == selCaseDirFix:
            folder = checkFolder(selCaseDirFix)
            colW = [self.tree_ctrl_1.GetColumnWidth(0),
                    self.tree_ctrl_1.GetColumnWidth(1),
                    self.tree_ctrl_1.GetColumnWidth(2),
                    self.tree_ctrl_1.GetColumnWidth(3),
                    self.tree_ctrl_1.GetColumnWidth(4)]
            name = self.tree_ctrl_1.GetItemText(item)
            i = len(currDir.split("/"))
            if x <= (i+1)*16:
                #解析caseとして設定
                toolTipText = u"ここをダブルクリックすると、\n  解析caseとして設定します。"
            elif x <= colW[0]:
                #folderOpen
                toolTipText = u"ここをダブルクリックすると、\n  folderを開きます。"
            elif x<= colW[0] + 16 and folder == "case":
                #解析caseとして設定（solverIcon)
                toolTipText = u"ここをダブルクリックすると、\n  解析caseとして設定します。"
            elif x<= colW[0]+colW[1] and folder == "case":
                #controlDictOpen
                toolTipText = u"ここをダブルクリックすると、\n  controlDictを開きます。"
            elif x<= colW[0]+colW[1]+colW[2]+colW[3]+colW[4] and folder == "case":
                #paraFoam起動
                toolTipText = u"ここをダブルクリックすると、\n  paraFoamを起動します。"
            else:
                toolTipText = u""
            if toolTipText != u"":
                self.tree_ctrl_1.SetToolTip(wx.ToolTip(toolTipText))
            else:
                self.tree_ctrl_1.SetToolTip(wx.ToolTip.Enable(False))
        else:
            self.tree_ctrl_1.SetToolTip(wx.ToolTip.Enable(False))

    #setDragFolders
    def setDragFolders(self, event):
        global clipBoard
        items = self.tree_ctrl_1.GetSelections()
        f = open(clipBoard, "w")
        f.write("foldersDrag\n")
        for item in items:
            directory = self.getDirectory(item)
            print "drag--->", directory
            f.write(directory.encode("utf-8") + "\n")
        f.close()

        print event(wx.MouseEvent.Dragging())
         


    """
    #dragCheck
    def dragCheck(self, event):
        global clipBoard
        f = open(clipBoard); contents = f.read(); f.close()
        lines = contents.decode("utf-8").split("\n")[:-1]   #最後に空文が入るので最後を削除
        if lines[0] == "foldersDrag":
            print "---------"
            #self.foldersPaste(lines[1:])
    """


    #------------Popup menu-------------
    # setPopupMenuItems
    #  ポップアップメニュー項目設定
    def setPopupMenuItems(self):
        menu = [u"開く", u"端末の起動", u"解析caseとして設定",
                "-", u"FOAM端末の起動", u"caseのコピー\tCtrl+A",
                u"フォルダコピー\tCtrl+C", u"貼付け(folder, case)\tCtrl+V",
                "-", u"フォルダ名変更...", u"新しいフォルダ追加", u"フォルダ削除...",
                "-", u"CADの起動", u"SalomeMecaの起動"]
        return menu

    def setPopupMenuItemsSolver(self):
        menu = [u"開く", "-", u"FOAM端末の起動",
                u"gridEditor起動...", u"fieldの編集...", u"fieldにデータセット...",
                u"controlDictの編集", u"properties(const)の編集...",
                u"dictionaries(sys)の編集...", u"meshの編集...",
                "-", u"solver実行", u"plotWatcherを実行", u"並列処理..."]
        return menu

    def setPopupMenuItemsNotSolveCase(self):
        menu = [u"開く", u"端末の起動", u"解析caseとして設定", "-", u"FOAM端末の起動"]
        return menu

    def setPopupMenuItemsResult(self):
        menu = [u"開く", "-", u"FOAM端末の起動",
                u"gridEditor起動...", u"残渣の確認(plotWatches起動)",
                u"結果フォルダの削除..."]
        return menu



    #
    # OnShowPopup
    #  ポップアップメニュー表示
    def OnShowPopup(self, event):
        global folderMsg
        item = event.GetItem()
        #self.tree_ctrl_1.SelectItem(item, item, True)
        #self.onItemActivated(event)
        pos = event.GetPoint()
        colWidth = [self.tree_ctrl_1.GetColumnWidth(0),
                    self.tree_ctrl_1.GetColumnWidth(1),
                    self.tree_ctrl_1.GetColumnWidth(2),
                    self.tree_ctrl_1.GetColumnWidth(3),
                    self.tree_ctrl_1.GetColumnWidth(4)]
        folderMsg = "no"
        item = self.tree_ctrl_1.GetSelection()
        selectDir = self.getDirectory(item)
        if pos[0] <= colWidth[0]:
            self.tree_ctrl_1.PopupMenu(self.popupmenu, pos)
        elif pos[0] <= colWidth[0] + colWidth[1]:
            if self.checkFolderMsg(selectDir) == "case":
                if selectDir == solveCaseFix:
                    self.tree_ctrl_1.PopupMenu(self.popupmenuSolver, pos)
                else:
                    self.tree_ctrl_1.PopupMenu(self.popupmenuNotSolveCase, pos)
        elif pos[0] <= colWidth[0]+colWidth[1]+colWidth[2]+colWidth[3]+colWidth[4]:
            if self.checkFolderMsg(selectDir) == "case":
                if selectDir == solveCaseFix:
                    self.tree_ctrl_1.PopupMenu(self.popupmenuResult, pos)
                else:
                    self.tree_ctrl_1.PopupMenu(self.popupmenuNotSolveCase, pos)
        folderMsg = "yes"
        return


    #folderのpopupメニュー----------------------
    def OnPopupItemSelected(self, event):
        global selCaseDir, solveCaseFix, editName, childrenList, rootDir
        PPitem = self.popupmenu.FindItemById(event.GetId())
        text = PPitem.GetText()
        #開く
        if (text == u"開く"):
            items = self.tree_ctrl_1.GetSelections()
            for item in items:
                directory = self.getDirectory(item)
                os.system(("nautilus " + directory).encode("utf-8"))
        #端末の起動
        elif (text == u"端末の起動"):
            self.runTerms()
        #FOAM端末の起動
        elif (text == u"FOAM端末の起動"):
            self.runFoamTerms()
        #解析caseとして設定
        elif (text == u"解析caseとして設定"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
        #新しいフォルダ追加
        elif (text == u"新しいフォルダ追加"):
            if self.checkMultiSelection(text)==False:
                self.insertFolder()
        #フォルダ削除
        elif (text == u"フォルダ削除..."):
            self.deleteFolders()
        #フォルダ名修正
        elif (text == u"フォルダ名変更..."):
            if self.checkMultiSelection(text)==False:
                self.renameFolder()
        #フォルダコピー
        elif (text == u"フォルダコピー"):
            self.foldersCopy()
        #caseのコピー
        elif (text == u"caseのコピー"):
            self.casesCopy()
        #貼付け(フォルダ, case)
        elif (text == u"貼付け(folder, case)"):
            if self.checkMultiSelection(text)==False:
                self.foldersCasesPaste()
        #CADの起動
        elif (text == u"CADの起動"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                #self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.selRunCAD(event)
        #SalomeMecaの起動
        elif (text == u"SalomeMecaの起動"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                #self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.selRunSalome(event)

    #solverのpopupメニュー ---------------------------
    def OnPopupItemSolverSelected(self, event):
        global selCaseDir, solveCaseFix, editName, childrenList, rootDir
        PPitem = self.popupmenuSolver.FindItemById(event.GetId())
        text = PPitem.GetText()
        #開く
        if (text == u"開く"):
            items = self.tree_ctrl_1.GetSelections()
            for item in items:
                directory = self.getDirectory(item)
                os.system(("nautilus " + directory).encode("utf-8"))
        #FOAM端末の起動
        elif (text == u"FOAM端末の起動"):
            if self.checkMultiSelection(text)==False:
                self.runFoamTerms()
        #gridEditor起動
        elif (text == u"gridEditor起動..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.openGridEditor(caseDir)
        #fieldの編集
        elif (text == u"fieldの編集..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.selEditField(event)
        #fieldにデータセット
        elif (text == u"fieldにデータセット..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.selDataSetClear(event)
        #controlDict, fvの編集
        elif (text == u"controlDictの編集"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.onCntDictOpen(event)
        #propertiesの編集
        elif (text == u"properties(const)の編集..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.editPropertiesFiles()
        #dictionariesの編集
        elif (text == u"dictionaries(sys)の編集..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.editDict()
        #meshの編集
        elif (text == u"meshの編集..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.selEditMesh(event)
        #solver実行
        elif (text == u"solver実行"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.selRunSolver(event)
        #plotWatcherを実行
        elif (text == u"plotWatcherを実行"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.runPlotWatcher()

        #並列処理
        elif (text == u"並列処理..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.onParallel(event)

    #resultのpopupメニュー --------------------
    def OnPopupItemResultSelected(self, event):
        global selCaseDir, solveCaseFix, editName, childrenList, rootDir
        PPitem = self.popupmenuResult.FindItemById(event.GetId())
        text = PPitem.GetText()
        #開く
        if (text == u"開く"):
            items = self.tree_ctrl_1.GetSelections()
            for item in items:
                directory = self.getDirectory(item)
                os.system(("nautilus " + directory).encode("utf-8"))
        #FOAM端末の起動
        elif (text == u"FOAM端末の起動"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.runFoamTerms()
        #gridEditor起動
        elif (text == u"gridEditor起動..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                caseDir = self.getDirectory(item)
                self.openGridEditor(caseDir)

        #残渣の確認(plotWatcher起動)
        elif (text == u"残渣の確認(plotWatches起動)"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.runPlotWatcher()
            
        #結果フォルダの削除
        elif (text == u"結果フォルダの削除..."):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
                self.deleteResultFolders()
                self.reloadItems()

    #NotSolveCaseのpopupメニュー --------------------
    def OnPopupItemNotSolveCaseSelected(self, event):
        global selCaseDir, solveCaseFix, editName, childrenList, rootDir
        PPitem = self.popupmenuNotSolveCase.FindItemById(event.GetId())
        text = PPitem.GetText()
        #開く
        if (text == u"開く"):
            items = self.tree_ctrl_1.GetSelections()
            for item in items:
                directory = self.getDirectory(item)
                os.system(("nautilus " + directory).encode("utf-8"))
        #端末の起動
        elif (text == u"端末の起動"):
            self.runTerms()
        #解析caseとして設定
        elif (text == u"解析caseとして設定"):
            if self.checkMultiSelection(text)==False:
                item = self.tree_ctrl_1.GetSelection()
                self.changeSolveCase(item)
        #FOAM端末の起動
        elif (text == u"FOAM端末の起動"):
            self.runFoamTerms()



    def checkMultiSelection(self, text):
        items = self.tree_ctrl_1.GetSelections()
        if len(items) == 1:
            return False
        else:
            self.errorMultiSelect(text)
            return True

    def errorMultiSelect(self, name):
        title = u"エラー"
        msg = u"複数行選択しています。\n「" + name + u"」は複数行選択できません。"
        self.errDialog(title, msg)


    def deleteResultFolders(self):
        global solveCaseFix
        #singleCore計算時の処理
        timeFolders = pyTreeFoam.getTimeFolders(solveCaseFix)
        ans = ""
        ansFlag = 0
        if len(timeFolders) > 1:
            title = u"結果フォルダの削除"
            msg = u"最小時間のフォルダのみ残し、それ以外をゴミ箱に移動します。"
            ans = self.okCancelDialog(title, msg)
            if ans == "OK":
                if selCaseDirFix != solveCaseFix:
                    msg = u"解析caseと選択しているcaseが異なっています。\n　解析caseの結果フォルダを削除しますか？"
                    ans = self.okCancelDialog(title, msg)
                    if ans == "OK":
                        ansFlag = 1
                else:
                    ansFlag = 1
                if ansFlag == 1:
                    i=1
                    while i<len(timeFolders):
                        delDir = solveCaseFix + "/" + timeFolders[i]
                        self.deleteFolder(delDir)
                        i += 1
        #並列計算時の処理
        [folders, files] = pyTreeFoam.getFoldersFiles(solveCaseFix)
        flag = 0
        ansFlag = 0
        for folder in folders:
            if folder[:len("processor")] == "processor":
                timeFolders = pyTreeFoam.getTimeFolders(solveCaseFix+"/processor0")
                if len(timeFolders) > 1:
                    flag = 1
                break
        if flag == 1:
            if ans == "":
                title = u"結果フォルダの削除"
                msg = u"最小時間のフォルダのみ残し、それ以外をゴミ箱に移動します。"
                ans = self.okCancelDialog(title, msg)
            if ans == "OK":
                if selCaseDirFix != solveCaseFix:
                    msg = u"解析caseと選択しているcaseが異なっています。\n　解析caseの結果フォルダを削除しますか？"
                    ans = self.okCancelDialog(title, msg)
                    if ans == "OK":
                        ansFlag = 1
                else:
                    ansFlag = 1
                if ansFlag == 1:
                    for folder in folders:
                        if folder[:len("processor")] == "processor":
                            delCaseDir = solveCaseFix + "/" + folder
                            timeFolders = pyTreeFoam.getTimeFolders(delCaseDir)
                            i = 1
                            while i<len(timeFolders):
                                delDir = delCaseDir + "/" + timeFolders[i]
                                self.deleteFolder(delDir)
                                i += 1


    def openGridEditor(self, caseDir):
        timeFolders = getResultTimeFolders(caseDir)
        wdir = solveCaseFix+"/system"
        startFrom = getFoamContents.controlDict(wdir, ["startFrom"])
        if len(timeFolders) != 0 and startFrom != None:
            if startFrom[0] == "startTime":
                timeFolder = timeFolders[0]
            elif startFrom[0] == "latestTime":
                timeFolder = timeFolders[-1]
            else:
                timeFolder = timeFolders[0]
            fieldDir = caseDir + "/" + timeFolder
            caseFolder = caseDir.split("/")[-1]
            title = ": " + caseFolder + "/" + timeFolder + "/."
            #FFlist = setEditFolders(fieldDir)
            #openGridEditorDialog内でFFlistを再設定している為、FFlistの設定は不要
            FFlist = []
            title = u"boundaryFieldの編集"
            mess = u"選択された boundaryField を編集します"
            openGridEditorDialog.Show(nTreeFoam, title, mess, FFlist, fieldDir, timeFolder)
        else:
            title = u"エラー"
            mess = u"OpenFOAMのCaseフォルダではありません！"
            self.errDialog(title, mess)

    def editPropertiesFiles(self):
        global solveCaseFix
        wdir = solveCaseFix + "/constant"
        [folders, files] = pyTreeFoam.getFoldersFiles(wdir)
        if len(folders) >= 1:
            FFlist = setEditFolders(wdir)
            title = u"...Properties File の編集"
            img = "oneFile"
            mess = u"編集する Properties file を選択してください。\n\nProperties File は、constant フォルダ内にあります。"
            editDir = wdir
            job = "editProperties"
            # properties File の選択と編集
            selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)
        else:
            print "constantフォルダ内にpolyMeshフォルダがありません!\n  current directoryを確認してください。"

    def editDict(self):
        global solveCaseFix
        wdir = solveCaseFix + "/system"
        FFlist = setEditFolders(wdir)
        title = u"Dict File の編集"
        img = "oneFaile"
        mess = u"編集するDict file を選択してください。\n\nDict file は、systemフォルダ内にあります。"
        editDir = wdir
        job = "editDictFile"
        # boundary File の選択と編集
        selectFolderFilesDialog.Show(job, title, img, mess, FFlist, editDir)


    def deleteFolders(self):
        global selCaseDir, solveCaseFix
        print "フォルダを削除します"
        self.writeStatus(u"フォルダを削除中...")
        items = self.tree_ctrl_1.GetSelections()
        for item in items:
            ans = self.checkDelete(item)
            if len(ans) != 0:
                break
        if len(ans) != 0:
            title = u"エラー"
            self.errDialog(title, ans)
            return

        title = u"フォルダの削除"
        mess = u"フォルダをゴミ箱へ移動します"
        ans = self.okCancelDialog(title, mess)
        if ans != "OK":
            return

        for item in items:
            delDir = self.getDirectory(item)
            error = self.deleteFolder(delDir)
            if error != "":
                title = u"エラー"
                result = self.okCancelDialog(title, error)
                if result != "OK":
                    break
        selCaseDir = "/".join(delDir.split("/")[:-1])
        self.saveSelCaseDirToConfig(selCaseDir)
        #Tree表示に削除を反映
        self.setSelectCase(self.convDirToList(selCaseDir))
        self.reloadItems()
        self.clearStatus()


    def checkDelete(self, item):
        delDir = self.getDirectory(item)
        mess = ""
        if delDir == rootDirFix:
            mess = u"rootDirを削除しようとしています。\nrootDirは、削除できません。"
            return mess
        if delDir == solveCaseFix:
            mess = u"解析caseを削除しようとしています。\n解析caseは、削除できません。\n削除する場合は、解析caseを変更してください。"
            return mess
        if len(solveCaseFix) > len(delDir):
            for (root, dirs, files) in os.walk(delDir):
                for name in files:
                    if root[:len(solveCaseFix)] == solveCaseFix:
                        mess = u"削除するfolder内に解析case解析caseが含まれます。\n解析caseは、削除できません。\n削除する場合は、解析caseを変更してください。"
                        return mess
        return mess


    def deleteFolder(self, delDir):
        mess = ""
        try:
            #ファイルをゴミ箱へ移動
            obj = gio.File(delDir.encode("utf-8"))
            obj.trash()
            #shutil.rmtree(delDir)
        except:
            mess = u"「" + delDir + u"」フォルダが削除できませんでした。\n　継続しますか？"
            print mess
        return mess


    def insertFolder(self):
        print "フォルダを追加します"
        self.writeStatus(u"フォルダを作成中...")
        item = self.tree_ctrl_1.GetSelection()
        itemDir = self.getDirectory(item)
        folderName = itemDir + "/newFolder"
        n = 0
        error = "NG"
        while error == "NG":
            newFolderName = folderName + "_" + str(n)
            try:
                os.mkdir(newFolderName)
                error = "OK"
            except:
                n += 1
        self.saveSelCaseDirToConfig(newFolderName)
        self.reloadItems()
        newItem = self.getItem(newFolderName)
        self.tree_ctrl_1.SelectItem(newItem, newItem, True)
        self.renameFolder()


    def renameFolder(self):
        global selCaseDir, solveCaseFix, editName, childrenList, rootDir
        self.writeStatus(u"フォルダ名を変更中...")
        item = self.tree_ctrl_1.GetSelection()
        saveDir = self.getDirectory(item)
        editName = self.tree_ctrl_1.GetItemText(item)
        if editName == rootDir:
            self.changeRoot(editName)
        else:
            parent = self.tree_ctrl_1.GetItemParent(item)
            childrenList = self.getChildrenList(parent)
            #self.tree_ctrl_1.EditLabel(item)
            inputTextDDialog.title = u"folder名の変更"
            inputTextDDialog.message = u"変更するfolder名を入力してください。"
            inputTextDDialog.inputText = editName
            dialog = inputTextDDialog.MyDialog(None)
            result = dialog.ShowModal()
            if inputTextDDialog.status == "OK":
                text = inputTextDDialog.inputText
            else:
                text = u""
            if text != u"":
                flag = 0
                for name in childrenList:
                    if name == text:
                        if name != editName:
                            msg = "フォルダ内に同じ名前があります。"
                            self.errDialog("エラー", msg)
                        flag = 1
                        break
                if flag == 0:
                    self.tree_ctrl_1.SetItemText(item, text)
                    parentDir = self.getDirectory(parent)
                    editDir = parentDir + "/" + editName
                    textDir = parentDir + "/" + text
                    shutil.move(editDir, textDir)
                    selCaseDir = textDir
                    self.saveSelCaseDirToConfig(selCaseDir)
                    if saveDir == solveCaseFix:
                        #変更するfolderが解析caseの場合
                        solveCaseFix = textDir
                        #caseName, solverをlabelにセット
                        caseItem = self.getItem(solveCaseFix)
                        self.changeSolveCase(caseItem)
                    self.saveSelCaseDirToConfig(selCaseDir)
                    self.reloadItems()
                    self.checkLinkAtGauge(parentDir, editDir, textDir)
        self.clearStatus()


    def checkLinkAtGauge(self, checkDir, oldDir, newDir):
        gaugeDDialog.job = "checkLink"
        gaugeDDialog.title = u"親folder以下のlinkをチェック中"
        gaugeDDialog.checkDir = checkDir
        gaugeDDialog.oldDir = oldDir
        gaugeDDialog.newDir = newDir
        dialog = gaugeDDialog.MyDialog(None)
        result = dialog.ShowModal()
        ans = gaugeDDialog.status
        if ans != "OK":
            #キャンセルした場合
            title = u"linkのチェック、修正"
            mess = u"linkのチェックをキャンセルしました。"
            self.okDialog(title, mess)
        return


    #  foldersCopy
    #    folderのコピー
    def foldersCopy(self):
        global clipBoard
        items = self.tree_ctrl_1.GetSelections()
        f = open(clipBoard, "w")
        f.write("foldersCopy\n")
        for item in items:
            directory = self.getDirectory(item)
            print "copy--->", directory
            f.write(directory.encode("utf-8") + "\n")
        f.close()

    #  casesCopy
    #    case（OpenFOAM）のコピー
    def casesCopy(self):
        global clipBoard
        items = self.tree_ctrl_1.GetSelections()
        f = open(clipBoard, "w")
        f.write("casesCopy\n")
        for item in items:
            caseDir = self.getDirectory(item)
            print "case copy--->", caseDir
            [copyDirs, copyFiles] = self.checkCaseDir(caseDir)
            if len(copyDirs) != 0:
                for copyDir in copyDirs:
                    f.write(copyDir.encode("utf-8") + "\n")
                for copyFile in copyFiles:
                    f.write(copyFile.encode("utf-8") + "\n")
            else:
                title = u"エラー"
                msg = u"「" + caseDir + u"」は、OpenFoamのcaseDirectoryではありません。\nコピーできません。"
                print msg
                self.errDialog(title, msg)
        f.close()

    def checkCaseDir(self, caseDir):
        ans = [[],[]]
        noans = [[],[]]
        [folders, files] = pyTreeFoam.getFoldersFiles(caseDir)
        caseFolder = []
        numFolder = []
        for folder in folders:
            try:
                a = float(folder)
                numFolder.append([a, folder])
            except:
                if checkFolder(caseDir + "/" + folder) != "case":
                    caseFolder.append(folder)
        nFolder = 0
        for folder in caseFolder:
            if folder == "system":
                nFolder += 1
            if folder == "constant":
                nFolder += 1
        if nFolder != 2:
            return noans
        if len(numFolder) == 0:
            return noans

        numFolder.sort()
        numIni = numFolder[0][1]
        caseFolder.append(numIni)
        for name in caseFolder:
            ans[0].append(caseDir + "/" + name)
        for name in files:
            ans[1].append(caseDir + "/" + name)
        return ans


    def foldersCasesPaste(self):
        f = open(clipBoard); contents = f.read(); f.close()
        lines = contents.decode("utf-8").split("\n")[:-1]   #最後に空文が入るので最後を削除
        if lines[0] == "foldersCopy":
            self.foldersPaste(lines[1:])
        elif lines[0] == "casesCopy":
            self.casesPaste(lines[1:])
        else:
            title = u"エラー"
            msg = u"クリップボードのデータは、貼り付ける事ができません。"
            self.errDialog(title, msg)

    def casesPaste(self, copyDirs):
        global clipBoard, selCaseDir
        item = self.tree_ctrl_1.GetSelection()
        itemDir = self.getDirectory(item)

        newDirs = []
        caseName = ""
        for copyDir in copyDirs:
            newCaseName = copyDir.split("/")[-2]
            if (caseName != newCaseName):
                caseName = newCaseName
                copyCaseDir = "/".join(copyDir.split("/")[:-1])
                newCaseDir = self.checkCopyDirFile(itemDir, copyCaseDir)
            newDir = self.checkCopyDirFile(newCaseDir, copyDir)
            if len(newDir) != 0:
                newDirs.append(newDir)
        if len(newDirs) != 0:
            self.pasteFoldersFilesAtGauge(copyDirs, newDirs)

            self.reloadItems()
            newItem = self.getItem(newCaseDir)
            self.tree_ctrl_1.SelectItem(newItem, newItem, True)
            self.reloadItems()
            item = self.getItem(itemDir)
            self.tree_ctrl_1.SelectItem(item, item, True)


    def foldersPaste(self, copyDirs):
        global clipBoard
        item = self.tree_ctrl_1.GetSelection()
        itemDir = self.getDirectory(item)
        error = 0
        #print copyDirs
        for copyDir in copyDirs:
            if os.path.isfile(copyDir) == True:
                error = 1
                break
        if error == 1:
            title = u"警告"
            mess = u"clipBoard内のデータにファイルが含まれています。\ncaseのコピーデータの可能性があります。\n　　継続しますか？。"
            ans = self.okCancelDialog(title, mess)
            if ans != "OK":
                return
        else:
            for copyDir in copyDirs:
                if os.path.isdir(copyDir) == True:
                    if copyDir.split("/")[-1] == "system":
                        title = u"警告"
                        mess = u"clipBoard内のデータに「system」フォルダが含まれています。\ncaseのコピーデータの可能性があります。\n　　継続しますか？"
                        ans = self.okCancelDialog(title, mess)
                        if ans != "OK":
                            return

        newDirs = []
        for copyDir in copyDirs:
            newDir = self.checkCopyDirFile(itemDir, copyDir)
            if len(newDir) != 0:
                newDirs.append(newDir)
        if len(newDirs) != 0:
            self.pasteFoldersFilesAtGauge(copyDirs, newDirs)

            self.reloadItems()
            newItem = self.getItem(newDir)
            self.tree_ctrl_1.SelectItem(newItem, newItem, True)
            self.reloadItems()
            newItem = self.getItem(itemDir)
            self.tree_ctrl_1.SelectItem(newItem, newItem, True)



    def checkCopyDirFile(self, itemDir, copyDir):
        newDir = itemDir + "/" + copyDir.split("/")[-1]
        if os.path.isdir(copyDir) == True:
            if len(glob.glob(newDir)) == 0:
                return newDir
            else:
                flag = 0
                n = 0
                while (flag == 0 and n < 10):
                    newDir_n = newDir + "_copy" + str(n)
                    if len(glob.glob(newDir_n)) == 0:
                        flag = 1
                    n += 1
                if flag == 1:
                    return newDir_n
                else:
                    return ""
        elif os.path.isfile(copyDir) == True:
            return newDir
        else:
            return ""


    def pasteFoldersFilesAtGauge(self, copyDirs, newDirs):
        gaugeDDialog.job = "copy"
        gaugeDDialog.title = u"folder, fileのコピー"
        gaugeDDialog.copyDirs = copyDirs
        gaugeDDialog.newDirs = newDirs
        dialog = gaugeDDialog.MyDialog(None)
        reult = dialog.ShowModal()
        ans = gaugeDDialog.status
        if ans == "NG":
            #コピーをキャンセルした場合
            title = u"fileのコピー"
            mess = u"コピーを中止しました"
            self.okDialog(title, mess)
        if ans == "ERR":
            #エラーが発生した場合
            title = u"エラー"
            [error, mess] = gaugeDDialog.error
            self.errDialog(title, mess)
        return


    def changeConfigTreeFoam(self, event):
        global configFile, treeFoamPath
        global paraFoamFix, salomeMecaFix, envOpenFOAMFix, cadFix, editor
        global OFversionFix
        #configTreeFoamを書き換え
        fileName = treeFoamPath + "/" + configFile
        changeConfigTreeFoamDDialog.configFile = fileName
        dialog = changeConfigTreeFoamDDialog.MyDialog(None)
        result = dialog.ShowModal()
        ans = changeConfigTreeFoamDDialog.status
        if ans == "OK":
            #configTreeFoamの内容を変数にセット
            configData = pyTreeFoam.readConfigTreeFoam()
            OFversion = configData[0]
            envOpenFOAMFix = configData[1]
            paraFoamFix = configData[4]
            salomeMecaFix = configData[5]
            cadFix = configData[6]
            editor = configData[7]
            #OpenFOAM環境設定表示を変更
            self.setLabelDataInFrame()
            #終了メッセージ
            title = u"configTreeFoamの変更"
            mess = u"configTreeFoamの内容を修正しました。"
            self.okDialog(title, mess)


    def changeRoot(self, editName):
        text = self.textDialog("rootの変更", "変更するrootDirを入力してください。", editName)
        if text != "":
            newName = glob.glob(text)
            if len(newName) == 1:
                self.saveRootDirToConfig(newName[0])
                self.reloadItems()
                msg = u"rootDirを変更しました。"
                self.okDialog(u"rootDirの変更", msg)
            else:
                msg = text + u" のフォルダは存在しません。"
                self.errDialog(u"エラー", msg)

    def runGridEditor(self, editDir):
        global solveCaseFix
        #title, dirを設定
        caseFolder = editDir.split("/")[-2]
        timeFolder = editDir.split("/")[-1]
        title = ": " + caseFolder + "/" + timeFolder + "/."
        #fieldDir = editDir + "/" + selectedFields[0][0]
        fieldDir = editDir
        boundaryDir = solveCaseFix + "/constant/polyMesh"
        #Grid表示
        comm = "runGridEditor.py "
        comm+= nTreeFoam + " '" + title + "' " + boundaryDir + " " + fieldDir
        comm+= " &"
        os.system(comm)

    def runSolver(self, directory):
        global envOpenFOAMFix
        if self.checkApp(directory, envOpenFOAMFix) == "OK":
            solver = getSolver(directory)
            if solver != "":
                #dir移動
                saveDir = os.getcwd()
                os.chdir(directory)
                #solve.logのクリア
                f=open("solve.log","w")
                f.close()
                #solverの実行ファイル作成
                caseName = directory.split("/")[-1]
                title =  "#!/bin/bash\n"
                envSet = ". " + envOpenFOAMFix + "\n"
                solverSet = solver + " | tee solve.log\n"
                cont = title + envSet + solverSet
                f=open("run","w")
                f.write(cont)
                f.close()
                #実行権付与
                os.system("chmod a+x run")
                #実行
                comm = "gnome-terminal -t " + caseName + " --geometry=120x20 --zoom=0.9 -x bash --rcfile run"
                os.system(comm)
                #dirを元に戻す
                os.chdir(saveDir)
            else:
                title = u"エラー"
                message = u"「controlDict」は、存在しません。\n"
                message = message + u"case directory が間違っています。"
                self.errDialog(title, message)
        else:
            title = u"エラー"
            message = u"環境設定file「bashrcFOAM」が存在しない為、FOAM端末が起動できません。\n"
            message = message + u"　　configTreeFoamファイルの内容を確認してください。"
            self.errDialog(title, message)

    def runPlotWatcher(self):
        global solveCaseFix, envOpenFOAMFix
        if self.checkApp(solveCaseFix, envOpenFOAMFix) == "OK":
            solver = getSolver(solveCaseFix)
            if solver != "":
                saveDir = os.getcwd()
                os.chdir(solveCaseFix)
                cont = "#!/ban/bash\n"
                cont = cont + ". " + envOpenFOAMFix + "\n"
                cont = cont + 'echo -n "running plotWatcher"\n'
                cont = cont + "pyFoamPlotWatcher.py solve.log 1> /dev/null 2> /dev/null\n"
                f = open("plotWatcher", "w")
                f.write(cont)
                f.close()
                os.system("chmod a+x plotWatcher")
                title = solveCaseFix
                if len(title) > 30:
                    title = "_..." + title[-30:]
                title = "plotWatcher:" + title
                comm = "gnome-terminal -t " + title + " --geometry=80x0 --hide-menubar -x bash --rcfile plotWatcher"
                os.system(comm)
                os.chdir(saveDir)
            else:
                title = u"エラー"
                message = u"「controlDict」は、存在しません。\n"
                message = message + u"case directory が間違っています。"
                self.errDialog(title, message)
        else:
            title = u"エラー"
            message = u"環境設定file「bashrcFOAM」が存在しない為、FOAM端末が起動できません。\n"
            message = message + u"　　configTreeFoamファイルの内容を確認してください。"
            self.errDialog(title, message)

    def runFoamTerms(self):
        items = self.tree_ctrl_1.GetSelections()
        for item in items:
            directory = self.getDirectory(item)
            self.runFoamTermDir(directory)

    def runFoamTerm(self):
        item = self.tree_ctrl_1.GetSelection()
        directory = self.getDirectory(item)
        self.runFoamTermDir(directory)

    def runFoamTermDir(self, directory):
        global envOpenFOAMFix
        if self.checkApp(directory, envOpenFOAMFix) == "OK":
            caseName = directory.split("/")[-1]
            comm = "cd " + directory + "; gnome-terminal -t " + caseName + " -x bash --rcfile " + envOpenFOAMFix + " &"
            os.system(comm)
        else:
            title = u"エラー"
            message = u"「" + envOpenFOAMFix + u"」は、存在しません。\n"
            message = message + u"FOAM端末の環境設定fileが間違っています。\n"
            message = message + u"  configFoamTreeの内容を確認してください。"
            self.errDialog(title, message)

    def runTerms(self):
        items = self.tree_ctrl_1.GetSelections()
        for item in items:
            directory = self.getDirectory(item)
            self.runTermDir(directory)

    def runTerm(self):
        item = self.tree_ctrl_1.GetSelection()
        directory = self.getDirectory(item)
        self.runTermDir(directory)
        
    def runTermDir(self, directory):
        comm =  "cd " + directory + ";"
        comm += "gnome-terminal &"
        os.system(comm)

    def checkApp(self, directory, App):
        flag = "NG"
        names = glob.glob(directory + "/" + App)
        if len(names) == 1:
            flag = "OK"
        else:
            names = glob.glob(App)
            if len(names) == 1:
                flag = "OK"
            else:
                paths = os.environ["PATH"].split(":")
                for path in paths:
                    names = glob.glob(path + "/" + App)
                    if len(names) == 1:
                        flag = "OK"
                        break
        return flag

    def runHelyxos(self):
        global solveCaseFix
        bashrc = envOpenFOAMFix
        comm = ". " + bashrc + ";"
        comm+= "runHelyxosDialog.py " + solveCaseFix + "&"
        os.system(comm)


    def runParaFoamDir(self, directory):
        global paraFoamFix
        if checkFolder(directory) == "case":
            if self.checkApp(directory, paraFoamFix) == "OK":
                comm = "cd " + directory + ";" + paraFoamFix + "&"
                os.system(comm.encode("utf-8"))
            else:
                title = u"エラー"
                message = u"「" + paraFoamFix + u"」は、存在しません。\n"
                message = message + u"実行fileが間違っています。"
                self.errDialog(title, message)
        else:
            title = u"エラー"
            message = u"指定されたフォルダ「" + directory + u"」は、OpenFOAMのCaseではありません。"
            self.errDialog(title, message)

    def runSalomeMeca(self, directory):
        global salomeMecaFix
        if self.checkApp(directory, salomeMecaFix) == "OK":
            comm = salomeMecaFix + " " + directory
            #comm = "cd " + directory + ";" + salomeMecaFix
            os.system(comm.encode("utf-8"))
        else:
            title = "エラー"
            message = u"「" + salomeMecaFix + u"」は、存在しません。\n"
            message = message + u"実行fileが間違っています。"
            self.errDialog(title, message)

    def runCAD(self, directory):
        global cadFix
        if self.checkApp(directory, cadFix) == "OK":
            #comm = "cd " + directory + ";" + cadFix + " &"
            comm = cadFix + " " + directory + " &"
            os.system(comm)
        else:
            title = u"エラー"
            message = u"「" + cadFix + u"」は、存在しません。\n"
            message = message + u"実行fileが間違っています。"
            self.errDialog(title, message)


    #---------config------------------------
    #
    # getConfig
    #
    def getConfig(self):
        global rootDirFix, selCaseDirFix
        global rootDir, selCaseDir, configFile

        rootDir = rootDirFix
        selCaseDir = selCaseDirFix

        message = ""
        if rootDir == "":
            rootDir = os.path.expanduser("~")
            rootDirFix = rootDir
            message = message + "rootDirが設定されていません。rootDirとしてhomeDirを設定しました。\n"
        if selCaseDir == "":
            selCaseDir = rootDir
            selCaseDirFix = selCaseDir
            message = message + "selCaseDirが設定されていません。selCaseDirとしてrootDirを設定しました。\n"
        if message != "":
            title = "エラー"
            self.errDialog(title, message)

    def saveRootDirToConfig(self, dirName):
        global rootDirFix
        rootDirFix = dirName

    def saveSelCaseDirToConfig(self, dirName):
        global selCaseDirFix
        selCaseDirFix = dirName


    #
    #  readConfig
    #    configFileの読み込み
    def readConfig(self):
        global rootDirFix, solveCaseFix, selCaseDirFix, configFile, paraFoamFix
        global treeFoamPath, salomeMecaFix, envOpenFOAMFix, cadFix, OFversionFix
        global editor, helyxosFix
        #configFileの読み込み
        configData = pyTreeFoam.readConfigTreeFoam()
        #変数をセット
        OFversion = configData[0]
        envOpenFOAMFix = configData[1]
        rootDirFix = configData[2]
        solveCaseFix = configData[3]
        paraFoamFix = configData[4]
        salomeMecaFix = configData[5]
        cadFix = configData[6]
        editor = configData[7]
        helyxosFix = configData[8]
        selCaseDirFix = solveCaseFix


    #
    #  writeConfig
    #    configFileの書き込み
    def writeConfig(self):
        global rootDirFix, solveCaseFix, selCaseDirFix, configFile, treeFoamPath
        os.chdir(treeFoamPath)
        f= codecs.open(configFile, "r", "utf-8")
        configContents = f.readlines()
        f.close()

        #rootDirの置き換え
        i=0
        flag = 0
        for line in configContents:
            word = line.split()
            if (len(word) > 0):
                if (word[0] == "rootDir"):
                    if type(rootDirFix) == str:
                        rootDirFix = rootDirFix.decode("utf-8")
                    configContents[i] = word[0] + "   " + rootDirFix + "\n"
                    flag = 1
                    break
            i=i+1
        if flag == 0:
            configContents.append("rootDir   " + rootDirFix + "\n")

        #solveCaseの置き換え
        i=0
        flag = 0
        for line in configContents:
            word = line.split()
            if (len(word) > 0):
                if (word[0] == "workDir"):
                    if type(solveCaseFix) == str:
                        solveCaseFix = solveCaseFix.decode("utf-8")
                    configContents[i] = word[0] + "   " + solveCaseFix + "\n"
                    flag = 1
                    break
            i=i+1
        if flag == 0:
            configContents.append("workDir   " + solveCaseFix + "\n")

        #ファイルに書込
        fu = codecs.open(configFile, "w", "utf-8")
        for line in configContents:
            fu.write(line)
        fu.close()


    #----------imageの読込み-------------
    def readImageFile(self):
        global imagePng, treeFoamPath
        #imageFileをセット
        imageName = []
        iconPath = treeFoamPath + "/icons/"
        for name in imagePng:
            fileName = iconPath + name
            image = wx.Image(fileName, wx.BITMAP_TYPE_ANY)
            imageName.append(image.ConvertToBitmap())
        #buttonへbitmapをセット
        self.bitmap_button_12.SetBitmapLabel(imageName[0])      #changeConfig
        self.bitmap_button_1.SetBitmapLabel(imageName[1])       #changeRoot
        self.bitmap_button_2.SetBitmapLabel(imageName[2])       #reload
        self.bitmap_button_3.SetBitmapLabel(imageName[3])       #newCase
        self.bitmap_button_4.SetBitmapLabel(imageName[4])       #setCase
        self.bitmap_button_16.SetBitmapLabel(imageName[5])      #exit
        self.bitmap_button_9.SetBitmapLabel(imageName[6])       #gridEditor
        self.bitmap_button_5.SetBitmapLabel(imageName[7])       #editFields
        self.bitmap_button_19.SetBitmapLabel(imageName[8])      #dataSetClear
        self.bitmap_button_6.SetBitmapLabel(imageName[9])       #editProperties
        self.bitmap_button_7.SetBitmapLabel(imageName[10])       #editDict
        self.bitmap_button_8.SetBitmapLabel(imageName[11])       #editMesh
        self.bitmap_button_13.SetBitmapLabel(imageName[12])      #runSolver
        self.bitmap_button_14.SetBitmapLabel(imageName[13])      #plotWatcher
        self.bitmap_button_15.SetBitmapLabel(imageName[14])      #parallel
        self.bitmap_button_20.SetBitmapLabel(imageName[15])      #runCAD
        self.bitmap_button_10.SetBitmapLabel(imageName[16])      #runFoamTerm
        self.bitmap_button_11.SetBitmapLabel(imageName[17])      #runParaview
        self.bitmap_button_17.SetBitmapLabel(imageName[18])      #runSalome


    #----------universal dialog-------------
    def okDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_INFORMATION) 
        dlg.ShowModal()
        dlg.Destroy()

    def okCancelDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                             wx.OK | wx.CANCEL | wx.ICON_INFORMATION) 
        ans = dlg.ShowModal()
        stat = ""
        if ans == wx.ID_OK:
            #dlg.Destroy()
            stat = "OK"
            #return "OK"
        else:
            #dlg.Destroy()
            stat = "CANCEL"
            #return "CANCEL"
        return stat

    def errDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_HAND) 
        dlg.ShowModal()
        dlg.Destroy()

    def warningDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_QUESTION) 
        dlg.ShowModal()
        dlg.Destroy()

    def textDialog(self, title, message, defText):
        dlg = wx.TextEntryDialog(self, message, title, defaultValue=defText)
        ansBtn = dlg.ShowModal()
        ansTxt = dlg.GetValue()
        if ansBtn == wx.ID_OK:
            text = ansTxt
        else:
            text = ""
        return text

    def writeStatus(self, text):
        self.frame_1_statusbar.SetStatusText(text)

    def clearStatus(self):
        global selectTreeDir
        path = selectTreeDir
        if path == "" or len(glob.glob(path))==0:
            path = "/home"
        s = os.statvfs(path.encode("utf-8"))
        #total = (int(((s.f_frsize * s.f_blocks) / 1024.0**3) * 100.0))/100.0
        #avail = (int(((s.f_frsize * s.f_bavail) / 1024.0**3) * 100.0))/100.0
        #合計
        total = (int(((s.f_frsize * s.f_blocks) / 1000.0**3) * 100.0))/100.0
        #利用可能な空き領域
        avail = (int(((s.f_frsize * s.f_bavail) / 1000.0**3) * 100.0))/100.0
        #空き領域（合計）
        favail = (int(((s.f_frsize * s.f_favail) / 1024.0**3) * 100.0))/100.0
        msg = "合計 " + str(total) + " GB,  空き " + str(avail) + " GB"
        self.frame_1_statusbar.SetStatusText(msg)

# end of class MyFrame




#
# getBashrcFoam
#   外からOpenFOAMの環境設定ファイルを読み込む時に使用
#   環境変数$TreeFoamPathから取得する
def getBashrcFoam():
    treeFoamPath = os.environ["TreeFoamPath"]
    if treeFoamPath == "":
        print "環境変数$TreeFoamPathが設定されていないので、隠しファイル~/.treeFoamInstallFolderから、TreeFoamPathを取得します。"
        homeFolder = os.environ["HOME"]
        f = open(homeFolder + "/.treeFoamInstallFolder")
        treeFoamPath = f.readline()
        f.close()
    f = open(treeFoamPath + "/configTreeFoam")
    configContents = f.readlines()
    f.close()
    for line in configContents:
        word = line.split()
        if (len(word) > 0):
            if (word[0] == "bashrcFOAM"):
                if len(word) >1:
                    envOpenFOAMFix = word[1]
                else:
                    envOpenFOAMFix = ""
                break
    return envOpenFOAMFix


def getSolver(selectDir):
    fileName = selectDir + "/system/controlDict"
    solver = ""
    if os.path.isfile(fileName) == True:
        f=open(fileName)
        for line in f.readlines():
            lineList = line.split()
            if len(lineList) >= 2:
                if lineList[0] == "application":
                    solver = lineList[1]
                    if solver[-1] == ";":
                        solver = solver[:-1]
                    break
        f.close()
    return solver

    

#
#  getResultTimeFolders
#    計算結果フォルダを取得
def getResultTimeFolders(wdir):
    folders = getFoldersOnlyCurrDir(wdir)
    ans = []
    for folder in folders:
        a = folder.split("/")[-1]
        try:
            num = float(a)
            ans.append([num,a])
        except:
            a = ""
    ans.sort()
    timeFolders = []
    for a in ans:
        timeFolders.append(a[1])
    return timeFolders

#
#  getNumberResult
#    計算結果folderの数を取得
def getNumberResult(selectDir):
    folders = getFoldersOnlyCurrDir(selectDir)
    ans = []
    for folder in folders:
        try:
            a = float(folder.split("/")[-1])
            ans.append(a)
        except:
            a = 1.0
    nRst = len(ans)
    if nRst < 2:
        for folder in folders:
            if folder.split("/")[-1] == "processor0":
                subFolders = getFoldersOnlyCurrDir(selectDir + "/processor0")
                ans = []
                for subFolder in subFolders:
                    try:
                        a = float(subFolder.split("/")[-1])
                        ans.append(a)
                    except:
                        a = 1.0
                nRst = len(ans)
                break
    ansR = []
    if nRst != 0:
        ansR.append(" " + str(nRst))
        ansR.append(" " + str(min(ans)))
        if min(ans) == max(ans):
            ansR.append("")
        else:
            ansR.append(" " + str(max(ans)))
    return ansR

#
#  checkFolder
#    folderのチェック
def checkFolder(folderDir):
    ans = ""
    if os.path.isdir(folderDir) == True:
        ans = "folder"
        folderName = folderDir.split("/")[-1]
        try:
            a=float(folderName)
            #数字フォルダ
            ans = "hide"
        except:
            if (folderName == "system" or
            folderName == "constant" or
            folderName == "model" or
            folderName == "solve.log.analyzed" or
            folderName[0] == "."):
                #禁止フォルダ
                ans = "hide"
            else:
                controlDict = folderDir + "/system/controlDict"
                if os.path.isfile(controlDict)==True:
                    #caseフォルダ
                    ans = "case"
                elif len(glob.glob(folderDir+"/*.hdf")) != 0:
                    ans = "salome" 
    else:
        #ファイル
        ans = "file"
    return ans

#
#  setEditFolders
#    編集するtimeフォルダ内のファイルとフォルダをセット
def setEditFolders(editDir):
    FFlist = []
    [folders, files] = pyTreeFoam.getFoldersFiles(editDir)
    fields = pyTreeFoam.getFieldNames(editDir)
    # カレントフォルダをFFlistにセット
    folderFiles = setFolderFiles(".",fields)
    FFlist.append(folderFiles)
    # フォルダをFFlistにセット
    i = 0
    while i < len(folders):
        # 各フォルダをFFlistにセット
        wdir = editDir + "/" + folders[i]
        fields = pyTreeFoam.getFieldNames(wdir)
        folderFiles = setFolderFiles(folders[i],fields)
        FFlist.append(folderFiles)
        i = i + 1
    return FFlist


def setFolderFiles(folder, files):
    fflist = []
    fflist.append(folder)
    for name in files:
        fflist.append(name)
    return fflist


#
#  getTreeDirOnlylRootChildren
#
def getTreeDirOnlylRootChildren(rootDir):
    global treeData
    treeData = pyTreeFoam.getTreeDirOnlylRootChildren(rootDir)

#
#  getTreeDirUntilCurrCase
#    currCaseまでのtreeデータ取得
#
def getTreeDirUntilCurrCase(rootDir, selCaseDir):
    global treeData, selCase
    #treeDataの取得
    treeData = pyTreeFoam.getTreeDirUntilCurrCase(rootDir, selCaseDir)
    #selCaseの設定
    selCaseNames = selCaseDir[len(rootDir)+1:].split("/")
    selCase=[]
    selCase.append(rootDir)
    for name in selCaseNames:
        selCase.append(name)

#
#　getFoldersOnlyCurrDir
#  　currDirのみのFolderを取得する
#
def getFoldersOnlyCurrDir(currDir):
    folders = []
    try:
        allItem = os.listdir(currDir)
        for item in allItem:
            newFolder = currDir + "/" + item
            if os.path.isdir(newFolder) == True:
                folders.append(newFolder)
    except:
        folders=[]
    return folders


#
#  endingTreeFoam
#    TreeFoamの終了処理
def endingTreeFoam(treeSize, colWidth):
    # TreeFoamの起動番号（nTreeFoam）を削除
    fileName = treeFoamPath + "/nRunningTreeFoam"
    f=open(fileName); contents=f.readlines(); f.close()
    newNoList = []
    for line in contents:
        NoList=line.split()
        if len(NoList) != 0:
            No = int(NoList[0])
            if No != int(nTreeFoam):
                newNoList.append(line)
    f=open(fileName,"w")
    for line in newNoList:
        f.write(line)
    f.close()
    #windowSizeの書き込み
    saveTreeSize(treeSize, colWidth)

#  saveTreeSize
#    treeFoamのwindowSizeを保存
def saveTreeSize(treeSize, colWidth):
    (h, w) = treeSize
    fileName = os.environ["TreeFoamPath"] + "/winSize"
    contents = []
    if len(glob.glob(fileName)) != 0:
        f=open(fileName); contents=f.readlines(); f.close()
        i=0; flag=0; flagCol = 0
        for line in contents:
            words = line.split()
            if words[0] == "treeFoam":
                words[1] = str(h)
                words[2] = str(w)
                contents[i] = words[0] + " " + words[1] + " " + words[2] + "\n"
                flag = 1
            if words[0] == "treeFoam_colWidth":
                contents[i] = words[0]
                contents[i] += " " + str(colWidth[0])
                contents[i] += " " + str(colWidth[1])
                contents[i] += " " + str(colWidth[2])
                contents[i] += " " + str(colWidth[3])
                contents[i] += " " + str(colWidth[4]) + "\n"
                flagCol = 1
            if flag == 1 and flagCol == 1:
                break
            i += 1
        if flag == 0:
            line = "treeFoam " + str(h) + " " + str(w) + "\n"
            contents.append(line)
        if flagCol == 0:
            line = "treeFoam_colWidth "
            line += " " + str(colWidth[0])
            line += " " + str(colWidth[1])
            line += " " + str(colWidth[2])
            line += " " + str(colWidth[3])
            line += " " + str(colWidth[4]) + "\n"
            contents.append(line)
    else:
        line = "treeFoam " + str(h) + " " + str(w) + "\n"
        contents.append(line)
        line = "treeFoam_colWidth "
        line += " " + str(colWidth[0])
        line += " " + str(colWidth[1])
        line += " " + str(colWidth[2])
        line += " " + str(colWidth[3])
        line += " " + str(colWidth[4]) + "\n"
        contents.append(line)
    f=open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()


#  getTreeFoamInstallDir
#    treeFoamのインストールdirを取得
def getTreeFoamInstallDir():
    homeFolder = os.path.expanduser("~")
    fileName = homeFolder + "/.treeFoamInstallFolder"
    f=open(fileName); treeFoamDir=f.readline(); f.close()
    return treeFoamDir

# initializeTreeFoam
#   初期化
def initializeTreeFoam():
    global treeFoamPath, nTreeFoam, version, clipBoard, treeSize
    #treeFoamのインストールフォルダの取得、隠しファイル作成
    treeFoamPath = os.getcwd()
    homeFolder = os.path.expanduser("~")
    fileName = homeFolder + "/.treeFoamInstallFolder"
    f=open(fileName, "w")
    f.write(treeFoamPath)
    f.close()
    #TreeFoamの起動No取得
    fileName = treeFoamPath + "/nRunningTreeFoam"
    f=open(fileName); contents = f.readlines(); f.close()
    No = -1
    for line in contents:
        NoList = line.split()
        if len(NoList) != 0:
            No = int(NoList[0])
    nTreeFoam = str(No + 1)
    contents.append(nTreeFoam + "\n")
    f=open(fileName,"w")
    for line in contents:
        f.write(line)
    f.close()
    #clipBoardを定義
    try:
        clipBoardPath = os.environ["TreeFoamPath"]
    except:
        clipBoardPath = os.environ["HOME"]
    clipBoard = clipBoardPath + "/clipBoard"
    #TeeFoamのバージョンを記録
    delFileName = treeFoamPath + "/Ver_*"
    command = "rm " + delFileName
    os.system(command)
    fileName = treeFoamPath + "/Ver_" + version
    f=open(fileName, "w"); f.close()
    print "TreeFoam ver", version, " (" + nTreeFoam + ") を起動しました。"
    #windowSizeの読み込み
    treeSize = getTreeSize()
    #OpenFOAMのバージョンを記録
    comm = ". " + pyTreeFoam.getBashrcFoam() + ";getOFversionWriteConfig.py"
    os.system(comm)

def getTreeSize():
    global treeSize, colWidth
    ans = treeSize
    fileName = os.environ["TreeFoamPath"] + "/winSize"
    if len(glob.glob(fileName)) != 0:
        f = open(fileName); lines = f.readlines(); f.close()
        for line in lines:
            words = line.split()
            if len(words) == 3:
                if words[0] == "treeFoam":
                    ans = (int(words[1]), int(words[2]))
            if len(words) == 6:
                if words[0] == "treeFoam_colWidth":
                    colWidth[0] = int(words[1])
                    colWidth[1] = int(words[2])
                    colWidth[2] = int(words[3])
                    colWidth[3] = int(words[4])
                    colWidth[4] = int(words[5])
    return ans



if __name__ == "__main__":
    #treeFoamPath = os.getcwd()
    initializeTreeFoam()
    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()
    frame_1 = MyFrame(None, -1, "")
    app.SetTopWindow(frame_1)
    frame_1.Show()
    app.MainLoop()

